@@ -51,9 +51,8 @@ from neutron.agent.l3 import l3_agent_extension_api as l3_ext_api
 from neutron.agent.l3 import l3_agent_extensions_manager as l3_ext_manager
 from neutron.agent.l3 import legacy_router
 from neutron.agent.l3 import namespace_manager
+from neutron.agent.l3 import namespaces as l3_namespaces
 from neutron.agent.linux import external_process
-from neutron.agent.linux import pd
-from neutron.agent.linux import utils as linux_utils
 from neutron.agent.metadata import driver as metadata_driver
 from neutron.agent import rpc as agent_rpc
 from neutron.common import utils
@@ -70,14 +69,13 @@ SYNC_ROUTERS_MIN_CHUNK_SIZE = 32
 PRIORITY_RELATED_ROUTER = 0
 PRIORITY_RPC = 1
 PRIORITY_SYNC_ROUTERS_TASK = 2
-PRIORITY_PD_UPDATE = 3
 
 # Actions
 DELETE_ROUTER = 1
 DELETE_RELATED_ROUTER = 2
 ADD_UPDATE_ROUTER = 3
 ADD_UPDATE_RELATED_ROUTER = 4
-PD_UPDATE = 5
+UPDATE_NETWORK = 5
 
 RELATED_ACTION_MAP = {DELETE_ROUTER: DELETE_RELATED_ROUTER,
                       ADD_UPDATE_ROUTER: ADD_UPDATE_RELATED_ROUTER}
@@ -93,7 +91,7 @@ def log_verbose_exc(message, router_payload):
                                           router_payload, indent=5))
 
 
-class L3PluginApi(object):
+class L3PluginApi:
     """Agent side of the l3 agent RPC API.
 
     API version history:
@@ -116,6 +114,7 @@ class L3PluginApi(object):
         1.11 Added get_host_ha_router_count
         1.12 Added get_networks
         1.13 Removed get_external_network_id
+        1.14 Removed process_prefix_update
     """
 
     def __init__(self, topic, host):
@@ -177,13 +176,6 @@ class L3PluginApi(object):
         return cctxt.cast(context, 'update_ha_routers_states',
                           host=self.host, states=states)
 
-    @utils.timecost
-    def process_prefix_update(self, context, prefix_update):
-        """Process prefix update whenever prefixes get changed."""
-        cctxt = self.client.prepare(version='1.6')
-        return cctxt.call(context, 'process_prefix_update',
-                          subnets=prefix_update)
-
     @utils.timecost
     def delete_agent_gateway_port(self, context, fip_net):
         """Delete Floatingip_agent_gateway_port."""
@@ -212,7 +204,7 @@ class L3PluginApi(object):
             context, 'get_networks', filters=filters, fields=fields)
 
 
-class RouterFactory(object):
+class RouterFactory:
 
     def __init__(self):
         self._routers = {}
@@ -329,17 +321,11 @@ class L3NATAgent(ha.AgentMixin,
         self._pool_size = ROUTER_PROCESS_GREENLET_MIN
         self._pool = eventlet.GreenPool(size=self._pool_size)
         self._queue = queue.ResourceProcessingQueue()
-        super(L3NATAgent, self).__init__(host=self.conf.host)
+        super().__init__(host=self.conf.host)
 
         self.target_ex_net_id = None
         self.use_ipv6 = netutils.is_ipv6_enabled()
 
-        self.pd = pd.PrefixDelegation(self.context, self.process_monitor,
-                                      self.driver,
-                                      self.plugin_rpc.process_prefix_update,
-                                      self.create_pd_router_update,
-                                      self.conf)
-
         # Consume network updates to trigger router resync
         consumers = [[topics.NETWORK, topics.UPDATE]]
         agent_rpc.create_consumers([self], topics.AGENT, consumers)
@@ -369,30 +355,23 @@ class L3NATAgent(ha.AgentMixin,
         if self.ha_router_count <= 0:
             return
 
-        # HA routers VRRP (keepalived) process count
-        vrrp_pcount = linux_utils.get_process_count_by_name("keepalived")
-        LOG.debug("VRRP process count %s.", vrrp_pcount)
-        # HA routers state change python monitor process count
-        vrrp_st_pcount = linux_utils.get_process_count_by_name(
-            "neutron-keepalived-state-change")
-        LOG.debug("neutron-keepalived-state-change process count %s.",
-                  vrrp_st_pcount)
+        # Only set HA ports down when host was rebooted so no net
+        # namespaces were still created.
+        if any(ns.startswith(l3_namespaces.NS_PREFIX) for ns in
+               self.namespaces_manager.list_all()):
+            LOG.debug("Network configuration already done. Skipping"
+                      " set HA port to DOWN state.")
+            return
 
-        # Due to the process structure design of keepalived and the current
-        # config of l3-ha router, it will run one main 'keepalived' process
-        # and a child  'VRRP' process. So in the following check, we divided
-        # number of processes by 2 to match the ha router count.
-        if (not (vrrp_pcount / 2 >= self.ha_router_count and
-                 vrrp_st_pcount >= self.ha_router_count)):
-            LOG.debug("Call neutron server to set HA port to DOWN state.")
-            try:
-                # We set HA network port status to DOWN to let l2 agent
-                # update it to ACTIVE after wiring. This allows us to spawn
-                # keepalived only when l2 agent finished wiring the port.
-                self.plugin_rpc.update_all_ha_network_port_statuses(
-                    self.context)
-            except Exception:
-                LOG.exception('update_all_ha_network_port_statuses failed')
+        LOG.debug("Call neutron server to set HA port to DOWN state.")
+        try:
+            # We set HA network port status to DOWN to let l2 agent
+            # update it to ACTIVE after wiring. This allows us to spawn
+            # keepalived only when l2 agent finished wiring the port.
+            self.plugin_rpc.update_all_ha_network_port_statuses(
+                self.context)
+        except Exception:
+            LOG.exception('update_all_ha_network_port_statuses failed')
 
     def _register_router_cls(self, factory):
         factory.register([], legacy_router.LegacyRouter)
@@ -481,8 +460,11 @@ class L3NATAgent(ha.AgentMixin,
 
     def _router_added(self, router_id, router):
         ri = self._create_router(router_id, router)
-        registry.notify(resources.ROUTER, events.BEFORE_CREATE,
-                        self, router=ri)
+        registry.publish(resources.ROUTER, events.BEFORE_CREATE, self,
+                         payload=events.DBEventPayload(
+                             self.context,
+                             resource_id=router_id,
+                             states=(ri,)))
 
         self.router_info[router_id] = ri
 
@@ -491,27 +473,31 @@ class L3NATAgent(ha.AgentMixin,
             ri.initialize(self.process_monitor)
         except Exception:
             with excutils.save_and_reraise_exception():
-                del self.router_info[router_id]
                 LOG.exception('Error while initializing router %s',
                               router_id)
-                self.namespaces_manager.ensure_router_cleanup(router_id)
-                try:
-                    ri.delete()
-                except Exception:
-                    LOG.exception('Error while deleting router %s',
-                                  router_id)
+                self._cleanup_failed_router(router_id, delete_router_info=True)
 
         self._resize_process_pool()
 
+    def _cleanup_failed_router(self, router_id, delete_router_info):
+        ri = self.router_info.pop(router_id)
+        self.namespaces_manager.ensure_router_cleanup(router_id)
+        try:
+            if delete_router_info:
+                ri.delete()
+        except Exception:
+            LOG.exception('Error while deleting router %s',
+                          router_id)
+
     def _safe_router_removed(self, router_id):
         """Try to delete a router and return True if successful."""
         # The l3_ext_manager API expects a router dict, look it up
         ri = self.router_info.get(router_id)
 
         try:
-            self._router_removed(ri, router_id)
             if ri:
                 self.l3_ext_manager.delete_router(self.context, ri.router)
+            self._router_removed(ri, router_id)
         except Exception:
             LOG.exception('Error while deleting router %s', router_id)
             return False
@@ -546,8 +532,14 @@ class L3NATAgent(ha.AgentMixin,
         except Exception:
             with excutils.save_and_reraise_exception():
                 self.router_info[router_id] = ri
+        LOG.debug("Router info %s delete action done, "
+                  "and it was removed from cache.", router_id)
 
-        registry.notify(resources.ROUTER, events.AFTER_DELETE, self, router=ri)
+        registry.publish(resources.ROUTER, events.AFTER_DELETE, self,
+                         payload=events.DBEventPayload(
+                             self.context,
+                             resource_id=router_id,
+                             states=(ri,)))
 
     def init_extension_manager(self, connection):
         l3_ext_manager.register_opts(self.conf)
@@ -595,16 +587,29 @@ class L3NATAgent(ha.AgentMixin,
         network_id = kwargs['network']['id']
         LOG.debug("Got network %s update", network_id)
         for ri in self.router_info.values():
-            LOG.debug("Checking if router %s is plugged to the network %s",
-                      ri, network_id)
-            ports = list(ri.internal_ports)
-            if ri.ex_gw_port:
-                ports.append(ri.ex_gw_port)
-            port_belongs = lambda p: p['network_id'] == network_id
-            if any(port_belongs(p) for p in ports):
-                update = queue.ResourceUpdate(
-                    ri.router_id, PRIORITY_SYNC_ROUTERS_TASK)
-                self._resync_router(update)
+            update = queue.ResourceUpdate(ri.router_id,
+                                          PRIORITY_RPC,
+                                          action=UPDATE_NETWORK,
+                                          resource=network_id)
+            self._queue.add(update)
+
+    def _process_network_update(self, router_id, network_id):
+
+        def _port_belongs(p):
+            return p['network_id'] == network_id
+
+        ri = self.router_info.get(router_id)
+        if not ri:
+            return
+        LOG.debug("Checking if router %s is plugged to the network %s",
+                  ri, network_id)
+        ports = list(ri.internal_ports)
+        if ri.ex_gw_port:
+            ports.append(ri.ex_gw_port)
+        if any(_port_belongs(p) for p in ports):
+            update = queue.ResourceUpdate(
+                ri.router_id, PRIORITY_SYNC_ROUTERS_TASK)
+            self._resync_router(update)
 
     def _process_router_if_compatible(self, router):
         # Either ex_net_id or handle_internal_only_routers must be set
@@ -613,16 +618,38 @@ class L3NATAgent(ha.AgentMixin,
             raise l3_exc.RouterNotCompatibleWithAgent(router_id=router['id'])
 
         if router['id'] not in self.router_info:
+            LOG.debug("Router %s info not in cache, "
+                      "will do the router add action.", router['id'])
             self._process_added_router(router)
         else:
+            LOG.debug("Router %s info in cache, "
+                      "will do the router update action.", router['id'])
             self._process_updated_router(router)
 
     def _process_added_router(self, router):
         self._router_added(router['id'], router)
         ri = self.router_info[router['id']]
         ri.router = router
-        ri.process()
-        registry.notify(resources.ROUTER, events.AFTER_CREATE, self, router=ri)
+        try:
+            ri.process()
+        except Exception:
+            with excutils.save_and_reraise_exception():
+                LOG.exception('Error while processing router %s',
+                              router['id'])
+                # NOTE(slaweq): deleting of the router info in the
+                # _cleanup_failed_router is avoided as in case of error,
+                # processing of the router will be retried on next call and
+                # that may lead to some race conditions e.g. with
+                # configuration of the DVR router's FIP gateway
+                self._cleanup_failed_router(router['id'],
+                                            delete_router_info=False)
+
+        registry.publish(resources.ROUTER, events.AFTER_CREATE, self,
+                         payload=events.DBEventPayload(
+                             self.context,
+                             resource_id=router['id'],
+                             states=(ri,)))
+
         self.l3_ext_manager.add_router(self.context, router)
 
     def _process_updated_router(self, router):
@@ -671,11 +698,18 @@ class L3NATAgent(ha.AgentMixin,
                 self.check_ha_state_for_router(
                     router['id'], router.get(lib_const.HA_ROUTER_STATE_KEY))
             ri.router = router
-            registry.notify(resources.ROUTER, events.BEFORE_UPDATE,
-                            self, router=ri)
+            registry.publish(resources.ROUTER, events.BEFORE_UPDATE, self,
+                             payload=events.DBEventPayload(
+                                 self.context,
+                                 resource_id=router['id'],
+                                 states=(ri,)))
+
             ri.process()
-            registry.notify(
-                resources.ROUTER, events.AFTER_UPDATE, self, router=ri)
+            registry.publish(resources.ROUTER, events.AFTER_UPDATE, self,
+                             payload=events.DBEventPayload(
+                                 self.context,
+                                 resource_id=router['id'],
+                                 states=(None, ri)))
             self.l3_ext_manager.update_router(self.context, router)
 
     def _resync_router(self, router_update,
@@ -690,72 +724,77 @@ class L3NATAgent(ha.AgentMixin,
         router_update.resource = None  # Force the agent to resync the router
         self._queue.add(router_update)
 
-    def _process_router_update(self):
+    def _process_update(self):
         if self._exiting:
             return
 
         for rp, update in self._queue.each_update_to_next_resource():
-            LOG.info("Starting router update for %s, action %s, priority %s, "
+            LOG.info("Starting processing update %s, action %s, priority %s, "
                      "update_id %s. Wait time elapsed: %.3f",
                      update.id, update.action, update.priority,
                      update.update_id,
                      update.time_elapsed_since_create)
-            if update.action == PD_UPDATE:
-                self.pd.process_prefix_update()
-                LOG.info("Finished a router update for %s IPv6 PD, "
-                         "update_id. %s. Time elapsed: %.3f",
-                         update.id, update.update_id,
-                         update.time_elapsed_since_start)
-                continue
+            if update.action == UPDATE_NETWORK:
+                self._process_network_update(
+                    router_id=update.id,
+                    network_id=update.resource)
+            else:
+                self._process_router_update(rp, update)
 
-            routers = [update.resource] if update.resource else []
+    def _process_router_update(self, rp, update):
+        LOG.info("Starting router update for %s, action %s, priority %s, "
+                 "update_id %s. Wait time elapsed: %.3f",
+                 update.id, update.action, update.priority,
+                 update.update_id,
+                 update.time_elapsed_since_create)
 
-            not_delete_no_routers = (update.action != DELETE_ROUTER and
-                                     not routers)
-            related_action = update.action in (DELETE_RELATED_ROUTER,
-                                               ADD_UPDATE_RELATED_ROUTER)
-            if not_delete_no_routers or related_action:
-                try:
-                    update.timestamp = timeutils.utcnow()
-                    routers = self.plugin_rpc.get_routers(self.context,
-                                                          [update.id])
-                except Exception:
-                    msg = "Failed to fetch router information for '%s'"
-                    LOG.exception(msg, update.id)
-                    self._resync_router(update)
-                    continue
+        routers = [update.resource] if update.resource else []
 
-                # For a related action, verify the router is still hosted here,
-                # since it could have just been deleted and we don't want to
-                # add it back.
-                if related_action:
-                    routers = [r for r in routers if r['id'] == update.id]
-
-            if not routers:
-                removed = self._safe_router_removed(update.id)
-                if not removed:
-                    self._resync_router(update)
-                else:
-                    # need to update timestamp of removed router in case
-                    # there are older events for the same router in the
-                    # processing queue (like events from fullsync) in order to
-                    # prevent deleted router re-creation
-                    rp.fetched_and_processed(update.timestamp)
-                LOG.info("Finished a router update for %s, update_id %s. "
-                         "Time elapsed: %.3f",
-                         update.id, update.update_id,
-                         update.time_elapsed_since_start)
-                continue
-
-            if not self._process_routers_if_compatible(routers, update):
+        not_delete_no_routers = (update.action != DELETE_ROUTER and
+                                 not routers)
+        related_action = update.action in (DELETE_RELATED_ROUTER,
+                                           ADD_UPDATE_RELATED_ROUTER)
+        if not_delete_no_routers or related_action:
+            try:
+                update.timestamp = timeutils.utcnow()
+                routers = self.plugin_rpc.get_routers(self.context,
+                                                      [update.id])
+            except Exception:
+                msg = "Failed to fetch router information for '%s'"
+                LOG.exception(msg, update.id)
                 self._resync_router(update)
-                continue
+                return
 
-            rp.fetched_and_processed(update.timestamp)
-            LOG.info("Finished a router update for %s, update_id %s. "
+            # For a related action, verify the router is still hosted here,
+            # since it could have just been deleted and we don't want to
+            # add it back.
+            if related_action:
+                routers = [r for r in routers if r['id'] == update.id]
+
+        if not routers:
+            if self._safe_router_removed(update.id):
+                # need to update timestamp of removed router in case
+                # there are older events for the same router in the
+                # processing queue (like events from fullsync) in order to
+                # prevent deleted router re-creation
+                rp.fetched_and_processed(update.timestamp)
+            else:
+                self._resync_router(update)
+            LOG.info("Finished a router delete for %s, update_id %s. "
                      "Time elapsed: %.3f",
                      update.id, update.update_id,
                      update.time_elapsed_since_start)
+            return
+
+        if not self._process_routers_if_compatible(routers, update):
+            self._resync_router(update)
+            return
+
+        rp.fetched_and_processed(update.timestamp)
+        LOG.info("Finished a router update for %s, update_id %s. "
+                 "Time elapsed: %.3f",
+                 update.id, update.update_id,
+                 update.time_elapsed_since_start)
 
     def _process_routers_if_compatible(self, routers, update):
         process_result = True
@@ -804,7 +843,7 @@ class L3NATAgent(ha.AgentMixin,
     def _process_routers_loop(self):
         LOG.debug("Starting _process_routers_loop")
         while not self._exiting:
-            self._pool.spawn_n(self._process_router_update)
+            self._pool.spawn_n(self._process_update)
 
     # NOTE(kevinbenton): this is set to 1 second because the actual interval
     # is controlled by a FixedIntervalLoopingCall in neutron/service.py that
@@ -909,15 +948,6 @@ class L3NATAgent(ha.AgentMixin,
         self._context.request_id = common_context.generate_request_id()
         return self._context
 
-    def after_start(self):
-        # Note: the FWaaS' vArmourL3NATAgent is a subclass of L3NATAgent. It
-        # calls this method here. So Removing this after_start() would break
-        # vArmourL3NATAgent. We need to find out whether vArmourL3NATAgent
-        # can have L3NATAgentWithStateReport as its base class instead of
-        # L3NATAgent.
-        eventlet.spawn_n(self._process_routers_loop)
-        LOG.info("L3 agent started")
-
     def stop(self):
         LOG.info("Stopping L3 agent")
         if self.conf.cleanup_on_shutdown:
@@ -925,19 +955,11 @@ class L3NATAgent(ha.AgentMixin,
             for router in self.router_info.values():
                 router.delete()
 
-    def create_pd_router_update(self):
-        router_id = None
-        update = queue.ResourceUpdate(router_id,
-                                      PRIORITY_PD_UPDATE,
-                                      timestamp=timeutils.utcnow(),
-                                      action=PD_UPDATE)
-        self._queue.add(update)
-
 
 class L3NATAgentWithStateReport(L3NATAgent):
 
     def __init__(self, host, conf=None):
-        super(L3NATAgentWithStateReport, self).__init__(host=host, conf=conf)
+        super().__init__(host=host, conf=conf)
         self.state_rpc = agent_rpc.PluginReportStateAPI(topics.REPORTS)
         self.failed_report_state = False
         self.agent_state = {
@@ -1008,8 +1030,6 @@ class L3NATAgentWithStateReport(L3NATAgent):
         # Do the report state before we do the first full sync.
         self._report_state()
 
-        self.pd.after_start()
-
     def agent_updated(self, context, payload):
         """Handle the agent_updated notification event."""
         self.fullsync = True
