@@ -51,7 +51,7 @@ class FipNamespace(namespaces.Namespace):
 
     def __init__(self, ext_net_id, agent_conf, driver, use_ipv6):
         name = self._get_ns_name(ext_net_id)
-        super(FipNamespace, self).__init__(
+        super().__init__(
             name, agent_conf, driver, use_ipv6)
 
         self._ext_net_id = ext_net_id
@@ -199,13 +199,16 @@ class FipNamespace(namespaces.Namespace):
 
         self.agent_gateway_port = ex_gw_port
 
+        cmd = ['sysctl', '-w', 'net.ipv4.neigh.%s.proxy_delay=1' %
+               interface_name]
+        ip_wrapper.netns.execute(cmd, check_exit_code=False, privsep_exec=True)
         cmd = ['sysctl', '-w', 'net.ipv4.conf.%s.proxy_arp=1' % interface_name]
         ip_wrapper.netns.execute(cmd, check_exit_code=False, privsep_exec=True)
 
     def create(self):
         LOG.debug("DVR: add fip namespace: %s", self.name)
         # parent class will ensure the namespace exists and turn-on forwarding
-        super(FipNamespace, self).create()
+        super().create()
         ip_lib.set_ip_nonlocal_bind_for_namespace(self.name, 1,
                                                   root_namespace=True)
 
@@ -236,7 +239,7 @@ class FipNamespace(namespaces.Namespace):
 
         # TODO(mrsmith): add LOG warn if fip count != 0
         LOG.debug('DVR: destroy fip namespace: %s', self.name)
-        super(FipNamespace, self).delete()
+        super().delete()
 
     def _check_for_gateway_ip_change(self, new_agent_gateway_port):
 
@@ -351,7 +354,7 @@ class FipNamespace(namespaces.Namespace):
             gw_ip = subnet.get('gateway_ip')
             if gw_ip:
                 is_gateway_not_in_subnet = not ipam_utils.check_subnet_ip(
-                                                subnet.get('cidr'), gw_ip)
+                    subnet.get('cidr'), gw_ip)
                 if is_gateway_not_in_subnet:
                     ipd.route.add_route(gw_ip, scope='link')
                 self._add_default_gateway_for_fip(gw_ip, ipd, tbl_index)
@@ -454,7 +457,7 @@ class FipNamespace(namespaces.Namespace):
         # add default route for the link local interface
         rtr_2_fip_dev.route.add_gateway(str(fip_2_rtr.ip), table=FIP_RT_TBL)
         v6_gateway = common_utils.cidr_to_ip(
-                ip_lib.get_ipv6_lladdr(fip_2_rtr_dev.link.address))
+            ip_lib.get_ipv6_lladdr(fip_2_rtr_dev.link.address))
         rtr_2_fip_dev.route.add_gateway(v6_gateway)
 
     def scan_fip_ports(self, ri):
