@@ -15,11 +15,11 @@
 
 import errno
 from os import path
+import queue as pqueue
 import re
 import threading
 import time
 
-import eventlet
 import netaddr
 from neutron_lib import constants
 from neutron_lib import exceptions
@@ -86,6 +86,8 @@ DEVICE_NAME_PATTERN = re.compile(r"(\d+?): (\S+?):.*")
 IP_ROUTE_METRIC_DEFAULT = {constants.IP_VERSION_4: 0,
                            constants.IP_VERSION_6: 1024}
 
+ARPING_SLEEP = 2
+
 
 def remove_interface_suffix(interface):
     """Remove a possible "<if>@<endpoint>" suffix from an interface' name.
@@ -110,7 +112,7 @@ class DADFailed(AddressNotReady):
 InvalidArgument = privileged.InvalidArgument
 
 
-class SubProcessBase(object):
+class SubProcessBase:
     def __init__(self, namespace=None,
                  log_fail_as_error=True):
         self.namespace = namespace
@@ -125,12 +127,11 @@ class SubProcessBase(object):
     def _run(self, options, command, args):
         if self.namespace:
             return self._as_root(options, command, args)
-        elif self.force_root:
+        if self.force_root:
             # Force use of the root helper to ensure that commands
             # will execute in dom0 when running under XenServer/XCP.
             return self._execute(options, command, args, run_as_root=True)
-        else:
-            return self._execute(options, command, args)
+        return self._execute(options, command, args)
 
     def _as_root(self, options, command, args, use_root_namespace=False):
         namespace = self.namespace if not use_root_namespace else None
@@ -155,7 +156,7 @@ class SubProcessBase(object):
 
 class IPWrapper(SubProcessBase):
     def __init__(self, namespace=None):
-        super(IPWrapper, self).__init__(namespace=namespace)
+        super().__init__(namespace=namespace)
         self.netns = IpNetnsCommand(self)
 
     def device(self, name):
@@ -285,9 +286,9 @@ class IPWrapper(SubProcessBase):
                 return True
         return False
 
-    def add_device_to_namespace(self, device, is_ovs_port=False):
+    def add_device_to_namespace(self, device):
         if self.namespace:
-            device.link.set_netns(self.namespace, is_ovs_port=is_ovs_port)
+            device.link.set_netns(self.namespace)
 
     def add_vlan(self, name, physical_interface, vlan_id):
         privileged.create_interface(name,
@@ -344,7 +345,7 @@ class IPWrapper(SubProcessBase):
 
 class IPDevice(SubProcessBase):
     def __init__(self, name, namespace=None, kind='link'):
-        super(IPDevice, self).__init__(namespace=namespace)
+        super().__init__(namespace=namespace)
         self._name = name
         self.kind = kind
         self.link = IpLinkCommand(self)
@@ -360,8 +361,8 @@ class IPDevice(SubProcessBase):
         return self.name
 
     def __repr__(self):
-        return "<IPDevice(name=%s, namespace=%s)>" % (self._name,
-                                                      self.namespace)
+        return "<IPDevice(name={}, namespace={})>".format(self._name,
+                                                          self.namespace)
 
     def exists(self):
         """Return True if the device exists in the namespace."""
@@ -441,7 +442,7 @@ class IPDevice(SubProcessBase):
         self._name = name
 
 
-class IpDeviceCommandBase(object):
+class IpDeviceCommandBase:
 
     def __init__(self, parent):
         self._parent = parent
@@ -477,7 +478,7 @@ class IpLinkCommand(IpDeviceCommandBase):
         privileged.set_link_attribute(
             self.name, self._parent.namespace, state='down')
 
-    def set_netns(self, namespace, is_ovs_port=False):
+    def set_netns(self, namespace):
         old_namespace = self._parent.namespace
         try:
             privileged.set_link_attribute(
@@ -580,7 +581,7 @@ class IpAddrCommand(IpDeviceCommandBase):
             for filter in filters:
                 if filter == 'permanent' and device['dynamic']:
                     return False
-                elif not device[filter]:
+                if not device[filter]:
                     return False
             return True
 
@@ -639,7 +640,7 @@ class IpAddrCommand(IpDeviceCommandBase):
 class IpRouteCommand(IpDeviceCommandBase):
 
     def __init__(self, parent, table=None):
-        super(IpRouteCommand, self).__init__(parent)
+        super().__init__(parent)
         self._table = table
 
     def add_gateway(self, gateway, metric=None, table=None, scope='global'):
@@ -693,7 +694,7 @@ class IpRouteCommand(IpDeviceCommandBase):
 
 class IPRoute(SubProcessBase):
     def __init__(self, namespace=None, table=None):
-        super(IPRoute, self).__init__(namespace=namespace)
+        super().__init__(namespace=namespace)
         self.name = None
         self.route = IpRouteCommand(self, table=table)
 
@@ -743,7 +744,7 @@ class IpNeighCommand(IpDeviceCommandBase):
                 self.delete(entry['dst'], entry['lladdr'])
 
 
-class IpNetnsCommand(object):
+class IpNetnsCommand:
 
     def __init__(self, parent):
         self._parent = parent
@@ -803,7 +804,8 @@ def device_exists(device_name, namespace=None):
     return IPDevice(device_name, namespace=namespace).exists()
 
 
-def device_exists_with_ips_and_mac(device_name, ip_cidrs, mac, namespace=None):
+def device_exists_with_ips_and_mac(device_name, ip_cidrs,
+                                   mac, namespace=None) -> bool:
     """Return True if the device with the given IP addresses and MAC address
     exists in the namespace.
     """
@@ -811,14 +813,10 @@ def device_exists_with_ips_and_mac(device_name, ip_cidrs, mac, namespace=None):
         device = IPDevice(device_name, namespace=namespace)
         if mac and mac != device.link.address:
             return False
-        device_ip_cidrs = [ip['cidr'] for ip in device.addr.list()]
-        for ip_cidr in ip_cidrs:
-            if ip_cidr not in device_ip_cidrs:
-                return False
+        device_ip_cidrs = {ip['cidr'] for ip in device.addr.list()}
     except RuntimeError:
         return False
-    else:
-        return True
+    return not bool(set(ip_cidrs) - device_ip_cidrs)
 
 
 def get_device_mac(device_name, namespace=None):
@@ -986,8 +984,7 @@ def list_network_namespaces(**kwargs):
     """
     if cfg.CONF.AGENT.use_helper_for_ns_read:
         return privileged.list_netns(**kwargs)
-    else:
-        return netns.listnetns(**kwargs)
+    return netns.listnetns(**kwargs)
 
 
 def network_namespace_exists(namespace, try_is_ready=False, **kwargs):
@@ -1006,8 +1003,7 @@ def network_namespace_exists(namespace, try_is_ready=False, **kwargs):
         nspath += '/' + namespace
         if cfg.CONF.AGENT.use_helper_for_ns_read:
             return priv_utils.path_exists(nspath)
-        else:
-            return path.exists(nspath)
+        return path.exists(nspath)
 
     try:
         privileged.open_namespace(namespace)
@@ -1066,7 +1062,7 @@ def _arping(ns_name, iface_name, address, count, log_exception):
     for i in range(count):
         if not first:
             # hopefully enough for kernel to get out of locktime loop
-            time.sleep(2)
+            time.sleep(ARPING_SLEEP)
             # On the second (and subsequent) arping calls, we can get a
             # "bind: Cannot assign requested address" error since
             # the IP address might have been deleted concurrently.
@@ -1118,8 +1114,7 @@ def _arping(ns_name, iface_name, address, count, log_exception):
 
 
 def send_ip_addr_adv_notif(
-        ns_name, iface_name, address, count=3, log_exception=True,
-        use_eventlet=True):
+        ns_name, iface_name, address, count=3, log_exception=True):
     """Send advance notification of an IP address assignment.
 
     If the address is in the IPv4 family, send gratuitous ARP.
@@ -1137,18 +1132,12 @@ def send_ip_addr_adv_notif(
     :param log_exception: (Optional) True if possible failures should be logged
                           on exception level. Otherwise they are logged on
                           WARNING level. Default is True.
-    :param use_eventlet: (Optional) True if the arping command will be spawned
-                         using eventlet, False to use Python threads
-                         (threading).
     """
     def arping():
         _arping(ns_name, iface_name, address, count, log_exception)
 
     if count > 0 and netaddr.IPAddress(address).version == 4:
-        if use_eventlet:
-            eventlet.spawn_n(arping)
-        else:
-            threading.Thread(target=arping).start()
+        threading.Thread(target=arping).start()
 
 
 def sysctl(cmd, namespace=None, log_fail_as_error=True):
@@ -1286,7 +1275,7 @@ def _parse_ip_rule(rule, ip_version):
     fwmark = rule['attrs'].get('FRA_FWMARK')
     if fwmark:
         fwmask = rule['attrs'].get('FRA_FWMASK')
-        parsed_rule['fwmark'] = '{0:#x}/{1:#x}'.format(fwmark, fwmask)
+        parsed_rule['fwmark'] = f'{fwmark:#x}/{fwmask:#x}'
     iifname = rule['attrs'].get('FRA_IIFNAME')
     if iifname:
         parsed_rule['iif'] = iifname
@@ -1540,7 +1529,7 @@ def ip_monitor(namespace, queue, event_stop, event_started):
         except EOFError:
             pass
 
-    _queue = eventlet.Queue()
+    _queue = pqueue.Queue()
     try:
         cache_devices = {}
         with privileged.get_iproute(namespace) as ip:
@@ -1557,7 +1546,7 @@ def ip_monitor(namespace, queue, event_stop, event_started):
                 ip_address = _queue.get(timeout=1)
                 LOG.debug("IP monitor %s; IP address to process: %s",
                           namespace, ip_address)
-            except eventlet.queue.Empty:
+            except pqueue.Empty:
                 continue
             if 'index' in ip_address and 'prefixlen' in ip_address:
                 index = ip_address['index']
@@ -1601,9 +1590,9 @@ def list_ip_routes(namespace, ip_version, scope=None, via=None, table=None,
     def get_proto(proto_number):
         if isinstance(proto_number, int) and proto_number in rtnl.rt_proto:
             return rtnl.rt_proto[proto_number]
-        elif isinstance(proto_number, str) and proto_number.isnumeric():
+        if isinstance(proto_number, str) and proto_number.isnumeric():
             return rtnl.rt_proto[int(proto_number)]
-        elif str(proto_number) in constants.IP_PROTOCOL_NUM_TO_NAME_MAP:
+        if str(proto_number) in constants.IP_PROTOCOL_NUM_TO_NAME_MAP:
             return constants.IP_PROTOCOL_NUM_TO_NAME_MAP[str(proto_number)]
 
     table = table if table else 'main'
@@ -1615,7 +1604,7 @@ def list_ip_routes(namespace, ip_version, scope=None, via=None, table=None,
     for route in routes:
         cidr = linux_utils.get_attr(route, 'RTA_DST')
         if cidr:
-            cidr = '%s/%s' % (cidr, route['dst_len'])
+            cidr = '{}/{}'.format(cidr, route['dst_len'])
         else:
             cidr = constants.IP_ANY[ip_version]
         table = int(linux_utils.get_attr(route, 'RTA_TABLE'))
