@@ -21,6 +21,7 @@ import re
 
 import eventlet
 import netaddr
+from neutron_lib.agent.common import constants as agent_consts
 from neutron_lib.callbacks import events as callbacks_events
 from neutron_lib.callbacks import registry as callbacks_registry
 from neutron_lib.callbacks import resources as callbacks_resources
@@ -60,17 +61,20 @@ def _replace_register(flow_params, register_number, register_value):
     try:
         reg_port = flow_params[register_value]
         del flow_params[register_value]
-        flow_params['reg{:d}'.format(register_number)] = reg_port
+        flow_params[f'reg{register_number:d}'] = reg_port
     except KeyError:
         pass
 
 
 def create_reg_numbers(flow_params):
     """Replace reg_(port|net) values with defined register numbers"""
-    _replace_register(flow_params, ovsfw_consts.REG_PORT, 'reg_port')
-    _replace_register(flow_params, ovsfw_consts.REG_NET, 'reg_net')
     _replace_register(
-        flow_params, ovsfw_consts.REG_REMOTE_GROUP, 'reg_remote_group')
+        flow_params, agent_consts.REG_PORT, agent_consts.PORT_REG_NAME)
+    _replace_register(
+        flow_params, agent_consts.REG_NET, agent_consts.NET_REG_NAME)
+    _replace_register(
+        flow_params, agent_consts.REG_REMOTE_GROUP,
+        agent_consts.REMOTE_GROUP_REG_NAME)
 
 
 def get_segmentation_id_from_other_config(bridge, port_name):
@@ -126,7 +130,7 @@ def get_tag_from_other_config(bridge, port_name):
             port_name=port_name, other_config=other_config)
 
 
-class SecurityGroup(object):
+class SecurityGroup:
     def __init__(self, id_):
         self.id = id_
         self.raw_rules = []
@@ -161,7 +165,7 @@ class SecurityGroup(object):
         return self.members.get(ethertype, [])
 
 
-class OFPort(object):
+class OFPort:
     def __init__(self, port_dict, ovs_port, vlan_tag, segment_id=None,
                  network_type=None, physical_network=None):
         self.id = port_dict['device']
@@ -213,7 +217,7 @@ class OFPort(object):
         self.neutron_port_dict = port_dict.copy()
 
 
-class SGPortMap(object):
+class SGPortMap:
     def __init__(self):
         self.ports = {}
         self.sec_groups = {}
@@ -262,7 +266,7 @@ class SGPortMap(object):
         sec_group.members = members
 
 
-class ConjIdMap(object):
+class ConjIdMap:
     """Handle conjunction ID allocations and deallocations."""
 
     CONJ_ID_BLOCK_SIZE = 8
@@ -270,7 +274,7 @@ class ConjIdMap(object):
 
     def __new__(cls, int_br):
         if not hasattr(cls, '_instance'):
-            cls._instance = super(ConjIdMap, cls).__new__(cls)
+            cls._instance = super().__new__(cls)
         return cls._instance
 
     def __init__(self, int_br):
@@ -348,7 +352,7 @@ class ConjIdMap(object):
         return a list of (remote_sg_id, conj_id), which are no longer
         in use.
         """
-        result = set([])
+        result = set()
         for k in list(self.id_map.keys()):
             if sg_id in k[0:2]:
                 conj_id = self.id_map.pop(k)
@@ -367,7 +371,7 @@ class ConjIdMap(object):
         return result
 
 
-class ConjIPFlowManager(object):
+class ConjIPFlowManager:
     """Manage conj_id allocation and remote securitygroups derived
     conjunction flows.
 
@@ -415,24 +419,26 @@ class ConjIPFlowManager(object):
             flow_state, addr_to_conj, direction, ethertype, vlan_tag)
         for conj_id_set in conj_id_to_remove:
             # Remove any remaining flow with remote SG/AG ID conj_id_to_remove
-            for current_ip, conj_ids in flow_state.items():
+            for (current_ip, current_mac), conj_ids in flow_state.items():
                 conj_ids_to_remove = conj_id_set & set(conj_ids)
-                self.driver.delete_flow_for_ip(
-                    current_ip, direction, ethertype, vlan_tag,
-                    conj_ids_to_remove)
+                self.driver.delete_flow_for_ip_and_mac(
+                    current_ip, current_mac, direction, ethertype,
+                    vlan_tag, conj_ids_to_remove)
 
         # NOTE(hangyang): Handle add/delete overlapped IPs among
         # remote security groups and remote address groups
-        removed_ips = set([str(netaddr.IPNetwork(addr[0]).cidr) for addr in (
-            set(flow_state.keys()) - set(addr_to_conj.keys()))])
+        removed_ips = {
+            str(netaddr.IPNetwork(addr).cidr)
+            for addr, _ in set(flow_state) - set(addr_to_conj)
+        }
         ip_to_conj = collections.defaultdict(set)
-        for addr, conj_ids in addr_to_conj.items():
+        for (addr, mac), conj_ids in addr_to_conj.items():
             # Addresses from remote security groups have mac addresses,
             # others from remote address groups have not.
-            ip_to_conj[str(netaddr.IPNetwork(addr[0]).cidr)].update(conj_ids)
+            ip_to_conj[str(netaddr.IPNetwork(addr).cidr)].update(conj_ids)
 
-        for addr in addr_to_conj.keys():
-            ip_cidr = str(netaddr.IPNetwork(addr[0]).cidr)
+        for addr, mac in addr_to_conj:
+            ip_cidr = str(netaddr.IPNetwork(addr).cidr)
             # When the overlapped IP in remote security group and remote
             # address group have different conjunction ids but with the
             # same priority offset, we need to combine the conj_ids together
@@ -440,7 +446,8 @@ class ConjIPFlowManager(object):
             # creation sequence.
             conj_ids = list(ip_to_conj[ip_cidr])
             conj_ids.sort()
-            if flow_state.get(addr) == conj_ids and ip_cidr not in removed_ips:
+            if (flow_state.get((addr, mac)) == conj_ids and
+                    ip_cidr not in removed_ips):
                 # When there are IP overlaps among remote security groups
                 # and remote address groups, removal of the overlapped ips
                 # from one remote group will also delete the flows for the
@@ -448,8 +455,8 @@ class ConjIPFlowManager(object):
                 # match flow priority or actions for different conjunction
                 # ids, therefore we need to recreate the affected flows.
                 continue
-            for flow in rules.create_flows_for_ip_address(
-                    addr, direction, ethertype, vlan_tag, conj_ids):
+            for flow in rules.create_flows_for_ip_address_and_mac(
+                    addr, mac, direction, ethertype, vlan_tag, conj_ids):
                 self.driver._add_flow(flow_group_id=ofport, **flow)
 
     def update_flows_for_vlan(self, vlan_tag, ofport, conj_id_to_remove=None):
@@ -611,7 +618,7 @@ class OVSFirewallDriver(firewall.FirewallDriver):
         dl_type = kwargs.get('dl_type')
         create_reg_numbers(kwargs)
         if isinstance(dl_type, int):
-            kwargs['dl_type'] = "0x{:04x}".format(dl_type)
+            kwargs['dl_type'] = f"0x{dl_type:04x}"
         if self._update_cookie:
             kwargs['cookie'] = self._update_cookie
         if self._deferred:
@@ -782,7 +789,7 @@ class OVSFirewallDriver(firewall.FirewallDriver):
             self.remove_port_filter(port)
             self._initialize_egress_no_port_security(port['device'])
             return
-        elif not self.is_port_managed(port):
+        if not self.is_port_managed(port):
             try:
                 self._remove_egress_no_port_security(port['device'])
             except exceptions.OVSFWPortNotHandled as e:
@@ -907,10 +914,10 @@ class OVSFirewallDriver(firewall.FirewallDriver):
         actions = ('set_field:{:d}->reg{:d},'
                    'set_field:{:d}->reg{:d},').format(
                        ofport,
-                       ovsfw_consts.REG_PORT,
+                       agent_consts.REG_PORT,
                        # This always needs the local vlan.
                        local_vlan,
-                       ovsfw_consts.REG_NET)
+                       agent_consts.REG_NET)
         if network_type == lib_const.TYPE_VLAN:
             actions += 'strip_vlan,resubmit(,{:d})'.format(
                 ovs_consts.BASE_INGRESS_TABLE)
@@ -959,9 +966,9 @@ class OVSFirewallDriver(firewall.FirewallDriver):
                     'set_field:{:d}->reg{:d},'
                     'resubmit(,{:d})'.format(
                         port.ofport,
-                        ovsfw_consts.REG_PORT,
+                        agent_consts.REG_PORT,
                         port.vlan_tag,
-                        ovsfw_consts.REG_NET,
+                        agent_consts.REG_NET,
                         ovs_consts.BASE_EGRESS_TABLE)
         )
 
@@ -981,9 +988,9 @@ class OVSFirewallDriver(firewall.FirewallDriver):
                         'set_field:{:d}->reg{:d},'
                         'strip_vlan,resubmit(,{:d})'.format(
                             port.ofport,
-                            ovsfw_consts.REG_PORT,
+                            agent_consts.REG_PORT,
                             port.vlan_tag,
-                            ovsfw_consts.REG_NET,
+                            agent_consts.REG_NET,
                             ovs_consts.BASE_INGRESS_TABLE),
             )
 
@@ -1048,9 +1055,9 @@ class OVSFirewallDriver(firewall.FirewallDriver):
                     'set_field:%d->reg%d,'
                     'resubmit(,%d)' % (
                         ovs_port.ofport,
-                        ovsfw_consts.REG_PORT,
+                        agent_consts.REG_PORT,
                         vlan_tag,
-                        ovsfw_consts.REG_NET,
+                        agent_consts.REG_NET,
                         ovs_consts.ACCEPT_OR_INGRESS_TABLE)
         )
         self._add_flow(
@@ -1108,6 +1115,16 @@ class OVSFirewallDriver(firewall.FirewallDriver):
                 actions='resubmit(,%d)' % (
                     ovs_consts.ACCEPTED_EGRESS_TRAFFIC_NORMAL_TABLE)
             )
+            self._add_flow(
+                table=ovs_consts.BASE_EGRESS_TABLE,
+                priority=95,
+                reg_port=port.ofport,
+                dl_type=lib_const.ETHERTYPE_RARP,
+                in_port=port.ofport,
+                dl_src=mac_addr,
+                actions='resubmit(,%d)' % (
+                    ovs_consts.ACCEPTED_EGRESS_TRAFFIC_NORMAL_TABLE)
+            )
             self._add_flow(
                 table=ovs_consts.BASE_EGRESS_TABLE,
                 priority=65,
@@ -1118,7 +1135,7 @@ class OVSFirewallDriver(firewall.FirewallDriver):
                 nw_src=ip_addr,
                 actions='ct(table={:d},zone=NXM_NX_REG{:d}[0..15])'.format(
                     ovs_consts.RULES_EGRESS_TABLE,
-                    ovsfw_consts.REG_NET)
+                    agent_consts.REG_NET)
             )
 
         # Apply mac/ip pairs for IPv6
@@ -1136,7 +1153,7 @@ class OVSFirewallDriver(firewall.FirewallDriver):
                 ipv6_src=ip_addr,
                 actions='ct(table={:d},zone=NXM_NX_REG{:d}[0..15])'.format(
                     ovs_consts.RULES_EGRESS_TABLE,
-                    ovsfw_consts.REG_NET)
+                    agent_consts.REG_NET)
             )
 
         # DHCP discovery
@@ -1224,7 +1241,7 @@ class OVSFirewallDriver(firewall.FirewallDriver):
                 reg_net=port.vlan_tag,
                 actions='set_field:{:d}->reg{:d},resubmit(,{:d})'.format(
                     port.ofport,
-                    ovsfw_consts.REG_PORT,
+                    agent_consts.REG_PORT,
                     ovs_consts.BASE_INGRESS_TABLE),
             )
         for ethertype in [lib_const.ETHERTYPE_IP, lib_const.ETHERTYPE_IPV6]:
@@ -1236,7 +1253,7 @@ class OVSFirewallDriver(firewall.FirewallDriver):
                 ct_state=ovsfw_consts.OF_STATE_NEW_NOT_ESTABLISHED,
                 actions='ct(commit,zone=NXM_NX_REG{:d}[0..15]),'
                         'resubmit(,{:d})'.format(
-                            ovsfw_consts.REG_NET,
+                            agent_consts.REG_NET,
                             ovs_consts.ACCEPTED_EGRESS_TRAFFIC_TABLE)
             )
         self._add_flow(
@@ -1266,7 +1283,7 @@ class OVSFirewallDriver(firewall.FirewallDriver):
             priority=12,
             dl_dst=mac,
             reg_net=vlan_tag,
-            actions='output:{:d}'.format(dst_port)
+            actions=f'output:{dst_port:d}'
         )
         # For packets from patch ports.
         self._add_flow(
@@ -1275,7 +1292,7 @@ class OVSFirewallDriver(firewall.FirewallDriver):
             priority=12,
             dl_dst=mac,
             dl_vlan=vlan_tag,
-            actions='strip_vlan,output:{:d}'.format(dst_port)
+            actions=f'strip_vlan,output:{dst_port:d}'
         )
 
         # The former flow may not match, that means the destination port is
@@ -1379,7 +1396,7 @@ class OVSFirewallDriver(firewall.FirewallDriver):
                 ct_state=ovsfw_consts.OF_STATE_ESTABLISHED,
                 actions="ct(commit,zone=NXM_NX_REG{:d}[0..15],"
                         "exec(set_field:{:s}->ct_mark))".format(
-                            ovsfw_consts.REG_NET,
+                            agent_consts.REG_NET,
                             ovsfw_consts.CT_MARK_INVALID)
             )
 
@@ -1397,7 +1414,7 @@ class OVSFirewallDriver(firewall.FirewallDriver):
                 dl_type=lib_const.ETHERTYPE_IPV6,
                 nw_proto=lib_const.PROTO_NUM_IPV6_ICMP,
                 icmp_type=icmp_type,
-                actions='output:{:d}'.format(port.ofport)
+                actions=f'output:{port.ofport:d}'
             )
 
     def _initialize_ingress(self, port):
@@ -1407,7 +1424,7 @@ class OVSFirewallDriver(firewall.FirewallDriver):
             priority=100,
             dl_type=lib_const.ETHERTYPE_ARP,
             reg_port=port.ofport,
-            actions='output:{:d}'.format(port.ofport)
+            actions=f'output:{port.ofport:d}'
         )
 
         # Allow custom ethertypes
@@ -1417,7 +1434,7 @@ class OVSFirewallDriver(firewall.FirewallDriver):
                 priority=100,
                 dl_type=permitted_ethertype,
                 reg_port=port.ofport,
-                actions='output:{:d}'.format(port.ofport))
+                actions=f'output:{port.ofport:d}')
 
         self._initialize_ingress_ipv6_icmp(port)
 
@@ -1433,7 +1450,7 @@ class OVSFirewallDriver(firewall.FirewallDriver):
                 nw_proto=lib_const.PROTO_NUM_UDP,
                 tp_src=src_port,
                 tp_dst=dst_port,
-                actions='output:{:d}'.format(port.ofport)
+                actions=f'output:{port.ofport:d}'
             )
 
         # Track untracked
@@ -1446,14 +1463,14 @@ class OVSFirewallDriver(firewall.FirewallDriver):
                 ct_state=ovsfw_consts.OF_STATE_NOT_TRACKED,
                 actions='ct(table={:d},zone=NXM_NX_REG{:d}[0..15])'.format(
                     ovs_consts.RULES_INGRESS_TABLE,
-                    ovsfw_consts.REG_NET)
+                    agent_consts.REG_NET)
             )
         self._add_flow(
             table=ovs_consts.BASE_INGRESS_TABLE,
             ct_state=ovsfw_consts.OF_STATE_TRACKED,
             priority=80,
             reg_port=port.ofport,
-            actions='resubmit(,{:d})'.format(ovs_consts.RULES_INGRESS_TABLE)
+            actions=f'resubmit(,{ovs_consts.RULES_INGRESS_TABLE:d})'
         )
 
     def _initialize_tracked_ingress(self, port):
@@ -1482,10 +1499,10 @@ class OVSFirewallDriver(firewall.FirewallDriver):
         # offload case. In case the explicitly_egress_direct is used the
         # pipeline don't contain action NORMAL so we don't have flood rule
         # issue.
-        actions = 'output:{:d}'.format(port.ofport)
+        actions = f'output:{port.ofport:d}'
         if (self.int_br.br.is_hw_offload_enabled and
                 not cfg.CONF.AGENT.explicitly_egress_direct):
-            actions = 'mod_vlan_vid:{:d},normal'.format(port.vlan_tag)
+            actions = f'mod_vlan_vid:{port.vlan_tag:d},normal'
         # Allow established and related connections
         for state in (ovsfw_consts.OF_STATE_ESTABLISHED_REPLY,
                       ovsfw_consts.OF_STATE_RELATED):
@@ -1514,7 +1531,7 @@ class OVSFirewallDriver(firewall.FirewallDriver):
                 ct_state=ovsfw_consts.OF_STATE_ESTABLISHED,
                 actions="ct(commit,zone=NXM_NX_REG{:d}[0..15],"
                         "exec(set_field:{:s}->ct_mark))".format(
-                            ovsfw_consts.REG_NET,
+                            agent_consts.REG_NET,
                             ovsfw_consts.CT_MARK_INVALID)
             )
 
@@ -1584,7 +1601,7 @@ class OVSFirewallDriver(firewall.FirewallDriver):
                         port, conj_id, direction, ethertype):
                     flow['actions'] = "set_field:{:d}->reg{:d},{:s}".format(
                         flow['conj_id'],
-                        ovsfw_consts.REG_REMOTE_GROUP,
+                        agent_consts.REG_REMOTE_GROUP,
                         flow['actions']
                     )
                     self._add_flow(**flow)
@@ -1611,8 +1628,7 @@ class OVSFirewallDriver(firewall.FirewallDriver):
 
     def _create_rules_generator_for_port(self, port):
         for sec_group in port.sec_groups:
-            for rule in sec_group.raw_rules:
-                yield rule
+            yield from sec_group.raw_rules
 
     def _create_remote_rules_generator_for_port(self, port):
         for sec_group in port.sec_groups:
@@ -1639,23 +1655,24 @@ class OVSFirewallDriver(firewall.FirewallDriver):
             self, flow_state, addr_to_conj, direction, ethertype, vlan_tag):
         # Remove rules for deleted IPs and action=conjunction(conj_id, 1/2)
         removed_ips = set(flow_state.keys()) - set(addr_to_conj.keys())
-        for removed_ip in removed_ips:
-            conj_ids = flow_state[removed_ip]
-            self.delete_flow_for_ip(removed_ip, direction, ethertype, vlan_tag,
-                                    conj_ids)
+        for removed_ip, removed_mac in removed_ips:
+            conj_ids = flow_state[(removed_ip, removed_mac)]
+            self.delete_flow_for_ip_and_mac(
+                removed_ip, removed_mac, direction,
+                ethertype, vlan_tag, conj_ids)
 
         if not cfg.CONF.AGENT.explicitly_egress_direct:
             return
 
-        for ip_addr in removed_ips:
+        for ip, mac in removed_ips:
             # Generate deletion template with bogus conj_id.
-            self.delete_flow_for_ip(ip_addr, direction, ethertype, vlan_tag,
-                                    [0])
+            self.delete_flow_for_ip_and_mac(
+                ip, mac, direction, ethertype, vlan_tag, [0])
 
-    def delete_flow_for_ip(self, ip_address, direction, ethertype,
-                           vlan_tag, conj_ids):
-        for flow in rules.create_flows_for_ip_address(
-                ip_address, direction, ethertype, vlan_tag, conj_ids):
+    def delete_flow_for_ip_and_mac(self, ip, mac, direction, ethertype,
+                                   vlan_tag, conj_ids):
+        for flow in rules.create_flows_for_ip_address_and_mac(
+                ip, mac, direction, ethertype, vlan_tag, conj_ids):
             # The following del statements are partly for
             # complying the OpenFlow spec. It forbids the use of
             # these field in non-strict delete flow messages, and
