@@ -16,6 +16,7 @@
 import collections
 
 import netaddr
+from neutron_lib.agent.common import constants as agent_consts
 from neutron_lib import constants as n_consts
 from neutron_lib.plugins.ml2 import ovs_constants as ovs_consts
 
@@ -171,7 +172,7 @@ def flow_priority_offset(rule, conjunction=False):
     if protocol in [n_consts.PROTO_NUM_ICMP, n_consts.PROTO_NUM_IPV6_ICMP]:
         if 'port_range_min' not in rule:
             return conj_offset + 1
-        elif 'port_range_max' not in rule:
+        if 'port_range_max' not in rule:
             return conj_offset + 2
     return conj_offset + 3
 
@@ -188,7 +189,7 @@ def create_flows_from_rule_and_port(rule, port, conjunction=False):
     flow_template = {
         'priority': 70 + flow_priority_offset(rule, conjunction),
         'dl_type': ovsfw_consts.ethertype_to_dl_type_map[ethertype],
-        'reg_port': port.ofport,
+        agent_consts.PORT_REG_NAME: port.ofport,
     }
 
     if is_valid_prefix(dst_ip_prefix):
@@ -210,7 +211,7 @@ def populate_flow_common(direction, flow_template, port):
     """Initialize common flow fields."""
     if direction == n_consts.INGRESS_DIRECTION:
         flow_template['table'] = ovs_consts.RULES_INGRESS_TABLE
-        flow_template['actions'] = "output:{:d}".format(port.ofport)
+        flow_template['actions'] = f"output:{port.ofport:d}"
     elif direction == n_consts.EGRESS_DIRECTION:
         flow_template['table'] = ovs_consts.RULES_EGRESS_TABLE
         # Traffic can be both ingress and egress, check that no ingress rules
@@ -241,10 +242,10 @@ def create_port_range_flows(flow_template, rule):
     if protocol is None:
         return []
     flows = []
-    src_port_match = '{:s}_src'.format(protocol)
+    src_port_match = f'{protocol:s}_src'
     src_port_min = rule.get('source_port_range_min')
     src_port_max = rule.get('source_port_range_max')
-    dst_port_match = '{:s}_dst'.format(protocol)
+    dst_port_match = f'{protocol:s}_dst'
     dst_port_min = rule.get('port_range_min')
     dst_port_max = rule.get('port_range_max')
 
@@ -312,7 +313,7 @@ def create_flows_for_ip_address_and_mac(ip_address, mac_address, direction,
 
     flow_template = {
         'dl_type': ovsfw_consts.ethertype_to_dl_type_map[ethertype],
-        'reg_net': vlan_tag,  # needed for project separation
+        agent_consts.NET_REG_NAME: vlan_tag,  # needed for project separation
     }
 
     ip_ver = utils.get_ip_version(ip_prefix)
@@ -348,7 +349,7 @@ def create_accept_flows(flow):
         flow['actions'] = (
             'ct(commit,zone=NXM_NX_REG{:d}[0..15]),{:s},'
             'resubmit(,{:d})'.format(
-                ovsfw_consts.REG_NET, flow['actions'],
+                agent_consts.REG_NET, flow['actions'],
                 ovs_consts.ACCEPTED_INGRESS_TRAFFIC_TABLE)
         )
     result.append(flow)
@@ -379,7 +380,7 @@ def create_conj_flows(port, conj_id, direction, ethertype):
         # The matching is redundant as it has been done by
         # conjunction(...,2/2) flows and flows can be summarized
         # without this.
-        'reg_port': port.ofport,
+        agent_consts.PORT_REG_NAME: port.ofport,
     }
     flow_template = populate_flow_common(direction, flow_template, port)
     flows = create_accept_flows(flow_template)
