@@ -17,7 +17,7 @@ from neutron_lib import constants
 
 
 def main():
-    proctitle = "%s (%s)" % (
+    proctitle = "{} ({})".format(
         constants.AGENT_PROCESS_DHCP, setproctitle.getproctitle())
     setproctitle.setproctitle(proctitle)
 
