@@ -57,7 +57,6 @@ DNSMASQ_VERSION_DHCP_RELEASE6 = '2.76'
 DNSMASQ_VERSION_HOST_ADDR6_LIST = '2.81'
 DNSMASQ_VERSION_SEGFAULT_ISSUE = '2.86'
 DIRECT_PORT_QOS_MIN_OVS_VERSION = '2.11'
-MINIMUM_DIBBLER_VERSION = '1.0.1'
 CONNTRACK_GRE_MODULE = 'nf_conntrack_proto_gre'
 OVN_NB_DB_SCHEMA_GATEWAY_CHASSIS = '5.7.0'
 OVN_NB_DB_SCHEMA_PORT_GROUP = '5.11.0'
@@ -148,15 +147,6 @@ def patch_supported():
         return port != ovs_lib.INVALID_OFPORT
 
 
-def nova_notify_supported():
-    try:
-        # pylint:disable=import-outside-toplevel
-        import neutron.notifiers.nova  # noqa since unused
-        return True
-    except ImportError:
-        return False
-
-
 def ofctl_arg_supported(cmd, **kwargs):
     """Verify if ovs-ofctl binary supports cmd with **kwargs.
 
@@ -182,8 +172,7 @@ def ofctl_arg_supported(cmd, **kwargs):
             LOG.exception("Unexpected exception while checking supported"
                           " feature via command: %s", full_args)
             return False
-        else:
-            return True
+        return True
 
 
 def arp_responder_supported():
@@ -258,7 +247,7 @@ def dnsmasq_local_service_supported():
         LOG.debug("Exception while checking dnsmasq version. "
                   "dnsmasq: No such file or directory")
         return False
-    elif returncode == 1:
+    if returncode == 1:
         return False
     return True
 
@@ -337,7 +326,7 @@ def bridge_firewalling_enabled():
     return True
 
 
-class KeepalivedIPv6Test(object):
+class KeepalivedIPv6Test:
     def __init__(self, ha_port, gw_port, gw_vip, default_gw):
         l3_config.register_l3_agent_config_opts(l3_config.OPTS, cfg.CONF)
         self.ha_port = ha_port
@@ -540,22 +529,6 @@ def conntrack_supported():
         return False
 
 
-def get_minimal_dibbler_version_supported():
-    return MINIMUM_DIBBLER_VERSION
-
-
-def dibbler_version_supported():
-    try:
-        cmd = ['dibbler-client',
-               'help']
-        out = agent_utils.execute(cmd)
-        return '-w' in out
-    except (OSError, RuntimeError, IndexError, ValueError) as e:
-        LOG.debug("Exception while checking minimal dibbler version. "
-                  "Exception: %s", e)
-        return False
-
-
 def _fix_ip_nonlocal_bind_root_value(original_value):
     current_value = ip_lib.get_ip_nonlocal_bind(namespace=None)
     if current_value != original_value:
