@@ -45,6 +45,13 @@ IPTABLES_MULTIPORT_ONLY_PROTOCOLS = [
 IPV6_ICMP_LEGACY_PROTO_LIST = [constants.PROTO_NAME_ICMP,
                                constants.PROTO_NAME_IPV6_ICMP_LEGACY]
 
+# Protocol 'any', which is treated the same way as 'no protocol' in the API.
+# The openstack client changes 'any' to None for the API call, but we want
+# to accept both as being the same, for example, if it was passed directly
+# via the curl command.
+PROTO_NAME_ANY = 'any'
+SG_RULE_PROTO_ANY = (None, PROTO_NAME_ANY)
+
 # Number of resources for neutron agent side functions to deal
 # with large sets.
 # Setting this value does not count on special conditions, it is just a human
@@ -87,12 +94,9 @@ TRAIT_NETWORK_TUNNEL = 'CUSTOM_NETWORK_TUNNEL_PROVIDER'
 # The lowest binding index for L3 agents and DHCP agents.
 LOWEST_AGENT_BINDING_INDEX = 1
 
-# Neutron-lib defines this with a /64 but it should be /128
-METADATA_V6_CIDR = constants.METADATA_V6_IP + '/128'
-
-# TODO(haleyb): move this constant to neutron_lib.constants
-IPV4_MIN_MTU = 68
-
-
 # TODO(ralonsoh): move this constant to neutron_lib.plugins.ml2.ovs_constants
 DEFAULT_BR_INT = 'br-int'
+
+# TODO(bpetermann): Remove here after neutron-lib is bumped
+DEVICE_OWNER_VPN_ROUTER_GW = (constants.DEVICE_OWNER_NETWORK_PREFIX +
+                              "vpn_router_gateway")
