@@ -63,8 +63,8 @@ def acl_direction(r, port=None, port_group=None):
         portdir = 'inport'
 
     if port:
-        return '%s == "%s"' % (portdir, port['id'])
-    return '%s == @%s' % (portdir, port_group)
+        return '{} == "{}"'.format(portdir, port['id'])
+    return '{} == @{}'.format(portdir, port_group)
 
 
 def acl_ethertype(r):
@@ -86,7 +86,7 @@ def acl_remote_ip_prefix(r, ip_version):
     if not r['normalized_cidr']:
         return ''
     src_or_dst = 'src' if r['direction'] == const.INGRESS_DIRECTION else 'dst'
-    return ' && %s.%s == %s' % (
+    return ' && {}.{} == {}'.format(
         ip_version, src_or_dst, r['normalized_cidr'])
 
 
@@ -148,8 +148,7 @@ def add_acls_for_drop_port_group(pg_name):
                "name": [],
                "severity": [],
                "direction": direction,
-               "match": '%s == @%s && ip' % (p, pg_name),
-               "meter": []}
+               "match": '{} == @{} && ip'.format(p, pg_name)}
         acl_list.append(acl)
     return acl_list
 
@@ -167,8 +166,7 @@ def drop_all_ip_traffic_for_port(port):
                "name": [],
                "severity": [],
                "direction": direction,
-               "match": '%s == "%s" && ip' % (p, port['id']),
-               "meter": [],
+               "match": '{} == "{}" && ip'.format(p, port['id']),
                "external_ids": {'neutron:lport': port['id']}}
         acl_list.append(acl)
     return acl_list
@@ -189,7 +187,6 @@ def add_sg_rule_acl_for_port_group(port_group, r, stateful, match):
            "severity": [],
            "direction": dir_map[r['direction']],
            "match": match,
-           "meter": [],
            ovn_const.OVN_SG_RULE_EXT_ID_KEY: r['id']}
     return acl
 
@@ -197,33 +194,29 @@ def add_sg_rule_acl_for_port_group(port_group, r, stateful, match):
 def _get_subnet_from_cache(plugin, admin_context, subnet_cache, subnet_id):
     if subnet_id in subnet_cache:
         return subnet_cache[subnet_id]
-    else:
-        subnet = plugin.get_subnet(admin_context, subnet_id)
-        if subnet:
-            subnet_cache[subnet_id] = subnet
-        return subnet
+    subnet = plugin.get_subnet(admin_context, subnet_id)
+    if subnet:
+        subnet_cache[subnet_id] = subnet
+    return subnet
 
 
 def _get_sg_ports_from_cache(plugin, admin_context, sg_ports_cache, sg_id):
     if sg_id in sg_ports_cache:
         return sg_ports_cache[sg_id]
-    else:
-        filters = {'security_group_id': [sg_id]}
-        sg_ports = plugin._get_port_security_group_bindings(
-            admin_context, filters)
-        if sg_ports:
-            sg_ports_cache[sg_id] = sg_ports
-        return sg_ports
+    filters = {'security_group_id': [sg_id]}
+    sg_ports = plugin._get_port_security_group_bindings(admin_context, filters)
+    if sg_ports:
+        sg_ports_cache[sg_id] = sg_ports
+    return sg_ports
 
 
 def _get_sg_from_cache(plugin, admin_context, sg_cache, sg_id):
     if sg_id in sg_cache:
         return sg_cache[sg_id]
-    else:
-        sg = plugin.get_security_group(admin_context, sg_id)
-        if sg:
-            sg_cache[sg_id] = sg
-        return sg
+    sg = plugin.get_security_group(admin_context, sg_id)
+    if sg:
+        sg_cache[sg_id] = sg
+    return sg
 
 
 def acl_remote_group_id(r, ip_version):
@@ -233,7 +226,7 @@ def acl_remote_group_id(r, ip_version):
     src_or_dst = 'src' if r['direction'] == const.INGRESS_DIRECTION else 'dst'
     addrset_name = utils.ovn_pg_addrset_name(r['remote_group_id'],
                                              ip_version)
-    return ' && %s.%s == $%s' % (ip_version, src_or_dst, addrset_name)
+    return ' && {}.{} == ${}'.format(ip_version, src_or_dst, addrset_name)
 
 
 def acl_remote_address_group_id(r, ip_version):
@@ -277,15 +270,12 @@ def _acl_columns_name_severity_supported(nb_idl):
     return ('name' in columns) and ('severity' in columns)
 
 
-def is_sg_stateful(sg, stateless_supported):
-    if stateless_supported:
-        return sg.get("stateful", True)
-    return True
+def is_sg_stateful(sg):
+    return sg.get("stateful", True)
 
 
-def add_acls_for_sg_port_group(ovn, security_group, txn,
-                               stateless_supported=True):
-    stateful = is_sg_stateful(security_group, stateless_supported)
+def add_acls_for_sg_port_group(ovn, security_group, txn):
+    stateful = is_sg_stateful(security_group)
     for r in security_group['security_group_rules']:
         acl = _add_sg_rule_acl_for_port_group(
             utils.ovn_port_group_name(security_group['id']), stateful, r)
@@ -297,30 +287,21 @@ def update_acls_for_security_group(plugin,
                                    ovn,
                                    security_group_id,
                                    security_group_rule,
-                                   is_add_acl=True,
-                                   stateless_supported=True):
+                                   is_add_acl=True):
 
     # Skip ACLs if security groups aren't enabled
     if not is_sg_enabled():
         return
 
-    # It's possible to have a security group created on one controller and
-    # then a security group rule created on a different controller quickly
-    # enough that the second controller does not yet see that security group
-    # in its local cache of the OVN northbound database. Check if the port
-    # group is present or not in the idl's local copy of the database before
-    # creating the security group rule.
-    pg_name = utils.ovn_port_group_name(security_group_id)
-    ovn.check_for_row_by_value_and_retry('Port_Group', 'name', pg_name)
-
     # Check if ACL log name and severity supported or not
     keep_name_severity = _acl_columns_name_severity_supported(ovn)
 
     sg = plugin.get_security_group(admin_context, security_group_id)
-    stateful = is_sg_stateful(sg, stateless_supported)
+    stateful = is_sg_stateful(sg)
 
     acl = _add_sg_rule_acl_for_port_group(
-        pg_name, stateful, security_group_rule)
+        utils.ovn_port_group_name(security_group_id),
+        stateful, security_group_rule)
     # Remove ACL log name and severity if not supported
     if is_add_acl:
         if not keep_name_severity:
