@@ -37,7 +37,7 @@ from oslo_log import log
 from oslo_serialization import jsonutils
 from oslo_utils import netutils
 from oslo_utils import strutils
-from ovsdbapp.backend.ovs_idl import rowview
+from ovsdbapp.backend.ovs_idl import idlutils
 from ovsdbapp import constants as ovsdbapp_const
 from pecan import util as pecan_util
 import tenacity
@@ -70,7 +70,7 @@ HAChassisGroupInfo = collections.namedtuple(
                            'ignore_chassis', 'external_ids'])
 
 
-class OvsdbClientCommand(object):
+class OvsdbClientCommand:
     _CONNECTION = 0
     _PRIVATE_KEY = 1
     _CERTIFICATE = 2
@@ -104,12 +104,12 @@ class OvsdbClientCommand(object):
             db = command[0]
         except IndexError:
             raise KeyError(
-                _("%s or %s schema must be specified in the command %s" % (
+                _("{} or {} schema must be specified in the command {}".format(
                     cls.OVN_Northbound, cls.OVN_Southbound, command)))
 
         if db not in (cls.OVN_Northbound, cls.OVN_Southbound):
             raise KeyError(
-                _("%s or %s schema must be specified in the command %s" % (
+                _("{} or {} schema must be specified in the command {}".format(
                     cls.OVN_Northbound, cls.OVN_Southbound, command)))
 
         cmd = ['ovsdb-client',
@@ -190,8 +190,7 @@ def ovn_context(txn_var_name='txn', idl_var_name='idl'):
                     else:
                         kwargs[txn_var_name] = new_txn
                     return f(*args, **kwargs)
-            else:
-                return f(*args, **kwargs)
+            return f(*args, **kwargs)
         return wrapped
     return decorator
 
@@ -202,7 +201,7 @@ def ovn_name(id):
     # is a UUID. If so then there will be no matches.
     # We prefix the UUID to enable us to use the Neutron UUID when
     # updating, deleting etc.
-    return "%s%s" % (constants.OVN_NAME_PREFIX, id)
+    return "{}{}".format(constants.OVN_NAME_PREFIX, id)
 
 
 def ovn_lrouter_port_name(id):
@@ -249,7 +248,7 @@ def ovn_addrset_name(sg_id, ip_version):
     #   as-<ip version>-<security group uuid>
     # with all '-' replaced with '_'. This replacement is necessary
     # because OVN doesn't support '-' in an address set name.
-    return ('as-%s-%s' % (ip_version, sg_id)).replace('-', '_')
+    return ('as-{}-{}'.format(ip_version, sg_id)).replace('-', '_')
 
 
 def ovn_pg_addrset_name(sg_id, ip_version):
@@ -258,7 +257,16 @@ def ovn_pg_addrset_name(sg_id, ip_version):
     #   pg-<security group uuid>-<ip version>
     # with all '-' replaced with '_'. This replacement is necessary
     # because OVN doesn't support '-' in an address set name.
-    return ('pg-%s-%s' % (sg_id, ip_version)).replace('-', '_')
+    return ('pg-{}-{}'.format(sg_id, ip_version)).replace('-', '_')
+
+
+def ovn_ag_addrset_name(ag_id, ip_version):
+    # The name of the address set for the given address group id and ip
+    # version. The format is:
+    #   ag-<address group uuid>-<ip version>
+    # with all '-' replaced with '_'. This replacement is necessary
+    # because OVN doesn't support '-' in an address set name.
+    return ('ag-%s-%s' % (ag_id, ip_version)).replace('-', '_')
 
 
 def ovn_port_group_name(sg_id):
@@ -386,8 +394,8 @@ def get_lsp_security_groups(port, skip_trusted_port=True):
 def is_lsp_enabled(lsp):
     """Return if a Logical Switch Port is enabled
 
-    This method mimics the OVN northd method ``lsp_is_enabled``
-    https://shorturl.at/rBSZ7. The "enabled" field can have three values:
+    This method mimics the OVN northd method ``lsp_is_enabled``. The "enabled"
+    field can have three values:
     * []: from older OVN versions, in this case the LSP is enabled by default.
     * [True]: the LSP is enabled.
     * [False]: the LSP is disabled.
@@ -401,8 +409,7 @@ def is_lsp_enabled(lsp):
 def is_lsp_up(lsp):
     """Return if a Logical Switch Port is UP
 
-    This method mimics the OVN northd method ``lsp_is_up``
-    https://shorturl.at/aoKR6
+    This method mimics the OVN northd method ``lsp_is_up``.
 
     :param lsp: ``ovs.db.Row`` with a Logical Switch Port register.
     :return: True if the port is UP, False if not.
@@ -549,10 +556,10 @@ def get_revision_number(resource, resource_type):
                          constants.TYPE_ROUTERS,
                          constants.TYPE_ROUTER_PORTS,
                          constants.TYPE_SECURITY_GROUPS,
+                         constants.TYPE_ADDRESS_GROUPS,
                          constants.TYPE_FLOATINGIPS, constants.TYPE_SUBNETS):
         return resource['revision_number']
-    else:
-        raise ovn_exc.UnknownResourceType(resource_type=resource_type)
+    raise ovn_exc.UnknownResourceType(resource_type=resource_type)
 
 
 def remove_macs_from_lsp_addresses(addresses):
@@ -565,9 +572,7 @@ def remove_macs_from_lsp_addresses(addresses):
     """
     ip_list = []
     for addr in addresses:
-        ip_list.extend([x for x in addr.split() if
-                        (netutils.is_valid_ipv4(x) or
-                         netutils.is_valid_ipv6(x))])
+        ip_list.extend([x for x in addr.split() if netutils.is_valid_ip(x)])
     return ip_list
 
 
@@ -627,7 +632,7 @@ def sort_ips_by_version(addresses):
 def is_lsp_router_port(neutron_port=None, lsp=None):
     if neutron_port:
         return neutron_port.get('device_owner') in const.ROUTER_PORT_OWNERS
-    elif lsp:
+    if lsp:
         return (lsp.external_ids.get(constants.OVN_DEVICE_OWNER_EXT_ID_KEY) in
                 const.ROUTER_PORT_OWNERS)
     return False
@@ -666,7 +671,7 @@ def get_system_dns_resolvers(resolver_file=DNS_RESOLVER_FILE):
     if not os.path.exists(resolver_file):
         return resolvers
 
-    with open(resolver_file, 'r') as rconf:
+    with open(resolver_file) as rconf:
         for line in rconf.readlines():
             if not line.startswith('nameserver'):
                 continue
@@ -752,11 +757,11 @@ def is_gateway_chassis_invalid(chassis_name, gw_chassis,
     """
     if chassis_name not in chassis_physnets:
         return True
-    elif physnet and physnet not in chassis_physnets.get(chassis_name):
+    if physnet and physnet not in chassis_physnets.get(chassis_name):
         return True
-    elif gw_chassis and chassis_name not in gw_chassis:
+    if gw_chassis and chassis_name not in gw_chassis:
         return True
-    elif az_hints and not set(az_hints) & set(chassis_with_azs.get(
+    if az_hints and not set(az_hints) & set(chassis_with_azs.get(
             chassis_name, [])):
         return True
     return False
@@ -910,14 +915,18 @@ def parse_ovn_lb_port_forwarding(ovn_rtr_lb_pfs):
         ovn_vips = ovn_lb.vips
         for vip, ips in ovn_vips.items():
             for ip in ips.split(','):
-                fip_dict_proto.add("{} {}".format(vip, ip))
+                fip_dict_proto.add(f"{vip} {ip}")
         fip_dict[protocol] = fip_dict_proto
         result[fip_id] = fip_dict
     return result
 
 
+def get_neutron_name(ovn_name):
+    return ovn_name.replace('neutron-', '', 1)
+
+
 def get_network_name_from_datapath(datapath):
-    return datapath.external_ids['name'].replace('neutron-', '')
+    return get_neutron_name(datapath.external_ids['name'])
 
 
 def is_port_external(port):
@@ -1105,16 +1114,14 @@ def _sync_ha_chassis_group(nb_idl, hcg_info, txn):
     candidates = _filter_candidates_for_ha_chassis_group(hcg_info)
 
     # Try to get the HA Chassis Group or create if it doesn't exist
-    ha_ch_grp_cmd = txn.add(nb_idl.ha_chassis_group_add(
-        hcg_info.group_name, may_exist=True,
-        external_ids=hcg_info.external_ids))
-
-    if isinstance(ha_ch_grp_cmd.result, rowview.RowView):
-        # The HA chassis group existed before this transaction.
-        ha_ch_grp = ha_ch_grp_cmd.result
-    else:
-        # The HA chassis group is being created in this transaction.
-        ha_ch_grp = None
+    ha_ch_grp = ha_ch_grp_cmd = None
+    try:
+        ha_ch_grp = nb_idl.ha_chassis_group_get(
+            hcg_info.group_name).execute(check_error=True)
+    except idlutils.RowNotFound:
+        ha_ch_grp_cmd = txn.add(nb_idl.ha_chassis_group_add(
+            hcg_info.group_name, may_exist=True,
+            external_ids=hcg_info.external_ids))
 
     max_chassis_number = constants.MAX_CHASSIS_IN_HA_GROUP
     priority = constants.HA_CHASSIS_GROUP_HIGHEST_PRIORITY
@@ -1315,8 +1322,8 @@ def validate_port_binding_and_virtual_port(
         return
 
     fixed_ips = port.get('fixed_ips', [])
-    subnet_ids = set([fixed_ip['subnet_id'] for fixed_ip in fixed_ips
-                      if 'subnet_id' in fixed_ip])
+    subnet_ids = {fixed_ip['subnet_id'] for fixed_ip in fixed_ips
+                  if 'subnet_id' in fixed_ip}
     if not subnet_ids:
         # If the port has no fixed_ips/subnets, it cannot be virtual.
         return
@@ -1340,7 +1347,8 @@ def get_requested_chassis(requested_chassis):
 
 
 # TODO(lucasagomes): Remove this function when the additional_chassis column
-# becomes the norm and older versions of OVN are no longer supported
+# becomes the norm and older versions of OVN are no longer supported (the field
+# was added in v22.09.0)
 def is_additional_chassis_supported(idl):
     return idl.is_col_present('Port_Binding', 'additional_chassis')
 
