@@ -18,10 +18,12 @@
 
 """Utilities and helper functions."""
 from collections import abc
+import datetime
 import functools
 import hashlib
 import hmac
 import importlib
+import math
 import os
 import os.path
 import re
@@ -31,11 +33,10 @@ import socket
 import sys
 import threading
 import time
+import typing
 import uuid
 import weakref
 
-import eventlet
-from eventlet.green import subprocess
 import netaddr
 from neutron_lib.api.definitions import availability_zone as az_def
 from neutron_lib import constants as n_const
@@ -54,7 +55,6 @@ from oslo_utils import uuidutils
 from oslo_utils import versionutils
 from osprofiler import profiler
 from sqlalchemy.dialects.mysql import dialect as mysql_dialect
-from sqlalchemy.dialects.postgresql import dialect as postgresql_dialect
 from sqlalchemy.dialects.sqlite import dialect as sqlite_dialect
 from sqlalchemy.sql.expression import func as sql_func
 
@@ -76,7 +76,7 @@ class WaitTimeout(Exception):
     """Default exception coming from wait_until_true() function."""
 
 
-class LockWithTimer(object):
+class LockWithTimer:
     def __init__(self, threshold):
         self._threshold = threshold
         self.timestamp = 0
@@ -111,7 +111,7 @@ def throttler(threshold=DEFAULT_THROTTLER_VALUE):
                         LOG.debug("Call of function %s scheduled, sleeping "
                                   "%.1f seconds", fname, time_to_wait)
                         # Decorated function has been called recently, wait.
-                        eventlet.sleep(time_to_wait)
+                        time.sleep(time_to_wait)
                     lock_with_timer.timestamp = time.time()
                 finally:
                     lock_with_timer.release()
@@ -127,14 +127,6 @@ def _subprocess_setup():
     signal.signal(signal.SIGPIPE, signal.SIG_DFL)
 
 
-def subprocess_popen(args, stdin=None, stdout=None, stderr=None, env=None,
-                     preexec_fn=_subprocess_setup, close_fds=True):
-
-    return subprocess.Popen(args, stdin=stdin, stdout=stdout, stderr=stderr,
-                            preexec_fn=preexec_fn, close_fds=close_fds,
-                            env=env)
-
-
 def get_first_host_ip(net, ip_version):
     return str(netaddr.IPAddress(net.first + 1, ip_version))
 
@@ -150,8 +142,8 @@ def get_dhcp_agent_device_id(network_id, host, segmentation_id=None):
     local_hostname = host.split('.')[0]
     host_uuid = uuid.uuid5(uuid.NAMESPACE_DNS, str(local_hostname))
     if not segmentation_id:
-        return 'dhcp%s-%s' % (host_uuid, network_id)
-    return 'dhcp%s-%s-%s' % (host_uuid, network_id, segmentation_id)
+        return 'dhcp{}-{}'.format(host_uuid, network_id)
+    return 'dhcp{}-{}-{}'.format(host_uuid, network_id, segmentation_id)
 
 
 def is_dns_servers_any_address(dns_servers, ip_version):
@@ -161,7 +153,7 @@ def is_dns_servers_any_address(dns_servers, ip_version):
             netaddr.IPNetwork(dns_servers[0]).ip == ip_any)
 
 
-class exception_logger(object):
+class exception_logger:
     """Wrap a function and log raised exception
 
     :param logger: the logger to log the exception default is LOG.exception
@@ -361,7 +353,7 @@ def is_version_greater_equal(version1, version2):
             versionutils.convert_version_to_tuple(version2))
 
 
-class DelayedStringRenderer(object):
+class DelayedStringRenderer:
     """Takes a callable and its args and calls when __str__ is called
 
     Useful for when an argument to a logging statement is expensive to
@@ -383,7 +375,7 @@ def _hex_format(port, mask=0):
     def hex_str(num):
         return format(num, '#06x')
     if mask > 0:
-        return "%s/%s" % (hex_str(port), hex_str(0xffff & ~mask))
+        return "{}/{}".format(hex_str(port), hex_str(0xffff & ~mask))
     return hex_str(port)
 
 
@@ -735,18 +727,18 @@ def wait_until_true(predicate, timeout=60, sleep=1, exception=None):
     :param exception: Exception instance to raise on timeout. If None is passed
                       (default) then WaitTimeout exception is raised.
     """
-    try:
-        with eventlet.Timeout(timeout):
-            while not predicate():
-                eventlet.sleep(sleep)
-    except eventlet.Timeout:
-        if exception is not None:
-            # pylint: disable=raising-bad-type
-            raise exception
-        raise WaitTimeout(_("Timed out after %d seconds") % timeout)
+    start_time = time.time()
+
+    while not predicate():
+        elapsed_time = time.time() - start_time
+        if elapsed_time > timeout:
+            raise exception if exception else WaitTimeout(
+                _("Timed out after %d seconds") % timeout
+            )
+        time.sleep(sleep)
 
 
-class classproperty(object):
+class classproperty:
     def __init__(self, f):
         self.func = f
 
@@ -846,9 +838,13 @@ def bytes_to_bits(value):
     return value * 8
 
 
-def bits_to_kilobits(value, base):
-    # NOTE(slaweq): round up that even 1 bit will give 1 kbit as a result
-    return int((value + (base - 1)) / base)
+def bits_to_kilobits(
+        value: typing.Union[int, float],
+        base: int
+) -> int:
+    # NOTE(slaweq): round up that even 1 bit will give 1 kbit as a result, but
+    # zero will return zero too.
+    return math.ceil(value / base)
 
 
 def disable_extension_by_service_plugin(core_plugin, service_plugin):
@@ -860,7 +856,7 @@ def disable_extension_by_service_plugin(core_plugin, service_plugin):
 
 
 def get_port_fixed_ips_set(port):
-    return set([ip["ip_address"] for ip in port.get("fixed_ips", [])])
+    return {ip["ip_address"] for ip in port.get("fixed_ips", [])}
 
 
 def port_ip_changed(new_port, original_port):
@@ -917,26 +913,6 @@ def collect_profiler_info():
         }
 
 
-def spawn(func, *args, **kwargs):
-    """As eventlet.spawn() but with osprofiler initialized in the new threads
-
-    osprofiler stores the profiler instance in thread local storage, therefore
-    in new threads (including eventlet threads) osprofiler comes uninitialized
-    by default. This spawn() is a stand-in replacement for eventlet.spawn()
-    but we re-initialize osprofiler in threads spawn()-ed.
-    """
-
-    profiler_info = collect_profiler_info()
-
-    @functools.wraps(func)
-    def wrapper(*args, **kwargs):
-        if profiler_info:
-            profiler.init(**profiler_info)
-        return func(*args, **kwargs)
-
-    return eventlet.spawn(wrapper, *args, **kwargs)
-
-
 def spawn_n(func, *args, **kwargs):
     """See spawn() above"""
 
@@ -948,7 +924,11 @@ def spawn_n(func, *args, **kwargs):
             profiler.init(**profiler_info)
         return func(*args, **kwargs)
 
-    return eventlet.spawn_n(wrapper, *args, **kwargs)
+    # NOTE(ralonsoh): the thread started and returned here is not tracked;
+    # the consumer should join() the thread in order to check that it finishes.
+    _thread = threading.Thread(target=wrapper, args=args, kwargs=kwargs)
+    _thread.start()
+    return _thread
 
 
 def timecost(f):
@@ -1020,12 +1000,11 @@ def get_sql_random_method(sql_dialect_name):
     """Return the SQL random method supported depending on the dialect."""
     # NOTE(ralonsoh): this method is a good candidate to be implemented in
     # oslo.db.
-    # https://www.postgresql.org/docs/8.2/functions-math.html
     # https://www.sqlite.org/c3ref/randomness.html
-    if sql_dialect_name in (postgresql_dialect.name, sqlite_dialect.name):
+    if sql_dialect_name == sqlite_dialect.name:
         return sql_func.random
     # https://dev.mysql.com/doc/refman/8.0/en/mathematical-functions.html
-    elif sql_dialect_name == mysql_dialect.name:
+    if sql_dialect_name == mysql_dialect.name:
         return sql_func.rand
 
 
@@ -1120,8 +1099,19 @@ def read_file(path: str) -> str:
         return ''
 
 
-def is_iterable_not_string(value):
-    """Return if a value is iterable but not a string type"""
-    return (isinstance(value, abc.Iterable) and
-            not isinstance(value, abc.ByteString) and
-            not isinstance(value, str))
+def ts_to_datetime(timestamp):
+    """Converts timestamp (in seconds) to datetime"""
+    return datetime.datetime.fromtimestamp(timestamp, tz=datetime.timezone.utc)
+
+
+def datetime_to_ts(_datetime):
+    """Converts datetime to timestamp in seconds"""
+    return int(datetime.datetime.timestamp(_datetime))
+
+
+def stringmap(data: abc.Mapping[str, typing.Any],
+              default: str = '') -> dict[str, str]:
+    result = {}
+    for key, value in data.items():
+        result[key] = default if value is None else str(value)
+    return result
