@@ -11,10 +11,12 @@
 #    under the License.
 
 from oslo_config import cfg
+from oslo_config import types
 from oslo_log import log as logging
 from ovsdbapp.backend.ovs_idl import vlog
 
 from neutron._i18n import _
+from neutron.common.ovn import constants as ovn_const
 from neutron.conf.agent import ovs_conf
 
 LOG = logging.getLogger(__name__)
@@ -25,24 +27,23 @@ EXTRA_LOG_LEVEL_DEFAULTS = [
 VLOG_LEVELS = {'CRITICAL': vlog.CRITICAL, 'ERROR': vlog.ERROR, 'WARNING':
                vlog.WARN, 'INFO': vlog.INFO, 'DEBUG': vlog.DEBUG}
 
-MIGRATE_MODE = "migrate"
-
 OVN_NB_GLOBAL = "ovn_nb_global"
 
 ovn_opts = [
-    cfg.StrOpt('ovn_nb_connection',
-               default='tcp:127.0.0.1:6641',
-               help=_('The connection string for the OVN_Northbound OVSDB.\n'
-                      'Use tcp:IP:PORT for TCP connection.\n'
-                      'Use ssl:IP:PORT for SSL connection. The '
-                      'ovn_nb_private_key, ovn_nb_certificate and '
-                      'ovn_nb_ca_cert are mandatory.\n'
-                      'Use unix:FILE for unix domain socket connection.\n'
-                      'Multiple connections can be specified by a comma '
-                      'separated string. See also: '
-                      'https://github.com/openvswitch/ovs/blob'
-                      '/ab4d3bfbef37c31331db5a9dbe7c22eb8d5e5e5f'
-                      '/python/ovs/db/idl.py#L215-L216')),
+    cfg.ListOpt('ovn_nb_connection',
+                default=['tcp:127.0.0.1:6641'],
+                item_type=types.String(regex=r'^(tcp|ssl|unix):.+'),
+                help=_('The connection string for the OVN_Northbound OVSDB.\n'
+                       'Use tcp:IP:PORT for TCP connection.\n'
+                       'Use ssl:IP:PORT for SSL connection. The '
+                       'ovn_nb_private_key, ovn_nb_certificate and '
+                       'ovn_nb_ca_cert are mandatory.\n'
+                       'Use unix:FILE for unix domain socket connection.\n'
+                       'Multiple connections can be specified by a comma '
+                       'separated string. See also: '
+                       'https://github.com/openvswitch/ovs/blob'
+                       '/ab4d3bfbef37c31331db5a9dbe7c22eb8d5e5e5f'
+                       '/python/ovs/db/idl.py#L215-L216')),
     cfg.StrOpt('ovn_nb_private_key',
                default='',
                help=_('The PEM file with private key for SSL connection to '
@@ -55,19 +56,20 @@ ovn_opts = [
                default='',
                help=_('The PEM file with CA certificate that OVN should use to'
                       ' verify certificates presented to it by SSL peers')),
-    cfg.StrOpt('ovn_sb_connection',
-               default='tcp:127.0.0.1:6642',
-               help=_('The connection string for the OVN_Southbound OVSDB.\n'
-                      'Use tcp:IP:PORT for TCP connection.\n'
-                      'Use ssl:IP:PORT for SSL connection. The '
-                      'ovn_sb_private_key, ovn_sb_certificate and '
-                      'ovn_sb_ca_cert are mandatory.\n'
-                      'Use unix:FILE for unix domain socket connection.\n'
-                      'Multiple connections can be specified by a comma '
-                      'separated string. See also: '
-                      'https://github.com/openvswitch/ovs/blob'
-                      '/ab4d3bfbef37c31331db5a9dbe7c22eb8d5e5e5f'
-                      '/python/ovs/db/idl.py#L215-L216')),
+    cfg.ListOpt('ovn_sb_connection',
+                default=['tcp:127.0.0.1:6642'],
+                item_type=types.String(regex=r'^(tcp|ssl|unix):.+'),
+                help=_('The connection string for the OVN_Southbound OVSDB.\n'
+                       'Use tcp:IP:PORT for TCP connection.\n'
+                       'Use ssl:IP:PORT for SSL connection. The '
+                       'ovn_sb_private_key, ovn_sb_certificate and '
+                       'ovn_sb_ca_cert are mandatory.\n'
+                       'Use unix:FILE for unix domain socket connection.\n'
+                       'Multiple connections can be specified by a comma '
+                       'separated string. See also: '
+                       'https://github.com/openvswitch/ovs/blob'
+                       '/ab4d3bfbef37c31331db5a9dbe7c22eb8d5e5e5f'
+                       '/python/ovs/db/idl.py#L215-L216')),
     cfg.StrOpt('ovn_sb_private_key',
                default='',
                help=_('The PEM file with private key for SSL connection to '
@@ -97,32 +99,34 @@ ovn_opts = [
                       'will be forced to at least 1000 milliseconds. Defaults '
                       'to 60 seconds.')),
     cfg.StrOpt('neutron_sync_mode',
-               default='log',
-               choices=('off', 'log', 'repair', MIGRATE_MODE),
+               default=ovn_const.OVN_DB_SYNC_MODE_LOG,
+               choices=[(ovn_const.OVN_DB_SYNC_MODE_OFF,
+                         "Synchronization is off."),
+                        (ovn_const.OVN_DB_SYNC_MODE_LOG,
+                         "During neutron-server startup, check to see if OVN "
+                         "is in sync with the Neutron database. "
+                         "Log warnings for any inconsistencies found so that "
+                         "an admin can investigate."),
+                        (ovn_const.OVN_DB_SYNC_MODE_REPAIR,
+                         "During neutron-server startup, automatically create "
+                         "resources found in Neutron but not in OVN. "
+                         "Also remove resources from OVN that are no longer "
+                         "found in Neutron."),
+                        (ovn_const.OVN_DB_SYNC_MODE_MIGRATE,
+                         "This mode is to OVS to OVN migration. "
+                         "It will sync the DB just like repair mode but it "
+                         "will additionally fix the Neutron DB resource from "
+                         "OVS to OVN.")],
                help=_('The synchronization mode of OVN_Northbound OVSDB '
-                      'with Neutron DB.\n'
-                      'off - synchronization is off \n'
-                      'log - during neutron-server startup, '
-                      'check to see if OVN is in sync with '
-                      'the Neutron database. '
-                      ' Log warnings for any inconsistencies found so'
-                      ' that an admin can investigate\n'
-                      'repair - during neutron-server startup, automatically'
-                      ' create resources found in Neutron but not in OVN.'
-                      ' Also remove resources from OVN'
-                      ' that are no longer in Neutron.'
-                      '%(migrate)s - This mode is to OVS to OVN migration. It'
-                      ' will sync the DB just like repair mode but it will'
-                      ' additionally fix the Neutron DB resource from OVS to'
-                      ' OVN.') % {'migrate': MIGRATE_MODE}),
+                      'with Neutron DB.')),
     cfg.StrOpt("ovn_l3_scheduler",
-               default='leastloaded',
-               choices=('leastloaded', 'chance'),
+               default=ovn_const.OVN_L3_SCHEDULER_LEASTLOADED,
+               choices=[(ovn_const.OVN_L3_SCHEDULER_LEASTLOADED,
+                         "Select chassis with fewest gateway ports."),
+                        (ovn_const.OVN_L3_SCHEDULER_CHANCE,
+                         "Select chassis randomly.")],
                help=_('The OVN L3 Scheduler type used to schedule router '
-                      'gateway ports on hypervisors/chassis.\n'
-                      'leastloaded - chassis with fewest gateway ports '
-                      'selected\n'
-                      'chance - chassis randomly selected')),
+                      'gateway ports on hypervisors/chassis.')),
     cfg.BoolOpt('enable_distributed_floating_ip',
                 default=False,
                 help=_('Enable distributed floating IP support.\n'
@@ -153,17 +157,24 @@ ovn_opts = [
                        "field is empty. If both subnet's dns_nameservers and "
                        "this option are empty, then the DNS resolvers on the "
                        "host running the neutron server will be used.")),
+    cfg.BoolOpt('dns_records_ovn_owned',
+                default=False,
+                help=_("Whether to consider DNS records local to OVN or not. "
+                       "For OVN version 24.03 and above if this option is set "
+                       "to True, DNS records will be treated local to the OVN "
+                       "controller and it will respond to the queries for the "
+                       "records and record types known to it, else it will "
+                       "forward them to the configured DNS server(s).")),
     cfg.DictOpt('ovn_dhcp4_global_options',
                 default={},
                 help=_("Dictionary of global DHCPv4 options which will be "
                        "automatically set on each subnet upon creation and "
                        "on all existing subnets when Neutron starts.\n"
                        "An empty value for a DHCP option will cause that "
-                       "option to be unset globally. Multiple values should "
-                       "be separated by semi-colon.\n"
+                       "option to be unset globally.\n"
                        "EXAMPLES:\n"
-                       "- ntp_server:1.2.3.4,wpad:1.2.3.5;1.2.3.6 - Set "
-                       "ntp_server and wpad\n"
+                       "- ntp_server:1.2.3.4,wpad:1.2.3.5 - Set ntp_server "
+                       "and wpad\n"
                        "- ntp_server:,wpad:1.2.3.5 - Unset ntp_server and "
                        "set wpad\n"
                        "See the ovn-nb(5) man page for available options.")),
@@ -173,16 +184,19 @@ ovn_opts = [
                        "automatically set on each subnet upon creation and "
                        "on all existing subnets when Neutron starts.\n"
                        "An empty value for a DHCPv6 option will cause that "
-                       "option to be unset globally. Multiple values should "
-                       "be separated by semi-colon.\n"
+                       "option to be unset globally.\n"
                        "See the ovn-nb(5) man page for available options.")),
     cfg.BoolOpt('ovn_emit_need_to_frag',
-                default=False,
+                default=True,
+                deprecated_for_removal=True,
+                deprecated_since='2025.1',
+                deprecated_reason=("The option is useful only on very old "
+                                   "Linux kernels (version < 5.2)."),
                 help=_('Configure OVN to emit "need to frag" packets in '
                        'case of MTU mismatches.\n'
-                       'Before enabling this option make sure that '
-                       'it is supported by the host kernel (version >= 5.2) '
-                       'or by checking the output of the following command:\n'
+                       'You may have to disable this option if you are '
+                       'running an old host kernel (version < 5.2). You may '
+                       'check the output of the following command:\n'
                        'ovs-appctl -t ovs-vswitchd dpif/show-dp-features '
                        'br-int | grep "Check pkt length action".')),
     cfg.BoolOpt('disable_ovn_dhcp_for_baremetal_ports',
@@ -196,15 +210,6 @@ ovn_opts = [
                        'provisioning over IPv6 this option should be set '
                        'to "True" and neutron-dhcp-agent should be used '
                        'instead. Defaults to "False".')),
-    cfg.BoolOpt('allow_stateless_action_supported',
-                default=True,
-                deprecated_for_removal=True,
-                deprecated_since="2023.1",
-                help=_('If OVN older than 21.06 is used together with '
-                       'Neutron, this option should be set to ``False`` in '
-                       'order to disable the ``stateful-security-group`` API '
-                       'extension as ``allow-stateless`` keyword is only '
-                       'supported by OVN >= 21.06.')),
     cfg.BoolOpt('localnet_learn_fdb',
                 default=False,
                 help=_('If enabled it will allow localnet ports to learn MAC '
@@ -237,6 +242,16 @@ ovn_opts = [
                        'connected to the router through any other routers, '
                        'similar to the default ML2/OVS behavior. Defaults to '
                        '"False".')),
+    cfg.StrOpt('live_migration_activation_strategy',
+               default="rarp",
+               choices=[("rarp",
+                         "Expect the hypervisor to send a Reverse ARP request "
+                         "through the migrated port after migration is "
+                         "complete."),
+                        ("",
+                         "A migrated port is immediately activated on "
+                         "the destination host.")],
+               help=_('Activation strategy to use for live migration.')),
 ]
 
 nb_global_opts = [
@@ -286,7 +301,7 @@ def list_opts():
 
 
 def get_ovn_nb_connection():
-    return cfg.CONF.ovn.ovn_nb_connection
+    return ','.join(cfg.CONF.ovn.ovn_nb_connection)
 
 
 def get_ovn_nb_private_key():
@@ -302,7 +317,7 @@ def get_ovn_nb_ca_cert():
 
 
 def get_ovn_sb_connection():
-    return cfg.CONF.ovn.ovn_sb_connection
+    return ','.join(cfg.CONF.ovn.ovn_sb_connection)
 
 
 def get_ovn_sb_private_key():
@@ -341,6 +356,10 @@ def is_ovn_distributed_floating_ip():
     return cfg.CONF.ovn.enable_distributed_floating_ip
 
 
+def get_ovn_lm_activation_strategy():
+    return cfg.CONF.ovn.live_migration_activation_strategy
+
+
 def get_ovn_vhost_sock_dir():
     return cfg.CONF.ovn.vhost_sock_dir
 
@@ -361,6 +380,10 @@ def get_dns_servers():
     return cfg.CONF.ovn.dns_servers
 
 
+def is_dns_records_ovn_owned():
+    return cfg.CONF.ovn.dns_records_ovn_owned
+
+
 def get_global_dhcpv4_opts():
     return cfg.CONF.ovn.ovn_dhcp4_global_options
 
