@@ -21,7 +21,6 @@ from neutron_lib.api.definitions import external_net as extnet_def
 from neutron_lib.api.definitions import ip_allocation as ipalloc_apidef
 from neutron_lib.api.definitions import port as port_def
 from neutron_lib.api.definitions import portbindings as portbindings_def
-from neutron_lib.api.definitions import provider_net as pnet_def
 from neutron_lib.api.definitions import subnetpool as subnetpool_def
 from neutron_lib.api import validators
 from neutron_lib.callbacks import events
@@ -58,7 +57,6 @@ from neutron.common import utils
 from neutron.conf import experimental as c_exp
 from neutron.db import db_base_plugin_common
 from neutron.db import ipam_pluggable_backend
-from neutron.db.models import segment as segment_db
 from neutron.db import models_v2
 from neutron.db import rbac_db_mixin as rbac_mixin
 from neutron.db import rbac_db_models
@@ -92,8 +90,8 @@ def _ensure_subnet_not_used(context, subnet_id):
 
 
 def _update_subnetpool_dict(orig_pool, new_pool):
-    updated = dict((k, v) for k, v in orig_pool.to_dict().items()
-                   if k not in orig_pool.synthetic_fields or k == 'shared')
+    updated = {k: v for k, v in orig_pool.to_dict().items()
+               if k not in orig_pool.synthetic_fields or k == 'shared'}
 
     new_pool = new_pool.copy()
     new_prefixes = new_pool.pop('prefixes', constants.ATTR_NOT_SPECIFIED)
@@ -132,33 +130,6 @@ def _port_filter_hook(context, original_model, conditions):
     return conditions
 
 
-def _network_result_filter_hook(query, filters):
-    # This filter matches the provider network attributes, defined in
-    # ``neutron_lib.api.definitions.provider_net.ATTRIBUTES``.
-    attr_to_field = {
-        pnet_def.NETWORK_TYPE: segment_db.NetworkSegment.network_type,
-        pnet_def.PHYSICAL_NETWORK: segment_db.NetworkSegment.physical_network,
-        pnet_def.SEGMENTATION_ID: segment_db.NetworkSegment.segmentation_id
-    }
-
-    if any(attr for attr in pnet_def.ATTRIBUTES if attr in filters):
-        query = query.join(
-            segment_db.NetworkSegment,
-            segment_db.NetworkSegment.network_id == models_v2.Network.id)
-    for attr in pnet_def.ATTRIBUTES:
-        if attr not in filters:
-            continue
-
-        value = filters[attr]
-        field = attr_to_field[attr]
-        if utils.is_iterable_not_string(value):
-            query = query.filter(field.in_(value))
-        else:
-            query = query.filter(field == value)
-
-    return query
-
-
 @registry.has_registry_receivers
 class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
                         neutron_plugin_base_v2.NeutronPluginBaseV2,
@@ -193,13 +164,7 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
             query_hook=_port_query_hook,
             filter_hook=_port_filter_hook,
             result_filters=None)
-        model_query.register_hook(
-            models_v2.Network,
-            'network',
-            query_hook=None,
-            filter_hook=None,
-            result_filters=_network_result_filter_hook)
-        return super(NeutronDbPluginV2, cls).__new__(cls, *args, **kwargs)
+        return super().__new__(cls, *args, **kwargs)
 
     @staticmethod
     def _validate_ipv6_pd():
@@ -287,11 +252,11 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
             tenant_to_check = policy['target_project']
 
         if tenant_to_check:
-            self.ensure_no_tenant_ports_on_network(
+            self.ensure_no_project_ports_on_network(
                 context, net['id'], net['tenant_id'], tenant_to_check)
 
-    def ensure_no_tenant_ports_on_network(self, context, network_id,
-                                          net_tenant_id, tenant_id):
+    def ensure_no_project_ports_on_network(self, context, network_id,
+                                           net_tenant_id, tenant_id):
         elevated = context.elevated()
         with db_api.CONTEXT_READER.using(elevated):
             ports = model_query.query_with_hooks(
@@ -944,7 +909,8 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
         if has_cidr:
             # turn the CIDR into a proper subnet
             net = netaddr.IPNetwork(s['cidr'])
-            subnet['subnet']['cidr'] = '%s/%s' % (net.network, net.prefixlen)
+            subnet['subnet']['cidr'] = '{}/{}'.format(
+                net.network, net.prefixlen)
 
         subnetpool_id = self._get_subnetpool_id(context, s)
         if not subnetpool_id and not has_cidr:
@@ -1038,7 +1004,7 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
 
         if new_cidr and ipv6_utils.is_ipv6_pd_enabled(s):
             # This is an ipv6 prefix delegation-enabled subnet being given an
-            # updated cidr by the process_prefix_update RPC
+            # updated cidr by the plugin.
             s['cidr'] = netaddr.IPNetwork(new_cidr, s['ip_version'])
             # Update gateway_ip and allocation pools based on new cidr
             s['gateway_ip'] = utils.get_first_host_ip(
@@ -1303,8 +1269,8 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
             context,
             network_id=affected_source_network_ids,
             ip_version=ip_version)
-        all_affected_subnet_ids = set(
-            [subnet.id for subnet in all_network_subnets])
+        all_affected_subnet_ids = {
+            subnet.id for subnet in all_network_subnets}
 
         # Use set difference to identify the subnets that would be
         # violating address scope affinity constraints if the subnet
