@@ -21,13 +21,14 @@ from oslo_utils import uuidutils
 from neutron._i18n import _
 from neutron.common import utils as common_utils
 from neutron.ipam import exceptions as ipam_exc
+from neutron.ipam import utils as ipam_utils
 
 
-class SubnetPool(object, metaclass=abc.ABCMeta):
+class SubnetPool(metaclass=abc.ABCMeta):
     """Represents a pool of IPs available inside an address scope."""
 
 
-class SubnetRequest(object, metaclass=abc.ABCMeta):
+class SubnetRequest(metaclass=abc.ABCMeta):
     """Carries the data needed to make a subnet request
 
     The data validated and carried by an instance of this class is the data
@@ -67,7 +68,7 @@ class SubnetRequest(object, metaclass=abc.ABCMeta):
                 if previous and pool.first <= previous.last:
                     raise ValueError(_("Ranges must not overlap"))
                 previous = pool
-            if 1 < len(allocation_pools):
+            if len(allocation_pools) > 1:
                 # Checks that all the ranges are in the same IP version.
                 # IPRange sorts first by ip version so we can get by with just
                 # checking the first and the last range having sorted them
@@ -104,11 +105,31 @@ class SubnetRequest(object, metaclass=abc.ABCMeta):
         if self.allocation_pools:
             if subnet_cidr.version != self.allocation_pools[0].version:
                 raise ipam_exc.IpamValueInvalid(_(
-                                "allocation_pools use the wrong ip version"))
+                    "allocation_pools use the wrong ip version"))
             for pool in self.allocation_pools:
                 if pool not in subnet_cidr:
                     raise ipam_exc.IpamValueInvalid(_(
-                                "allocation_pools are not in the subnet"))
+                        "allocation_pools are not in the subnet"))
+
+    @staticmethod
+    def _validate_gateway_ip_in_subnet(subnet_cidr, gateway_ip):
+        """Validates if the Gateway IP is in subnet CIDR if needed
+
+        If the Gateway (GW) IP address is in the subnet CIDR, we need to make
+        sure that the user has not used the IPs reserved to represent the
+        network or the broadcast domain.
+
+        If the Gateway is not in the subnet CIDR, we do not validate it.
+        Therefore, for such cases, it is assumed that its access is on link.
+        """
+        if not gateway_ip:
+            return
+
+        if ipam_utils.check_gateway_invalid_in_subnet(subnet_cidr, gateway_ip):
+            raise ipam_exc.IpamValueInvalid(_(
+                'Gateway IP %(gateway_ip)s cannot be the network or broadcast '
+                'IP address %(subnet_cidr)s' % {'gateway_ip': gateway_ip,
+                                                'subnet_cidr': subnet_cidr}))
 
 
 class AnySubnetRequest(SubnetRequest):
@@ -131,7 +152,7 @@ class AnySubnetRequest(SubnetRequest):
             max allowed.
         :type prefixlen: int
         """
-        super(AnySubnetRequest, self).__init__(
+        super().__init__(
             tenant_id=tenant_id,
             subnet_id=subnet_id,
             gateway_ip=gateway_ip,
@@ -163,7 +184,7 @@ class SpecificSubnetRequest(SubnetRequest):
             the version of the address scope being used.
         :type subnet: netaddr.IPNetwork or convertible to one
         """
-        super(SpecificSubnetRequest, self).__init__(
+        super().__init__(
             tenant_id=tenant_id,
             subnet_id=subnet_id,
             gateway_ip=gateway_ip,
@@ -171,6 +192,7 @@ class SpecificSubnetRequest(SubnetRequest):
 
         self._subnet_cidr = netaddr.IPNetwork(subnet_cidr)
         self._validate_with_subnet(self._subnet_cidr)
+        self._validate_gateway_ip_in_subnet(self._subnet_cidr, self.gateway_ip)
 
     @property
     def subnet_cidr(self):
@@ -181,7 +203,7 @@ class SpecificSubnetRequest(SubnetRequest):
         return self._subnet_cidr.prefixlen
 
 
-class AddressRequest(object, metaclass=abc.ABCMeta):
+class AddressRequest(metaclass=abc.ABCMeta):
     """Abstract base class for address requests"""
 
 
@@ -193,7 +215,7 @@ class SpecificAddressRequest(AddressRequest):
         :param address: The address being requested
         :type address: A netaddr.IPAddress or convertible to one.
         """
-        super(SpecificAddressRequest, self).__init__()
+        super().__init__()
         self._address = netaddr.IPAddress(address)
 
     @property
@@ -208,7 +230,7 @@ class BulkAddressRequest(AddressRequest):
         :param num_addresses: The quantity of IP addresses being requested
         :type num_addresses: int
         """
-        super(BulkAddressRequest, self).__init__()
+        super().__init__()
         self._num_addresses = num_addresses
 
     @property
@@ -229,7 +251,7 @@ class AutomaticAddressRequest(SpecificAddressRequest):
     EUI64 = 'eui64'
 
     def _generate_eui64_address(self, **kwargs):
-        if set(kwargs) != set(['prefix', 'mac']):
+        if set(kwargs) != {'prefix', 'mac'}:
             raise ipam_exc.AddressCalculationFailure(
                 address_type='eui-64',
                 reason=_('must provide exactly 2 arguments - cidr and MAC'))
@@ -254,14 +276,14 @@ class AutomaticAddressRequest(SpecificAddressRequest):
         if not address_generator:
             raise ipam_exc.InvalidAddressType(address_type=address_type)
         address = address_generator(self, **kwargs)
-        super(AutomaticAddressRequest, self).__init__(address)
+        super().__init__(address)
 
 
 class RouterGatewayAddressRequest(AddressRequest):
     """Used to request allocating the special router gateway address."""
 
 
-class AddressRequestFactory(object):
+class AddressRequestFactory:
     """Builds request using ip info
 
     Additional parameters(port and context) are not used in default
@@ -282,26 +304,28 @@ class AddressRequestFactory(object):
         """
         if ip_dict.get('ip_address'):
             return SpecificAddressRequest(ip_dict['ip_address'])
-        elif ip_dict.get('eui64_address'):
+        if ip_dict.get('eui64_address'):
             return AutomaticAddressRequest(prefix=ip_dict['subnet_cidr'],
                                            mac=ip_dict['mac'])
-        elif (port['device_owner'] == constants.DEVICE_OWNER_DHCP or
-              port['device_owner'] == constants.DEVICE_OWNER_DISTRIBUTED):
+        if (port['device_owner'] == constants.DEVICE_OWNER_DHCP or
+                port['device_owner'] == constants.DEVICE_OWNER_DISTRIBUTED):
             # preserve previous behavior of DHCP ports choosing start of pool
             return PreferNextAddressRequest()
-        else:
-            return AnyAddressRequest()
+        return AnyAddressRequest()
 
 
-class SubnetRequestFactory(object):
+class SubnetRequestFactory:
     """Builds request using subnet info"""
 
     @classmethod
     def get_request(cls, context, subnet, subnetpool):
         cidr = subnet.get('cidr')
+        cidr = cidr if validators.is_attr_set(cidr) else None
+        gateway_ip = subnet.get('gateway_ip')
+        gateway_ip = gateway_ip if validators.is_attr_set(gateway_ip) else None
         subnet_id = subnet.get('id', uuidutils.generate_uuid())
-        is_any_subnetpool_request = not validators.is_attr_set(cidr)
 
+        is_any_subnetpool_request = not (cidr or gateway_ip)
         if is_any_subnetpool_request:
             prefixlen = subnet['prefixlen']
             if not validators.is_attr_set(prefixlen):
@@ -312,9 +336,20 @@ class SubnetRequestFactory(object):
                 subnet_id,
                 common_utils.ip_version_from_int(subnetpool['ip_version']),
                 prefixlen)
-        else:
-            return SpecificSubnetRequest(subnet['tenant_id'],
-                                         subnet_id,
-                                         cidr,
-                                         subnet.get('gateway_ip'),
-                                         subnet.get('allocation_pools'))
+        alloc_pools = subnet.get('allocation_pools')
+        alloc_pools = (
+            alloc_pools if validators.is_attr_set(alloc_pools) else None)
+        if not cidr and gateway_ip:
+            prefixlen = subnet['prefixlen']
+            if not validators.is_attr_set(prefixlen):
+                prefixlen = int(subnetpool['default_prefixlen'])
+            gw_ip_net = netaddr.IPNetwork('%s/%s' % (gateway_ip, prefixlen))
+            cidr = gw_ip_net.cidr
+
+        # TODO(ralonsoh): "tenant_id" reference should be removed.
+        project_id = subnet.get('project_id') or subnet['tenant_id']
+        return SpecificSubnetRequest(project_id,
+                                     subnet_id,
+                                     cidr,
+                                     gateway_ip=gateway_ip,
+                                     allocation_pools=alloc_pools)
