@@ -15,6 +15,7 @@
 
 import contextlib
 
+from keystoneauth1 import exceptions as ks_exceptions
 from keystoneauth1 import loading as ks_loading
 from neutron_lib.callbacks import events
 from neutron_lib.callbacks import registry
@@ -31,7 +32,9 @@ from oslo_context import context as common_context
 from oslo_log import log as logging
 from oslo_utils import uuidutils
 from sqlalchemy.orm import attributes as sql_attr
+import tenacity
 
+from neutron.common import eventlet_utils
 from neutron.notifiers import batch_notifier
 
 
@@ -45,9 +48,16 @@ NEUTRON_NOVA_EVENT_STATUS_MAP = {constants.PORT_STATUS_ACTIVE: 'completed',
                                  constants.PORT_STATUS_DOWN: 'completed'}
 NOVA_API_VERSION = "2.1"
 
+NOTIFIER_ENABLE_DEFAULT = True
+# NOTE(ralonsoh): the Nova notifier can be called simultaneously by several RPC
+# callbacks from the agents (DHCP, L2), trying to update the provisioning
+# status of a port. In order to handle each context notifier enable flag, a
+# thread local variable is used.
+_notifier_store = eventlet_utils.get_threading_local()
+
 
 @registry.has_registry_receivers
-class Notifier(object):
+class Notifier:
 
     _instance = None
 
@@ -68,16 +78,14 @@ class Notifier(object):
             if ext.name == "server_external_events"]
         self.batch_notifier = batch_notifier.BatchNotifier(
             cfg.CONF.send_events_interval, self.send_events)
-        self._enabled = True
 
     @contextlib.contextmanager
     def context_enabled(self, enabled):
-        stored_enabled = self._enabled
         try:
-            self._enabled = enabled
+            _notifier_store.enable = enabled
             yield
         finally:
-            self._enabled = stored_enabled
+            _notifier_store.enable = NOTIFIER_ENABLE_DEFAULT
 
     def _get_nova_client(self):
         global_id = common_context.generate_request_id()
@@ -172,11 +180,13 @@ class Notifier(object):
         if port and self._is_compute_port(port):
             if action == 'delete_port':
                 return self._get_port_delete_event(port)
-            else:
-                return self._get_network_changed_event(port)
+            return self._get_network_changed_event(port)
 
     def _can_notify(self, port):
-        if not self._enabled:
+        if getattr(_notifier_store, 'enable', None) is None:
+            _notifier_store.enable = NOTIFIER_ENABLE_DEFAULT
+
+        if not _notifier_store.enable:
             LOG.debug("Nova notifier disabled")
             return False
 
@@ -258,15 +268,28 @@ class Notifier(object):
              'tag': port.id})
         self.send_port_status(None, None, port)
 
+    @tenacity.retry(
+        retry=tenacity.retry_if_exception_type(
+            ks_exceptions.RetriableConnectionFailure),
+        wait=tenacity.wait_exponential(multiplier=0.01, max=1),
+        stop=tenacity.stop_after_delay(1),
+        after=tenacity.after_log(LOG, logging.DEBUG))
     def send_events(self, batched_events):
         LOG.debug("Sending events: %s", batched_events)
         novaclient = self._get_nova_client()
         try:
             response = novaclient.server_external_events.create(
                 batched_events)
+        except ks_exceptions.EndpointNotFound:
+            LOG.exception("Nova endpoint not found, invalidating the session")
+            self.session.invalidate()
         except nova_exceptions.NotFound:
             LOG.debug("Nova returned NotFound for event: %s",
                       batched_events)
+        except ks_exceptions.RetriableConnectionFailure:
+            raise
+            # next clause handles all exceptions
+            # so reraise for retry decorator
         except Exception:
             LOG.exception("Failed to notify nova on events: %s",
                           batched_events)
@@ -282,11 +305,15 @@ class Notifier(object):
                 except KeyError:
                     response_error = True
                     continue
-                if code != 200:
-                    LOG.warning("Nova event: %s returned with failed "
-                                "status", event)
+                if hasattr(response, 'request_ids'):
+                    msg = f"Nova event matching {response.request_ids}"
                 else:
-                    LOG.info("Nova event response: %s", event)
+                    msg = "Nova event"
+                if code != 200:
+                    LOG.warning("%s: %s returned with failed "
+                                "status", msg, event)
+                else:
+                    LOG.info("%s response: %s", msg, event)
             if response_error:
                 LOG.error("Error response returned from nova: %s",
                           response)
