@@ -11,6 +11,7 @@
 #    under the License.
 
 import netaddr
+from neutron_lib.api.definitions import external_net
 from neutron_lib.api import validators
 from neutron_lib import constants as const
 from neutron_lib.db import model_query
@@ -64,9 +65,9 @@ class DNSNameServer(base.NeutronDbObject):
         if not _pager.sorts:
             # (NOTE) True means ASC, False is DESC
             _pager.sorts = [('order', True)]
-        return super(DNSNameServer, cls).get_objects(context, _pager,
-                                                     validate_filters,
-                                                     **kwargs)
+        return super().get_objects(context, _pager,
+                                   validate_filters,
+                                   **kwargs)
 
 
 @base.NeutronObjectRegistry.register
@@ -89,7 +90,7 @@ class Route(base.NeutronDbObject):
     @classmethod
     def modify_fields_from_db(cls, db_obj):
         # TODO(korzen) remove this method when IP and CIDR decorator ready
-        result = super(Route, cls).modify_fields_from_db(db_obj)
+        result = super().modify_fields_from_db(db_obj)
         if 'destination' in result:
             result['destination'] = net_utils.AuthenticIPNetwork(
                 result['destination'])
@@ -100,7 +101,7 @@ class Route(base.NeutronDbObject):
     @classmethod
     def modify_fields_to_db(cls, fields):
         # TODO(korzen) remove this method when IP and CIDR decorator ready
-        result = super(Route, cls).modify_fields_to_db(fields)
+        result = super().modify_fields_to_db(fields)
         if 'destination' in result:
             result['destination'] = cls.filter_to_str(result['destination'])
         if 'nexthop' in fields:
@@ -134,7 +135,7 @@ class IPAllocationPool(base.NeutronDbObject):
     @classmethod
     def modify_fields_from_db(cls, db_obj):
         # TODO(korzen) remove this method when IP and CIDR decorator ready
-        result = super(IPAllocationPool, cls).modify_fields_from_db(db_obj)
+        result = super().modify_fields_from_db(db_obj)
         if 'start' in result:
             result['start'] = netaddr.IPAddress(result['start'])
         if 'end' in result:
@@ -144,7 +145,7 @@ class IPAllocationPool(base.NeutronDbObject):
     @classmethod
     def modify_fields_to_db(cls, fields):
         # TODO(korzen) remove this method when IP and CIDR decorator ready
-        result = super(IPAllocationPool, cls).modify_fields_to_db(fields)
+        result = super().modify_fields_to_db(fields)
         if 'first_ip' in result:
             result['first_ip'] = cls.filter_to_str(result['first_ip'])
         if 'last_ip' in result:
@@ -250,7 +251,7 @@ class Subnet(base.NeutronDbObject):
     }
 
     def __init__(self, context=None, **kwargs):
-        super(Subnet, self).__init__(context, **kwargs)
+        super().__init__(context, **kwargs)
         self.add_extra_filter_name('shared')
 
     def obj_load_attr(self, attrname):
@@ -262,7 +263,7 @@ class Subnet(base.NeutronDbObject):
             return self._load_service_types()
         if attrname == 'external':
             return self._load_external()
-        super(Subnet, self).obj_load_attr(attrname)
+        super().obj_load_attr(attrname)
 
     def _load_dns_publish_fixed_ip(self, db_obj=None):
         if db_obj:
@@ -318,8 +319,23 @@ class Subnet(base.NeutronDbObject):
         setattr(self, 'external', external_network)
         self.obj_reset_changes(['external'])
 
+    @classmethod
+    def get_objects(cls, context, _pager=None, validate_filters=True,
+                    fields=None, return_db_obj=False, **kwargs):
+        external = kwargs.pop(external_net.EXTERNAL, None)
+        if isinstance(external, list):
+            external = external[0]
+        subnets = super().get_objects(
+            context, _pager=_pager, validate_filters=validate_filters,
+            fields=fields, return_db_obj=return_db_obj, **kwargs)
+
+        if external is not None:
+            return [subnet for subnet in subnets if
+                    subnet.external == external]
+        return subnets
+
     def from_db_object(self, db_obj):
-        super(Subnet, self).from_db_object(db_obj)
+        super().from_db_object(db_obj)
         self._load_dns_publish_fixed_ip(db_obj)
         self._load_shared(db_obj)
         self._load_service_types(db_obj)
@@ -328,7 +344,7 @@ class Subnet(base.NeutronDbObject):
     @classmethod
     def modify_fields_from_db(cls, db_obj):
         # TODO(korzen) remove this method when IP and CIDR decorator ready
-        result = super(Subnet, cls).modify_fields_from_db(db_obj)
+        result = super().modify_fields_from_db(db_obj)
         if 'cidr' in result:
             result['cidr'] = net_utils.AuthenticIPNetwork(result['cidr'])
         if 'gateway_ip' in result and result['gateway_ip'] is not None:
@@ -338,7 +354,7 @@ class Subnet(base.NeutronDbObject):
     @classmethod
     def modify_fields_to_db(cls, fields):
         # TODO(korzen) remove this method when IP and CIDR decorator ready
-        result = super(Subnet, cls).modify_fields_to_db(fields)
+        result = super().modify_fields_to_db(fields)
         if 'cidr' in result:
             result['cidr'] = cls.filter_to_str(result['cidr'])
         if 'gateway_ip' in result and result['gateway_ip'] is not None:
@@ -578,7 +594,7 @@ class NetworkSubnetLock(base.NeutronDbObject):
 
     @classmethod
     def lock_subnet(cls, context, network_id, subnet_id):
-        subnet_lock = super(NetworkSubnetLock, cls).get_object(
+        subnet_lock = super().get_object(
             context, network_id=network_id)
         if subnet_lock:
             subnet_lock.subnet_id = subnet_id
