@@ -19,6 +19,7 @@ from oslo_config import cfg
 
 import neutron.agent.agent_extensions_manager
 import neutron.agent.securitygroups_rpc
+import neutron.api.wsgi
 import neutron.conf.agent.agent_extensions_manager
 import neutron.conf.agent.common
 import neutron.conf.agent.database.agents_db
@@ -35,6 +36,7 @@ import neutron.conf.db.dvr_mac_db
 import neutron.conf.db.extraroute_db
 import neutron.conf.db.l3_agentschedulers_db
 import neutron.conf.db.l3_dvr_db
+import neutron.conf.db.l3_extra_gws_db
 import neutron.conf.db.l3_gwmode_db
 import neutron.conf.db.l3_hamode_db
 import neutron.conf.experimental
@@ -43,7 +45,6 @@ import neutron.conf.extensions.conntrack_helper
 import neutron.conf.plugins.ml2.config
 import neutron.conf.plugins.ml2.drivers.agent
 import neutron.conf.plugins.ml2.drivers.driver_type
-import neutron.conf.plugins.ml2.drivers.linuxbridge
 import neutron.conf.plugins.ml2.drivers.macvtap
 import neutron.conf.plugins.ml2.drivers.mech_sriov.agent_common
 import neutron.conf.plugins.ml2.drivers.mech_sriov.mech_sriov_conf
@@ -51,6 +52,7 @@ import neutron.conf.plugins.ml2.drivers.openvswitch.mech_ovs_conf
 import neutron.conf.plugins.ml2.drivers.ovs_conf
 import neutron.conf.quota
 import neutron.conf.service
+import neutron.conf.services.extdns_designate_driver
 import neutron.conf.services.logging
 import neutron.conf.services.metering_agent
 import neutron.conf.wsgi
@@ -58,7 +60,6 @@ import neutron.db.migration.cli
 import neutron.extensions.l3
 import neutron.extensions.securitygroup
 import neutron.plugins.ml2.drivers.mech_sriov.agent.common.config
-import neutron.wsgi
 
 
 AUTH_GROUPS_OPTS = {
@@ -76,7 +77,8 @@ AUTH_GROUPS_OPTS = {
         }
     },
     'ironic': {},
-    'placement': {}
+    'placement': {},
+    'designate': {}
 }
 
 CONF = cfg.CONF
@@ -116,6 +118,10 @@ def list_placement_auth_opts():
     return list_auth_opts('placement')
 
 
+def list_designate_auth_opts():
+    return list_auth_opts('designate')
+
+
 def list_agent_opts():
     return [
         ('agent',
@@ -162,7 +168,8 @@ def list_db_opts():
              neutron.conf.db.dvr_mac_db.DVR_MAC_ADDRESS_OPTS,
              neutron.conf.db.l3_dvr_db.ROUTER_DISTRIBUTED_OPTS,
              neutron.conf.db.l3_agentschedulers_db.L3_AGENTS_SCHEDULER_OPTS,
-             neutron.conf.db.l3_hamode_db.L3_HA_OPTS)
+             neutron.conf.db.l3_hamode_db.L3_HA_OPTS,
+             neutron.conf.db.l3_extra_gws_db.L3_EXTRA_GWS_OPTS)
          ),
         ('database',
          neutron.db.migration.cli.get_engine_config())
@@ -191,6 +198,9 @@ def list_opts():
          itertools.chain(
              neutron.conf.common.placement_opts)
          ),
+        ('designate',
+         neutron.conf.services.extdns_designate_driver.designate_opts
+         ),
         ('quotas', neutron.conf.quota.core_quota_opts)
     ]
 
@@ -225,28 +235,9 @@ def list_dhcp_agent_opts():
              neutron.conf.agent.dhcp.DHCP_AGENT_OPTS,
              neutron.conf.agent.dhcp.DHCP_OPTS,
              neutron.conf.agent.dhcp.DNSMASQ_OPTS)
-         )
-    ]
-
-
-def list_linux_bridge_opts():
-    return [
-        ('DEFAULT',
-         neutron.conf.service.RPC_EXTRA_OPTS),
-        ('linux_bridge',
-         neutron.conf.plugins.ml2.drivers.linuxbridge.bridge_opts),
-        ('vxlan',
-         neutron.conf.plugins.ml2.drivers.linuxbridge.vxlan_opts),
-        ('agent',
-         itertools.chain(
-             neutron.conf.plugins.ml2.drivers.agent.agent_opts,
-             neutron.conf.agent.agent_extensions_manager.
-             AGENT_EXT_MANAGER_OPTS)
          ),
-        ('securitygroup',
-         neutron.conf.agent.securitygroups_rpc.security_group_opts),
-        ('network_log',
-         neutron.conf.services.logging.log_driver_opts)
+        (meta_conf.RATE_LIMITING_GROUP,
+         meta_conf.METADATA_RATE_LIMITING_OPTS)
     ]
 
 
@@ -257,13 +248,14 @@ def list_l3_agent_opts():
              neutron.conf.agent.l3.config.OPTS,
              neutron.conf.service.SERVICE_OPTS,
              neutron.conf.agent.l3.ha.OPTS,
-             neutron.conf.agent.common.PD_DRIVER_OPTS,
              neutron.conf.agent.common.RA_OPTS)
          ),
         ('agent',
          neutron.conf.agent.agent_extensions_manager.AGENT_EXT_MANAGER_OPTS),
         ('network_log',
-         neutron.conf.services.logging.log_driver_opts)
+         neutron.conf.services.logging.log_driver_opts),
+        (meta_conf.RATE_LIMITING_GROUP,
+         meta_conf.METADATA_RATE_LIMITING_OPTS)
     ]
 
 
@@ -346,7 +338,11 @@ def list_ovs_opts():
         ('dhcp',
          itertools.chain(
              neutron.conf.plugins.ml2.drivers.ovs_conf.dhcp_opts,
-             neutron.conf.agent.common.DHCP_PROTOCOL_OPTS))
+             neutron.conf.agent.common.DHCP_PROTOCOL_OPTS)),
+        ('metadata',
+         itertools.chain(
+             neutron.conf.plugins.ml2.drivers.ovs_conf.metadata_opts,
+             meta_conf.METADATA_PROXY_HANDLER_OPTS))
     ]
 
 
