@@ -14,13 +14,11 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from neutron._i18n import _LE
+from neutron_lib.api.definitions import portbindings
 from neutron_lib import constants
+from neutron_lib.plugins.ml2 import api
 from oslo_log import log
 
-from neutron.extensions import portbindings
-from neutron.plugins.common import constants as p_constants
-from neutron.plugins.ml2 import driver_api as api
 from neutron.plugins.ml2.drivers.macvtap import macvtap_common
 from neutron.plugins.ml2.drivers import mech_agent
 
@@ -40,13 +38,20 @@ class MacvtapMechanismDriver(mech_agent.SimpleAgentMechanismDriverBase):
     """
 
     def __init__(self):
-        super(MacvtapMechanismDriver, self).__init__(
+        vif_details = {portbindings.CAP_PORT_FILTER: False,
+                       portbindings.VIF_DETAILS_CONNECTIVITY:
+                           self.connectivity}
+        super().__init__(
             constants.AGENT_TYPE_MACVTAP,
             portbindings.VIF_TYPE_MACVTAP,
-            {portbindings.CAP_PORT_FILTER: False})
+            vif_details)
+
+    @property
+    def connectivity(self):
+        return portbindings.CONNECTIVITY_L2
 
     def get_allowed_network_types(self, agent):
-        return [p_constants.TYPE_FLAT, p_constants.TYPE_VLAN]
+        return [constants.TYPE_FLAT, constants.TYPE_VLAN]
 
     def get_mappings(self, agent):
         return agent['configurations'].get('interface_mappings', {})
@@ -55,6 +60,10 @@ class MacvtapMechanismDriver(mech_agent.SimpleAgentMechanismDriverBase):
         """Macvtap driver vlan transparency support."""
         return False
 
+    def check_vlan_qinq(self, context):
+        """Currently Macvtap driver doesn't support QinQ vlan."""
+        return False
+
     def _is_live_migration(self, context):
         # We cannot just check if
         # context.original['host_id'] != context.current['host_id']
@@ -63,13 +72,15 @@ class MacvtapMechanismDriver(mech_agent.SimpleAgentMechanismDriverBase):
         # context.original['host_id'] is set to the failed host.
         # The only safe way to detect a migration is to look into the binding
         # profiles 'migrating_to' attribute, which is set by Nova since patch
-        # https://review.openstack.org/#/c/275073/.
+        # https://review.opendev.org/#/c/275073/.
+        if not context.original:
+            # new port
+            return False
         port_profile = context.original.get(portbindings.PROFILE)
         if port_profile and port_profile.get('migrating_to', None):
             LOG.debug("Live migration with profile %s detected.", port_profile)
             return True
-        else:
-            return False
+        return False
 
     def try_to_bind_segment_for_agent(self, context, segment, agent):
         if self.check_segment_for_agent(segment, agent):
@@ -78,7 +89,7 @@ class MacvtapMechanismDriver(mech_agent.SimpleAgentMechanismDriverBase):
             interface = mappings[segment['physical_network']]
             network_type = segment[api.NETWORK_TYPE]
 
-            if network_type == p_constants.TYPE_VLAN:
+            if network_type == constants.TYPE_VLAN:
                 vlan_id = segment[api.SEGMENTATION_ID]
                 macvtap_src = macvtap_common.get_vlan_device_name(interface,
                                                                   vlan_id)
@@ -99,15 +110,15 @@ class MacvtapMechanismDriver(mech_agent.SimpleAgentMechanismDriverBase):
                 if orig_source != macvtap_src:
                     source_host = context.original[portbindings.HOST_ID]
                     target_host = agent['host']
-                    LOG.error(_LE("Vif binding denied by mechanism driver. "
-                                  "MacVTap source device '%(target_dev)s' on "
-                                  "the migration target '%(target_host)s'is "
-                                  "not equal to device '%(source_dev)s' on "
-                                  "the migration source '%(source_host)s. "
-                                  "Make sure that the "
-                                  "interface mapping of macvtap "
-                                  "agent on both hosts is equal "
-                                  "for the physical network '%(physnet)s'!"),
+                    LOG.error("Vif binding denied by mechanism driver. "
+                              "MacVTap source device '%(target_dev)s' on "
+                              "the migration target '%(target_host)s'is "
+                              "not equal to device '%(source_dev)s' on "
+                              "the migration source '%(source_host)s. "
+                              "Make sure that the "
+                              "interface mapping of macvtap "
+                              "agent on both hosts is equal "
+                              "for the physical network '%(physnet)s'!",
                               {'source_dev': orig_source,
                                'target_dev': macvtap_src,
                                'target_host': target_host,
