@@ -18,6 +18,7 @@ import sys
 
 import netaddr
 from neutron_lib import constants as n_const
+from neutron_lib.plugins.ml2 import ovs_constants
 from oslo_config import cfg
 from oslo_log import log as logging
 import oslo_messaging
@@ -27,15 +28,15 @@ from osprofiler import profiler
 from neutron.agent.common import ovs_lib
 from neutron.agent.linux.openvswitch_firewall import firewall as ovs_firewall
 from neutron.common import utils as n_utils
-from neutron.plugins.ml2.drivers.openvswitch.agent.common import constants
+from neutron.ipam import utils as ipam_utils
 
 LOG = logging.getLogger(__name__)
 
 
 # A class to represent a DVR-hosted subnet including vif_ports resident on
 # that subnet
-class LocalDVRSubnetMapping(object):
-    def __init__(self, subnet, csnat_ofport=constants.OFPORT_INVALID):
+class LocalDVRSubnetMapping:
+    def __init__(self, subnet, csnat_ofport=ovs_constants.OFPORT_INVALID):
         # set of compute ports on this dvr subnet
         self.compute_ports = {}
         # set of dvr router interfaces on this subnet
@@ -87,7 +88,7 @@ class LocalDVRSubnetMapping(object):
         return self.dvr_ports
 
 
-class OVSPort(object):
+class OVSPort:
     def __init__(self, id, ofport, mac, device_owner):
         self.id = id
         self.mac = mac
@@ -136,15 +137,15 @@ class OVSPort(object):
 
 
 @profiler.trace_cls("ovs_dvr_agent")
-class OVSDVRNeutronAgent(object):
+class OVSDVRNeutronAgent:
     '''Implements OVS-based DVR (Distributed Virtual Router) agent'''
     # history
     #   1.0 Initial version
 
     def __init__(self, context, plugin_rpc, integ_br, tun_br,
                  bridge_mappings, phys_brs, int_ofports, phys_ofports,
-                 patch_int_ofport=constants.OFPORT_INVALID,
-                 patch_tun_ofport=constants.OFPORT_INVALID,
+                 patch_int_ofport=ovs_constants.OFPORT_INVALID,
+                 patch_tun_ofport=ovs_constants.OFPORT_INVALID,
                  host=None, enable_tunneling=False,
                  enable_distributed_routing=False):
         self.context = context
@@ -247,13 +248,14 @@ class OVSDVRNeutronAgent(object):
         self.int_br.setup_canary_table()
 
         # Insert 'drop' action as the default for Table DVR_TO_SRC_MAC
-        self.int_br.install_drop(table_id=constants.DVR_TO_SRC_MAC, priority=1)
+        self.int_br.install_drop(
+            table_id=ovs_constants.DVR_TO_SRC_MAC, priority=1)
 
-        self.int_br.install_drop(table_id=constants.DVR_TO_SRC_MAC_PHYSICAL,
-                                 priority=1)
+        self.int_br.install_drop(
+            table_id=ovs_constants.DVR_TO_SRC_MAC_PHYSICAL, priority=1)
 
         for physical_network in self.bridge_mappings:
-            self.int_br.install_drop(table_id=constants.LOCAL_SWITCHING,
+            self.int_br.install_drop(table_id=ovs_constants.LOCAL_SWITCHING,
                                      priority=2,
                                      in_port=self.int_ofports[
                                          physical_network])
@@ -263,16 +265,20 @@ class OVSDVRNeutronAgent(object):
         if not self.enable_tunneling:
             return
 
-        self.tun_br.install_goto(dest_table_id=constants.DVR_PROCESS,
-                                 priority=1,
-                                 in_port=self.patch_int_ofport)
+        self._setup_dvr_flows_on_tun_br(self.tun_br, self.patch_int_ofport)
+
+    @staticmethod
+    def _setup_dvr_flows_on_tun_br(tun_br, patch_int_ofport):
+        tun_br.install_goto(dest_table_id=ovs_constants.DVR_PROCESS,
+                            priority=1,
+                            in_port=patch_int_ofport)
 
         # table-miss should be sent to learning table
-        self.tun_br.install_goto(table_id=constants.DVR_NOT_LEARN,
-                                 dest_table_id=constants.LEARN_FROM_TUN)
+        tun_br.install_goto(table_id=ovs_constants.DVR_NOT_LEARN,
+                            dest_table_id=ovs_constants.LEARN_FROM_TUN)
 
-        self.tun_br.install_goto(table_id=constants.DVR_PROCESS,
-                                 dest_table_id=constants.PATCH_LV_TO_TUN)
+        tun_br.install_goto(table_id=ovs_constants.DVR_PROCESS,
+                            dest_table_id=ovs_constants.PATCH_LV_TO_TUN)
 
     def setup_dvr_flows_on_phys_br(self, bridge_mappings=None):
         '''Setup up initial dvr flows into br-phys'''
@@ -281,20 +287,20 @@ class OVSDVRNeutronAgent(object):
             self.phys_brs[physical_network].install_goto(
                 in_port=self.phys_ofports[physical_network],
                 priority=2,
-                dest_table_id=constants.DVR_PROCESS_PHYSICAL)
+                dest_table_id=ovs_constants.DVR_PROCESS_PHYSICAL)
             self.phys_brs[physical_network].install_goto(
                 priority=1,
-                dest_table_id=constants.DVR_NOT_LEARN_PHYSICAL)
+                dest_table_id=ovs_constants.DVR_NOT_LEARN_PHYSICAL)
             self.phys_brs[physical_network].install_goto(
-                table_id=constants.DVR_PROCESS_PHYSICAL,
+                table_id=ovs_constants.DVR_PROCESS_PHYSICAL,
                 priority=0,
-                dest_table_id=constants.LOCAL_VLAN_TRANSLATION)
+                dest_table_id=ovs_constants.LOCAL_VLAN_TRANSLATION)
             self.phys_brs[physical_network].install_drop(
-                table_id=constants.LOCAL_VLAN_TRANSLATION,
+                table_id=ovs_constants.LOCAL_VLAN_TRANSLATION,
                 in_port=self.phys_ofports[physical_network],
                 priority=2)
             self.phys_brs[physical_network].install_normal(
-                table_id=constants.DVR_NOT_LEARN_PHYSICAL,
+                table_id=ovs_constants.DVR_NOT_LEARN_PHYSICAL,
                 priority=1)
 
     def _add_dvr_mac_for_phys_br(self, physical_network, mac):
@@ -306,7 +312,7 @@ class OVSDVRNeutronAgent(object):
 
     def _add_arp_dvr_mac_for_phys_br(self, physical_network, mac):
         self.int_br.add_dvr_gateway_mac_arp_vlan(
-                mac=mac, port=self.int_ofports[physical_network])
+            mac=mac, port=self.int_ofports[physical_network])
 
     def _remove_dvr_mac_for_phys_br(self, physical_network, mac):
         # REVISIT(yamamoto): match in_port as well?
@@ -321,7 +327,7 @@ class OVSDVRNeutronAgent(object):
 
     def _add_arp_dvr_mac_for_tun_br(self, mac):
         self.int_br.add_dvr_gateway_mac_arp_tun(
-                mac=mac, port=self.patch_tun_ofport)
+            mac=mac, port=self.patch_tun_ofport)
 
     def _remove_dvr_mac_for_tun_br(self, mac):
         self.int_br.remove_dvr_mac_tun(mac=mac, port=self.patch_tun_ofport)
@@ -427,7 +433,7 @@ class OVSDVRNeutronAgent(object):
         ldm.set_dvr_owned(True)
 
         vlan_to_use = lvm.vlan
-        if lvm.network_type in constants.DVR_PHYSICAL_NETWORK_TYPES:
+        if lvm.network_type in ovs_constants.DVR_PHYSICAL_NETWORK_TYPES:
             vlan_to_use = lvm.segmentation_id
 
         subnet_info = ldm.get_subnet_info()
@@ -488,11 +494,11 @@ class OVSDVRNeutronAgent(object):
             dvr_mac=self.dvr_mac_address,
             rtr_port=port.ofport)
 
-        if lvm.network_type in constants.DVR_PHYSICAL_NETWORK_TYPES:
+        if lvm.network_type in ovs_constants.DVR_PHYSICAL_NETWORK_TYPES:
             # TODO(vivek) remove the IPv6 related flows once SNAT is not
             # used for IPv6 DVR.
             br = self.phys_brs[lvm.physical_network]
-        if lvm.network_type in constants.TUNNEL_NETWORK_TYPES:
+        if lvm.network_type in ovs_constants.TUNNEL_NETWORK_TYPES:
             br = self.tun_br
         # TODO(vivek) remove the IPv6 related flows once SNAT is not
         # used for IPv6 DVR.
@@ -515,8 +521,22 @@ class OVSDVRNeutronAgent(object):
         self.local_ports[port.vif_id] = ovsport
         ldm.add_dvr_ofport(port.vif_id, port.ofport)
 
+        if (ip_version == n_const.IP_VERSION_4 and
+                subnet_info.get('gateway_mac')):
+            # Change ARP reply destination MAC address from
+            # dvr_host_mac to gateway_mac.
+            self.int_br.change_arp_destination_mac(
+                target_mac_address=subnet_info['gateway_mac'],
+                orig_mac_address=self.dvr_mac_address)
+
     def _bind_port_on_dvr_subnet(self, port, lvm, fixed_ips,
                                  device_owner):
+        ports = self.plugin_rpc.get_ports(self.context,
+                                          filters={'id': [port.vif_id]})
+        aaps = []
+        if len(ports) == 1:
+            aaps = ports[0].get("allowed_address_pairs", [])
+
         # Handle new compute port added use-case
         subnet_uuid = None
         for ips in fixed_ips:
@@ -550,15 +570,27 @@ class OVSDVRNeutronAgent(object):
                 ovsport.add_subnet(subnet_uuid)
                 self.local_ports[port.vif_id] = ovsport
             vlan_to_use = lvm.vlan
-            if lvm.network_type in constants.DVR_PHYSICAL_NETWORK_TYPES:
+            if lvm.network_type in ovs_constants.DVR_PHYSICAL_NETWORK_TYPES:
                 vlan_to_use = lvm.segmentation_id
             # create a rule for this vm port
+            dst_port = ovsport.get_ofport()
             self.int_br.install_dvr_to_src_mac(
                 network_type=lvm.network_type,
                 vlan_tag=vlan_to_use,
                 gateway_mac=subnet_info['gateway_mac'],
                 dst_mac=ovsport.get_mac(),
-                dst_port=ovsport.get_ofport())
+                dst_port=dst_port)
+            for aap in aaps:
+                aap_ip_cidr = netaddr.IPNetwork(aap['ip_address'])
+                if n_utils.is_cidr_host(str(aap_ip_cidr.cidr)):
+                    if ipam_utils.check_subnet_ip(
+                            ldm.subnet['cidr'], str(aap_ip_cidr.ip)):
+                        self.int_br.install_dvr_to_src_mac(
+                            network_type=lvm.network_type,
+                            vlan_tag=vlan_to_use,
+                            gateway_mac=subnet_info['gateway_mac'],
+                            dst_mac=aap["mac_address"],
+                            dst_port=dst_port)
 
     def _bind_centralized_snat_port_on_dvr_subnet(self, port, lvm,
                                                   fixed_ips, device_owner):
@@ -610,7 +642,7 @@ class OVSDVRNeutronAgent(object):
         ovsport.add_subnet(subnet_uuid)
         self.local_ports[port.vif_id] = ovsport
         vlan_to_use = lvm.vlan
-        if lvm.network_type in constants.DVR_PHYSICAL_NETWORK_TYPES:
+        if lvm.network_type in ovs_constants.DVR_PHYSICAL_NETWORK_TYPES:
             vlan_to_use = lvm.segmentation_id
         self.int_br.install_dvr_to_src_mac(
             network_type=lvm.network_type,
@@ -625,8 +657,8 @@ class OVSDVRNeutronAgent(object):
             return
 
         if (local_vlan_map.network_type not in
-                (constants.TUNNEL_NETWORK_TYPES +
-                 constants.DVR_PHYSICAL_NETWORK_TYPES)):
+                (ovs_constants.TUNNEL_NETWORK_TYPES +
+                 ovs_constants.DVR_PHYSICAL_NETWORK_TYPES)):
             LOG.debug("DVR: Port %s is with network_type %s not supported"
                       " for dvr plumbing", port.vif_id,
                       local_vlan_map.network_type)
@@ -638,9 +670,10 @@ class OVSDVRNeutronAgent(object):
                      "%(ofport)s, rebinding.",
                      {'vif': port.vif_id, 'ofport': port.ofport})
             self.unbind_port_from_dvr(port, local_vlan_map)
-            if port.ofport in (ovs_lib.INVALID_OFPORT,
-                               ovs_lib.UNASSIGNED_OFPORT):
-                return
+
+        if port.ofport in (ovs_lib.INVALID_OFPORT,
+                           ovs_lib.UNASSIGNED_OFPORT):
+            return
 
         if device_owner == n_const.DEVICE_OWNER_DVR_INTERFACE:
             self._bind_distributed_router_interface_port(port,
@@ -667,7 +700,7 @@ class OVSDVRNeutronAgent(object):
         network_type = lvm.network_type
         physical_network = lvm.physical_network
         vlan_to_use = lvm.vlan
-        if network_type in constants.DVR_PHYSICAL_NETWORK_TYPES:
+        if network_type in ovs_constants.DVR_PHYSICAL_NETWORK_TYPES:
             vlan_to_use = lvm.segmentation_id
         # ensure we process for all the subnets laid on this removed port
         for sub_uuid in subnet_set:
@@ -707,16 +740,16 @@ class OVSDVRNeutronAgent(object):
                 gateway_mac=port.vif_mac,
                 dvr_mac=self.dvr_mac_address,
                 rtr_port=port.ofport)
-            if (ldm.get_csnat_ofport() == constants.OFPORT_INVALID and
+            if (ldm.get_csnat_ofport() == ovs_constants.OFPORT_INVALID and
                     len(ldm.get_dvr_ofports()) <= 1):
                 # if there is no csnat port for this subnet and if this is
                 # the last dvr port in the subnet, remove this subnet from
                 # local_dvr_map, as no dvr (or) csnat ports available on this
                 # agent anymore
                 self.local_dvr_map.pop(sub_uuid, None)
-            if network_type in constants.DVR_PHYSICAL_NETWORK_TYPES:
+            if network_type in ovs_constants.DVR_PHYSICAL_NETWORK_TYPES:
                 br = self.phys_brs[physical_network]
-            if network_type in constants.TUNNEL_NETWORK_TYPES:
+            if network_type in ovs_constants.TUNNEL_NETWORK_TYPES:
                 br = self.tun_br
             if ip_version == 4:
                 if subnet_info['gateway_ip']:
@@ -734,9 +767,16 @@ class OVSDVRNeutronAgent(object):
                 self.firewall.delete_accepted_egress_direct_flow(
                     subnet_info['gateway_mac'], lvm.vlan)
 
-        if lvm.network_type in constants.DVR_PHYSICAL_NETWORK_TYPES:
+            if (ip_version == n_const.IP_VERSION_4 and
+                    subnet_info.get('gateway_mac')):
+                # remove ARP reply destination MAC address change flow
+                self.int_br.delete_arp_destination_change(
+                    target_mac_address=subnet_info['gateway_mac'],
+                    orig_mac_address=self.dvr_mac_address)
+
+        if lvm.network_type in ovs_constants.DVR_PHYSICAL_NETWORK_TYPES:
             br = self.phys_brs[physical_network]
-        if lvm.network_type in constants.TUNNEL_NETWORK_TYPES:
+        if lvm.network_type in ovs_constants.TUNNEL_NETWORK_TYPES:
             br = self.tun_br
         br.delete_dvr_process(vlan_tag=lvm.vlan, vif_mac=port.vif_mac)
 
@@ -744,6 +784,12 @@ class OVSDVRNeutronAgent(object):
         self.local_ports.pop(port.vif_id, None)
 
     def _unbind_port_on_dvr_subnet(self, port, lvm):
+        ports = self.plugin_rpc.get_ports(self.context,
+                                          filters={'id': [port.vif_id]})
+        aaps = []
+        if len(ports) == 1:
+            aaps = ports[0].get("allowed_address_pairs", [])
+
         ovsport = self.local_ports[port.vif_id]
         # This confirms that this compute port being removed belonged
         # to a dvr hosted subnet.
@@ -753,15 +799,34 @@ class OVSDVRNeutronAgent(object):
         for sub_uuid in subnet_ids:
             if sub_uuid not in self.local_dvr_map:
                 continue
+            if aaps:
+                local_compute_ports = (
+                    self.plugin_rpc.get_ports_on_host_by_subnet(
+                        self.context, self.host, sub_uuid))
+                local_aap_macs = set()
+                for lport in local_compute_ports:
+                    if lport.id != port.vif_id:
+                        local_aap_macs.update({
+                            aap["mac_address"] for aap in lport.get(
+                                "allowed_address_pairs", [])})
             ldm = self.local_dvr_map[sub_uuid]
             ldm.remove_compute_ofport(port.vif_id)
             vlan_to_use = lvm.vlan
-            if lvm.network_type in constants.DVR_PHYSICAL_NETWORK_TYPES:
+            if lvm.network_type in ovs_constants.DVR_PHYSICAL_NETWORK_TYPES:
                 vlan_to_use = lvm.segmentation_id
             # first remove this vm port rule
             self.int_br.delete_dvr_to_src_mac(
                 network_type=lvm.network_type,
                 vlan_tag=vlan_to_use, dst_mac=ovsport.get_mac())
+        for aap in aaps:
+            aap_ip_cidr = netaddr.IPNetwork(aap['ip_address'])
+            if n_utils.is_cidr_host(str(aap_ip_cidr.cidr)):
+                if ipam_utils.check_subnet_ip(ldm.subnet['cidr'],
+                                              str(aap_ip_cidr.ip)):
+                    if aap["mac_address"] not in local_aap_macs:
+                        self.int_br.delete_dvr_to_src_mac(
+                            network_type=lvm.network_type,
+                            vlan_tag=vlan_to_use, dst_mac=aap["mac_address"])
         # release port state
         self.local_ports.pop(port.vif_id, None)
 
@@ -775,9 +840,9 @@ class OVSDVRNeutronAgent(object):
         if sub_uuid not in self.local_dvr_map:
             return
         ldm = self.local_dvr_map[sub_uuid]
-        ldm.set_csnat_ofport(constants.OFPORT_INVALID)
+        ldm.set_csnat_ofport(ovs_constants.OFPORT_INVALID)
         vlan_to_use = lvm.vlan
-        if lvm.network_type in constants.DVR_PHYSICAL_NETWORK_TYPES:
+        if lvm.network_type in ovs_constants.DVR_PHYSICAL_NETWORK_TYPES:
             vlan_to_use = lvm.segmentation_id
         # then remove csnat port rule
         self.int_br.delete_dvr_to_src_mac(
