@@ -40,12 +40,8 @@ from neutron.services.segments import db as segments_db
 
 LOG = log.getLogger(__name__)
 
-SYNC_MODE_OFF = 'off'
-SYNC_MODE_LOG = 'log'
-SYNC_MODE_REPAIR = 'repair'
 
-
-class OvnDbSynchronizer(object, metaclass=abc.ABCMeta):
+class OvnDbSynchronizer(metaclass=abc.ABCMeta):
 
     def __init__(self, core_plugin, ovn_api, ovn_driver):
         self.ovn_driver = ovn_driver
@@ -71,7 +67,7 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
     """Synchronizer class for NB."""
 
     def __init__(self, core_plugin, ovn_api, sb_ovn, mode, ovn_driver):
-        super(OvnNbSynchronizer, self).__init__(
+        super().__init__(
             core_plugin, ovn_api, ovn_driver)
         self.mode = mode
         self.l3_plugin = directory.get_plugin(plugin_constants.L3)
@@ -91,10 +87,10 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
         if utils.is_ovn_l3(self.l3_plugin):
             self.l3_plugin._nb_ovn.ovsdb_connection.stop()
             self.l3_plugin._sb_ovn.ovsdb_connection.stop()
-        super(OvnNbSynchronizer, self).stop()
+        super().stop()
 
     def do_sync(self):
-        if self.mode == SYNC_MODE_OFF:
+        if self.mode == ovn_const.OVN_DB_SYNC_MODE_OFF:
             LOG.debug("Neutron sync mode is off, not checking OVN "
                       "Northbound DB for consistency")
             return
@@ -149,7 +145,7 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
         @var   acl_list_dict: Dictionary of acl-lists based on lport as key
         @return: acl_list-dict
         """
-        lswitch_names = set([])
+        lswitch_names = set()
         for network in self.core_plugin.get_networks(context):
             lswitch_names.add(network['id'])
         acl_dict, ignore1, ignore2 = (
@@ -196,11 +192,12 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
             LOG.warning('Number of Port Groups to add: %d, remove: %d',
                         len(add_pgs), len(remove_pgs))
 
-        if self.mode == SYNC_MODE_REPAIR and (add_pgs or remove_pgs):
+        if self.mode == ovn_const.OVN_DB_SYNC_MODE_REPAIR and (
+                add_pgs or remove_pgs):
             if add_pgs:
                 db_ports = self.core_plugin.get_ports(ctx)
-                ovn_ports = set(p.name for p in
-                                self.ovn_api.lsp_list().execute())
+                ovn_ports = {p.name for p in
+                             self.ovn_api.lsp_list().execute()}
             with self.ovn_api.transaction(check_error=True) as txn:
                 pg = ovn_const.OVN_DROP_PORT_GROUP_NAME
                 # Process default drop port group first
@@ -260,24 +257,17 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
         LOG.debug('OVN-NB Sync ACLs started @ %s', str(datetime.now()))
 
         neutron_acls = []
-        # if allow-stateless supported, we have to fetch groups to determine if
-        # stateful is set
-        if self._ovn_client.is_allow_stateless_supported():
-            for sg in self.core_plugin.get_security_groups(ctx):
-                stateful = sg.get("stateful", True)
-                pg_name = utils.ovn_port_group_name(sg['id'])
-                for sgr in self.core_plugin.get_security_group_rules(
-                        ctx, {'security_group_id': sg['id']}):
-                    neutron_acls.append(
-                        acl_utils._add_sg_rule_acl_for_port_group(
-                            pg_name, stateful, sgr)
-                    )
-        else:
-            # TODO(ihrachys) remove when min OVN version >= 21.06
-            for sgr in self.core_plugin.get_security_group_rules(ctx):
-                pg_name = utils.ovn_port_group_name(sgr['security_group_id'])
-                neutron_acls.append(acl_utils._add_sg_rule_acl_for_port_group(
-                    pg_name, True, sgr))
+        # we have to fetch groups to determine if stateful is set
+        for sg in self.core_plugin.get_security_groups(ctx):
+            stateful = sg.get("stateful", True)
+            pg_name = utils.ovn_port_group_name(sg['id'])
+            for sgr in self.core_plugin.get_security_group_rules(
+                    ctx, {'security_group_id': sg['id']}):
+                neutron_acls.append(
+                    acl_utils._add_sg_rule_acl_for_port_group(
+                        pg_name, stateful, sgr)
+                )
+
         # Sort the acls in the Neutron database according to the security
         # group rule ID for easy comparison in the future.
         neutron_acls.sort(key=lambda x: x[ovn_const.OVN_SG_RULE_EXT_ID_KEY])
@@ -358,7 +348,7 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                         {'add': num_acls_to_add,
                          'remove': num_acls_to_remove})
 
-        if (self.mode == SYNC_MODE_REPAIR and
+        if (self.mode == ovn_const.OVN_DB_SYNC_MODE_REPAIR and
                 (num_acls_to_add or num_acls_to_remove)):
             one_time_pg_resync = True
             while True:
@@ -692,7 +682,7 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                 continue
             LOG.warning("Router found in Neutron but not in "
                         "OVN NB DB, router id=%s", router['id'])
-            if self.mode == SYNC_MODE_REPAIR:
+            if self.mode == ovn_const.OVN_DB_SYNC_MODE_REPAIR:
                 try:
                     LOG.warning("Creating the router %s in OVN NB DB",
                                 router['id'])
@@ -733,7 +723,7 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
         for rp_id, rrport in db_router_ports.items():
             LOG.warning("Router Port found in Neutron but not in OVN NB "
                         "DB, router port_id=%s", rrport['id'])
-            if self.mode == SYNC_MODE_REPAIR:
+            if self.mode == ovn_const.OVN_DB_SYNC_MODE_REPAIR:
                 try:
                     LOG.warning("Creating the router port %s in OVN NB DB",
                                 rrport['id'])
@@ -748,7 +738,7 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
             LOG.warning("Router Port port_id=%s needs to be updated in OVN NB "
                         "DB as network(s) have changed",
                         rport['id'])
-            if self.mode == SYNC_MODE_REPAIR:
+            if self.mode == ovn_const.OVN_DB_SYNC_MODE_REPAIR:
                 try:
                     LOG.warning(
                         "Updating networks on router port %s in OVN NB DB",
@@ -762,7 +752,7 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
             for lrouter in del_lrouters_list:
                 LOG.warning("Router found in OVN NB DB but not in "
                             "Neutron, router id=%s", lrouter['name'])
-                if self.mode == SYNC_MODE_REPAIR:
+                if self.mode == ovn_const.OVN_DB_SYNC_MODE_REPAIR:
                     LOG.warning("Deleting the router %s from OVN NB DB",
                                 lrouter['name'])
                     txn.add(self.ovn_api.lr_del(
@@ -771,7 +761,7 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
             for lrport_info in del_lrouter_ports_list:
                 LOG.warning("Router Port found in OVN NB DB but not in "
                             "Neutron, port_id=%s", lrport_info['port'])
-                if self.mode == SYNC_MODE_REPAIR:
+                if self.mode == ovn_const.OVN_DB_SYNC_MODE_REPAIR:
                     LOG.warning("Deleting the port %s from OVN NB DB",
                                 lrport_info['port'])
                     txn.add(self.ovn_api.delete_lrouter_port(
@@ -783,7 +773,7 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                     LOG.warning("Router %(id)s static routes %(route)s "
                                 "found in Neutron but not in OVN NB DB",
                                 {'id': sroute['id'], 'route': sroute['add']})
-                    if self.mode == SYNC_MODE_REPAIR:
+                    if self.mode == ovn_const.OVN_DB_SYNC_MODE_REPAIR:
                         LOG.warning("Add static routes %s to OVN NB DB",
                                     sroute['add'])
                         for route in sroute['add']:
@@ -801,7 +791,7 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                     LOG.warning("Router %(id)s static routes %(route)s "
                                 "found in OVN NB DB but not in Neutron",
                                 {'id': sroute['id'], 'route': sroute['del']})
-                    if self.mode == SYNC_MODE_REPAIR:
+                    if self.mode == ovn_const.OVN_DB_SYNC_MODE_REPAIR:
                         LOG.warning("Delete static routes %s from OVN NB DB",
                                     sroute['del'])
                         routes_to_delete = [
@@ -815,7 +805,7 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                     LOG.warning("Router %(id)s floating IPs %(fip)s "
                                 "found in OVN NB DB but not in Neutron",
                                 {'id': fip['id'], 'fip': fip['del']})
-                    if self.mode == SYNC_MODE_REPAIR:
+                    if self.mode == ovn_const.OVN_DB_SYNC_MODE_REPAIR:
                         LOG.warning(
                             "Delete floating IPs %s from OVN NB DB",
                             fip['del'])
@@ -826,7 +816,7 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                     LOG.warning("Router %(id)s floating IPs %(fip)s "
                                 "found in Neutron but not in OVN NB DB",
                                 {'id': fip['id'], 'fip': fip['add']})
-                    if self.mode == SYNC_MODE_REPAIR:
+                    if self.mode == ovn_const.OVN_DB_SYNC_MODE_REPAIR:
                         LOG.warning("Add floating IPs %s to OVN NB DB",
                                     fip['add'])
                         for nat in fip['add']:
@@ -839,7 +829,7 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                                 "IPs %(fip)s found in OVN NB DB but not in "
                                 "Neutron",
                                 {'id': pf['id'], 'fip': pf['del']})
-                    if self.mode == SYNC_MODE_REPAIR:
+                    if self.mode == ovn_const.OVN_DB_SYNC_MODE_REPAIR:
                         LOG.warning(
                             "Delete port forwarding for floating IPs %s from "
                             "OVN NB DB",
@@ -851,7 +841,7 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                                 "IPs %(fip)s Neutron out of sync or missing "
                                 "in OVN NB DB",
                                 {'id': pf['id'], 'fip': pf['add']})
-                    if self.mode == SYNC_MODE_REPAIR:
+                    if self.mode == ovn_const.OVN_DB_SYNC_MODE_REPAIR:
                         LOG.warning("Add port forwarding for floating IPs %s "
                                     "to OVN NB DB",
                                     pf['add'])
@@ -864,7 +854,7 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                     LOG.warning("Router %(id)s SNAT %(snat)s "
                                 "found in OVN NB DB but not in Neutron",
                                 {'id': snat['id'], 'snat': snat['del']})
-                    if self.mode == SYNC_MODE_REPAIR:
+                    if self.mode == ovn_const.OVN_DB_SYNC_MODE_REPAIR:
                         LOG.warning("Delete SNATs %s from OVN NB DB",
                                     snat['del'])
                         for nat in snat['del']:
@@ -877,7 +867,7 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                     LOG.warning("Router %(id)s SNAT %(snat)s "
                                 "found in Neutron but not in OVN NB DB",
                                 {'id': snat['id'], 'snat': snat['add']})
-                    if self.mode == SYNC_MODE_REPAIR:
+                    if self.mode == ovn_const.OVN_DB_SYNC_MODE_REPAIR:
                         LOG.warning("Add SNATs %s to OVN NB DB",
                                     snat['add'])
                         for nat in snat['add']:
@@ -925,7 +915,7 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
         for subnet_id, subnet in db_subnets.items():
             LOG.warning('DHCP options for subnet %s present in '
                         'Neutron but out of sync with OVN NB DB', subnet_id)
-            if self.mode == SYNC_MODE_REPAIR:
+            if self.mode == ovn_const.OVN_DB_SYNC_MODE_REPAIR:
                 # If neutron-server is running we could race and find a
                 # subnet without a cached network, just skip it to avoid
                 # a KeyError below.
@@ -954,7 +944,7 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
             LOG.warning('Out of sync DHCP options for subnet %s '
                         'found in OVN NB DB which need to be deleted',
                         dhcp_opt['external_ids']['subnet_id'])
-            if self.mode == SYNC_MODE_REPAIR:
+            if self.mode == ovn_const.OVN_DB_SYNC_MODE_REPAIR:
                 LOG.warning('Deleting DHCP options for subnet %s ',
                             dhcp_opt['external_ids']['subnet_id'])
                 txn_commands.append(self.ovn_api.delete_dhcp_options(
@@ -967,7 +957,7 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
         LOG.debug('OVN-NB Sync DHCP options for Neutron subnets completed @ '
                   '%s', str(datetime.now()))
 
-    def _sync_port_dhcp_options(self, ctx, ports_need_sync_dhcp_opts,
+    def _sync_port_dhcp_options(self, ports_need_sync_dhcp_opts,
                                 ovn_port_dhcpv4_opts, ovn_port_dhcpv6_opts):
         LOG.debug('OVN-NB Sync DHCP options for Neutron ports with extra '
                   'dhcp options assigned started')
@@ -977,7 +967,7 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                         constants.IP_VERSION_6: 'dhcpv6_options'}
         ovn_port_dhcp_opts = {constants.IP_VERSION_4: ovn_port_dhcpv4_opts,
                               constants.IP_VERSION_6: ovn_port_dhcpv6_opts}
-        if self.mode == SYNC_MODE_REPAIR:
+        if self.mode == ovn_const.OVN_DB_SYNC_MODE_REPAIR:
             for port in ports_need_sync_dhcp_opts:
                 LOG.warning('Updating DHCP options for port %s in OVN NB DB',
                             port['id'])
@@ -1017,7 +1007,7 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                      'subnet_id': dhcp_opt['external_ids']['subnet_id'],
                      'port_id': port_id})
 
-                if self.mode == SYNC_MODE_REPAIR:
+                if self.mode == ovn_const.OVN_DB_SYNC_MODE_REPAIR:
                     LOG.warning('Deleting port DHCPv%d options for '
                                 '(subnet %s, port %s)', ip_v,
                                 dhcp_opt['external_ids']['subnet_id'], port_id)
@@ -1049,7 +1039,7 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
             if not metadata_ports:
                 LOG.warning('Missing metadata port found in Neutron for '
                             'network %s', net['id'])
-                if self.mode == SYNC_MODE_REPAIR:
+                if self.mode == ovn_const.OVN_DB_SYNC_MODE_REPAIR:
                     try:
                         # Create the missing port in both Neutron and OVN.
                         LOG.warning('Creating missing metadata port in '
@@ -1066,7 +1056,7 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                 for port in metadata_ports[1:]:
                     LOG.warning('Unnecessary DHCP port %s for network %s '
                                 'found in Neutron', port['id'], net['id'])
-                    if self.mode == SYNC_MODE_REPAIR:
+                    if self.mode == ovn_const.OVN_DB_SYNC_MODE_REPAIR:
                         LOG.warning('Deleting unnecessary DHCP port %s for '
                                     'network %s', port['id'], net['id'])
                         self.core_plugin.delete_port(ctx, port['id'])
@@ -1076,14 +1066,14 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                     LOG.warning('Metadata port %s for network %s found in '
                                 'Neutron but not in OVN NB DB',
                                 port['id'], net['id'])
-                    if self.mode == SYNC_MODE_REPAIR:
+                    if self.mode == ovn_const.OVN_DB_SYNC_MODE_REPAIR:
                         LOG.warning('Creating metadata port %s for network '
                                     '%s in OVN NB DB',
                                     port['id'], net['id'])
                         self._create_port_in_ovn(ctx, port)
                     db_ports.pop(port['id'])
 
-            if self.mode == SYNC_MODE_REPAIR:
+            if self.mode == ovn_const.OVN_DB_SYNC_MODE_REPAIR:
                 try:
                     # Make sure that this port has an IP address in all the
                     # subnets
@@ -1155,10 +1145,10 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
             else:
                 del_lswitchs_list.append(lswitch)
 
-        for net_id, network in db_networks.items():
+        for network in db_networks.values():
             LOG.warning("Network found in Neutron but not in "
                         "OVN NB DB, network_id=%s", network['id'])
-            if self.mode == SYNC_MODE_REPAIR:
+            if self.mode == ovn_const.OVN_DB_SYNC_MODE_REPAIR:
                 try:
                     LOG.warning('Creating network %s in OVN NB DB',
                                 network['id'])
@@ -1179,18 +1169,18 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
         for port_id, port in db_ports.items():
             LOG.warning("Port found in Neutron but not in OVN NB "
                         "DB, port_id=%s", port['id'])
-            if self.mode == SYNC_MODE_REPAIR:
+            if self.mode == ovn_const.OVN_DB_SYNC_MODE_REPAIR:
                 try:
                     LOG.warning('Creating the port %s in OVN NB DB',
                                 port['id'])
                     self._create_port_in_ovn(ctx, port)
                     if port_id in ovn_all_dhcp_options['ports_v4']:
-                        dhcp_disable, lsp_opts = utils.get_lsp_dhcp_opts(
+                        __, lsp_opts = utils.get_lsp_dhcp_opts(
                             port, constants.IP_VERSION_4)
                         if lsp_opts:
                             ovn_all_dhcp_options['ports_v4'].pop(port_id)
                     if port_id in ovn_all_dhcp_options['ports_v6']:
-                        dhcp_disable, lsp_opts = utils.get_lsp_dhcp_opts(
+                        __, lsp_opts = utils.get_lsp_dhcp_opts(
                             port, constants.IP_VERSION_6)
                         if lsp_opts:
                             ovn_all_dhcp_options['ports_v6'].pop(port_id)
@@ -1202,7 +1192,7 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
             for lswitch in del_lswitchs_list:
                 LOG.warning("Network found in OVN NB DB but not in "
                             "Neutron, network_id=%s", lswitch['name'])
-                if self.mode == SYNC_MODE_REPAIR:
+                if self.mode == ovn_const.OVN_DB_SYNC_MODE_REPAIR:
                     LOG.warning('Deleting network %s from OVN NB DB',
                                 lswitch['name'])
                     txn.add(self.ovn_api.ls_del(lswitch['name']))
@@ -1215,12 +1205,12 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                             "network_id=%(net)s segment_id=%(seg)s",
                             {'net': network['id'],
                              'seg': segment['id']})
-                if self.mode == SYNC_MODE_REPAIR:
+                if self.mode == ovn_const.OVN_DB_SYNC_MODE_REPAIR:
                     LOG.warning('Creating provider network port %s in '
                                 'OVN NB DB',
                                 utils.ovn_provnet_port_name(segment['id']))
                     self._ovn_client.create_provnet_port(
-                        network['id'], segment, txn=txn)
+                        network['id'], segment, txn=txn, network=network)
 
             for provnet_port_info in del_provnet_ports_list:
                 network = provnet_port_info['network']
@@ -1231,7 +1221,7 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                             "port_name=%(lport)s",
                             {'net': network,
                              'seg': lport})
-                if self.mode == SYNC_MODE_REPAIR:
+                if self.mode == ovn_const.OVN_DB_SYNC_MODE_REPAIR:
                     LOG.warning('Deleting provider network port %s from '
                                 'OVN NB DB', lport)
                     txn.add(self.ovn_api.delete_lswitch_port(
@@ -1241,7 +1231,7 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
             for lport_info in del_lports_list:
                 LOG.warning("Port found in OVN NB DB but not in "
                             "Neutron, port_id=%s", lport_info['port'])
-                if self.mode == SYNC_MODE_REPAIR:
+                if self.mode == ovn_const.OVN_DB_SYNC_MODE_REPAIR:
                     LOG.warning('Deleting port %s from OVN NB DB',
                                 lport_info['port'])
                     txn.add(self.ovn_api.delete_lswitch_port(
@@ -1260,14 +1250,14 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                             ovn_all_dhcp_options['ports_v6'].pop(
                                 lport_info['port'])['uuid']))
 
-        self._sync_port_dhcp_options(ctx, ports_need_sync_dhcp_opts,
+        self._sync_port_dhcp_options(ports_need_sync_dhcp_opts,
                                      ovn_all_dhcp_options['ports_v4'],
                                      ovn_all_dhcp_options['ports_v6'])
         LOG.debug('OVN-NB Sync networks, ports and DHCP options completed @ '
                   '%s', str(datetime.now()))
 
     def sync_port_dns_records(self, ctx):
-        if self.mode != SYNC_MODE_REPAIR:
+        if self.mode != ovn_const.OVN_DB_SYNC_MODE_REPAIR:
             return
         LOG.debug('OVN-NB Sync port DNS records started @ %s',
                   str(datetime.now()))
@@ -1343,7 +1333,7 @@ class OvnSbSynchronizer(OvnDbSynchronizer):
     """Synchronizer class for SB."""
 
     def __init__(self, core_plugin, ovn_api, ovn_driver):
-        super(OvnSbSynchronizer, self).__init__(
+        super().__init__(
             core_plugin, ovn_api, ovn_driver)
         self.l3_plugin = directory.get_plugin(plugin_constants.L3)
 
