@@ -29,7 +29,6 @@ from neutron_lib.plugins import utils as plugin_utils
 from oslo_config import cfg
 from oslo_db import exception as db_exc
 from oslo_log import log
-from oslo_utils import uuidutils
 from sqlalchemy import or_
 
 from neutron._i18n import _
@@ -56,7 +55,7 @@ class _TunnelTypeDriverBase(helpers.SegmentTypeDriver, metaclass=abc.ABCMeta):
     BULK_SIZE = 100
 
     def __init__(self, model):
-        super(_TunnelTypeDriverBase, self).__init__(model)
+        super().__init__(model)
         self.segmentation_key = next(iter(self.primary_keys))
 
     @abc.abstractmethod
@@ -146,21 +145,12 @@ class _TunnelTypeDriverBase(helpers.SegmentTypeDriver, metaclass=abc.ABCMeta):
                  {'type': self.get_type(), 'range': current_range})
 
     @db_api.retry_db_errors
-    def _populate_new_default_network_segment_ranges(self):
+    def _populate_new_default_network_segment_ranges(self, start_time):
         ctx = context.get_admin_context()
-        for tun_min, tun_max in self.tunnel_ranges:
-            res = {
-                'id': uuidutils.generate_uuid(),
-                'name': '',
-                'default': True,
-                'shared': True,
-                'network_type': self.get_type(),
-                'minimum': tun_min,
-                'maximum': tun_max}
-            with db_api.CONTEXT_WRITER.using(ctx):
-                new_default_range_obj = (
-                    range_obj.NetworkSegmentRange(ctx, **res))
-                new_default_range_obj.create()
+        with db_api.CONTEXT_WRITER.using(ctx):
+            for tun_min, tun_max in self.tunnel_ranges:
+                range_obj.NetworkSegmentRange.new_default(
+                    ctx, self.get_type(), None, tun_min, tun_max, start_time)
 
     @db_api.retry_db_errors
     def _get_network_segment_ranges_from_db(self):
@@ -174,9 +164,9 @@ class _TunnelTypeDriverBase(helpers.SegmentTypeDriver, metaclass=abc.ABCMeta):
 
         return ranges
 
-    def initialize_network_segment_range_support(self):
-        self._delete_expired_default_network_segment_ranges()
-        self._populate_new_default_network_segment_ranges()
+    def initialize_network_segment_range_support(self, start_time):
+        self._delete_expired_default_network_segment_ranges(start_time)
+        self._populate_new_default_network_segment_ranges(start_time)
         # Override self.tunnel_ranges with the network segment range
         # information from DB and then do a sync_allocations since the
         # segment range service plugin has not yet been loaded at this
@@ -199,6 +189,17 @@ class _TunnelTypeDriverBase(helpers.SegmentTypeDriver, metaclass=abc.ABCMeta):
         tunnel_col = getattr(self.model, self.segmentation_key)
         ctx = context.get_admin_context()
         with db_api.CONTEXT_WRITER.using(ctx):
+            # Check if the allocations are updated: if the total number of
+            # allocations for this tunnel type matches the allocations of the
+            # specific IDs, fast exit in that case.
+            # If another worker handled that before or the table was updated
+            # in a previous Neutron API restart, this section will end here.
+            num_allocs = ctx.session.query(self.model).filter(
+                tunnel_col.in_(tunnel_ids)).count()
+            num_allocs_total = ctx.session.query(self.model).count()
+            if len(tunnel_ids) == num_allocs == num_allocs_total:
+                return
+
             # remove from table unallocated tunnels not currently allocatable
             # fetch results as list via all() because we'll be iterating
             # through them twice
@@ -240,7 +241,7 @@ class _TunnelTypeDriverBase(helpers.SegmentTypeDriver, metaclass=abc.ABCMeta):
                 raise exc.InvalidInput(error_message=msg)
 
     def get_mtu(self, physical_network=None):
-        seg_mtu = super(_TunnelTypeDriverBase, self).get_mtu()
+        seg_mtu = super().get_mtu()
         mtu = []
         if seg_mtu > 0:
             mtu.append(seg_mtu)
@@ -339,7 +340,7 @@ class ML2TunnelTypeDriver(_TunnelTypeDriverBase, metaclass=abc.ABCMeta):
 class EndpointTunnelTypeDriver(ML2TunnelTypeDriver):
 
     def __init__(self, segment_model, endpoint_model):
-        super(EndpointTunnelTypeDriver, self).__init__(segment_model)
+        super().__init__(segment_model)
         self.endpoint_model = endpoint_model.db_model
         self.segmentation_key = next(iter(self.primary_keys))
 
@@ -396,7 +397,7 @@ class EndpointTunnelTypeDriver(ML2TunnelTypeDriver):
         return endpoint
 
 
-class TunnelRpcCallbackMixin(object):
+class TunnelRpcCallbackMixin:
 
     def setup_tunnel_callback_mixin(self, notifier, type_manager):
         self._notifier = notifier
@@ -429,67 +430,65 @@ class TunnelRpcCallbackMixin(object):
             raise exc.InvalidInput(error_message=msg)
 
         driver = self._type_manager.drivers.get(tunnel_type)
-        if driver:
-            # The given conditional statements will verify the following
-            # things:
-            # 1. If host is not passed from an agent, it is a legacy mode.
-            # 2. If passed host and tunnel_ip are not found in the DB,
-            #    it is a new endpoint.
-            # 3. If host is passed from an agent and it is not found in DB
-            #    but the passed tunnel_ip is found, delete the endpoint
-            #    from DB and add the endpoint with (tunnel_ip, host),
-            #    it is an upgrade case.
-            # 4. If passed host is found in DB and passed tunnel ip is not
-            #    found, delete the endpoint belonging to that host and
-            #    add endpoint with latest (tunnel_ip, host), it is a case
-            #    where local_ip of an agent got changed.
-            # 5. If the passed host had another ip in the DB the host-id has
-            #    roamed to a different IP then delete any reference to the new
-            #    local_ip or the host id. Don't notify tunnel_delete for the
-            #    old IP since that one could have been taken by a different
-            #    agent host-id (neutron-ovs-cleanup should be used to clean up
-            #    the stale endpoints).
-            #    Finally create a new endpoint for the (tunnel_ip, host).
-            if host:
-                host_endpoint = driver.obj.get_endpoint_by_host(host)
-                ip_endpoint = driver.obj.get_endpoint_by_ip(tunnel_ip)
-
-                if (ip_endpoint and ip_endpoint.host is None and
-                        host_endpoint is None):
-                    driver.obj.delete_endpoint(ip_endpoint.ip_address)
-                elif (ip_endpoint and ip_endpoint.host != host):
-                    LOG.info(
-                        "Tunnel IP %(ip)s was used by host %(host)s and "
-                        "will be assigned to %(new_host)s",
-                        {'ip': ip_endpoint.ip_address,
-                         'host': ip_endpoint.host,
-                         'new_host': host})
-                    driver.obj.delete_endpoint_by_host_or_ip(
-                        host, ip_endpoint.ip_address)
-                elif (host_endpoint and host_endpoint.ip_address != tunnel_ip):
-                    # Notify all other listening agents to delete stale tunnels
-                    self._notifier.tunnel_delete(
-                        rpc_context, host_endpoint.ip_address, tunnel_type)
-                    driver.obj.delete_endpoint(host_endpoint.ip_address)
-
-            tunnel = driver.obj.add_endpoint(tunnel_ip, host)
-            tunnels = driver.obj.get_endpoints()
-            entry = {'tunnels': tunnels}
-            # Notify all other listening agents
-            self._notifier.tunnel_update(rpc_context, tunnel.ip_address,
-                                         tunnel_type)
-            # Return the list of tunnels IP's to the agent
-            return entry
-        else:
+        if not driver:
             msg = (_("Network type value %(type)s not supported, "
-                    "host: %(host)s with tunnel IP: %(ip)s") %
-                    {'type': tunnel_type,
-                     'host': host or 'legacy mode (no host provided by agent)',
-                     'ip': tunnel_ip})
+                     "host: %(host)s with tunnel IP: %(ip)s") %
+                   {'type': tunnel_type,
+                    'host': host or 'legacy mode (no host provided by agent)',
+                    'ip': tunnel_ip})
             raise exc.InvalidInput(error_message=msg)
 
+        # The given conditional statements will verify the following things:
+        # 1. If host is not passed from an agent, it is a legacy mode.
+        # 2. If passed host and tunnel_ip are not found in the DB, it is a new
+        #    endpoint.
+        # 3. If host is passed from an agent and it is not found in DB but the
+        #    passed tunnel_ip is found, delete the endpoint from DB and add the
+        #    endpoint with (tunnel_ip, host), it is an upgrade case.
+        # 4. If passed host is found in DB and passed tunnel ip is not found,
+        #    delete the endpoint belonging to that host and add endpoint with
+        #    latest (tunnel_ip, host), it is a case where local_ip of an agent
+        #    got changed.
+        # 5. If the passed host had another ip in the DB the host-id has roamed
+        #    to a different IP then delete any reference to the new local_ip or
+        #    the host id. Don't notify tunnel_delete for the old IP since that
+        #    one could have been taken by a different agent host-id
+        #    (neutron-ovs-cleanup should be used to clean up the stale
+        #    endpoints). Finally create a new endpoint for the (tunnel_ip,
+        #    host).
+        if host:
+            host_endpoint = driver.obj.get_endpoint_by_host(host)
+            ip_endpoint = driver.obj.get_endpoint_by_ip(tunnel_ip)
 
-class TunnelAgentRpcApiMixin(object):
+            if (ip_endpoint and ip_endpoint.host is None and
+                    host_endpoint is None):
+                driver.obj.delete_endpoint(ip_endpoint.ip_address)
+            elif (ip_endpoint and ip_endpoint.host != host):
+                LOG.info(
+                    "Tunnel IP %(ip)s was used by host %(host)s and "
+                    "will be assigned to %(new_host)s",
+                    {'ip': ip_endpoint.ip_address,
+                     'host': ip_endpoint.host,
+                     'new_host': host})
+                driver.obj.delete_endpoint_by_host_or_ip(
+                    host, ip_endpoint.ip_address)
+            elif (host_endpoint and host_endpoint.ip_address != tunnel_ip):
+                # Notify all other listening agents to delete stale tunnels
+                self._notifier.tunnel_delete(
+                    rpc_context, host_endpoint.ip_address, tunnel_type)
+                driver.obj.delete_endpoint(host_endpoint.ip_address)
+
+        tunnel = driver.obj.add_endpoint(tunnel_ip, host)
+        tunnels = driver.obj.get_endpoints()
+        entry = {'tunnels': tunnels}
+        # Notify all other listening agents
+        self._notifier.tunnel_update(rpc_context, tunnel.ip_address,
+                                     tunnel_type)
+        # Return the list of tunnels IP's to the agent
+        return entry
+
+
+class TunnelAgentRpcApiMixin:
 
     def _get_tunnel_update_topic(self):
         return topics.get_topic_name(self.topic,
