@@ -26,7 +26,6 @@ from neutron_lib.plugins.ml2 import api
 from neutron_lib.plugins import utils as plugin_utils
 from oslo_config import cfg
 from oslo_log import log
-from oslo_utils import uuidutils
 
 from neutron._i18n import _
 from neutron.conf.plugins.ml2.drivers import driver_type
@@ -53,29 +52,20 @@ class VlanTypeDriver(helpers.SegmentTypeDriver):
     """
 
     def __init__(self):
-        super(VlanTypeDriver, self).__init__(vlanalloc.VlanAllocation)
+        super().__init__(vlanalloc.VlanAllocation)
         self.model_segmentation_id = vlan_alloc_model.VlanAllocation.vlan_id
         self._parse_network_vlan_ranges()
 
     @db_api.retry_db_errors
-    def _populate_new_default_network_segment_ranges(self):
+    def _populate_new_default_network_segment_ranges(self, start_time):
         ctx = context.get_admin_context()
-        for (physical_network, vlan_ranges) in (
-                self.network_vlan_ranges.items()):
-            for vlan_min, vlan_max in vlan_ranges:
-                res = {
-                    'id': uuidutils.generate_uuid(),
-                    'name': '',
-                    'default': True,
-                    'shared': True,
-                    'network_type': p_const.TYPE_VLAN,
-                    'physical_network': physical_network,
-                    'minimum': vlan_min,
-                    'maximum': vlan_max}
-                with db_api.CONTEXT_WRITER.using(ctx):
-                    new_default_range_obj = (
-                        range_obj.NetworkSegmentRange(ctx, **res))
-                    new_default_range_obj.create()
+        with db_api.CONTEXT_WRITER.using(ctx):
+            for (physical_network, vlan_ranges) in (
+                    self.network_vlan_ranges.items()):
+                for vlan_min, vlan_max in vlan_ranges:
+                    range_obj.NetworkSegmentRange.new_default(
+                        ctx, self.get_type(), physical_network, vlan_min,
+                        vlan_max, start_time)
 
     def _parse_network_vlan_ranges(self):
         try:
@@ -181,9 +171,9 @@ class VlanTypeDriver(helpers.SegmentTypeDriver):
             self._sync_vlan_allocations()
         LOG.info("VlanTypeDriver initialization complete")
 
-    def initialize_network_segment_range_support(self):
-        self._delete_expired_default_network_segment_ranges()
-        self._populate_new_default_network_segment_ranges()
+    def initialize_network_segment_range_support(self, start_time):
+        self._delete_expired_default_network_segment_ranges(start_time)
+        self._populate_new_default_network_segment_ranges(start_time)
         # Override self.network_vlan_ranges with the network segment range
         # information from DB and then do a sync_allocations since the
         # segment range service plugin has not yet been loaded at this
@@ -226,12 +216,6 @@ class VlanTypeDriver(helpers.SegmentTypeDriver):
                            {'min': p_const.MIN_VLAN_TAG,
                             'max': p_const.MAX_VLAN_TAG})
                     raise exc.InvalidInput(error_message=msg)
-            else:
-                if not ranges.get(physical_network):
-                    msg = (_("Physical network %s requires segmentation_id "
-                             "to be specified when creating a provider "
-                             "network") % physical_network)
-                    raise exc.InvalidInput(error_message=msg)
         elif segmentation_id is not None:
             msg = _("segmentation_id requires physical_network for VLAN "
                     "provider network")
@@ -326,7 +310,7 @@ class VlanTypeDriver(helpers.SegmentTypeDriver):
                          'physical_network': physical_network})
 
     def get_mtu(self, physical_network):
-        seg_mtu = super(VlanTypeDriver, self).get_mtu()
+        seg_mtu = super().get_mtu()
         mtu = []
         if seg_mtu > 0:
             mtu.append(seg_mtu)
