@@ -16,22 +16,23 @@
 import functools
 
 import eventlet
+from neutron_lib.callbacks import events
+from neutron_lib.callbacks import registry
+from neutron_lib.callbacks import resources
+from neutron_lib import context as n_context
+from neutron_lib.plugins.ml2 import ovs_constants as ovs_agent_constants
+from neutron_lib.services.trunk import constants
 from oslo_concurrency import lockutils
 from oslo_context import context as o_context
 from oslo_log import log as logging
 import oslo_messaging
 from oslo_serialization import jsonutils
 
-from neutron._i18n import _, _LE
+from neutron._i18n import _
 from neutron.agent.common import ovs_lib
 from neutron.api.rpc.handlers import resources_rpc
-from neutron.callbacks import events
-from neutron.callbacks import registry
 from neutron.common import utils as common_utils
-from neutron import context as n_context
-from neutron.plugins.ml2.drivers.openvswitch.agent.common \
-    import constants as ovs_agent_constants
-from neutron.services.trunk import constants
+from neutron.services.trunk.drivers.openvswitch.agent import exceptions
 from neutron.services.trunk.drivers.openvswitch.agent \
     import trunk_manager as tman
 from neutron.services.trunk.drivers.openvswitch import constants as t_const
@@ -84,8 +85,8 @@ def bridge_has_port(bridge, is_port_predicate):
     try:
         ifaces = bridge.get_iface_name_list()
     except RuntimeError as e:
-        LOG.error(_LE("Cannot obtain interface list for bridge %(bridge)s: "
-                      "%(err)s"),
+        LOG.error("Cannot obtain interface list for bridge %(bridge)s: "
+                  "%(err)s",
                   {'bridge': bridge.br_name,
                    'err': e})
         return False
@@ -93,12 +94,15 @@ def bridge_has_port(bridge, is_port_predicate):
     return any(iface for iface in ifaces if is_port_predicate(iface))
 
 
+def _is_instance_port(port_name):
+    return not is_trunk_service_port(port_name)
+
+
 def bridge_has_instance_port(bridge):
     """True if there is an OVS port that doesn't have bridge or patch ports
        prefix.
     """
-    is_instance_port = lambda p: not is_trunk_service_port(p)
-    return bridge_has_port(bridge, is_instance_port)
+    return bridge_has_port(bridge, _is_instance_port)
 
 
 def bridge_has_service_port(bridge):
@@ -107,7 +111,8 @@ def bridge_has_service_port(bridge):
     return bridge_has_port(bridge, is_trunk_service_port)
 
 
-class OVSDBHandler(object):
+@registry.has_registry_receivers
+class OVSDBHandler:
     """It listens to OVSDB events to create the physical resources associated
     to a logical trunk in response to OVSDB events (such as VM boot and/or
     delete).
@@ -119,18 +124,16 @@ class OVSDBHandler(object):
         self.trunk_manager = trunk_manager
         self.trunk_rpc = agent.TrunkStub()
 
-        registry.subscribe(self.process_trunk_port_events,
-                           ovs_agent_constants.OVSDB_RESOURCE,
-                           events.AFTER_READ)
-
     @property
     def context(self):
         self._context.request_id = o_context.generate_request_id()
         return self._context
 
+    @registry.receives(ovs_agent_constants.OVSDB_RESOURCE, [events.AFTER_READ])
     def process_trunk_port_events(
-            self, resource, event, trigger, ovsdb_events):
+            self, resource, event, trigger, payload):
         """Process added and removed port events coming from OVSDB monitor."""
+        ovsdb_events = payload.latest_state
         for port_event in ovsdb_events['added']:
             port_name = port_event['name']
             if is_trunk_bridge(port_name):
@@ -184,13 +187,13 @@ class OVSDBHandler(object):
         try:
             self._wire_trunk(bridge, self._get_parent_port(bridge), rewire)
         except oslo_messaging.MessagingException as e:
-            LOG.error(_LE("Got messaging error while processing trunk bridge "
-                          "%(bridge_name)s: %(err)s"),
+            LOG.error("Got messaging error while processing trunk bridge "
+                      "%(bridge_name)s: %(err)s",
                       {'bridge_name': bridge.br_name,
                        'err': e})
-        except RuntimeError as e:
-            LOG.error(_LE("Failed to get parent port for bridge "
-                          "%(bridge_name)s: %(err)s"),
+        except exceptions.ParentPortNotFound as e:
+            LOG.error("Failed to get parent port for bridge "
+                      "%(bridge_name)s: %(err)s",
                       {'bridge_name': bridge.br_name,
                        'err': e})
 
@@ -218,7 +221,7 @@ class OVSDBHandler(object):
             self.unwire_subports_for_trunk(trunk_id, subport_ids)
             self.trunk_manager.remove_trunk(trunk_id, parent_port_id)
         except tman.TrunkManagerError as te:
-            LOG.error(_LE("Removing trunk %(trunk_id)s failed: %(err)s"),
+            LOG.error("Removing trunk %(trunk_id)s failed: %(err)s",
                       {'trunk_id': port['external_ids']['trunk_id'],
                        'err': te})
         else:
@@ -236,14 +239,14 @@ class OVSDBHandler(object):
             return []
         try:
             ports = bridge.get_ports_attributes(
-                            'Interface', columns=['name', 'external_ids'])
+                'Interface', columns=['name', 'external_ids'])
             return [
                 self.trunk_manager.get_port_uuid_from_external_ids(port)
                 for port in ports if is_subport(port['name'])
             ]
         except (RuntimeError, tman.TrunkManagerError) as e:
-            LOG.error(_LE("Failed to get subports for bridge %(bridge)s: "
-                          "%(err)s"), {'bridge': bridge.br_name, 'err': e})
+            LOG.error("Failed to get subports for bridge %(bridge)s: "
+                      "%(err)s", {'bridge': bridge.br_name, 'err': e})
             return []
 
     def wire_subports_for_trunk(self, context, trunk_id, subports,
@@ -263,9 +266,9 @@ class OVSDBHandler(object):
                                                 subports_mac[subport.port_id],
                                                 subport.segmentation_id)
             except tman.TrunkManagerError as te:
-                LOG.error(_LE("Failed to add subport with port ID "
-                              "%(subport_port_id)s to trunk with ID "
-                              "%(trunk_id)s: %(err)s"),
+                LOG.error("Failed to add subport with port ID "
+                          "%(subport_port_id)s to trunk with ID "
+                          "%(trunk_id)s: %(err)s",
                           {'subport_port_id': subport.port_id,
                            'trunk_id': trunk_id,
                            'err': te})
@@ -275,43 +278,55 @@ class OVSDBHandler(object):
         try:
             self._update_trunk_metadata(
                 trunk_bridge, parent_port, trunk_id, subport_ids)
-        except RuntimeError:
-            LOG.error(_LE("Failed to store metadata for trunk %s"), trunk_id)
+        except (RuntimeError, exceptions.ParentPortNotFound) as e:
+            LOG.error("Failed to store metadata for trunk %(trunk_id)s: "
+                      "%(reason)s", {'trunk_id': trunk_id, 'reason': e})
             # NOTE(status_police): Trunk bridge has stale metadata now, it
             # might cause troubles during deletion. Signal a DEGRADED status;
             # if the user undo/redo the operation things may go back to
             # normal.
-            return constants.DEGRADED_STATUS
+            return constants.TRUNK_DEGRADED_STATUS
 
         LOG.debug("Added trunk: %s", trunk_id)
         return self._get_current_status(subports, subport_ids)
 
-    def unwire_subports_for_trunk(self, trunk_id, subport_ids):
-        """Destroy OVS ports associated to the logical subports."""
+    def _remove_sub_ports(self, trunk_id, subport_ids):
         ids = []
         for subport_id in subport_ids:
             try:
                 self.trunk_manager.remove_sub_port(trunk_id, subport_id)
                 ids.append(subport_id)
             except tman.TrunkManagerError as te:
-                LOG.error(_LE("Removing subport %(subport_id)s from trunk "
-                              "%(trunk_id)s failed: %(err)s"),
+                LOG.error("Removing subport %(subport_id)s from trunk "
+                          "%(trunk_id)s failed: %(err)s",
                           {'subport_id': subport_id,
                            'trunk_id': trunk_id,
                            'err': te})
+        return ids
+
+    def unwire_subports_for_trunk(self, trunk_id, subport_ids):
+        """Destroy OVS ports associated to the logical subports."""
+        ids = self._remove_sub_ports(trunk_id, subport_ids)
         try:
             # OVS bridge and port to be determined by _update_trunk_metadata
             bridge = None
             port = None
             self._update_trunk_metadata(
                 bridge, port, trunk_id, subport_ids, wire=False)
-        except RuntimeError:
+        except RuntimeError as e:
             # NOTE(status_police): Trunk bridge has stale metadata now, it
             # might cause troubles during deletion. Signal a DEGRADED status;
             # if the user undo/redo the operation things may go back to
             # normal.
-            LOG.error(_LE("Failed to store metadata for trunk %s"), trunk_id)
-            return constants.DEGRADED_STATUS
+            LOG.error("Failed to store metadata for trunk %(trunk_id)s: "
+                      "%(reason)s", {'trunk_id': trunk_id, 'reason': e})
+            return constants.TRUNK_DEGRADED_STATUS
+        except exceptions.ParentPortNotFound as e:
+            # If a user deletes/migrates a VM and remove subports from a trunk
+            # in short sequence, there is a chance that we hit this spot in
+            # that the trunk may still be momentarily bound to the agent. We
+            # should not mark the status as DEGRADED in this case.
+            LOG.debug(e)
 
         return self._get_current_status(subport_ids, ids)
 
@@ -327,9 +342,7 @@ class OVSDBHandler(object):
         for trunk_br_port in trunk_br_ports:
             if not is_trunk_service_port(trunk_br_port['name']):
                 return trunk_br_port
-        raise RuntimeError(
-            "Can't find parent port for trunk bridge %s" %
-            trunk_bridge.br_name)
+        raise exceptions.ParentPortNotFound(bridge=trunk_bridge.br_name)
 
     def _wire_trunk(self, trunk_br, port, rewire=False):
         """Wire trunk bridge with integration bridge.
@@ -350,43 +363,56 @@ class OVSDBHandler(object):
             parent_port_id = (
                 self.trunk_manager.get_port_uuid_from_external_ids(port))
             trunk = self.trunk_rpc.get_trunk_details(ctx, parent_port_id)
-        except tman.TrunkManagerError as te:
-            LOG.error(_LE("Can't obtain parent port ID from port %s"),
+        except tman.TrunkManagerError:
+            LOG.error("Can't obtain parent port ID from port %s",
                       port['name'])
             return
         except resources_rpc.ResourceNotFound:
-            LOG.error(_LE("Port %s has no trunk associated."), parent_port_id)
+            LOG.error("Port %s has no trunk associated.", parent_port_id)
             return
 
         try:
+            registry.publish(
+                resources.TRUNK, events.BEFORE_CREATE, self,
+                payload=events.DBEventPayload(ctx, resource_id=trunk.id,
+                                              desired_state=trunk))
             self.trunk_manager.create_trunk(
                 trunk.id, trunk.port_id,
                 port['external_ids'].get('attached-mac'))
         except tman.TrunkManagerError as te:
-            LOG.error(_LE("Failed to create trunk %(trunk_id)s: %(err)s"),
+            LOG.error("Failed to create trunk %(trunk_id)s: %(err)s",
                       {'trunk_id': trunk.id,
                        'err': te})
             # NOTE(status_police): Trunk couldn't be created so it ends in
             # ERROR status and resync can fix that later.
-            self.report_trunk_status(ctx, trunk.id, constants.ERROR_STATUS)
+            self.report_trunk_status(
+                ctx, trunk.id, constants.TRUNK_ERROR_STATUS)
             return
 
         # We need to remove stale subports
+        unwire_status = constants.TRUNK_ACTIVE_STATUS
         if rewire:
             old_subport_ids = self.get_connected_subports_for_trunk(trunk.id)
             subports = {p['port_id'] for p in trunk.sub_ports}
             subports_to_delete = set(old_subport_ids) - subports
             if subports_to_delete:
-                self.unwire_subports_for_trunk(trunk.id, subports_to_delete)
+                unwire_status = self.unwire_subports_for_trunk(
+                    trunk.id, subports_to_delete)
 
         # NOTE(status_police): inform the server whether the operation
         # was a partial or complete success. Do not inline status.
         # NOTE: in case of rewiring we readd ports that are already present on
         # the bridge because e.g. the segmentation ID might have changed (e.g.
         # agent crashed, port was removed and readded with a different seg ID)
-        status = self.wire_subports_for_trunk(
+        wire_status = self.wire_subports_for_trunk(
             ctx, trunk.id, trunk.sub_ports,
             trunk_bridge=trunk_br, parent_port=port)
+
+        if (unwire_status == wire_status and
+                wire_status == constants.TRUNK_ACTIVE_STATUS):
+            status = constants.TRUNK_ACTIVE_STATUS
+        else:
+            status = constants.TRUNK_DEGRADED_STATUS
         self.report_trunk_status(ctx, trunk.id, status)
 
     def _set_trunk_metadata(self, trunk_bridge, port, trunk_id, subport_ids):
@@ -424,8 +450,13 @@ class OVSDBHandler(object):
         :param subport_ids: subports affecting the metadata.
         :param wire: if True subport_ids are added, otherwise removed.
         """
-        trunk_bridge = trunk_bridge or ovs_lib.OVSBridge(
-            utils.gen_trunk_br_name(trunk_id))
+        bridge_name = utils.gen_trunk_br_name(trunk_id)
+        trunk_bridge = trunk_bridge or ovs_lib.OVSBridge(bridge_name)
+        if not trunk_bridge.bridge_exists(bridge_name):
+            LOG.info(
+                "Could not update trunk metadata in a non-existent bridge "
+                "(likely already deleted by nova/os-vif): %s", bridge_name)
+            return
         port = port or self._get_parent_port(trunk_bridge)
         _port_id, _trunk_id, old_subports = self._get_trunk_metadata(port)
         if wire:
@@ -445,9 +476,8 @@ class OVSDBHandler(object):
         # a trunk_update_status to report the latest trunk status, but there
         # can be exceptions (e.g. unwire_subports_for_trunk).
         if len(expected_subports) != len(actual_subports):
-            return constants.DEGRADED_STATUS
-        else:
-            return constants.ACTIVE_STATUS
+            return constants.TRUNK_DEGRADED_STATUS
+        return constants.TRUNK_ACTIVE_STATUS
 
     def _is_vm_connected(self, bridge):
         """True if an instance is connected to bridge, False otherwise."""
@@ -458,10 +488,10 @@ class OVSDBHandler(object):
                 bridge_has_port_predicate,
                 timeout=self.timeout)
             return True
-        except eventlet.TimeoutError:
+        except common_utils.WaitTimeout:
             LOG.error(
-                _LE('No port present on trunk bridge %(br_name)s '
-                    'in %(timeout)d seconds.'),
+                'No port present on trunk bridge %(br_name)s '
+                'in %(timeout)d seconds.',
                 {'br_name': bridge.br_name,
                  'timeout': self.timeout})
         return False
