@@ -21,7 +21,6 @@ from neutron_lib.plugins.ml2 import ovs_constants
 from oslo_log import log as logging
 
 from neutron.agent.common import async_process
-from neutron.agent.linux import iptables_manager
 from neutron.common import utils as common_utils
 from neutron.plugins.ml2.common import constants as comm_consts
 
@@ -47,15 +46,6 @@ def extract_mod_nw_tos_action(flows):
     return tos_mark
 
 
-def extract_dscp_value_from_iptables_rules(rules):
-    pattern = (r"^-A neutron-linuxbri-qos-.* -j DSCP "
-               "--set-dscp (?P<dscp_value>0x[A-Fa-f0-9]+)$")
-    for rule in rules:
-        m = re.match(pattern, rule)
-        if m:
-            return int(m.group("dscp_value"), 16)
-
-
 def wait_until_bandwidth_limit_rule_applied(check_function, port_vif, rule):
     def _bandwidth_limit_rule_applied():
         bw_rule = check_function(port_vif)
@@ -98,7 +88,7 @@ def wait_until_pkt_meter_rule_applied_ovs(bridge, port_vif, port_id,
     def _pkt_rate_limit_rule_applied():
         port_num = bridge.get_port_ofport(port_vif)
         port_vlan = bridge.get_port_tag_by_name(port_vif)
-        key = "%s_%s_%s" % (type_, port_id, direction)
+        key = "{}_{}_{}".format(type_, port_id, direction)
         meter_id = bridge.get_value_from_other_config(
             port_vif, key, value_type=int)
 
@@ -114,26 +104,11 @@ def wait_until_pkt_meter_rule_applied_ovs(bridge, port_vif, port_id,
                                           dl_dst=str(mac))
         if mac:
             return bool(flows) and meter_id
-        else:
-            return not bool(flows) and not meter_id
+        return not bool(flows) and not meter_id
 
     common_utils.wait_until_true(_pkt_rate_limit_rule_applied)
 
 
-def wait_until_dscp_marking_rule_applied_linuxbridge(namespace, port_vif,
-                                                     expected_rule):
-
-    iptables = iptables_manager.IptablesManager(
-        namespace=namespace)
-
-    def _dscp_marking_rule_applied():
-        mangle_rules = iptables.get_rules_for_table("mangle")
-        dscp_mark = extract_dscp_value_from_iptables_rules(mangle_rules)
-        return dscp_mark == expected_rule
-
-    common_utils.wait_until_true(_dscp_marking_rule_applied)
-
-
 def wait_for_dscp_marked_packet(sender_vm, receiver_vm, dscp_mark):
     cmd = [
         "tcpdump", "-i", receiver_vm.port.name, "-nlt",
@@ -160,7 +135,7 @@ def wait_for_dscp_marked_packet(sender_vm, receiver_vm, dscp_mark):
             return
         tcpdump_stderr_lines.append(line)
 
-    tcpdump_stdout_lines = [line for line in tcpdump_async.iter_stdout()]
+    tcpdump_stdout_lines = list(tcpdump_async.iter_stdout())
     LOG.debug("Captured output lines from tcpdump. Stdout: %s; Stderr: %s",
               tcpdump_stdout_lines, tcpdump_stderr_lines)
 
