@@ -15,12 +15,12 @@
 import functools
 
 import fixtures
+import netaddr
 from neutron_lib import constants
+from neutron_lib.plugins.ml2 import ovs_constants as ovs_consts
+from oslo_config import cfg
 from oslo_utils import uuidutils
 
-from neutron.agent import firewall
-from neutron.agent.linux import ip_lib
-from neutron.common import constants as n_consts
 from neutron.common import utils as common_utils
 from neutron.tests.common import machine_fixtures
 from neutron.tests.common import net_helpers
@@ -42,8 +42,8 @@ class ConnectionTesterException(Exception):
 def _validate_direction(f):
     @functools.wraps(f)
     def wrap(self, direction, *args, **kwargs):
-        if direction not in (firewall.INGRESS_DIRECTION,
-                             firewall.EGRESS_DIRECTION):
+        if direction not in (constants.INGRESS_DIRECTION,
+                             constants.EGRESS_DIRECTION):
             raise ConnectionTesterException('Unknown direction %s' % direction)
         return f(self, direction, *args, **kwargs)
     return wrap
@@ -79,9 +79,9 @@ class ConnectionTester(fixtures.Fixture):
     UDP = net_helpers.NetcatTester.UDP
     TCP = net_helpers.NetcatTester.TCP
     ICMP = constants.PROTO_NAME_ICMP
-    ARP = n_consts.ETHERTYPE_NAME_ARP
-    INGRESS = firewall.INGRESS_DIRECTION
-    EGRESS = firewall.EGRESS_DIRECTION
+    ARP = constants.ETHERTYPE_NAME_ARP
+    INGRESS = constants.INGRESS_DIRECTION
+    EGRESS = constants.EGRESS_DIRECTION
 
     def __init__(self, ip_cidr):
         self.ip_cidr = ip_cidr
@@ -173,15 +173,15 @@ class ConnectionTester(fixtures.Fixture):
 
     def _test_icmp_connectivity(self, direction, protocol, src_port, dst_port):
         src_namespace, ip_address = self._get_namespace_and_address(direction)
-        ip_version = ip_lib.get_ip_version(ip_address)
+        ip_version = common_utils.get_ip_version(ip_address)
         icmp_timeout = ICMP_VERSION_TIMEOUTS[ip_version]
         try:
             net_helpers.assert_ping(src_namespace, ip_address,
                                     timeout=icmp_timeout)
         except RuntimeError:
             raise ConnectionTesterException(
-                "ICMP packets can't get from %s namespace to %s address" % (
-                    src_namespace, ip_address))
+                "ICMP packets can't get from {} namespace to {} "
+                "address".format(src_namespace, ip_address))
 
     def _test_arp_connectivity(self, direction, protocol, src_port, dst_port):
         src_namespace, ip_address = self._get_namespace_and_address(direction)
@@ -205,8 +205,8 @@ class ConnectionTester(fixtures.Fixture):
         except ConnectionTesterException:
             pass
         else:
-            dst_port_info = str()
-            src_port_info = str()
+            dst_port_info = ''
+            src_port_info = ''
             if dst_port is not None:
                 dst_port_info = " and destination port %d" % dst_port
             if src_port is not None:
@@ -223,26 +223,23 @@ class ConnectionTester(fixtures.Fixture):
                                       dst_port=None):
         nc_params = (direction, protocol, src_port, dst_port)
         nc_tester = self._nc_testers.get(nc_params)
-        if nc_tester:
-            if nc_tester.is_established:
-                try:
-                    nc_tester.test_connectivity()
-                except RuntimeError:
-                    raise ConnectionTesterException(
-                        "Established %s connection with protocol %s, source "
-                        "port %s and destination port %s can no longer "
-                        "communicate")
-            else:
-                nc_tester.stop_processes()
-                raise ConnectionTesterException(
-                    '%s connection with protocol %s, source port %s and '
-                    'destination port %s is not established' % nc_params)
-        else:
+        if not nc_tester:
             raise ConnectionTesterException(
                 "Attempting to test established %s connection with protocol %s"
                 ", source port %s and destination port %s that hasn't been "
                 "established yet by calling establish_connection()"
                 % nc_params)
+        if not nc_tester.is_established:
+            nc_tester.stop_processes()
+            raise ConnectionTesterException(
+                '%s connection with protocol %s, source port %s and '
+                'destination port %s is not established' % nc_params)
+        try:
+            nc_tester.test_connectivity()
+        except RuntimeError:
+            raise ConnectionTesterException(
+                "Established %s connection with protocol %s, source port %s "
+                "and destination port %s can no longer communicate")
 
     def assert_no_established_connection(self, direction, protocol,
                                          src_port=None, dst_port=None):
@@ -305,7 +302,8 @@ class ConnectionTester(fixtures.Fixture):
         except KeyError:
             src_namespace, dst_address = self._get_namespace_and_address(
                 direction)
-            pinger = net_helpers.Pinger(src_namespace, dst_address)
+            pinger = net_helpers.Pinger(
+                src_namespace, dst_address, interval=0.3)
             self._pingers[direction] = pinger
         return pinger
 
@@ -358,6 +356,12 @@ class ConnectionTester(fixtures.Fixture):
                 "At least one packet got reply from %s namespace to %s "
                 "address." % self._get_namespace_and_address(direction)))
 
+    def set_peer_port_as_patch_port(self):
+        pass
+
+    def set_peer_port_as_vm_port(self):
+        pass
+
 
 class OVSBaseConnectionTester(ConnectionTester):
 
@@ -371,12 +375,12 @@ class OVSBaseConnectionTester(ConnectionTester):
 
     @staticmethod
     def set_tag(port_name, bridge, tag):
-        bridge.set_db_attribute('Port', port_name, 'tag', tag)
-        other_config = bridge.db_get_val(
-            'Port', port_name, 'other_config')
-        other_config['tag'] = tag
-        bridge.set_db_attribute(
-            'Port', port_name, 'other_config', other_config)
+        ovsdb = bridge.ovsdb
+        with ovsdb.transaction() as txn:
+            txn.add(ovsdb.db_set('Port', port_name, ('tag', tag)))
+            txn.add(
+                ovsdb.db_add(
+                    'Port', port_name, 'other_config', {'tag': str(tag)}))
 
 
 class OVSConnectionTester(OVSBaseConnectionTester):
@@ -391,9 +395,18 @@ class OVSConnectionTester(OVSBaseConnectionTester):
 
     """
 
+    def __init__(self, ip_cidr, br_int_cls):
+        super().__init__(ip_cidr)
+        self.br_int_cls = br_int_cls
+
     def _setUp(self):
-        super(OVSConnectionTester, self)._setUp()
-        self.bridge = self.useFixture(net_helpers.OVSBridgeFixture()).bridge
+        super()._setUp()
+        br_name = self.useFixture(
+            net_helpers.OVSBridgeFixture()).bridge.br_name
+        self.bridge = self.br_int_cls(br_name)
+        self.bridge.set_secure_mode()
+        self.bridge.setup_controllers(cfg.CONF)
+        self.bridge.setup_default_table()
         machines = self.useFixture(
             machine_fixtures.PeerMachines(
                 self.bridge, self.ip_cidr)).machines
@@ -414,9 +427,47 @@ class OVSConnectionTester(OVSBaseConnectionTester):
 
     def set_vm_tag(self, tag):
         self.set_tag(self._vm.port.name, self.bridge, tag)
+        self._vm.port.vlan_tag = tag
 
     def set_peer_tag(self, tag):
         self.set_tag(self._peer.port.name, self.bridge, tag)
+        self._peer.port.vlan_tag = tag
+
+    def set_peer_port_as_patch_port(self):
+        """As packets coming from tunneling bridges are always tagged with
+        local VLAN tag, this flows will simulate the behavior.
+        """
+        self.bridge.add_flow(
+            table=ovs_consts.LOCAL_SWITCHING,
+            priority=110,
+            vlan_tci=0,
+            in_port=self.bridge.get_port_ofport(self._peer.port.name),
+            actions='mod_vlan_vid:0x%x,'
+                    'resubmit(,%d)' % (
+                        self._peer.port.vlan_tag,
+                        ovs_consts.LOCAL_SWITCHING)
+        )
+        self.bridge.add_flow(
+            table=ovs_consts.TRANSIENT_TABLE,
+            priority=4,
+            dl_vlan='0x%x' % self._peer.port.vlan_tag,
+            actions='strip_vlan,normal'
+        )
+
+    def set_peer_port_as_vm_port(self):
+        """Remove flows simulating traffic from tunneling bridges.
+
+        This method is opposite to set_peer_port_as_patch_port().
+        """
+        self.bridge.delete_flows(
+            table=ovs_consts.LOCAL_SWITCHING,
+            vlan_tci=0,
+            in_port=self.bridge.get_port_ofport(self._peer.port.name),
+        )
+        self.bridge.delete_flows(
+            table=ovs_consts.TRANSIENT_TABLE,
+            dl_vlan='0x%x' % self._peer.port.vlan_tag,
+        )
 
 
 class OVSTrunkConnectionTester(OVSBaseConnectionTester):
@@ -433,11 +484,11 @@ class OVSTrunkConnectionTester(OVSBaseConnectionTester):
     """
 
     def __init__(self, ip_cidr, br_trunk_name):
-        super(OVSTrunkConnectionTester, self).__init__(ip_cidr)
+        super().__init__(ip_cidr)
         self._br_trunk_name = br_trunk_name
 
     def _setUp(self):
-        super(OVSTrunkConnectionTester, self)._setUp()
+        super()._setUp()
         self.bridge = self.useFixture(
             net_helpers.OVSBridgeFixture()).bridge
         self.br_trunk = self.useFixture(
@@ -462,13 +513,11 @@ class OVSTrunkConnectionTester(OVSBaseConnectionTester):
 
         """
 
-        ip_wrap = ip_lib.IPWrapper(self._vm.namespace)
-        dev_name = self._vm.port.name + ".%d" % vlan
-        ip_wrap.add_vlan(dev_name, self._vm.port.name, vlan)
-        dev = ip_wrap.device(dev_name)
-        dev.addr.add(ip_cidr)
-        dev.link.set_up()
-        self._ip_vlan = ip_cidr.partition('/')[0]
+        network = netaddr.IPNetwork(ip_cidr)
+        net_helpers.create_vlan_interface(
+            self._vm.namespace, self._vm.port.name,
+            self.vm_mac_address, network, vlan)
+        self._ip_vlan = str(network.ip)
         ip_cidr = net_helpers.increment_ip_cidr(ip_cidr, 1)
         self._peer2 = self.useFixture(machine_fixtures.FakeMachine(
             self.bridge, ip_cidr))
@@ -515,9 +564,18 @@ class LinuxBridgeConnectionTester(ConnectionTester):
 
     """
 
+    def __init__(self, *args, **kwargs):
+        self.bridge_name = kwargs.pop('bridge_name', None)
+        super().__init__(*args, **kwargs)
+
     def _setUp(self):
-        super(LinuxBridgeConnectionTester, self)._setUp()
-        self.bridge = self.useFixture(net_helpers.LinuxBridgeFixture()).bridge
+        super()._setUp()
+        bridge_args = {}
+        if self.bridge_name:
+            bridge_args = {'prefix': self.bridge_name,
+                           'prefix_is_full_name': True}
+        self.bridge = self.useFixture(
+            net_helpers.LinuxBridgeFixture(**bridge_args)).bridge
         machines = self.useFixture(
             machine_fixtures.PeerMachines(
                 self.bridge, self.ip_cidr)).machines
@@ -538,4 +596,4 @@ class LinuxBridgeConnectionTester(ConnectionTester):
 
     def flush_arp_tables(self):
         self.bridge.neigh.flush(4, 'all')
-        super(LinuxBridgeConnectionTester, self).flush_arp_tables()
+        super().flush_arp_tables()
