@@ -38,7 +38,7 @@ CMD_FOLDER = 'agents'
 class ProcessFixture(fixtures.Fixture):
     def __init__(self, test_name, process_name, exec_name, config_filenames,
                  namespace=None, slice_name=None):
-        super(ProcessFixture, self).__init__()
+        super().__init__()
         self.test_name = test_name
         self.process_name = process_name
         self.exec_name = exec_name
@@ -67,7 +67,7 @@ class ProcessFixture(fixtures.Fixture):
         fileutils.ensure_tree(log_dir, mode=0o755)
 
         timestamp = datetime.datetime.now().strftime("%Y-%m-%d--%H-%M-%S-%f")
-        log_file = "%s--%s.log" % (self.process_name, timestamp)
+        log_file = "{}--{}.log".format(self.process_name, timestamp)
         run_as_root = bool(self.namespace)
         exec_name = (self.exec_name
                      if run_as_root
@@ -106,7 +106,7 @@ class ProcessFixture(fixtures.Fixture):
             # run unprivileged if run_as_root is False.
             run_as_root=True,
         )
-        common_utils.wait_until_true(self.service_is_active)
+        fullstack_base.wait_until_true(self.service_is_active)
         LOG.debug("Process started: %s", self.process_name)
 
     def stop(self, kill_signal=None):
@@ -114,6 +114,9 @@ class ProcessFixture(fixtures.Fixture):
             return
 
         if kill_signal:
+            # systemd in ubuntu noble returns invalid-argument for some child
+            # processes
+            check_exit_code = False
             stop_cmd = [
                 'systemctl',
                 'kill',
@@ -124,11 +127,13 @@ class ProcessFixture(fixtures.Fixture):
             msg = (f'Process killed with signal {kill_signal}: '
                    f'{self.process_name}')
         else:
+            check_exit_code = True
             stop_cmd = ['systemctl', 'stop', '--no-block', self.unit_name]
             msg = f'Process stopped: {self.process_name}'
 
-        utils.execute(stop_cmd, run_as_root=True)
-        common_utils.wait_until_true(self.process_is_not_running)
+        utils.execute(stop_cmd, run_as_root=True,
+                      check_exit_code=check_exit_code)
+        fullstack_base.wait_until_true(self.process_is_not_running)
         LOG.debug(msg)
 
     def restart(self, executor=None):
@@ -141,17 +146,17 @@ class ProcessFixture(fixtures.Fixture):
                     self.unit_name,
                 ]
                 utils.execute(restart_cmd, run_as_root=True)
-                common_utils.wait_until_true(self.service_is_active)
+                fullstack_base.wait_until_true(self.service_is_active)
             else:
                 self.start()
 
         LOG.debug("Restarting process: %s", self.process_name)
 
-        if executor is None:
-            _restart()
-        else:
+        if executor is not None:
             return executor.submit(_restart)
 
+        _restart()
+
     @property
     def service_state(self):
         cmd = ['systemctl', 'is-active', self.unit_name]
@@ -175,7 +180,7 @@ class ProcessFixture(fixtures.Fixture):
 class RabbitmqEnvironmentFixture(fixtures.Fixture):
 
     def __init__(self, host="127.0.0.1"):
-        super(RabbitmqEnvironmentFixture, self).__init__()
+        super().__init__()
         self.host = host
 
     def _setUp(self):
@@ -216,7 +221,7 @@ class NeutronServerFixture(ServiceFixture):
     def __init__(self, env_desc, host_desc,
                  test_name, neutron_cfg_fixture, plugin_cfg_fixture,
                  service_cfg_fixtures=None):
-        super(NeutronServerFixture, self).__init__()
+        super().__init__()
         self.env_desc = env_desc
         self.host_desc = host_desc
         self.test_name = test_name
@@ -239,7 +244,7 @@ class NeutronServerFixture(ServiceFixture):
             exec_name=self.NEUTRON_SERVER,
             config_filenames=config_filenames))
 
-        common_utils.wait_until_true(self.server_is_live)
+        fullstack_base.wait_until_true(self.server_is_live)
 
     def server_is_live(self):
         try:
@@ -259,7 +264,7 @@ class OVSAgentFixture(ServiceFixture):
 
     def __init__(self, env_desc, host_desc,
                  test_name, neutron_cfg_fixture, agent_cfg_fixture):
-        super(OVSAgentFixture, self).__init__()
+        super().__init__()
         self.env_desc = env_desc
         self.host_desc = host_desc
         self.test_name = test_name
@@ -291,7 +296,7 @@ class OVSAgentFixture(ServiceFixture):
 class PlacementFixture(ServiceFixture):
 
     def __init__(self, env_desc, host_desc, test_name, placement_cfg_fixture):
-        super(PlacementFixture, self).__init__()
+        super().__init__()
         self.env_desc = env_desc
         self.host_desc = host_desc
         self.test_name = test_name
@@ -309,11 +314,32 @@ class PlacementFixture(ServiceFixture):
             config_filenames=[self.placement_cfg_fixture.filename]))
 
 
+class MetadataFixture(ServiceFixture):
+
+    def __init__(self, env_desc, host_desc, test_name, metadata_cfg_fixture):
+        super().__init__()
+        self.env_desc = env_desc
+        self.host_desc = host_desc
+        self.test_name = test_name
+        self.metadata_cfg_fixture = metadata_cfg_fixture
+        self.metadata_config = self.metadata_cfg_fixture.config
+
+    def _setUp(self):
+        self.process_fixture = self.useFixture(ProcessFixture(
+            test_name=self.test_name,
+            process_name='metadata',
+            exec_name=shutil.which(
+                'metadata.py', path=os.path.join(fullstack_base.ROOTDIR,
+                                                 'servers')
+            ),
+            config_filenames=[self.metadata_cfg_fixture.filename]))
+
+
 class SRIOVAgentFixture(ServiceFixture):
 
     def __init__(self, env_desc, host_desc,
                  test_name, neutron_cfg_fixture, agent_cfg_fixture):
-        super(SRIOVAgentFixture, self).__init__()
+        super().__init__()
         self.env_desc = env_desc
         self.host_desc = host_desc
         self.test_name = test_name
@@ -335,43 +361,12 @@ class SRIOVAgentFixture(ServiceFixture):
             config_filenames=config_filenames))
 
 
-class LinuxBridgeAgentFixture(ServiceFixture):
-
-    def __init__(self, env_desc, host_desc, test_name,
-                 neutron_cfg_fixture, agent_cfg_fixture,
-                 namespace=None):
-        super(LinuxBridgeAgentFixture, self).__init__()
-        self.env_desc = env_desc
-        self.host_desc = host_desc
-        self.test_name = test_name
-        self.neutron_cfg_fixture = neutron_cfg_fixture
-        self.neutron_config = self.neutron_cfg_fixture.config
-        self.agent_cfg_fixture = agent_cfg_fixture
-        self.agent_config = agent_cfg_fixture.config
-        self.namespace = namespace
-        self.hostname = self.neutron_config['DEFAULT']['host']
-
-    def _setUp(self):
-        config_filenames = [self.neutron_cfg_fixture.filename,
-                            self.agent_cfg_fixture.filename]
-        process_name = f'{constants.AGENT_PROCESS_LINUXBRIDGE}-{self.hostname}'
-
-        self.process_fixture = self.useFixture(
-            ProcessFixture(
-                test_name=self.test_name,
-                process_name=process_name,
-                slice_name=self.hostname,
-                exec_name=constants.AGENT_PROCESS_LINUXBRIDGE,
-                config_filenames=config_filenames,
-                namespace=self.namespace
-            )
-        )
-
-
 class NamespaceCleanupFixture(ServiceFixture):
 
+    namespace_pattern = None
+
     def _setUp(self):
-        super(NamespaceCleanupFixture, self)._setUp()
+        super()._setUp()
         self.addCleanup(self.clean_namespaces)
 
     def clean_namespaces(self):
@@ -384,7 +379,7 @@ class NamespaceCleanupFixture(ServiceFixture):
         """
 
         for namespace in ip_lib.list_network_namespaces():
-            if (getattr(self, 'namespace_pattern') and
+            if (self.namespace_pattern and
                     self.namespace_pattern.match(namespace)):
                 try:
                     ip_lib.delete_network_namespace(namespace)
@@ -398,7 +393,7 @@ class L3AgentFixture(NamespaceCleanupFixture):
     def __init__(self, env_desc, host_desc, test_name,
                  neutron_cfg_fixture, l3_agent_cfg_fixture,
                  namespace=None):
-        super(L3AgentFixture, self).__init__()
+        super().__init__()
         self.env_desc = env_desc
         self.host_desc = host_desc
         self.test_name = test_name
@@ -408,7 +403,7 @@ class L3AgentFixture(NamespaceCleanupFixture):
         self.hostname = self.neutron_cfg_fixture.config['DEFAULT']['host']
 
     def _setUp(self):
-        super(L3AgentFixture, self)._setUp()
+        super()._setUp()
 
         self.plugin_config = self.l3_agent_cfg_fixture.config
 
@@ -446,7 +441,7 @@ class DhcpAgentFixture(NamespaceCleanupFixture):
 
     def __init__(self, env_desc, host_desc, test_name,
                  neutron_cfg_fixture, agent_cfg_fixture, namespace=None):
-        super(DhcpAgentFixture, self).__init__()
+        super().__init__()
         self.env_desc = env_desc
         self.host_desc = host_desc
         self.test_name = test_name
@@ -455,7 +450,7 @@ class DhcpAgentFixture(NamespaceCleanupFixture):
         self.namespace = namespace
 
     def _setUp(self):
-        super(DhcpAgentFixture, self)._setUp()
+        super()._setUp()
 
         self.plugin_config = self.agent_cfg_fixture.config
 
