@@ -60,7 +60,7 @@ class TestL3Agent(base.BaseFullStackTestCase):
         def is_port_status_active():
             port = self.client.show_port(port_id)
             return port['port']['status'] == 'ACTIVE'
-        common_utils.wait_until_true(lambda: is_port_status_active(), sleep=1)
+        base.wait_until_true(lambda: is_port_status_active(), sleep=1)
 
     def _create_and_attach_subnet(
             self, tenant_id, subnet_cidr, network_id, router_id):
@@ -178,7 +178,7 @@ class TestL3Agent(base.BaseFullStackTestCase):
             suffix = agent.get_namespace_suffix()
         else:
             suffix = self.environment.hosts[0].l3_agent.get_namespace_suffix()
-        return "%s@%s" % (namespace, suffix)
+        return "{}@{}".format(namespace, suffix)
 
     def _get_l3_agents_with_ha_state(
             self, router_id, ha_state=None):
@@ -253,7 +253,7 @@ class TestL3Agent(base.BaseFullStackTestCase):
             qrouter_ns = self._get_namespace(router['id'])
         ip = ip_lib.IPWrapper(qrouter_ns)
         try:
-            common_utils.wait_until_true(get_router_gw_interface)
+            base.wait_until_true(get_router_gw_interface)
         except common_utils.WaitTimeout:
             self.fail('Router gateway interface "qg-*" not found')
 
@@ -261,7 +261,7 @@ class TestL3Agent(base.BaseFullStackTestCase):
         tc_wrapper = l3_tc_lib.FloatingIPTcCommand(
             interface_name,
             namespace=qrouter_ns)
-        common_utils.wait_until_true(
+        base.wait_until_true(
             functools.partial(
                 self._wait_until_filters_set,
                 tc_wrapper),
@@ -332,7 +332,7 @@ class TestLegacyL3Agent(TestL3Agent):
                 network_type='vlan', l2_pop=False,
                 qos=True),
             host_descriptions)
-        super(TestLegacyL3Agent, self).setUp(env)
+        super().setUp(env)
 
     def test_mtu_update(self):
         tenant_id = uuidutils.generate_uuid()
@@ -347,7 +347,7 @@ class TestLegacyL3Agent(TestL3Agent):
         self.assert_namespace_exists(namespace)
 
         ip = ip_lib.IPWrapper(namespace)
-        common_utils.wait_until_true(lambda: ip.get_devices())
+        base.wait_until_true(lambda: ip.get_devices())
 
         devices = ip.get_devices()
         self.assertEqual(1, len(devices))
@@ -358,7 +358,7 @@ class TestLegacyL3Agent(TestL3Agent):
 
         mtu -= 1
         network = self.safe_client.update_network(network['id'], mtu=mtu)
-        common_utils.wait_until_true(lambda: ri_dev.link.mtu == mtu)
+        base.wait_until_true(lambda: ri_dev.link.mtu == mtu)
 
     def test_gateway_ip_changed(self):
         self._test_gateway_ip_changed()
@@ -400,7 +400,7 @@ class TestHAL3Agent(TestL3Agent):
                 agent_down_time=30,
                 qos=True),
             host_descriptions)
-        super(TestHAL3Agent, self).setUp(env)
+        super().setUp(env)
 
     def _is_ha_router_active_on_one_agent(self, router_id):
         agents = self.client.list_l3_agent_hosting_routers(router_id)
@@ -411,13 +411,13 @@ class TestHAL3Agent(TestL3Agent):
         tenant_id = uuidutils.generate_uuid()
         router = self.safe_client.create_router(tenant_id, ha=True)
 
-        common_utils.wait_until_true(
+        base.wait_until_true(
             lambda:
             len(self.client.list_l3_agent_hosting_routers(
                 router['id'])['agents']) == 2,
             timeout=90)
 
-        common_utils.wait_until_true(
+        base.wait_until_true(
             functools.partial(
                 self._is_ha_router_active_on_one_agent,
                 router['id']),
@@ -467,7 +467,7 @@ class TestHAL3Agent(TestL3Agent):
                 self.safe_client))
         external.block_until_boot()
 
-        common_utils.wait_until_true(
+        base.wait_until_true(
             functools.partial(
                 self._is_ha_router_active_on_one_agent,
                 router_id),
@@ -475,10 +475,27 @@ class TestHAL3Agent(TestL3Agent):
 
         # Test external connectivity, failover, test again
         pinger = net_helpers.Pinger(vm.namespace, external.ip, interval=0.1)
+        netcat_tcp = net_helpers.NetcatTester(
+            vm.namespace,
+            external.namespace,
+            external.ip,
+            3333,
+            net_helpers.NetcatTester.TCP,
+        )
+        netcat_udp = net_helpers.NetcatTester(
+            vm.namespace,
+            external.namespace,
+            external.ip,
+            3334,
+            net_helpers.NetcatTester.UDP,
+        )
+
         pinger.start()
 
         # Ensure connectivity before disconnect
         vm.block_until_ping(external.ip)
+        netcat_tcp.establish_connection()
+        netcat_udp.establish_connection()
 
         get_active_hosts = functools.partial(
             self._get_hosts_with_ha_state,
@@ -491,7 +508,7 @@ class TestHAL3Agent(TestL3Agent):
             return len(active_hosts) == 1
 
         try:
-            common_utils.wait_until_true(
+            base.wait_until_true(
                 is_one_host_active_for_router, timeout=15)
         except common_utils.WaitTimeout:
             pass
@@ -499,8 +516,10 @@ class TestHAL3Agent(TestL3Agent):
         # Test one last time:
         active_hosts = get_active_hosts()
         if len(active_hosts) != 1:
-            self.fail('Number of active hosts for router: %s' %
-                      len(active_hosts))
+            self.fail('Number of active hosts for router: {}\n'
+                      'Hosts: {}\n'
+                      'Router: {}'.format(
+                          len(active_hosts), active_hosts, router_id))
         active_host = active_hosts[0]
         backup_host = next(
             h for h in self.environment.hosts if h != active_host)
@@ -518,9 +537,14 @@ class TestHAL3Agent(TestL3Agent):
         vm.block_until_ping(external.ip)
         LOG.debug(f'Connectivity restored after {datetime.now() - start}')
 
+        # Ensure connection tracking states are synced to now active router
+        netcat_tcp.test_connectivity()
+        netcat_udp.test_connectivity()
+        LOG.debug(f'Connections restored after {datetime.now() - start}')
+
         # Assert the backup host got active
         timeout = self.environment.env_desc.agent_down_time * 1.2
-        common_utils.wait_until_true(
+        base.wait_until_true(
             lambda: backup_host in get_active_hosts(),
             timeout=timeout,
         )
@@ -530,7 +554,7 @@ class TestHAL3Agent(TestL3Agent):
             # Assert the previously active host is no longer active if it was
             # killed or shutdown. In the disconnect case both hosts will stay
             # active, but one host is disconnected from the data plane.
-            common_utils.wait_until_true(
+            base.wait_until_true(
                 lambda: active_host not in get_active_hosts(),
                 timeout=timeout,
             )
@@ -538,11 +562,12 @@ class TestHAL3Agent(TestL3Agent):
 
         # Stop probing processes
         pinger.stop()
+        netcat_tcp.stop_processes()
+        netcat_udp.stop_processes()
 
         # With the default advert_int of 2s the keepalived master timeout is
-        # about 6s. Assert less than 90 lost packets (9 seconds) plus 30 to
-        # account for CI infrastructure variability
-        threshold = 120
+        # about 6s. Assert less than 90 lost packets (9 seconds)
+        threshold = 90
 
         lost = pinger.sent - pinger.received
         message = (f'Sent {pinger.sent} packets, received {pinger.received} '
@@ -560,13 +585,14 @@ class TestHAL3Agent(TestL3Agent):
         self._test_ha_router_failover('disconnect')
 
     def _get_keepalived_state(self, keepalived_state_file):
-        with open(keepalived_state_file, "r") as fd:
+        with open(keepalived_state_file) as fd:
             return fd.read()
 
     def _get_state_file_for_primary_agent(self, router_id):
         for host in self.environment.hosts:
             keepalived_state_file = os.path.join(
-                host.neutron_config.state_path, "ha_confs", router_id, "state")
+                host.neutron_config.config.DEFAULT.state_path,
+                "ha_confs", router_id, "state")
 
             if self._get_keepalived_state(keepalived_state_file) == "primary":
                 return keepalived_state_file
@@ -579,12 +605,12 @@ class TestHAL3Agent(TestL3Agent):
         ext_net, ext_sub = self._create_external_network_and_subnet(tenant_id)
         router = self.safe_client.create_router(tenant_id, ha=True,
                                                 external_network=ext_net['id'])
-        common_utils.wait_until_true(
+        base.wait_until_true(
             lambda:
             len(self.client.list_l3_agent_hosting_routers(
                 router['id'])['agents']) == 2,
             timeout=90)
-        common_utils.wait_until_true(
+        base.wait_until_true(
             functools.partial(
                 self._is_ha_router_active_on_one_agent,
                 router['id']),
@@ -630,13 +656,13 @@ class TestHAL3Agent(TestL3Agent):
 
         external_vm = self._create_external_vm(ext_net, ext_sub)
 
-        common_utils.wait_until_true(
+        base.wait_until_true(
             lambda:
             len(self.client.list_l3_agent_hosting_routers(
                 router['id'])['agents']) == 2,
             timeout=90)
 
-        common_utils.wait_until_true(
+        base.wait_until_true(
             functools.partial(
                 self._is_ha_router_active_on_one_agent,
                 router['id']),
