@@ -33,6 +33,7 @@ from neutron.common import utils
 from neutron.conf.agent import common as agent_config
 from neutron.conf.agent import ovs_conf as ovs_agent_config
 from neutron.conf import common as common_config
+from neutron.conf.plugins.ml2 import config as ml2_config
 from neutron.conf.plugins.ml2.drivers import agent
 from neutron.conf.plugins.ml2.drivers import ovs_conf
 from neutron.plugins.ml2.drivers.openvswitch.agent.extension_drivers \
@@ -45,7 +46,7 @@ from neutron.tests.common import net_helpers
 from neutron.tests.functional.agent.linux import base
 
 
-class OVSOFControllerHelper(object):
+class OVSOFControllerHelper:
     """Helper class that runs os-ken openflow controller."""
 
     def start_of_controller(self, conf):
@@ -110,7 +111,7 @@ class OVSOFControllerHelper(object):
 class OVSAgentTestFramework(base.BaseOVSLinuxTestCase, OVSOFControllerHelper):
 
     def setUp(self):
-        super(OVSAgentTestFramework, self).setUp()
+        super().setUp()
         agent_rpc = ('neutron.plugins.ml2.drivers.openvswitch.agent.'
                      'ovs_neutron_agent.OVSPluginApi')
         mock.patch(agent_rpc).start()
@@ -140,13 +141,11 @@ class OVSAgentTestFramework(base.BaseOVSLinuxTestCase, OVSOFControllerHelper):
         agent_config.register_agent_state_opts_helper(config)
         ovs_agent_config.register_ovs_agent_opts(config)
         ext_manager.register_opts(config)
+        ml2_config.register_ml2_plugin_opts(cfg=config)
         return config
 
     def _configure_agent(self):
         config = self._get_config_opts()
-        config.set_override(
-            'interface_driver',
-            'neutron.agent.linux.interface.OVSInterfaceDriver')
         config.set_override('integration_bridge', self.br_int, "OVS")
         config.set_override('tunnel_bridge', self.br_tun, "OVS")
         config.set_override('int_peer_patch_port', self.patch_tun, "OVS")
