@@ -36,7 +36,7 @@ METADATA_REQUEST_SLEEP = 5
 TOO_MANY_REQUESTS_CODE = '429'
 
 
-class MetadataFakeProxyHandler(object):
+class MetadataFakeProxyHandler:
 
     def __init__(self, status):
         self.status = status
@@ -120,15 +120,15 @@ class MetadataL3AgentTestCase(framework.L3AgentTestFramework):
                 raw_headers = machine.execute(cmd)
                 break
             except RuntimeError as e:
-                if 'Connection refused' in str(e):
-                    time.sleep(METADATA_REQUEST_SLEEP)
-                    i += METADATA_REQUEST_SLEEP
-                else:
+                if 'Connection refused' not in str(e):
                     if router:
                         self._log_router_interfaces_configuration(router)
 
-                    self.fail('metadata proxy unreachable '
-                              'on %s before timeout' % cmd[-1])
+                    self.fail(
+                        'metadata proxy unreachable on %s before timeout' %
+                        cmd[-1])
+                time.sleep(METADATA_REQUEST_SLEEP)
+                i += METADATA_REQUEST_SLEEP
 
         if i > CONNECTION_REFUSED_TIMEOUT:
             self.fail('Timed out waiting metadata proxy to become available')
@@ -274,7 +274,7 @@ class UnprivilegedUserMetadataL3AgentTestCase(MetadataL3AgentTestCase):
     SOCKET_MODE = 0o664
 
     def setUp(self):
-        super(UnprivilegedUserMetadataL3AgentTestCase, self).setUp()
+        super().setUp()
         self.agent.conf.set_override('metadata_proxy_user', '65534')
 
 
@@ -290,6 +290,6 @@ class UnprivilegedUserGroupMetadataL3AgentTestCase(MetadataL3AgentTestCase):
     SOCKET_MODE = 0o666
 
     def setUp(self):
-        super(UnprivilegedUserGroupMetadataL3AgentTestCase, self).setUp()
+        super().setUp()
         self.agent.conf.set_override('metadata_proxy_user', '65534')
         self.agent.conf.set_override('metadata_proxy_group', '65534')
