@@ -20,6 +20,7 @@ from unittest import mock
 import eventlet
 import fixtures
 import netaddr
+from neutron_lib.api import converters
 from neutron_lib import constants as lib_const
 from oslo_config import fixture as fixture_config
 from oslo_log import log as logging
@@ -33,7 +34,7 @@ from neutron.agent.linux import external_process
 from neutron.agent.linux import interface
 from neutron.agent.linux import ip_lib
 from neutron.agent.linux import utils
-from neutron.agent.metadata import driver as metadata_driver
+from neutron.agent.metadata import driver_base
 from neutron.common import utils as common_utils
 from neutron.conf.agent import common as config
 from neutron.tests.common import net_helpers
@@ -45,10 +46,10 @@ LOG = logging.getLogger(__name__)
 
 class DHCPAgentOVSTestFramework(base.BaseSudoTestCase):
 
-    _DHCP_PORT_MAC_ADDRESS = netaddr.EUI("24:77:03:7d:00:4c")
-    _DHCP_PORT_MAC_ADDRESS.dialect = netaddr.mac_unix
-    _TENANT_PORT_MAC_ADDRESS = netaddr.EUI("24:77:03:7d:00:3a")
-    _TENANT_PORT_MAC_ADDRESS.dialect = netaddr.mac_unix
+    _DHCP_PORT_MAC_ADDRESS = converters.convert_to_sanitized_mac_address(
+        '24:77:03:7d:00:4c')
+    _TENANT_PORT_MAC_ADDRESS = converters.convert_to_sanitized_mac_address(
+        '24:77:03:7d:00:3a')
 
     _IP_ADDRS = {
         4: {'addr': '192.168.10.11',
@@ -59,7 +60,7 @@ class DHCPAgentOVSTestFramework(base.BaseSudoTestCase):
             'gateway': '2001:db8:0:1::c0a8:a01'}, }
 
     def setUp(self):
-        super(DHCPAgentOVSTestFramework, self).setUp()
+        super().setUp()
         config.setup_logging()
         self.conf_fixture = self.useFixture(fixture_config.Config())
         self.conf = self.conf_fixture.conf
@@ -76,18 +77,17 @@ class DHCPAgentOVSTestFramework(base.BaseSudoTestCase):
             'interface_driver',
             'neutron.agent.linux.interface.OVSInterfaceDriver')
         self.conf.set_override('report_interval', 0, 'AGENT')
-        br_int = self.useFixture(net_helpers.OVSBridgeFixture()).bridge
-        self.conf.set_override('integration_bridge', br_int.br_name, 'OVS')
+        self.br_int = self.useFixture(net_helpers.OVSBridgeFixture()).bridge
+        self.conf.set_override('integration_bridge', self.br_int.br_name,
+                               'OVS')
 
         self.mock_plugin_api = mock.patch(
             'neutron.agent.dhcp.agent.DhcpPluginApi').start().return_value
         mock.patch('neutron.agent.rpc.PluginReportStateAPI').start()
+        self.conf.set_override('check_child_processes_interval', 1, 'AGENT')
         self.agent = agent.DhcpAgentWithStateReport('localhost')
 
         self.ovs_driver = interface.OVSInterfaceDriver(self.conf)
-
-        self.conf.set_override('check_child_processes_interval', 1, 'AGENT')
-
         mock.patch('neutron.agent.common.ovs_lib.'
                    'OVSBridge._set_port_dead').start()
 
@@ -133,7 +133,7 @@ class DHCPAgentOVSTestFramework(base.BaseSudoTestCase):
     def create_port_dict(self, network_id, subnet_id, mac_address,
                          ip_version=lib_const.IP_VERSION_4, ip_address=None):
         ip_address = (self._IP_ADDRS[ip_version]['addr']
-            if not ip_address else ip_address)
+                      if not ip_address else ip_address)
         port_dict = dhcp.DictModel(id=uuidutils.generate_uuid(),
                                    name="foo",
                                    mac_address=mac_address,
@@ -222,6 +222,10 @@ class DHCPAgentOVSTestFramework(base.BaseSudoTestCase):
 
     def assert_good_allocation_for_port(self, network, port):
         vif_name = self.get_interface_name(network.id, port)
+        tag = self.br_int.ovsdb.db_get('Port', vif_name, 'tag').execute(
+            check_error=True)
+        self.assertEqual([], tag)
+
         self._run_dhclient(vif_name, network)
 
         predicate = lambda: len(
@@ -266,7 +270,7 @@ class DHCPAgentOVSTestFramework(base.BaseSudoTestCase):
             self.conf,
             network.id,
             network.namespace,
-            service=metadata_driver.HAPROXY_SERVICE)
+            service=driver_base.HAPROXY_SERVICE)
 
 
 class DHCPAgentOVSTestCase(DHCPAgentOVSTestFramework):
@@ -312,9 +316,8 @@ class DHCPAgentOVSTestCase(DHCPAgentOVSTestFramework):
         network, port = self._get_network_port_for_allocation_test()
         network.ports.append(port)
         self.configure_dhcp_for_network(network=network)
-        bad_mac_address = netaddr.EUI(self._TENANT_PORT_MAC_ADDRESS.value + 1)
-        bad_mac_address.dialect = netaddr.mac_unix
-        port.mac_address = str(bad_mac_address)
+        port.mac_address = converters.convert_to_sanitized_mac_address(
+            '24:77:03:7d:00:4d')
         self._plug_port_for_dhcp_request(network, port)
         self.assert_bad_allocation_for_port(network, port)
 
@@ -418,43 +421,3 @@ class DHCPAgentOVSTestCase(DHCPAgentOVSTestFramework):
             exception=RuntimeError("'dhcp_ready_on_ports' not be called"))
         self.mock_plugin_api.dhcp_ready_on_ports.assert_called_with(
             ports_to_send)
-
-    def test_dhcp_processing_pool_size(self):
-        mock.patch.object(self.agent, 'call_driver').start().return_value = (
-            True)
-        self.agent.update_isolated_metadata_proxy = mock.Mock()
-        self.agent.disable_isolated_metadata_proxy = mock.Mock()
-
-        network_info_1 = self.network_dict_for_dhcp()
-        self.configure_dhcp_for_network(network=network_info_1)
-        self.assertEqual(agent.DHCP_PROCESS_GREENLET_MIN,
-                         self.agent._pool.size)
-
-        network_info_2 = self.network_dict_for_dhcp()
-        self.configure_dhcp_for_network(network=network_info_2)
-        self.assertEqual(agent.DHCP_PROCESS_GREENLET_MIN,
-                         self.agent._pool.size)
-
-        network_info_list = [network_info_1, network_info_2]
-        for _i in range(agent.DHCP_PROCESS_GREENLET_MAX + 1):
-            ni = self.network_dict_for_dhcp()
-            self.configure_dhcp_for_network(network=ni)
-            network_info_list.append(ni)
-
-        self.assertEqual(agent.DHCP_PROCESS_GREENLET_MAX,
-                         self.agent._pool.size)
-
-        for network in network_info_list:
-            self.agent.disable_dhcp_helper(network.id)
-
-        agent_network_info_len = len(self.agent.cache.get_network_ids())
-        if agent_network_info_len < agent.DHCP_PROCESS_GREENLET_MIN:
-            self.assertEqual(agent.DHCP_PROCESS_GREENLET_MIN,
-                             self.agent._pool.size)
-        elif (agent.DHCP_PROCESS_GREENLET_MIN <= agent_network_info_len <=
-              agent.DHCP_PROCESS_GREENLET_MAX):
-            self.assertEqual(agent_network_info_len,
-                             self.agent._pool.size)
-        else:
-            self.assertEqual(agent.DHCP_PROCESS_GREENLET_MAX,
-                             self.agent._pool.size)
