@@ -80,12 +80,10 @@ class TestOVSAgent(base.OVSAgentTestFramework):
 
         # verify no stale drop flows
         self.assertEqual(0,
-            num_ports_with_drop_flows(
-                ofports,
-                self.agent.int_br.dump_flows(
-                    ovs_constants.LOCAL_SWITCHING
-                )
-            ))
+                         num_ports_with_drop_flows(
+                             ofports,
+                             self.agent.int_br.dump_flows(
+                                 ovs_constants.LOCAL_SWITCHING)))
 
     def _check_datapath_type_netdev(self, expected, default=False):
         if not default:
@@ -304,9 +302,14 @@ class TestOVSAgent(base.OVSAgentTestFramework):
         net_helpers.assert_ping(ns_phys, ip_int)
         net_helpers.assert_ping(self.namespace, ip_phys)
 
-        with net_helpers.async_ping(ns_phys, [ip_int, ip_phys]) as done:
-            self.agent.setup_physical_bridges(self.agent.bridge_mappings)
+        with net_helpers.async_ping(ns_phys, [ip_int]) as done:
             while not done():
+                self.agent.setup_physical_bridges(self.agent.bridge_mappings)
+                time.sleep(0.25)
+
+        with net_helpers.async_ping(self.namespace, [ip_phys]) as done:
+            while not done():
+                self.agent.setup_physical_bridges(self.agent.bridge_mappings)
                 time.sleep(0.25)
 
     def test_noresync_after_port_gone(self):
@@ -346,7 +349,7 @@ class TestOVSAgent(base.OVSAgentTestFramework):
 
 class TestOVSAgentExtensionConfig(base.OVSAgentTestFramework):
     def setUp(self):
-        super(TestOVSAgentExtensionConfig, self).setUp()
+        super().setUp()
         self.config.set_override('extensions', ['qos'], 'agent')
         self.agent = self.create_agent(create_tunnels=False)
 
