@@ -33,13 +33,12 @@ from neutron.tests.functional import base
 from neutron.tests.functional.cmd import process_spawn
 
 GET_NAMESPACES = 'neutron.agent.linux.ip_lib.list_network_namespaces'
-TEST_INTERFACE_DRIVER = 'neutron.agent.linux.interface.OVSInterfaceDriver'
 NUM_SUBPROCESSES = 6
 
 
 class NetnsCleanupTest(base.BaseSudoTestCase):
     def setUp(self):
-        super(NetnsCleanupTest, self).setUp()
+        super().setUp()
 
         self.get_namespaces_p = mock.patch(GET_NAMESPACES)
         self.get_namespaces = self.get_namespaces_p.start()
@@ -52,7 +51,6 @@ class NetnsCleanupTest(base.BaseSudoTestCase):
         args.append('--force')
 
         self.conf = netns_cleanup.setup_conf()
-        self.conf.set_override('interface_driver', TEST_INTERFACE_DRIVER)
         self.config_parse(conf=self.conf, args=args)
 
     def test_cleanup_network_namespaces_cleans_dhcp_and_l3_namespaces(self):
@@ -79,7 +77,7 @@ class NetnsCleanupTest(base.BaseSudoTestCase):
                 timeout=15)
         except eventlet.Timeout:
             num_spawned_procs = self._get_num_spawned_procs()
-            err_str = ("Expected number/spawned number: {0}/{1}\nProcess "
+            err_str = ("Expected number/spawned number: {}/{}\nProcess "
                        "information:\n".format(num_spawned_procs,
                                                procs_launched))
             cmd = ['ps', '-f', '-u', 'root']
@@ -144,22 +142,22 @@ class NetnsCleanupTest(base.BaseSudoTestCase):
                 # Otherwise, it won't find the necessary packages such as
                 # oslo_config
                 ip.netns.execute(command,
-                                 addl_env={'PATH':
-                                           os.environ.get('PATH')})
+                                 addl_env={'PATH': os.environ.get('PATH')},
+                                 privsep_exec=True)
         return proc_count
 
     @staticmethod
     def _get_num_spawned_procs():
         cmd = ['ps', '-f', '-u', 'root']
         out = utils.execute(cmd, run_as_root=True)
-        return sum([1 for line in out.splitlines() if 'process_spawn' in line])
+        return sum(1 for line in out.splitlines() if 'process_spawn' in line)
 
 
 class TestNETNSCLIConfig(basetest.BaseTestCase):
 
     def setup_config(self, args=None):
         self.conf = netns_cleanup.setup_conf()
-        super(TestNETNSCLIConfig, self).setup_config(args=args)
+        super().setup_config(args=args)
 
     def test_netns_opts_registration(self):
         self.assertFalse(self.conf.force)
