@@ -64,13 +64,27 @@ migration.log_warning = LOG.warning
 migration.log_info = LOG.info
 
 
+def render_url_str(_url):
+    """Render a ``URL`` instance as string and the password in clear text"""
+    try:
+        return _url.render_as_string(hide_password=False)
+    except AttributeError:
+        # NOTE(ralonsoh): ``URL`` objects from SQLAlchemy<2.0.0 don't have
+        # ``render_as_string`` method but it is not necessary.
+        return str(_url)
+
+
 def upgrade(engine, alembic_config, branch_name='heads'):
-    cfg.CONF.set_override('connection', engine.url, group='database')
+    url_str = render_url_str(engine.url)
+    cfg.CONF.set_override('connection', url_str, group='database')
     migration.do_alembic_command(alembic_config, 'upgrade',
                                  branch_name)
 
 
-class _TestModelsMigrations(test_migrations.ModelsMigrationsSync):
+class TestModelsMigrations(test_migrations.ModelsMigrationsSync,
+                           testlib_api.MySQLTestCaseMixin,
+                           testlib_api.SqlTestCaseLight,
+                           functional_base.BaseLoggingTestCase):
     '''Test for checking of equality models state and migrations.
 
     For the opportunistic testing you need to set up a db named
@@ -78,14 +92,6 @@ class _TestModelsMigrations(test_migrations.ModelsMigrationsSync):
     'openstack_citest' on localhost.
     The test will then use that db and user/password combo to run the tests.
 
-    For PostgreSQL on Ubuntu this can be done with the following commands::
-
-        sudo -u postgres psql
-        postgres=# create user openstack_citest with createdb login password
-                  'openstack_citest';
-        postgres=# create database openstack_citest with owner
-                   openstack_citest;
-
     For MySQL on Ubuntu this can be done with the following commands::
 
         mysql -u root
@@ -144,7 +150,7 @@ class _TestModelsMigrations(test_migrations.ModelsMigrationsSync):
 
     def setUp(self):
         config.register_common_config_options()
-        super(_TestModelsMigrations, self).setUp()
+        super().setUp()
         self.cfg = self.useFixture(config_fixture.Config())
         self.cfg.config(core_plugin='ml2')
         self.alembic_config = migration.get_neutron_config()
@@ -167,7 +173,7 @@ class _TestModelsMigrations(test_migrations.ModelsMigrationsSync):
                                  name in external.TABLES):
             return False
 
-        return super(_TestModelsMigrations, self).include_object(
+        return super().include_object(
             object_, name, type_, reflected, compare_to)
 
     def filter_metadata_diff(self, diff):
@@ -199,13 +205,13 @@ class _TestModelsMigrations(test_migrations.ModelsMigrationsSync):
 
     def test_upgrade_expand_branch(self):
         # Verify that "command neutron-db-manage upgrade --expand" works
-        #  without errors. Check this for both MySQL and PostgreSQL.
+        # without errors.
         upgrade(self.engine, self.alembic_config,
                 branch_name='%s@head' % migration.EXPAND_BRANCH)
 
     def test_upgrade_contract_branch(self):
         # Verify that "command neutron-db-manage upgrade --contract" works
-        # without errors. Check this for both MySQL and PostgreSQL.
+        # without errors.
         upgrade(self.engine, self.alembic_config,
                 branch_name='%s@head' % migration.CONTRACT_BRANCH)
 
@@ -250,7 +256,7 @@ class _TestModelsMigrations(test_migrations.ModelsMigrationsSync):
                 if not get_excepted_elements:
                     continue
                 explanation = getattr(get_excepted_elements, '__doc__', "")
-                if len(explanation) < 1:
+                if not explanation:
                     self.fail("%s() requires docstring with explanation" %
                               '.'.join([m.module.__name__,
                                         get_excepted_elements.__name__]))
@@ -301,7 +307,8 @@ class _TestModelsMigrations(test_migrations.ModelsMigrationsSync):
 
         find_migration_exceptions()
         engine = self.engine
-        cfg.CONF.set_override('connection', engine.url, group='database')
+        url_str = render_url_str(engine.url)
+        cfg.CONF.set_override('connection', url_str, group='database')
 
         with engine.begin() as connection:
             self.alembic_config.attributes['connection'] = connection
@@ -331,7 +338,8 @@ class _TestModelsMigrations(test_migrations.ModelsMigrationsSync):
     # contradiction to blueprint online-upgrades
     def test_forbid_offline_migrations_starting_newton(self):
         engine = self.get_engine()
-        cfg.CONF.set_override('connection', engine.url, group='database')
+        url_str = render_url_str(engine.url)
+        cfg.CONF.set_override('connection', url_str, group='database')
         # the following revisions are Newton heads
         for revision in ('5cd92597d11d', '5c85685d616d'):
             migration.do_alembic_command(
@@ -340,15 +348,10 @@ class _TestModelsMigrations(test_migrations.ModelsMigrationsSync):
             self.alembic_config, 'unused'),
             msg='Offline contract migration scripts are forbidden for Ocata+')
 
-
-class TestModelsMigrationsMySQL(testlib_api.MySQLTestCaseMixin,
-                                _TestModelsMigrations,
-                                testlib_api.SqlTestCaseLight,
-                                functional_base.BaseLoggingTestCase):
-
     def test_check_mysql_engine(self):
         engine = self.get_engine()
-        cfg.CONF.set_override('connection', engine.url, group='database')
+        url_str = render_url_str(engine.url)
+        cfg.CONF.set_override('connection', url_str, group='database')
         with engine.begin() as connection:
             self.alembic_config.attributes['connection'] = connection
             migration.do_alembic_command(self.alembic_config, 'upgrade',
@@ -364,21 +367,12 @@ class TestModelsMigrationsMySQL(testlib_api.MySQLTestCaseMixin,
                    table != 'alembic_version']
             self.assertEqual(0, len(res), "%s non InnoDB tables created" % res)
 
-    def test_models_sync(self):
-        super(TestModelsMigrationsMySQL, self).test_models_sync()
-
-
-class TestModelsMigrationsPostgreSQL(testlib_api.PostgreSQLTestCaseMixin,
-                                     _TestModelsMigrations,
-                                     testlib_api.SqlTestCaseLight):
-    pass
-
 
 class TestSanityCheck(testlib_api.SqlTestCaseLight):
     BUILD_SCHEMA = False
 
     def setUp(self):
-        super(TestSanityCheck, self).setUp()
+        super().setUp()
         self.alembic_config = migration.get_neutron_config()
         self.alembic_config.neutron_config = cfg.CONF
 
@@ -393,7 +387,7 @@ class TestSanityCheck(testlib_api.SqlTestCaseLight):
             sqlalchemy.Column('router_id', sqlalchemy.String(36)),
             sqlalchemy.Column('l3_agent_id', sqlalchemy.String(36)))
 
-        with self.engine.connect() as conn:
+        with self.engine.begin() as conn:
             ha_router_agent_port_bindings.create(conn)
             self.addCleanup(self._drop_table, ha_router_agent_port_bindings)
             # NOTE(haleyb): without this disabled, pylint complains
@@ -418,7 +412,7 @@ class TestSanityCheck(testlib_api.SqlTestCaseLight):
             sqlalchemy.Column('port_id', sqlalchemy.String(36)),
             sqlalchemy.Column('port_type', sqlalchemy.String(255)))
 
-        with self.engine.connect() as conn:
+        with self.engine.begin() as conn:
             routerports.create(conn)
             self.addCleanup(self._drop_table, routerports)
             # NOTE(haleyb): without this disabled, pylint complains
@@ -443,7 +437,7 @@ class TestSanityCheck(testlib_api.SqlTestCaseLight):
             sqlalchemy.Column('fixed_port_id', sqlalchemy.String(36)),
             sqlalchemy.Column('fixed_ip_address', sqlalchemy.String(64)))
 
-        with self.engine.connect() as conn:
+        with self.engine.begin() as conn:
             floatingips.create(conn)
             self.addCleanup(self._drop_table, floatingips)
             # NOTE(haleyb): without this disabled, pylint complains
@@ -470,7 +464,7 @@ class TestSanityCheck(testlib_api.SqlTestCaseLight):
             sqlalchemy.Column('fixed_port_id', sqlalchemy.String(36)),
             sqlalchemy.Column('fixed_ip_address', sqlalchemy.String(64)))
 
-        with self.engine.connect() as conn:
+        with self.engine.begin() as conn:
             floatingips.create(conn)
             self.addCleanup(self._drop_table, floatingips)
             # NOTE(haleyb): without this disabled, pylint complains
@@ -493,15 +487,14 @@ class TestSanityCheck(testlib_api.SqlTestCaseLight):
 class TestWalkDowngrade(oslotest_base.BaseTestCase):
 
     def setUp(self):
-        super(TestWalkDowngrade, self).setUp()
+        super().setUp()
         self.alembic_config = migration.get_neutron_config()
         self.alembic_config.neutron_config = cfg.CONF
 
     def test_no_downgrade(self):
         script_dir = alembic_script.ScriptDirectory.from_config(
             self.alembic_config)
-        versions = [v for v in script_dir.walk_revisions(base='base',
-                                                         head='heads')]
+        versions = list(script_dir.walk_revisions(base='base', head='heads'))
         failed_revisions = []
         for version in versions:
             if hasattr(version.module, 'downgrade'):
@@ -512,7 +505,8 @@ class TestWalkDowngrade(oslotest_base.BaseTestCase):
             return True
 
 
-class _TestWalkMigrations(object):
+class TestWalkMigrations(testlib_api.MySQLTestCaseMixin,
+                         testlib_api.SqlTestCaseLight):
     '''This will add framework for testing schema migration
        for different backends.
 
@@ -521,11 +515,11 @@ class _TestWalkMigrations(object):
     BUILD_SCHEMA = False
 
     def execute_cmd(self, cmd=None):
-        proc = subprocess.Popen(cmd, stdout=subprocess.PIPE,
-                                stderr=subprocess.STDOUT, shell=True)
-        output = proc.communicate()[0]
-        self.assertEqual(0, proc.returncode, 'Command failed with '
-                         'output:\n%s' % output)
+        with subprocess.Popen(cmd, stdout=subprocess.PIPE,
+                              stderr=subprocess.STDOUT, shell=True) as proc:
+            output = proc.communicate()[0]
+            self.assertEqual(0, proc.returncode, 'Command failed with '
+                             'output:\n%s' % output)
 
     def _get_alembic_config(self, uri):
         db_config = migration.get_neutron_config()
@@ -562,38 +556,25 @@ class _TestWalkMigrations(object):
             migration.do_alembic_command(config, 'upgrade', dest)
             check(engine, data)
 
+    # NOTE(slaweq): this workaround is taken from Manila patch:
+    # https://review.opendev.org/#/c/291397/
+    # Set 5 minutes timeout for case of running it on very slow nodes/VMs.
+    # Note, that this test becomes slower with each addition of new DB
+    # migration. On fast nodes it can take about 5-10 secs having Mitaka set of
+    # migrations.
+    @test_base.set_timeout(600)
     def test_walk_versions(self):
         """Test migrations ability to upgrade and downgrade.
 
         """
         engine = self.engine
-        config = self._get_alembic_config(engine.url)
+        url_str = render_url_str(engine.url)
+        config = self._get_alembic_config(url_str)
         revisions = self._revisions()
+        upgrade_dest = None
         for dest, curr in revisions:
             self._migrate_up(config, engine, dest, curr)
+            upgrade_dest = dest
 
-        if dest:
-            migration.do_alembic_command(config, 'upgrade', dest)
-
-
-class TestWalkMigrationsMySQL(testlib_api.MySQLTestCaseMixin,
-                              _TestWalkMigrations,
-                              testlib_api.SqlTestCaseLight):
-
-    # NOTE(slaweq): this workaround is taken from Manila patch:
-    # https://review.opendev.org/#/c/291397/
-    # Set 5 minutes timeout for case of running it on
-    # very slow nodes/VMs. Note, that this test becomes slower with each
-    # addition of new DB migration. On fast nodes it can take about 5-10
-    # secs having Mitaka set of migrations. 'pymysql' works much slower
-    # on slow nodes than 'psycopg2' and because of that this increased
-    # timeout is required only when for testing with 'mysql' backend.
-    @test_base.set_timeout(600)
-    def test_walk_versions(self):
-        super(TestWalkMigrationsMySQL, self).test_walk_versions()
-
-
-class TestWalkMigrationsPostgreSQL(testlib_api.PostgreSQLTestCaseMixin,
-                                   _TestWalkMigrations,
-                                   testlib_api.SqlTestCaseLight):
-    pass
+        if upgrade_dest:
+            migration.do_alembic_command(config, 'upgrade', upgrade_dest)
