@@ -58,7 +58,7 @@ class TestRootController(test_functional.PecanFunctionalTest):
     base_url = '/'
 
     def setUp(self):
-        super(TestRootController, self).setUp()
+        super().setUp()
         self.setup_service_plugin()
         self.plugin = directory.get_plugin()
         self.ctx = context.get_admin_context()
@@ -154,7 +154,7 @@ class TestExtensionsController(TestRootController):
     def test_get(self):
         # Fetch any extension supported by plugins
         test_alias = self._get_supported_extensions().pop()
-        response = self.app.get('%s/%s' % (self.base_url, test_alias))
+        response = self.app.get('{}/{}'.format(self.base_url, test_alias))
         self.assertEqual(response.status_int, 200)
         json_body = jsonutils.loads(response.body)
         self.assertEqual(test_alias, json_body['extension']['alias'])
@@ -192,7 +192,7 @@ class TestQuotasController(test_functional.PecanFunctionalTest):
     def test_index_admin(self):
         # NOTE(salv-orlando): The quota controller has an hardcoded check for
         # admin-ness for this operation, which is supposed to return quotas for
-        # all tenants. Such check is "vestigial" from the home-grown WSGI and
+        # all projects. Such check is "vestigial" from the home-grown WSGI and
         # shall be removed
         response = self.app.get('%s.json' % self.base_url,
                                 headers={'X-Project-Id': 'admin',
@@ -213,7 +213,7 @@ class TestQuotasController(test_functional.PecanFunctionalTest):
         self._verify_default_limits(json_body)
 
     def test_get(self):
-        # It is not ok to access another tenant's limits
+        # It is not ok to access another project's limits
         url = '%s/foo.json' % self.base_url
         response = self.app.get(url, expect_errors=True)
         self.assertEqual(403, response.status_int)
@@ -269,7 +269,7 @@ class TestQuotasController(test_functional.PecanFunctionalTest):
         json_body = jsonutils.loads(response.body)
         found = False
         for qs in json_body['quotas']:
-            if qs['tenant_id'] == 'foo':
+            if qs['project_id'] == 'foo':
                 found = True
         self.assertTrue(found)
 
@@ -283,7 +283,7 @@ class TestQuotasController(test_functional.PecanFunctionalTest):
         self.assertEqual(200, response.status_int)
         json_body = jsonutils.loads(response.body)
         for qs in json_body['quotas']:
-            self.assertNotEqual('foo', qs['tenant_id'])
+            self.assertNotEqual('foo', qs['project_id'])
 
     def test_quotas_get_defaults(self):
         response = self.app.get('%s/foo/default.json' % self.base_url,
@@ -294,13 +294,14 @@ class TestQuotasController(test_functional.PecanFunctionalTest):
         json_body = jsonutils.loads(response.body)
         self._verify_default_limits(json_body)
 
-    def test_get_tenant_info(self):
-        response = self.app.get('%s/tenant.json' % self.base_url,
-                                headers={'X-Project-Id': 'admin',
-                                         'X-Roles': 'admin'})
-        self.assertEqual(200, response.status_int)
-        json_body = jsonutils.loads(response.body)
-        self.assertEqual('admin', json_body['tenant']['tenant_id'])
+    def test_get_project_info(self):
+        for key in ('project', 'tenant'):
+            response = self.app.get('{}/{}.json'.format(self.base_url, key),
+                                    headers={'X-Project-Id': 'admin',
+                                             'X-Roles': 'admin'})
+            self.assertEqual(200, response.status_int)
+            json_body = jsonutils.loads(response.body)
+            self.assertEqual('admin', json_body[key][key + '_id'])
 
 
 class TestResourceController(TestRootController):
@@ -310,7 +311,7 @@ class TestResourceController(TestRootController):
     base_url = '/v2.0'
 
     def setUp(self):
-        super(TestResourceController, self).setUp()
+        super().setUp()
         policy.init()
         self.addCleanup(policy.reset)
         self._gen_port()
@@ -318,11 +319,11 @@ class TestResourceController(TestRootController):
     def _gen_port(self):
         network_id = self.plugin.create_network(context.get_admin_context(), {
             'network':
-            {'name': 'pecannet', 'tenant_id': 'tenid', 'shared': False,
+            {'name': 'pecannet', 'project_id': 'tenid', 'shared': False,
              'admin_state_up': True, 'status': 'ACTIVE'}})['id']
         self.port = self.plugin.create_port(context.get_admin_context(), {
             'port':
-            {'tenant_id': 'tenid', 'network_id': network_id,
+            {'project_id': 'tenid', 'network_id': network_id,
              'fixed_ips': n_const.ATTR_NOT_SPECIFIED,
              'mac_address': '00:11:22:33:44:55',
              'admin_state_up': True, 'device_id': 'FF',
@@ -342,7 +343,7 @@ class TestResourceController(TestRootController):
         query_params = ['fields=%s' % field for field in fields]
         url = '/v2.0/ports.json'
         if query_params:
-            url = '%s?%s' % (url, '&'.join(query_params))
+            url = '{}?{}'.format(url, '&'.join(query_params))
         list_resp = self.app.get(url, headers={'X-Project-Id': 'tenid'})
         self.assertEqual(200, list_resp.status_int)
         for item in jsonutils.loads(list_resp.body).get('ports', []):
@@ -364,7 +365,7 @@ class TestResourceController(TestRootController):
         self._test_get_collection_with_fields_selector(fields=[])
 
     def test_project_id_in_mandatory_fields(self):
-        # ports only specifies that tenant_id is mandatory, but project_id
+        # ports only specifies that project_id is mandatory, but project_id
         # should still be passed to the plugin.
         mock_get = mock.patch.object(self.plugin, 'get_ports',
                                      return_value=[]).start()
@@ -384,10 +385,10 @@ class TestResourceController(TestRootController):
         # Explicitly require an attribute which is also 'required_by_policy'.
         # The attribute should not be stripped while generating the response
         item_resp = self.app.get(
-            '/v2.0/ports/%s.json?fields=id&fields=tenant_id' % self.port['id'],
-            headers={'X-Project-Id': 'tenid'})
+            '/v2.0/ports/%s.json?fields=id&fields=project_id' %
+            self.port['id'], headers={'X-Project-Id': 'tenid'})
         self.assertEqual(200, item_resp.status_int)
-        self._check_item(['id', 'tenant_id'],
+        self._check_item(['id', 'project_id'],
                          jsonutils.loads(item_resp.body)['port'])
 
     def test_duped_and_empty_fields_stripped(self):
@@ -406,7 +407,7 @@ class TestResourceController(TestRootController):
             '/v2.0/ports.json',
             params={'port': {'network_id': self.port['network_id'],
                              'admin_state_up': True,
-                             'tenant_id': 'tenid'}},
+                             'project_id': 'tenid'}},
             headers={'X-Project-Id': 'tenid'})
         self.assertEqual(response.status_int, 201)
 
@@ -426,7 +427,7 @@ class TestResourceController(TestRootController):
                 '/v2.0/ports.json',
                 params={'port': {'network_id': self.port['network_id'],
                                  'admin_state_up': True,
-                                 'tenant_id': 'tenid'}},
+                                 'project_id': 'tenid'}},
                 headers={'X-Project-Id': 'tenid'})
             self.assertEqual(201, response.status_int)
 
@@ -439,7 +440,7 @@ class TestResourceController(TestRootController):
         self.assertEqual(1, len(json_body))
         self.assertIn('port', json_body)
         self.assertEqual('test', json_body['port']['name'])
-        self.assertEqual('tenid', json_body['port']['tenant_id'])
+        self.assertEqual('tenid', json_body['port']['project_id'])
 
     def test_delete(self):
         response = self.app.delete('/v2.0/ports/%s.json' % self.port['id'],
@@ -486,11 +487,11 @@ class TestResourceController(TestRootController):
         response = self.app.post_json(
             '/v2.0/ports.json',
             params={'ports': [{'network_id': self.port['network_id'],
-                             'admin_state_up': True,
-                             'tenant_id': 'tenid'},
-                             {'network_id': self.port['network_id'],
+                               'admin_state_up': True,
+                               'project_id': 'tenid'},
+                              {'network_id': self.port['network_id'],
                               'admin_state_up': True,
-                              'tenant_id': 'tenid'}]
+                               'project_id': 'tenid'}]
                     },
             headers={'X-Project-Id': 'tenid'})
         self.assertEqual(201, response.status_int)
@@ -516,13 +517,13 @@ class TestResourceController(TestRootController):
         port_response = self.app.post_json(
                 '/v2.0/ports.json',
                 params={'ports': [{'network_id': self.port['network_id'],
-                                 'admin_state_up': True,
-                                 'security_groups': [sg_id],
-                                 'tenant_id': 'tenid'},
-                                 {'network_id': self.port['network_id'],
+                                   'admin_state_up': True,
+                                   'security_groups': [sg_id],
+                                   'project_id': 'tenid'},
+                                  {'network_id': self.port['network_id'],
                                   'admin_state_up': True,
-                                 'security_groups': [sg_id],
-                                  'tenant_id': 'tenid'}]
+                                   'security_groups': [sg_id],
+                                   'project_id': 'tenid'}]
                         },
                 headers={'X-Project-Id': 'tenid'})
         self.assertEqual(201, port_response.status_int)
@@ -538,11 +539,11 @@ class TestResourceController(TestRootController):
         response = self.app.post_json(
             '/v2.0/ports.json',
             params={'ports': [{'network_id': self.port['network_id'],
-                             'admin_state_up': True,
-                             'tenant_id': 'tenid'},
-                             {'network_id': self.port['network_id'],
+                               'admin_state_up': True,
+                               'project_id': 'tenid'},
+                              {'network_id': self.port['network_id'],
                               'admin_state_up': True,
-                              'tenant_id': 'tenid'}]
+                               'project_id': 'tenid'}]
                     },
             headers={'X-Project-Id': 'tenid'})
         self.assertEqual(response.status_int, 201)
@@ -555,14 +556,14 @@ class TestResourceController(TestRootController):
         response = self.app.post_json(
             '/v2.0/ports.json',
             params={'ports': [{'network_id': self.port['network_id'],
-                             'admin_state_up': True,
-                             'tenant_id': 'tenid'},
-                             {'network_id': self.port['network_id'],
+                               'admin_state_up': True,
+                               'project_id': 'tenid'},
+                              {'network_id': self.port['network_id'],
                               'admin_state_up': True,
-                              'tenant_id': 'tenid'},
-                             {'network_id': 'bad_net_id',
+                               'project_id': 'tenid'},
+                              {'network_id': 'bad_net_id',
                               'admin_state_up': True,
-                              'tenant_id': 'tenid'}]
+                               'project_id': 'tenid'}]
                     },
             headers={'X-Project-Id': 'tenid'},
             expect_errors=True)
@@ -579,7 +580,7 @@ class TestResourceController(TestRootController):
             '/v2.0/ports.json',
             params={'ports': [{'network_id': self.port['network_id'],
                                'admin_state_up': True,
-                               'tenant_id': 'tenid'}]
+                               'project_id': 'tenid'}]
                     },
             headers={'X-Project-Id': 'tenid'})
         self.assertEqual(response.status_int, 201)
@@ -593,18 +594,20 @@ class TestPaginationAndSorting(test_functional.PecanFunctionalTest):
     RESOURCE_COUNT = 6
 
     def setUp(self):
-        super(TestPaginationAndSorting, self).setUp()
+        super().setUp()
         policy.init()
         self.addCleanup(policy.reset)
         self.plugin = directory.get_plugin()
         self.ctx = context.get_admin_context()
+        self._project_id = 'project_id1'
         self._create_networks(self.RESOURCE_COUNT)
         self.networks = self._get_collection()['networks']
 
     def _create_networks(self, count=1):
         network_ids = []
         for index in range(count):
-            network = {'name': 'pecannet-%d' % index, 'tenant_id': 'tenid',
+            network = {'name': 'pecannet-%d' % index,
+                       'project_id': self._project_id,
                        'shared': False, 'admin_state_up': True,
                        'status': 'ACTIVE'}
             network_id = self.plugin.create_network(
@@ -628,11 +631,13 @@ class TestPaginationAndSorting(test_functional.PecanFunctionalTest):
             query_params.append('sort_key=%s' % sort_key)
         if sort_dir:
             query_params.append('sort_dir=%s' % sort_dir)
-        query_params.extend(['%s%s' % ('fields=', field) for field in fields])
+        query_params.extend(['{}{}'.format('fields=', field)
+                            for field in fields])
         url = '/v2.0/%s.json' % collection
         if query_params:
-            url = '%s?%s' % (url, '&'.join(query_params))
-        list_resp = self.app.get(url, headers={'X-Project-Id': 'tenid'})
+            url = '{}?{}'.format(url, '&'.join(query_params))
+        list_resp = self.app.get(url,
+                                 headers={'X-Project-Id': self._project_id})
         self.assertEqual(200, list_resp.status_int)
         return list_resp.json
 
@@ -712,7 +717,7 @@ class TestPaginationAndSorting(test_functional.PecanFunctionalTest):
 class TestRequestProcessing(TestRootController):
 
     def setUp(self):
-        super(TestRequestProcessing, self).setUp()
+        super().setUp()
         mock.patch('neutron.pecan_wsgi.hooks.notifier.registry').start()
         # request.context is thread-local storage so it has to be accessed by
         # the controller. We can capture it into a list here to assert on after
@@ -735,9 +740,9 @@ class TestRequestProcessing(TestRootController):
 
     def test_context_set_in_request(self):
         self.app.get('/v2.0/ports.json',
-                     headers={'X-Project-Id': 'tenant_id'})
-        self.assertEqual('tenant_id',
-                         self.captured_context['neutron_context'].tenant_id)
+                     headers={'X-Project-Id': 'project_id'})
+        self.assertEqual('project_id',
+                         self.captured_context['neutron_context'].project_id)
 
     def test_core_resource_identified(self):
         self.app.get('/v2.0/ports.json')
@@ -846,7 +851,7 @@ class TestRouterController(TestResourceController):
             'service_plugins',
             ['neutron.services.l3_router.l3_router_plugin.L3RouterPlugin',
              'neutron.services.flavors.flavors_plugin.FlavorsPlugin'])
-        super(TestRouterController, self).setUp()
+        super().setUp()
         policy.init()
         self.addCleanup(policy.reset)
         plugin = directory.get_plugin()
@@ -889,7 +894,7 @@ class TestRouterController(TestResourceController):
 class TestDHCPAgentShimControllers(test_functional.PecanFunctionalTest):
 
     def setUp(self):
-        super(TestDHCPAgentShimControllers, self).setUp()
+        super().setUp()
         policy.init()
         policy._ENFORCER.set_rules(
             oslo_policy.Rules.from_dict(
@@ -928,8 +933,8 @@ class TestDHCPAgentShimControllers(test_functional.PecanFunctionalTest):
             headers=headers)
         self.assertIn(self.agent.id,
                       [a['id'] for a in response.json['agents']])
-        self.app.delete('/v2.0/agents/%(a)s/dhcp-networks/%(n)s.json' % {
-            'a': self.agent.id, 'n': self.network['id']}, headers=headers)
+        self.app.delete('/v2.0/agents/{a}/dhcp-networks/{n}.json'.format(
+            a=self.agent.id, n=self.network['id']), headers=headers)
         response = self.app.get(
             '/v2.0/networks/%s/dhcp-agents.json' % self.network['id'],
             headers=headers)
@@ -944,7 +949,7 @@ class TestL3AgentShimControllers(test_functional.PecanFunctionalTest):
             'service_plugins',
             ['neutron.services.l3_router.l3_router_plugin.L3RouterPlugin',
              'neutron.services.flavors.flavors_plugin.FlavorsPlugin'])
-        super(TestL3AgentShimControllers, self).setUp()
+        super().setUp()
         policy.init()
         policy._ENFORCER.set_rules(
             oslo_policy.Rules.from_dict(
@@ -983,8 +988,8 @@ class TestL3AgentShimControllers(test_functional.PecanFunctionalTest):
         self.assertIn(self.agent.id,
                       [a['id'] for a in response.json['agents']])
         response = self.app.delete(
-            '/v2.0/agents/%(a)s/l3-routers/%(n)s.json' % {
-                'a': self.agent.id, 'n': self.router['id']}, headers=headers)
+            '/v2.0/agents/{a}/l3-routers/{n}.json'.format(
+                a=self.agent.id, n=self.router['id']), headers=headers)
         self.assertEqual(204, response.status_int)
         self.assertFalse(response.body)
         response = self.app.get(
@@ -1001,7 +1006,7 @@ class TestShimControllers(test_functional.PecanFunctionalTest):
         fake_plugin = pecan_utils.FakePlugin()
         plugins = {pecan_utils.FakePlugin.PLUGIN_TYPE: fake_plugin}
         new_extensions = {fake_ext.get_alias(): fake_ext}
-        super(TestShimControllers, self).setUp(
+        super().setUp(
             service_plugins=plugins, extensions=new_extensions)
         policy.init()
         policy._ENFORCER.set_rules(
@@ -1016,7 +1021,7 @@ class TestShimControllers(test_functional.PecanFunctionalTest):
         collection = pecan_utils.FakeExtension.HYPHENATED_COLLECTION.replace(
             '_', '-')
         resource = pecan_utils.FakeExtension.HYPHENATED_RESOURCE
-        url = '/v2.0/{}/something.json'.format(collection)
+        url = f'/v2.0/{collection}/something.json'
         resp = self.app.get(url)
         self.assertEqual(200, resp.status_int)
         self.assertEqual({resource: {'fake': 'something'}}, resp.json)
@@ -1024,7 +1029,7 @@ class TestShimControllers(test_functional.PecanFunctionalTest):
     def test_hyphenated_collection_controller_not_shimmed(self):
         body_collection = pecan_utils.FakeExtension.HYPHENATED_COLLECTION
         uri_collection = body_collection.replace('_', '-')
-        url = '/v2.0/{}.json'.format(uri_collection)
+        url = f'/v2.0/{uri_collection}.json'
         resp = self.app.get(url)
         self.assertEqual(200, resp.status_int)
         self.assertEqual({body_collection: [{'fake': 'fake'}]}, resp.json)
@@ -1036,7 +1041,7 @@ class TestShimControllers(test_functional.PecanFunctionalTest):
         sub_resource_collection = (
             pecan_utils.FakeExtension.FAKE_SUB_RESOURCE_COLLECTION)
         temp_id = uuidutils.generate_uuid()
-        url = '/v2.0/{0}/{1}/{2}'.format(
+        url = '/v2.0/{}/{}/{}'.format(
             uri_collection,
             temp_id,
             sub_resource_collection.replace('_', '-'))
@@ -1052,19 +1057,19 @@ class TestMemberActionController(test_functional.PecanFunctionalTest):
         fake_plugin = pecan_utils.FakePlugin()
         plugins = {pecan_utils.FakePlugin.PLUGIN_TYPE: fake_plugin}
         new_extensions = {fake_ext.get_alias(): fake_ext}
-        super(TestMemberActionController, self).setUp(
+        super().setUp(
             service_plugins=plugins, extensions=new_extensions)
         hyphen_collection = pecan_utils.FakeExtension.HYPHENATED_COLLECTION
         self.collection = hyphen_collection.replace('_', '-')
 
     def test_get_member_action_controller(self):
-        url = '/v2.0/{}/something/boo_meh.json'.format(self.collection)
+        url = f'/v2.0/{self.collection}/something/boo_meh.json'
         resp = self.app.get(url)
         self.assertEqual(200, resp.status_int)
         self.assertEqual({'boo_yah': 'something'}, resp.json)
 
     def test_put_member_action_controller(self):
-        url = '/v2.0/{}/something/put_meh.json'.format(self.collection)
+        url = f'/v2.0/{self.collection}/something/put_meh.json'
         resp = self.app.put_json(url, params={'it_matters_not': 'ok'})
         self.assertEqual(200, resp.status_int)
         self.assertEqual({'poo_yah': 'something'}, resp.json)
@@ -1083,13 +1088,13 @@ class TestMemberActionController(test_functional.PecanFunctionalTest):
         self.assertEqual(404, resp.status_int)
 
     def test_put_on_get_member_action(self):
-        url = '/v2.0/{}/something/boo_meh.json'.format(self.collection)
+        url = f'/v2.0/{self.collection}/something/boo_meh.json'
         resp = self.app.put_json(url, params={'it_matters_not': 'ok'},
                                  expect_errors=True)
         self.assertEqual(405, resp.status_int)
 
     def test_get_on_put_member_action(self):
-        url = '/v2.0/{}/something/put_meh.json'.format(self.collection)
+        url = f'/v2.0/{self.collection}/something/put_meh.json'
         resp = self.app.get(url, expect_errors=True)
         self.assertEqual(405, resp.status_int)
 
@@ -1100,7 +1105,7 @@ class TestParentSubresourceController(test_functional.PecanFunctionalTest):
         fake_plugin = pecan_utils.FakePlugin()
         plugins = {pecan_utils.FakePlugin.PLUGIN_TYPE: fake_plugin}
         new_extensions = {fake_ext.get_alias(): fake_ext}
-        super(TestParentSubresourceController, self).setUp(
+        super().setUp(
             service_plugins=plugins, extensions=new_extensions)
         policy.init()
         policy._ENFORCER.set_rules(
@@ -1115,21 +1120,21 @@ class TestParentSubresourceController(test_functional.PecanFunctionalTest):
                                 FAKE_PARENT_SUBRESOURCE_COLLECTION)
 
     def test_get_duplicate_parent_resource(self):
-        url = '/v2.0/{}'.format(self.fake_collection)
+        url = f'/v2.0/{self.fake_collection}'
         resp = self.app.get(url)
         self.assertEqual(200, resp.status_int)
         self.assertEqual({'fake_duplicates': [{'fake': 'fakeduplicates'}]},
                          resp.json)
 
     def test_get_duplicate_parent_resource_item(self):
-        url = '/v2.0/{}/something'.format(self.fake_collection)
+        url = f'/v2.0/{self.fake_collection}/something'
         resp = self.app.get(url)
         self.assertEqual(200, resp.status_int)
         self.assertEqual({'fake_duplicate': {'fake': 'something'}}, resp.json)
 
     def test_get_parent_resource_and_duplicate_subresources(self):
-        url = '/v2.0/{0}/something/{1}'.format(self.collection,
-                                               self.fake_collection)
+        url = '/v2.0/{}/something/{}'.format(self.collection,
+                                             self.fake_collection)
         resp = self.app.get(url)
         self.assertEqual(200, resp.status_int)
         self.assertEqual({'fake_duplicates': [{'fake': 'something'}]},
@@ -1143,8 +1148,8 @@ class TestParentSubresourceController(test_functional.PecanFunctionalTest):
                 {'get_meh_meh_fake_duplicate': ''}
             )
         )
-        url = '/v2.0/{0}/something/{1}'.format(self.collection,
-                                               self.fake_collection)
+        url = '/v2.0/{}/something/{}'.format(self.collection,
+                                             self.fake_collection)
         resp = self.app.get(url)
         self.assertEqual(200, resp.status_int)
         self.assertEqual({'fake_duplicates': [{'fake': 'something'}]},
@@ -1154,7 +1159,7 @@ class TestParentSubresourceController(test_functional.PecanFunctionalTest):
 class TestExcludeAttributePolicy(test_functional.PecanFunctionalTest):
 
     def setUp(self):
-        super(TestExcludeAttributePolicy, self).setUp()
+        super().setUp()
         policy.init()
         self.addCleanup(policy.reset)
         plugin = directory.get_plugin()
@@ -1165,8 +1170,7 @@ class TestExcludeAttributePolicy(test_functional.PecanFunctionalTest):
 
     def test_get_networks(self):
         response = self.app.get('/v2.0/networks/%s.json' % self.network_id,
-                headers={'X-Project-Id': 'tenid'})
+                                headers={'X-Project-Id': 'tenid'})
         json_body = jsonutils.loads(response.body)
         self.assertEqual(response.status_int, 200)
         self.assertEqual('tenid', json_body['network']['project_id'])
-        self.assertEqual('tenid', json_body['network']['tenant_id'])
