@@ -10,61 +10,11 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from neutron_lib.api import extensions as api_extensions
+from neutron_lib import constants
+
 from neutron.api import extensions
-from neutron.api.v2 import attributes
 from neutron.api.v2 import base
-from neutron.pecan_wsgi import controllers
-from neutron.pecan_wsgi.controllers import utils as pecan_utils
-
-
-class FakeSingularCollectionExtension(extensions.ExtensionDescriptor):
-
-    COLLECTION = 'topologies'
-    RESOURCE = 'topology'
-
-    RAM = {
-        COLLECTION: {
-            'fake': {'is_visible': True}
-        }
-    }
-
-    @classmethod
-    def get_name(cls):
-        return ""
-
-    @classmethod
-    def get_alias(cls):
-        return "fake-sc"
-
-    @classmethod
-    def get_description(cls):
-        return ""
-
-    @classmethod
-    def get_updated(cls):
-        return "2099-07-23T10:00:00-00:00"
-
-    def get_extended_resources(self, version):
-        if version == "2.0":
-            return self.RAM
-        else:
-            return {}
-
-    def get_pecan_controllers(self):
-        ctrllr = controllers.CollectionsController(
-            self.RESOURCE, self.RESOURCE)
-        return [pecan_utils.PecanResourceExtension(self.RESOURCE, ctrllr)]
-
-
-class FakeSingularCollectionPlugin(object):
-
-    supported_extension_aliases = ['fake-sc']
-
-    def get_topology(self, context, id_, fields=None):
-        return {'fake': id_}
-
-    def get_topologies(self, context, filters=None, fields=None):
-        return [{'fake': 'fake'}]
 
 
 def create_network(context, plugin):
@@ -85,7 +35,7 @@ def create_subnet(context, plugin, network_id):
          {'tenant_id': 'tenid',
           'network_id': network_id,
           'name': 'pecansub',
-          'ip_version': 4,
+          'ip_version': constants.IP_VERSION_4,
           'cidr': '10.20.30.0/24',
           'gateway_ip': '10.20.30.1',
           'enable_dhcp': True,
@@ -105,27 +55,39 @@ def create_router(context, l3_plugin):
           'admin_state_up': True}})
 
 
-class FakeExtension(extensions.ExtensionDescriptor):
+class FakeExtension(api_extensions.ExtensionDescriptor):
 
     HYPHENATED_RESOURCE = 'meh_meh'
     HYPHENATED_COLLECTION = HYPHENATED_RESOURCE + 's'
+    FAKE_PARENT_SUBRESOURCE_COLLECTION = 'fake_duplicates'
+    FAKE_SUB_RESOURCE_COLLECTION = 'fake_subresources'
+
+    RESOURCE_ATTRIBUTE_MAP = {
+        'meh_mehs': {
+            'fake': {'is_visible': True}
+        },
+        'fake_duplicates': {
+            'fake': {'is_visible': True}
+        }
+    }
 
     SUB_RESOURCE_ATTRIBUTE_MAP = {
         'fake_subresources': {
             'parent': {
                 'collection_name': (
-                    HYPHENATED_COLLECTION),
-                'member_name': HYPHENATED_RESOURCE},
+                    'meh_mehs'),
+                'member_name': 'meh_meh'},
             'parameters': {'foo': {'is_visible': True},
                            'bar': {'is_visible': True}
                            }
-        }
-    }
-    FAKE_SUB_RESOURCE_COLLECTION = 'fake_subresources'
-
-    RAM = {
-        HYPHENATED_COLLECTION: {
-            'fake': {'is_visible': True}
+        },
+        'fake_duplicates': {
+            'parent': {
+                'collection_name': (
+                    'meh_mehs'),
+                'member_name': 'meh_meh'},
+            'parameters': {'fake': {'is_visible': True}
+                           }
         }
     }
 
@@ -146,21 +108,25 @@ class FakeExtension(extensions.ExtensionDescriptor):
         return "meh"
 
     def get_resources(self):
-        collection = self.HYPHENATED_COLLECTION.replace('_', '-')
-        params = self.RAM.get(self.HYPHENATED_COLLECTION, {})
-        attributes.PLURALS.update({self.HYPHENATED_COLLECTION:
-                                   self.HYPHENATED_RESOURCE})
-        member_actions = {'put_meh': 'PUT', 'boo_meh': 'GET'}
+        """Returns Ext Resources."""
+        resources = []
         fake_plugin = FakePlugin()
-        controller = base.create_resource(
-            collection, self.HYPHENATED_RESOURCE, FakePlugin(),
-            params, allow_bulk=True, allow_pagination=True,
-            allow_sorting=True, member_actions=member_actions)
-        resources = [extensions.ResourceExtension(
-            collection, controller, attr_map=params,
-            member_actions=member_actions)]
+        for collection_name in self.RESOURCE_ATTRIBUTE_MAP:
+            resource_name = collection_name[:-1]
+            params = self.RESOURCE_ATTRIBUTE_MAP.get(collection_name, {})
+            member_actions = {'put_meh': 'PUT', 'boo_meh': 'GET'}
+            if collection_name == self.HYPHENATED_COLLECTION:
+                collection_name = collection_name.replace('_', '-')
+            controller = base.create_resource(
+                collection_name, resource_name, fake_plugin,
+                params, allow_bulk=True, allow_pagination=True,
+                allow_sorting=True, member_actions=member_actions)
+            resource = extensions.ResourceExtension(
+                collection_name, controller, attr_map=params)
+            resources.append(resource)
+
         for collection_name in self.SUB_RESOURCE_ATTRIBUTE_MAP:
-            resource_name = collection_name
+            resource_name = collection_name[:-1]
             parent = self.SUB_RESOURCE_ATTRIBUTE_MAP[collection_name].get(
                 'parent')
             params = self.SUB_RESOURCE_ATTRIBUTE_MAP[collection_name].get(
@@ -170,7 +136,6 @@ class FakeExtension(extensions.ExtensionDescriptor):
                                               fake_plugin, params,
                                               allow_bulk=True,
                                               parent=parent)
-
             resource = extensions.ResourceExtension(
                 collection_name,
                 controller, parent,
@@ -182,12 +147,11 @@ class FakeExtension(extensions.ExtensionDescriptor):
 
     def get_extended_resources(self, version):
         if version == "2.0":
-            return self.RAM
-        else:
-            return {}
+            return self.RESOURCE_ATTRIBUTE_MAP
+        return {}
 
 
-class FakePlugin(object):
+class FakePlugin:
 
     PLUGIN_TYPE = 'fake-ext-plugin'
     supported_extension_aliases = ['fake-ext']
@@ -202,6 +166,16 @@ class FakePlugin(object):
     def get_meh_mehs(self, context, filters=None, fields=None):
         return [{'fake': 'fake'}]
 
+    def get_fake_duplicate(self, context, id_, fields=None):
+        return {'fake': id_}
+
+    def get_fake_duplicates(self, context, filters=None, fields=None):
+        return [{'fake': 'fakeduplicates'}]
+
+    def get_meh_meh_fake_duplicates(self, context, id_, fields=None,
+                                    filters=None):
+        return [{'fake': id_}]
+
     def get_meh_meh_fake_subresources(self, context, id_, fields=None,
                                       filters=None):
         return {'foo': id_}
@@ -209,5 +183,5 @@ class FakePlugin(object):
     def put_meh(self, context, id_, data):
         return {'poo_yah': id_}
 
-    def boo_meh(self, context, id_, fields=None):
+    def boo_meh(self, context, id_):
         return {'boo_yah': id_}
