@@ -28,12 +28,12 @@ from neutron_lib import constants as n_const
 from neutron_lib import context as n_context
 from neutron_lib.exceptions import l3 as lib_l3_exc
 from oslo_utils import uuidutils
+from sqlalchemy.dialects.mysql import dialect as mysql_dialect
 
 from neutron.common.ovn import constants as ovn_const
 from neutron.common.ovn import utils
 from neutron.conf.plugins.ml2.drivers.ovn import ovn_conf as ovn_config
 from neutron.db import ovn_revision_numbers_db as db_rev
-from neutron.objects import servicetype as servicetype_obj
 from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import maintenance
 from neutron.services.portforwarding import constants as pf_consts
 from neutron.tests.functional import base
@@ -42,16 +42,20 @@ from neutron.tests.functional.services.logapi.drivers.ovn \
 
 from neutron.tests.unit.api import test_extensions
 from neutron.tests.unit.extensions import test_extraroute
+from neutron.tests.unit import testlib_api
+
 
 CFG_NEW_BURST = 50
 CFG_NEW_RATE = 150
 
 
-class _TestMaintenanceHelper(base.TestOVNFunctionalBase):
+class _TestMaintenanceHelper(testlib_api.MySQLTestCaseMixin,
+                             base.TestOVNFunctionalBase):
     """A helper class to keep the code more organized."""
 
     def setUp(self):
-        super(_TestMaintenanceHelper, self).setUp()
+        super().setUp()
+        self.assertEqual(mysql_dialect.name, self.db.engine.dialect.name)
         self._ovn_client = self.mech_driver._ovn_client
         self._l3_ovn_client = self.l3_plugin._ovn_client
         ext_mgr = test_extraroute.ExtraRouteTestExtensionManager()
@@ -122,7 +126,7 @@ class _TestMaintenanceHelper(base.TestOVNFunctionalBase):
                 return row
 
     def _set_global_dhcp_opts(self, ip_version, opts):
-        opt_string = ','.join(['{0}:{1}'.format(key, value)
+        opt_string = ','.join([f'{key}:{value}'
                                for key, value
                                in opts.items()])
         if ip_version == n_const.IP_VERSION_6:
@@ -751,9 +755,10 @@ class TestMaintenance(_TestMaintenanceHelper):
     def test_check_for_igmp_snooping_enabled(self):
         cfg.CONF.set_override('igmp_snooping_enable', False, group='OVS')
         net = self._create_network('net')
-        ls = self.nb_api.db_find('Logical_Switch',
+        ls = self.nb_api.db_find(
+            'Logical_Switch',
             ('name', '=', utils.ovn_name(net['id']))).execute(
-            check_error=True)[0]
+                check_error=True)[0]
 
         self.assertEqual('false', ls['other_config'][ovn_const.MCAST_SNOOP])
         self.assertEqual(
@@ -767,9 +772,10 @@ class TestMaintenance(_TestMaintenanceHelper):
         self.assertRaises(periodics.NeverAgain,
                           self.maint.check_for_igmp_snoop_support)
 
-        ls = self.nb_api.db_find('Logical_Switch',
+        ls = self.nb_api.db_find(
+            'Logical_Switch',
             ('name', '=', utils.ovn_name(net['id']))).execute(
-            check_error=True)[0]
+                check_error=True)[0]
 
         self.assertEqual('true', ls['other_config'][ovn_const.MCAST_SNOOP])
         self.assertEqual(
@@ -947,11 +953,11 @@ class TestMaintenance(_TestMaintenanceHelper):
         def _verify_lb(test, protocol, vip_ext_port, vip_int_port):
             ovn_lbs = self._find_pf_lb(router_id, fip_id)
             test.assertEqual(1, len(ovn_lbs))
-            test.assertEqual('pf-floatingip-{}-{}'.format(fip_id, protocol),
+            test.assertEqual(f'pf-floatingip-{fip_id}-{protocol}',
                              ovn_lbs[0].name)
             test.assertEqual(
-                {'{}:{}'.format(fip_ip, vip_ext_port):
-                 '{}:{}'.format(p1_ip, vip_int_port)},
+                {f'{fip_ip}:{vip_ext_port}':
+                 f'{p1_ip}:{vip_int_port}'},
                 ovn_lbs[0].vips)
 
         ext_net = self._create_network('ext_networktest', external=True)
@@ -1191,8 +1197,8 @@ class TestMaintenance(_TestMaintenanceHelper):
             'fixed_ip_address': logical_ip}}
 
         # Create floating IP without gateway_port
-        with mock.patch.object(utils,
-                'is_nat_gateway_port_supported', return_value=False):
+        with mock.patch.object(
+                utils, 'is_nat_gateway_port_supported', return_value=False):
             fip = self.l3_plugin.create_floatingip(self.context, fip_info)
 
         self.assertEqual(router['id'], fip['router_id'])
@@ -1208,7 +1214,8 @@ class TestMaintenance(_TestMaintenanceHelper):
 
         # Call the maintenance task and check that the value has been
         # updated in the NAT rule
-        self.assertRaises(periodics.NeverAgain,
+        self.assertRaises(
+            periodics.NeverAgain,
             self.maint.update_nat_floating_ip_with_gateway_port_reference)
 
         rules = self.nb_api.get_all_logical_routers_with_rports()[0]
@@ -1219,32 +1226,6 @@ class TestMaintenance(_TestMaintenanceHelper):
         else:
             self.assertNotIn('gateway_port', fip_rule)
 
-    def test_add_provider_resource_association_to_routers(self):
-        provider_name = 'ovn'
-        router1 = self._create_router(uuidutils.generate_uuid())
-        router2 = self._create_router(uuidutils.generate_uuid())
-
-        # NOTE(ralonsoh): now each Neutron router will create a
-        # ``ProviderResourceAssociation`` register. In order to be able to test
-        # the maintenance method, the router2 ``ProviderResourceAssociation``
-        # register is deleted.
-        servicetype_obj.ProviderResourceAssociation.delete_objects(
-            self.context, resource_id=router2['id'])
-        pra_list = servicetype_obj.ProviderResourceAssociation.get_objects(
-            self.context, provider_name=provider_name)
-        pra_res_ids = set(pra.resource_id for pra in pra_list)
-        self.assertIn(router1['id'], pra_res_ids)
-        self.assertNotIn(router2['id'], pra_res_ids)
-
-        self.assertRaises(
-            periodics.NeverAgain,
-            self.maint.add_provider_resource_association_to_routers)
-        pra_list = servicetype_obj.ProviderResourceAssociation.get_objects(
-            self.context, provider_name=provider_name)
-        pra_res_ids = set(pra.resource_id for pra in pra_list)
-        self.assertIn(router1['id'], pra_res_ids)
-        self.assertIn(router2['id'], pra_res_ids)
-
     def test_remove_invalid_gateway_chassis_from_unbound_lrp(self):
         net1 = self._create_network(uuidutils.generate_uuid(), external=True)
         subnet1 = self._create_subnet(uuidutils.generate_uuid(), net1['id'])
@@ -1337,7 +1318,7 @@ class TestMaintenance(_TestMaintenanceHelper):
 
         # Create static routes via Neutron
         with mock.patch.object(self.nb_api,
-                'add_static_route', columns=None):
+                               'add_static_route', columns=None):
             self.l3_plugin.update_router(
                 self.context, router['id'],
                 {'router': {'routes': [{'destination': '10.10.0.0/24',
@@ -1370,7 +1351,7 @@ class TestMaintenance(_TestMaintenanceHelper):
         for route in sroute_info:
             if route.ip_prefix == '10.10.0.0/24':
                 self.assertEqual(route.external_ids,
-                    {ovn_const.OVN_LRSR_EXT_ID_KEY: 'true'})
+                                 {ovn_const.OVN_LRSR_EXT_ID_KEY: 'true'})
             if route.ip_prefix == '20.0.0.0/24':
                 self.assertEqual({}, route.external_ids)
             if route.ip_prefix == '30.0.0.0/24':
@@ -1379,18 +1360,18 @@ class TestMaintenance(_TestMaintenanceHelper):
         # Call the maintenance task and check that the value has been
         # updated in the external_ids.
         self.assertRaises(periodics.NeverAgain,
-            self.maint.update_router_static_routes)
+                          self.maint.update_router_static_routes)
 
         sroutes = self.nb_api.get_all_logical_routers_static_routes()[0]
         sroute_info = sroutes['static_routes']
         for route in sroute_info:
             if route.ip_prefix == '10.10.0.0/24':
                 self.assertEqual(route.external_ids,
-                    {ovn_const.OVN_LRSR_EXT_ID_KEY: 'true'})
+                                 {ovn_const.OVN_LRSR_EXT_ID_KEY: 'true'})
             # Check if the OVN static route was updated with the Neutron key
             if route.ip_prefix == '20.0.0.0/24':
                 self.assertEqual(route.external_ids,
-                    {ovn_const.OVN_LRSR_EXT_ID_KEY: 'true'})
+                                 {ovn_const.OVN_LRSR_EXT_ID_KEY: 'true'})
             # Check if the externally managed OVN static route remains
             # without the Neutron key.
             if route.ip_prefix == '30.0.0.0/24':
@@ -1444,6 +1425,34 @@ class TestMaintenance(_TestMaintenanceHelper):
             original_value=True,
             config_value=True)
 
+    def test_set_ovn_owned_dns_option(self):
+        neutron_net = self._create_network('network1')
+        ls_name = utils.ovn_name(neutron_net['id'])
+        with mock.patch.object(
+                self._ovn_client, 'is_dns_required_for_port',
+                return_value=True):
+            self._create_port('portdns', neutron_net['id'])
+
+        ls, ls_dns_record = self.nb_api.get_ls_and_dns_record(ls_name)
+
+        # Assert that option is not set
+        self.assertNotEqual(
+            ls_dns_record.options.get('ovn-owned'), 'true')
+
+        # Override config
+        cfg.CONF.set_override(
+            'dns_records_ovn_owned', True, group='ovn')
+
+        # Call the maintenance task and check that the option has been
+        # updated in the DNS record
+        self.assertRaises(
+            periodics.NeverAgain,
+            self.maint.set_ovn_owned_dns_option)
+
+        # Assert that option is not set
+        self.assertEqual(
+            ls_dns_record.options.get('ovn-owned'), 'true')
+
 
 class TestLogMaintenance(_TestMaintenanceHelper,
                          test_log_driver.LogApiTestCaseBase):
@@ -1460,7 +1469,8 @@ class TestLogMaintenance(_TestMaintenanceHelper,
         # Check a meter and fair meter exist
         self.assertTrue(self.nb_api._tables['Meter'].rows)
         self.assertTrue(self.nb_api._tables['Meter_Band'].rows)
-        self.assertEqual(len([*self.nb_api._tables['Meter'].rows.values()]),
+        self.assertEqual(
+            len([*self.nb_api._tables['Meter'].rows.values()]),
             len([*self.nb_api._tables['Meter_Band'].rows.values()]))
         self._check_meters_consistency()
         # Update burst and rate limit values on the configuration
