@@ -18,7 +18,6 @@ from neutron_lib.api.definitions import provider_net
 from neutron_lib import constants
 from oslo_config import cfg
 from oslo_utils import strutils
-from sqlalchemy.dialects.mysql import dialect as mysql_dialect
 
 from neutron.common.ovn import constants as ovn_const
 from neutron.common.ovn import utils as ovn_utils
@@ -26,16 +25,13 @@ from neutron.conf.plugins.ml2.drivers.ovn import ovn_conf as ovn_config
 from neutron.tests.functional import base
 from neutron.tests.unit.api import test_extensions
 from neutron.tests.unit.extensions import test_l3
-from neutron.tests.unit import testlib_api
 
 
-class TestOVNClient(testlib_api.MySQLTestCaseMixin,
-                    base.TestOVNFunctionalBase,
+class TestOVNClient(base.TestOVNFunctionalBase,
                     test_l3.L3NatTestCaseMixin):
 
     def setUp(self, **kwargs):
         super().setUp(**kwargs)
-        self.assertEqual(mysql_dialect.name, self.db.engine.dialect.name)
         ext_mgr = test_l3.L3TestExtensionManager()
         self.ext_api = test_extensions.setup_extensions_middleware(ext_mgr)
 
@@ -122,7 +118,7 @@ class TestOVNClient(testlib_api.MySQLTestCaseMixin,
                     self.assertTrue(strutils.bool_from_string(
                         lrp.external_ids[ovn_const.OVN_ROUTER_IS_EXT_GW]))
                     hcg = self.nb_api.lookup('HA_Chassis_Group',
-                                            ovn_utils.ovn_name(router_id))
+                                             ovn_utils.ovn_name(router_id))
                     self.assertIsNotNone(hcg)
 
                     # Remove the external GW port.
@@ -186,95 +182,6 @@ class TestOVNClient(testlib_api.MySQLTestCaseMixin,
     def test_router_reside_chassis_redirect_non_dvr_geneve_net(self):
         self._test_router_reside_chassis_redirect(False, 'geneve')
 
-    def _check_gw_lrp_mtu(self, router_id, mtu):
-        # Find gateway LRP and check the MTU value.
-        lr = self.nb_api.lookup('Logical_Router',
-                                ovn_utils.ovn_name(router_id))
-        for lrp in lr.ports:
-            if strutils.bool_from_string(
-                    lrp.external_ids[
-                        ovn_const.OVN_ROUTER_IS_EXT_GW]):
-                self.assertEqual(mtu, int(lrp.options['gateway_mtu']))
-                return
-
-        self.fail('Gateway Logical_Router_Port not found for '
-                  'router %s' % router_id)
-
-    def test_update_network_lrp_mtu_updated(self):
-        cfg.CONF.set_override('ovn_emit_need_to_frag', True, group='ovn')
-        self.add_fake_chassis('host1', enable_chassis_as_gw=True, azs=[])
-        net_ext_args = {provider_net.NETWORK_TYPE: 'geneve',
-                        external_net.EXTERNAL: True,
-                        mtu_def.MTU: 1300}
-        net_int_args = {provider_net.NETWORK_TYPE: 'geneve',
-                        mtu_def.MTU: 1400}
-        with self.network(
-                'test-ext-net', as_admin=True,
-                arg_list=tuple(net_ext_args.keys()), **net_ext_args) as \
-                net_ext, self.network(
-                'test-int-net', as_admin=True,
-                arg_list=tuple(net_int_args.keys()), **net_int_args) as \
-                net_int:
-            with self.subnet(net_ext, cidr='10.1.0.0/24'), \
-                    self.subnet(net_int, cidr='10.2.0.0/24') as snet_int:
-                ext_gw = {'network_id': net_ext['network']['id']}
-                with self.router(external_gateway_info=ext_gw) as router:
-                    router_id = router['router']['id']
-                    self._router_interface_action(
-                        'add', router_id, snet_int['subnet']['id'],
-                        None)
-
-                    self._check_gw_lrp_mtu(router_id, 1300)
-
-                    # Update external network MTU.
-                    net_ext_args = {'network': {mtu_def.MTU: 1350}}
-                    req = self.new_update_request('networks', net_ext_args,
-                                                  net_ext['network']['id'])
-                    req.get_response(self.api)
-                    self._check_gw_lrp_mtu(router_id, 1350)
-
-    def test_add_new_router_interfaces_lrp_mtu_updated(self):
-        mtu_ext = 1350
-        internal_interfaces = [{'cidr': '10.2.0.0/24', 'mtu': mtu_ext - 10},
-                               {'cidr': '10.3.0.0/24', 'mtu': mtu_ext + 10},
-                               {'cidr': '10.4.0.0/24', 'mtu': mtu_ext - 20}]
-        cfg.CONF.set_override('ovn_emit_need_to_frag', True, group='ovn')
-        self.add_fake_chassis('host1', enable_chassis_as_gw=True, azs=[])
-        net_ext_args = {provider_net.NETWORK_TYPE: 'geneve',
-                        external_net.EXTERNAL: True,
-                        mtu_def.MTU: mtu_ext}
-        # Store the network's MTUs connected to the router.
-        router_attached_net_mtus = {mtu_ext}
-        with self.network(
-                'test-ext-net', as_admin=True,
-                arg_list=tuple(net_ext_args.keys()), **net_ext_args) as \
-                net_ext:
-            ext_gw = {'network_id': net_ext['network']['id']}
-            with self.subnet(net_ext, cidr='10.1.0.0/24'), \
-                    self.router(external_gateway_info=ext_gw) as router:
-                router_id = router['router']['id']
-                for _int in internal_interfaces:
-                    # Attach a new internal interface to the router. If the new
-                    # network MTU is lower than the minimum MTU connected, the
-                    # GW LRP.options.gateway_mtu will be updated.
-                    net_int_args = {provider_net.NETWORK_TYPE: 'geneve',
-                                    mtu_def.MTU: _int['mtu']}
-                    router_attached_net_mtus.add(_int['mtu'])
-                    net_int = self._make_network(
-                        self.fmt, name='nettest', admin_state_up=True,
-                        **net_int_args)
-                    snet_int = self._make_subnet(
-                        self.fmt, net_int, constants.ATTR_NOT_SPECIFIED,
-                        _int['cidr'])
-                    self._router_interface_action(
-                        'add', router_id, snet_int['subnet']['id'],
-                        None)
-
-                    # The GW LRP "gateway_mtu" value should be the minimum
-                    # MTU connected to the router.
-                    self._check_gw_lrp_mtu(router_id,
-                                           min(router_attached_net_mtus))
-
     def test_process_address_group(self):
         def _find_address_set_for_ag():
             as_v4 = self.nb_api.lookup(
@@ -316,15 +223,89 @@ class TestOVNClient(testlib_api.MySQLTestCaseMixin,
         ovn_client.delete_address_group(self.context, ag['id'])
         self.assertEqual((None, None), _find_address_set_for_ag())
 
-    def test_create_subnet_with_dhcp_options(self):
-        cfg.CONF.set_override('ovn_dhcp4_global_options',
-                              'ntp_server:1.2.3.4;1.2.3.5,wpad:1.2.3.6',
-                              group='ovn')
-        with self.network('test-ovn-client') as net:
-            with self.subnet(net):
-                dhcp_options = self.nb_api.dhcp_options_list().execute(
-                    check_error=True)[0]
-                self.assertEqual('{1.2.3.4, 1.2.3.5}',
-                                 dhcp_options.options['ntp_server'])
-                self.assertEqual('1.2.3.6',
-                                 dhcp_options.options['wpad'])
+    def _check_gw_lrp_mtu(self, router_id, mtu):
+        # Find gateway LRP and check the MTU value.
+        lr = self.nb_api.lookup('Logical_Router',
+                                ovn_utils.ovn_name(router_id))
+        for lrp in lr.ports:
+            if strutils.bool_from_string(
+                    lrp.external_ids[
+                        ovn_const.OVN_ROUTER_IS_EXT_GW]):
+                self.assertEqual(mtu, int(lrp.options['gateway_mtu']))
+                return
+
+        self.fail('Gateway Logical_Router_Port not found for '
+                  'router %s' % router_id)
+
+    def test_update_network_lrp_mtu_updated(self):
+        self.add_fake_chassis('host1', enable_chassis_as_gw=True, azs=[])
+        net_ext_args = {provider_net.NETWORK_TYPE: 'geneve',
+                        external_net.EXTERNAL: True,
+                        mtu_def.MTU: 1300}
+        net_int_args = {provider_net.NETWORK_TYPE: 'geneve',
+                        mtu_def.MTU: 1400}
+        with self.network(
+                'test-ext-net', as_admin=True,
+                arg_list=tuple(net_ext_args.keys()), **net_ext_args) as \
+                net_ext, self.network(
+                'test-int-net', as_admin=True,
+                arg_list=tuple(net_int_args.keys()), **net_int_args) as \
+                net_int:
+            with self.subnet(net_ext, cidr='10.1.0.0/24'), \
+                    self.subnet(net_int, cidr='10.2.0.0/24') as snet_int:
+                ext_gw = {'network_id': net_ext['network']['id']}
+                with self.router(external_gateway_info=ext_gw) as router:
+                    router_id = router['router']['id']
+                    self._router_interface_action(
+                        'add', router_id, snet_int['subnet']['id'],
+                        None)
+
+                    self._check_gw_lrp_mtu(router_id, 1300)
+
+                    # Update external network MTU.
+                    net_ext_args = {'network': {mtu_def.MTU: 1350}}
+                    req = self.new_update_request('networks', net_ext_args,
+                                                  net_ext['network']['id'])
+                    req.get_response(self.api)
+                    self._check_gw_lrp_mtu(router_id, 1350)
+
+    def test_add_new_router_interfaces_lrp_mtu_updated(self):
+        mtu_ext = 1350
+        internal_interfaces = [{'cidr': '10.2.0.0/24', 'mtu': mtu_ext - 10},
+                               {'cidr': '10.3.0.0/24', 'mtu': mtu_ext + 10},
+                               {'cidr': '10.4.0.0/24', 'mtu': mtu_ext - 20}]
+        self.add_fake_chassis('host1', enable_chassis_as_gw=True, azs=[])
+        net_ext_args = {provider_net.NETWORK_TYPE: 'geneve',
+                        external_net.EXTERNAL: True,
+                        mtu_def.MTU: mtu_ext}
+        # Store the network's MTUs connected to the router.
+        router_attached_net_mtus = {mtu_ext}
+        with self.network(
+                'test-ext-net', as_admin=True,
+                arg_list=tuple(net_ext_args.keys()), **net_ext_args) as \
+                net_ext:
+            ext_gw = {'network_id': net_ext['network']['id']}
+            with self.subnet(net_ext, cidr='10.1.0.0/24'), \
+                    self.router(external_gateway_info=ext_gw) as router:
+                router_id = router['router']['id']
+                for _int in internal_interfaces:
+                    # Attach a new internal interface to the router. If the new
+                    # network MTU is lower than the minimum MTU connected, the
+                    # GW LRP.options.gateway_mtu will be updated.
+                    net_int_args = {provider_net.NETWORK_TYPE: 'geneve',
+                                    mtu_def.MTU: _int['mtu']}
+                    router_attached_net_mtus.add(_int['mtu'])
+                    net_int = self._make_network(
+                        self.fmt, name='nettest', admin_state_up=True,
+                        **net_int_args)
+                    snet_int = self._make_subnet(
+                        self.fmt, net_int, constants.ATTR_NOT_SPECIFIED,
+                        _int['cidr'])
+                    self._router_interface_action(
+                        'add', router_id, snet_int['subnet']['id'],
+                        None)
+
+                    # The GW LRP "gateway_mtu" value should be the minimum
+                    # MTU connected to the router.
+                    self._check_gw_lrp_mtu(router_id,
+                                           min(router_attached_net_mtus))
