@@ -48,7 +48,7 @@ FIP_RULE_PRIO_LIST = [['fip_1', 'fixed_ip_1', 'prio_1'],
 class TestDvrRouterOperations(base.BaseTestCase):
 
     def setUp(self):
-        super(TestDvrRouterOperations, self).setUp()
+        super().setUp()
         mock.patch('eventlet.spawn').start()
         self.conf = agent_config.setup_conf()
         self.conf.register_opts(base_config.core_opts)
@@ -287,6 +287,15 @@ class TestDvrRouterOperations(base.BaseTestCase):
         self.assertEqual(sorted(ret, key=lambda ret: ret[0]),
                          fip_rule_prio_list)
 
+    @mock.patch.object(router_info.RouterInfo, 'initialize')
+    def test_initialize_dvr_local_router(self, super_initialize):
+        ri = self._create_router()
+        self.mock_load_fip.assert_not_called()
+
+        ri.initialize(self.process_monitor)
+        super_initialize.assert_called_once_with(self.process_monitor)
+        self.mock_load_fip.assert_called_once()
+
     def test_get_floating_ips_dvr(self):
         router = mock.MagicMock()
         router.get.return_value = [{'host': HOSTNAME},
@@ -323,9 +332,10 @@ class TestDvrRouterOperations(base.BaseTestCase):
                                                return_value=rtr_2_fip_name)
         ri.fip_ns = instance
         dnat_from_floatingip_to_fixedip = (
-            'PREROUTING', '-d %s/32 -i %s -j DNAT --to-destination %s' % (
+            'PREROUTING', '-d {}/32 -i {} -j DNAT --to-destination {}'.format(
                 floating_ip, rtr_2_fip_name, fixed_ip))
-        to_source = '-s %s/32 -j SNAT --to-source %s' % (fixed_ip, floating_ip)
+        to_source = '-s {}/32 -j SNAT --to-source {}'.format(
+            fixed_ip, floating_ip)
 
         if ri.iptables_manager.random_fully:
             to_source += ' --random-fully'
@@ -360,7 +370,7 @@ class TestDvrRouterOperations(base.BaseTestCase):
                                                return_value=rtr_2_fip_name)
         ri.fip_ns = instance
         mark_traffic_to_floating_ip = (
-            'floatingip', '-d %s/32 -i %s -j MARK --set-xmark %s' % (
+            'floatingip', '-d {}/32 -i {} -j MARK --set-xmark {}'.format(
                 floating_ip, rtr_2_fip_name, internal_mark))
         mark_traffic_from_fixed_ip = (
             'FORWARD', '-s %s/32 -j $float-snat' % fixed_ip)
@@ -535,10 +545,10 @@ class TestDvrRouterOperations(base.BaseTestCase):
         self.assertIsNone(ri._get_internal_port(mock.sentinel.subnet_id2))
 
     def test__get_snat_idx_ipv4(self):
-        ip_cidr = '101.12.13.00/24'
+        ip_cidr = '101.12.13.0/24'
         ri = self._create_router(mock.MagicMock())
         snat_idx = ri._get_snat_idx(ip_cidr)
-        # 0x650C0D00 is numerical value of 101.12.13.00
+        # 0x650C0D00 is numerical value of 101.12.13.0
         self.assertEqual(0x650C0D00, snat_idx)
 
     def test__get_snat_idx_ipv6(self):
@@ -585,18 +595,7 @@ class TestDvrRouterOperations(base.BaseTestCase):
                                       'subnet_id': subnet_id}],
                        'allowed_address_pairs': [
                            {'ip_address': '10.20.30.40',
-                            'mac_address': '00:11:22:33:44:55'}]},
-                      {'mac_address': '11:22:33:44:55:66',
-                       'device_owner': lib_constants.DEVICE_OWNER_LOADBALANCER,
-                       'fixed_ips': [{'ip_address': '1.2.3.5',
-                                      'prefixlen': 24,
-                                      'subnet_id': subnet_id}]},
-                      {'mac_address': '22:33:44:55:66:77',
-                       'device_owner':
-                           lib_constants.DEVICE_OWNER_LOADBALANCERV2,
-                       'fixed_ips': [{'ip_address': '1.2.3.6',
-                                      'prefixlen': 24,
-                                      'subnet_id': subnet_id}]}]
+                            'mac_address': '00:11:22:33:44:55'}]}]
 
         self.plugin_api.get_ports_by_subnet.return_value = test_ports
 
@@ -950,8 +949,8 @@ class TestDvrRouterOperations(base.BaseTestCase):
 
     @mock.patch.object(dvr_edge_rtr.DvrEdgeRouter,
                        'remove_centralized_floatingip')
-    def test_remove_centralized_floatingip(self,
-                                super_remove_centralized_floatingip):
+    def test_remove_centralized_floatingip(
+            self, super_remove_centralized_floatingip):
         agent = l3_agent.L3NATAgent(HOSTNAME, self.conf)
         agent.conf.agent_mode = lib_constants.L3_AGENT_MODE_DVR_SNAT
         router = l3_test_common.prepare_router_data(num_internal_ports=2)
@@ -1012,6 +1011,6 @@ class TestDvrRouterOperations(base.BaseTestCase):
                         "KeepalivedManager.check_processes",
                         return_value=False):
             ri.initialize(mock.Mock())
-            with open(ri._ha_state_path, "r") as f:
+            with open(ri._ha_state_path) as f:
                 state = f.readline()
                 self.assertEqual("backup", state)
