@@ -58,7 +58,7 @@ def makePort(datapath=None, type='', mac=None, external_ids=None,
 
 class ConfFixture(config_fixture.Config):
     def setUp(self):
-        super(ConfFixture, self).setUp()
+        super().setUp()
         ovn_meta_conf.register_meta_conf_opts(meta_conf.SHARED_OPTS, self.conf)
         ovn_meta_conf.register_meta_conf_opts(
             meta_conf.UNIX_DOMAIN_METADATA_PROXY_OPTS, self.conf)
@@ -74,7 +74,7 @@ class TestMetadataAgent(base.BaseTestCase):
     fake_conf_fixture = ConfFixture(fake_conf)
 
     def setUp(self):
-        super(TestMetadataAgent, self).setUp()
+        super().setUp()
         self.useFixture(self.fake_conf_fixture)
         self.log_p = mock.patch.object(agent, 'LOG')
         self.log = self.log_p.start()
@@ -91,8 +91,9 @@ class TestMetadataAgent(base.BaseTestCase):
         self.ports = []
         for i in range(0, 3):
             self.ports.append(makePort(
-                datapath=DatapathInfo(uuid=str(uuid.uuid4()),
-                external_ids={'name': 'neutron-%d' % i})))
+                datapath=DatapathInfo(
+                    uuid=str(uuid.uuid4()),
+                    external_ids={'name': 'neutron-%d' % i})))
         self.agent.sb_idl.get_ports_on_chassis.return_value = self.ports
 
     def test_sync(self):
@@ -168,13 +169,13 @@ class TestMetadataAgent(base.BaseTestCase):
         """
 
         datapath_1 = DatapathInfo(uuid='uuid1',
-            external_ids={'name': 'neutron-1'})
+                                  external_ids={'name': 'neutron-1'})
         datapath_2 = DatapathInfo(uuid='uuid2',
-            external_ids={'name': 'neutron-2'})
+                                  external_ids={'name': 'neutron-2'})
         datapath_3 = DatapathInfo(uuid='uuid3',
-            external_ids={'name': 'neutron-3'})
+                                  external_ids={'name': 'neutron-3'})
         datapath_4 = DatapathInfo(uuid='uuid4',
-            external_ids={'name': 'neutron-4'})
+                                  external_ids={'name': 'neutron-4'})
 
         ports = [
             makePort(datapath_1, type=''),
@@ -186,7 +187,7 @@ class TestMetadataAgent(base.BaseTestCase):
         ]
 
         with mock.patch.object(self.agent.sb_idl, 'get_ports_on_chassis',
-                              return_value=ports):
+                               return_value=ports):
             self.assertEqual([ports[0], ports[1], ports[3], ports[4]],
                              self.agent.get_networks_port_bindings())
 
@@ -295,8 +296,8 @@ class TestMetadataAgent(base.BaseTestCase):
         """
         self.ipv6.return_value = ipv6_enabled
         lla = 'fe80::f816:3eff:fe63:8dc5/64'
-        current_namespace_cidrs = set([n_const.METADATA_CIDR, '10.0.1.0/31',
-                                       lla])
+        current_namespace_cidrs = {n_const.METADATA_CIDR, '10.0.1.0/31',
+                                   lla}
         datapath_port_ips = ['10.0.0.2', '10.0.0.3', '10.0.1.5']
         metadaport_subnet_cidrs = ['10.0.0.0/30', '10.0.1.0/28', '11.0.1.2/24']
 
@@ -376,7 +377,7 @@ class TestMetadataAgent(base.BaseTestCase):
         """
         network_id = '1'
         datapath = DatapathInfo(uuid='test123',
-            external_ids={'name': 'neutron-{}'.format(network_id)})
+                                external_ids={'name': f'neutron-{network_id}'})
 
         with mock.patch.object(
                 self.agent.sb_idl, 'get_metadata_port',
@@ -393,7 +394,7 @@ class TestMetadataAgent(base.BaseTestCase):
         """
         network_id = '1'
         datapath = DatapathInfo(uuid='test123',
-            external_ids={'name': 'neutron-{}'.format(network_id)})
+                                external_ids={'name': f'neutron-{network_id}'})
         metadadata_port = makePort(datapath,
                                    mac=['NO_MAC_HERE 1.2.3.4'],
                                    external_ids={'neutron:cidrs':
@@ -413,7 +414,7 @@ class TestMetadataAgent(base.BaseTestCase):
         """
         network_id = '1'
         datapath = DatapathInfo(uuid='test123',
-            external_ids={'name': 'neutron-{}'.format(network_id)})
+                                external_ids={'name': f'neutron-{network_id}'})
         datapath_ports = [makePort(datapath, type='not_vif_type')]
         metadadata_port = makePort(datapath,
                                    mac=['fa:16:3e:22:65:18 1.2.3.4'],
@@ -421,9 +422,9 @@ class TestMetadataAgent(base.BaseTestCase):
                                                  '10.204.0.10/29'})
 
         with mock.patch.object(self.agent.sb_idl, 'get_metadata_port',
-                    return_value=metadadata_port),\
+                               return_value=metadadata_port),\
                 mock.patch.object(self.agent.sb_idl, 'get_ports_on_chassis',
-                    return_value=datapath_ports),\
+                                  return_value=datapath_ports),\
                 mock.patch.object(self.agent, 'teardown_datapath') as tdp:
             self.assertIsNone(self.agent._get_provision_params(datapath))
             tdp.assert_called_once_with(network_id)
@@ -439,21 +440,21 @@ class TestMetadataAgent(base.BaseTestCase):
         metada_port_logical_port = "3b66c176-199b-48ec-8331-c1fd3f6e2b44"
 
         datapath = DatapathInfo(uuid='test123',
-            external_ids={'name': 'neutron-{}'.format(network_id)})
+                                external_ids={'name': f'neutron-{network_id}'})
         datapath_ports = [makePort(datapath,
-                                   mac=['fa:16:3e:e7:ac {}'.format(port_ip)])]
+                                   mac=[f'fa:16:3e:e7:ac {port_ip}'])]
         metadadata_port = makePort(datapath,
                                    mac=[
-                                       '{} 10.204.0.1'.format(metada_port_mac)
+                                       f'{metada_port_mac} 10.204.0.1'
                                    ],
                                    external_ids={'neutron:cidrs':
                                                  metada_port_subnet_cidr},
                                    logical_port=metada_port_logical_port)
 
         with mock.patch.object(self.agent.sb_idl, 'get_metadata_port',
-                return_value=metadadata_port),\
+                               return_value=metadadata_port),\
             mock.patch.object(self.agent.sb_idl, 'get_ports_on_chassis',
-                return_value=datapath_ports):
+                              return_value=datapath_ports):
             actual_params = self.agent._get_provision_params(datapath)
 
         net_name, datapath_port_ips, any_ip6, metadata_port_info = (
@@ -467,7 +468,7 @@ class TestMetadataAgent(base.BaseTestCase):
         else:
             self.assertTrue(any_ip6)
         self.assertEqual(metada_port_mac, metadata_port_info.mac)
-        self.assertSetEqual(set([metada_port_subnet_cidr]),
+        self.assertSetEqual({metada_port_subnet_cidr},
                             metadata_port_info.ip_addresses)
         self.assertEqual(metada_port_logical_port,
                          metadata_port_info.logical_port)
