@@ -15,6 +15,9 @@
 
 from unittest import mock
 
+from neutron_lib.api import attributes
+from neutron_lib.api.definitions import subnet as subnet_def
+from neutron_lib.api.definitions import subnet_external_network as sen_def
 from oslo_policy import policy as base_policy
 from oslo_utils import uuidutils
 
@@ -25,7 +28,12 @@ from neutron.tests.unit.conf.policies import test_base as base
 class SubnetAPITestCase(base.PolicyBaseTestCase):
 
     def setUp(self):
-        super(SubnetAPITestCase, self).setUp()
+        # Extend subnet "subnet-external-network" extension. This extension
+        # is not loaded in the unit tests.
+        rname = subnet_def.COLLECTION_NAME
+        attributes.RESOURCES[rname].update(
+            sen_def.RESOURCE_ATTRIBUTE_MAP[rname])
+        super().setUp()
 
         self.network = {
             'id': uuidutils.generate_uuid(),
@@ -35,17 +43,23 @@ class SubnetAPITestCase(base.PolicyBaseTestCase):
             'id': uuidutils.generate_uuid(),
             'tenant_id': self.alt_project_id,
             'project_id': self.alt_project_id}
+        self.ext_alt_network = {
+            'id': uuidutils.generate_uuid(),
+            'tenant_id': self.alt_project_id,
+            'project_id': self.alt_project_id}
 
         networks = {
             self.network['id']: self.network,
-            self.alt_network['id']: self.alt_network}
+            self.alt_network['id']: self.alt_network,
+            self.ext_alt_network['id']: self.ext_alt_network,
+        }
 
         self.target = {
             'project_id': self.project_id,
             'tenant_id': self.project_id,
             'network_id': self.network['id'],
             'ext_parent_network_id': self.network['id']}
-        # This network belongs to "project_id", but not the network that
+        # This subnet belongs to "project_id", but not the network that
         # belongs to "alt_project_id".
         self.target_net_alt_target = {
             'project_id': self.project_id,
@@ -57,6 +71,14 @@ class SubnetAPITestCase(base.PolicyBaseTestCase):
             'tenant_id': self.alt_project_id,
             'network_id': self.alt_network['id'],
             'ext_parent_network_id': self.alt_network['id']}
+        # Both the subnet and the network belongs to "alt_project_id" and the
+        # network is external.
+        self.target_net_ext_alt_target = {
+            'project_id': self.alt_project_id,
+            'tenant_id': self.alt_project_id,
+            'network_id': self.ext_alt_network['id'],
+            'ext_parent_network_id': self.ext_alt_network['id'],
+            'router:external': True}
         # This is the case where the network belongs to the project but not
         # the subnet.
         self.alt_target_own_net = {
@@ -78,7 +100,7 @@ class SubnetAPITestCase(base.PolicyBaseTestCase):
 class SystemAdminTests(SubnetAPITestCase):
 
     def setUp(self):
-        super(SystemAdminTests, self).setUp()
+        super().setUp()
         self.context = self.system_admin_ctx
 
     def test_create_subnet(self):
@@ -138,6 +160,24 @@ class SystemAdminTests(SubnetAPITestCase):
             self.context, 'create_subnet:service_types',
             self.alt_target_own_net)
 
+    def test_create_subnet_tags(self):
+        self.assertRaises(
+            base_policy.InvalidScope,
+            policy.enforce,
+            self.context, 'create_subnet:tags', self.target)
+        self.assertRaises(
+            base_policy.InvalidScope,
+            policy.enforce,
+            self.context, 'create_subnet:tags', self.target_net_alt_target)
+        self.assertRaises(
+            base_policy.InvalidScope,
+            policy.enforce,
+            self.context, 'create_subnet:tags', self.alt_target)
+        self.assertRaises(
+            base_policy.InvalidScope,
+            policy.enforce,
+            self.context, 'create_subnet:tags', self.alt_target_own_net)
+
     def test_get_subnet(self):
         self.assertRaises(
             base_policy.InvalidScope,
@@ -147,6 +187,10 @@ class SystemAdminTests(SubnetAPITestCase):
             base_policy.InvalidScope,
             policy.enforce,
             self.context, 'get_subnet', self.target_net_alt_target)
+        self.assertRaises(
+            base_policy.InvalidScope,
+            policy.enforce,
+            self.context, 'get_subnet', self.target_net_ext_alt_target)
         self.assertRaises(
             base_policy.InvalidScope,
             policy.enforce,
@@ -174,23 +218,27 @@ class SystemAdminTests(SubnetAPITestCase):
             policy.enforce,
             self.context, 'get_subnet:segment_id', self.alt_target_own_net)
 
-    def test_get_subnets_tags(self):
+    def test_get_subnet_tags(self):
         self.assertRaises(
             base_policy.InvalidScope,
             policy.enforce,
-            self.context, 'get_subnets_tags', self.target)
+            self.context, 'get_subnet:tags', self.target)
         self.assertRaises(
             base_policy.InvalidScope,
             policy.enforce,
-            self.context, 'get_subnets_tags', self.target_net_alt_target)
+            self.context, 'get_subnet:tags', self.target_net_alt_target)
         self.assertRaises(
             base_policy.InvalidScope,
             policy.enforce,
-            self.context, 'get_subnets_tags', self.alt_target)
+            self.context, 'get_subnet:tags', self.target_net_ext_alt_target)
         self.assertRaises(
             base_policy.InvalidScope,
             policy.enforce,
-            self.context, 'get_subnets_tags', self.alt_target_own_net)
+            self.context, 'get_subnet:tags', self.alt_target)
+        self.assertRaises(
+            base_policy.InvalidScope,
+            policy.enforce,
+            self.context, 'get_subnet:tags', self.alt_target_own_net)
 
     def test_update_subnet(self):
         self.assertRaises(
@@ -249,23 +297,23 @@ class SystemAdminTests(SubnetAPITestCase):
             self.context, 'update_subnet:service_types',
             self.alt_target_own_net)
 
-    def test_update_subnets_tags(self):
+    def test_update_subnet_tags(self):
         self.assertRaises(
             base_policy.InvalidScope,
             policy.enforce,
-            self.context, 'update_subnets_tags', self.target)
+            self.context, 'update_subnet:tags', self.target)
         self.assertRaises(
             base_policy.InvalidScope,
             policy.enforce,
-            self.context, 'update_subnets_tags', self.target_net_alt_target)
+            self.context, 'update_subnet:tags', self.target_net_alt_target)
         self.assertRaises(
             base_policy.InvalidScope,
             policy.enforce,
-            self.context, 'update_subnets_tags', self.alt_target)
+            self.context, 'update_subnet:tags', self.alt_target)
         self.assertRaises(
             base_policy.InvalidScope,
             policy.enforce,
-            self.context, 'update_subnets_tags', self.alt_target_own_net)
+            self.context, 'update_subnet:tags', self.alt_target_own_net)
 
     def test_delete_subnet(self):
         self.assertRaises(
@@ -285,43 +333,43 @@ class SystemAdminTests(SubnetAPITestCase):
             policy.enforce,
             self.context, 'delete_subnet', self.alt_target_own_net)
 
-    def test_delete_subnets_tags(self):
+    def test_delete_subnet_tags(self):
         self.assertRaises(
             base_policy.InvalidScope,
             policy.enforce,
-            self.context, 'delete_subnets_tags', self.target)
+            self.context, 'delete_subnet:tags', self.target)
         self.assertRaises(
             base_policy.InvalidScope,
             policy.enforce,
-            self.context, 'delete_subnets_tags', self.target_net_alt_target)
+            self.context, 'delete_subnet:tags', self.target_net_alt_target)
         self.assertRaises(
             base_policy.InvalidScope,
             policy.enforce,
-            self.context, 'delete_subnets_tags', self.alt_target)
+            self.context, 'delete_subnet:tags', self.alt_target)
         self.assertRaises(
             base_policy.InvalidScope,
             policy.enforce,
-            self.context, 'delete_subnets_tags', self.alt_target_own_net)
+            self.context, 'delete_subnet:tags', self.alt_target_own_net)
 
 
 class SystemMemberTests(SystemAdminTests):
 
     def setUp(self):
-        super(SystemMemberTests, self).setUp()
+        super().setUp()
         self.context = self.system_member_ctx
 
 
 class SystemReaderTests(SystemMemberTests):
 
     def setUp(self):
-        super(SystemReaderTests, self).setUp()
+        super().setUp()
         self.context = self.system_reader_ctx
 
 
 class AdminTests(SubnetAPITestCase):
 
     def setUp(self):
-        super(AdminTests, self).setUp()
+        super().setUp()
         self.context = self.project_admin_ctx
 
     def test_create_subnet(self):
@@ -368,12 +416,28 @@ class AdminTests(SubnetAPITestCase):
                 self.context, 'create_subnet:service_types',
                 self.alt_target_own_net))
 
+    def test_create_subnet_tags(self):
+        self.assertTrue(
+            policy.enforce(self.context, 'create_subnet:tags', self.target))
+        self.assertTrue(
+            policy.enforce(self.context, 'create_subnet:tags',
+                           self.target_net_alt_target))
+        self.assertTrue(
+            policy.enforce(self.context, 'create_subnet:tags',
+                           self.alt_target))
+        self.assertTrue(
+            policy.enforce(self.context, 'create_subnet:tags',
+                           self.alt_target_own_net))
+
     def test_get_subnet(self):
         self.assertTrue(
             policy.enforce(self.context, 'get_subnet', self.target))
         self.assertTrue(
             policy.enforce(self.context, 'get_subnet',
                            self.target_net_alt_target))
+        self.assertTrue(
+            policy.enforce(self.context, 'get_subnet',
+                           self.target_net_ext_alt_target))
         self.assertTrue(
             policy.enforce(self.context, 'get_subnet', self.alt_target))
         self.assertTrue(
@@ -394,16 +458,19 @@ class AdminTests(SubnetAPITestCase):
                 self.context, 'get_subnet:segment_id',
                 self.alt_target_own_net))
 
-    def test_get_subnets_tags(self):
+    def test_get_subnet_tags(self):
         self.assertTrue(
-            policy.enforce(self.context, 'get_subnets_tags', self.target))
+            policy.enforce(self.context, 'get_subnet:tags', self.target))
         self.assertTrue(
-            policy.enforce(self.context, 'get_subnets_tags',
+            policy.enforce(self.context, 'get_subnet:tags',
                            self.target_net_alt_target))
         self.assertTrue(
-            policy.enforce(self.context, 'get_subnets_tags', self.alt_target))
+            policy.enforce(self.context, 'get_subnet:tags',
+                           self.target_net_ext_alt_target))
         self.assertTrue(
-            policy.enforce(self.context, 'get_subnets_tags',
+            policy.enforce(self.context, 'get_subnet:tags', self.alt_target))
+        self.assertTrue(
+            policy.enforce(self.context, 'get_subnet:tags',
                            self.alt_target_own_net))
 
     def test_update_subnet(self):
@@ -446,17 +513,17 @@ class AdminTests(SubnetAPITestCase):
             policy.enforce(
                 self.context, 'update_subnet:service_types', self.alt_target))
 
-    def test_update_subnets_tags(self):
+    def test_update_subnet_tags(self):
         self.assertTrue(
-            policy.enforce(self.context, 'update_subnets_tags', self.target))
+            policy.enforce(self.context, 'update_subnet:tags', self.target))
         self.assertTrue(
-            policy.enforce(self.context, 'update_subnets_tags',
+            policy.enforce(self.context, 'update_subnet:tags',
                            self.target_net_alt_target))
         self.assertTrue(
-            policy.enforce(self.context, 'update_subnets_tags',
+            policy.enforce(self.context, 'update_subnet:tags',
                            self.alt_target))
         self.assertTrue(
-            policy.enforce(self.context, 'update_subnets_tags',
+            policy.enforce(self.context, 'update_subnet:tags',
                            self.alt_target_own_net))
 
     def test_delete_subnet(self):
@@ -471,25 +538,25 @@ class AdminTests(SubnetAPITestCase):
             policy.enforce(self.context, 'delete_subnet',
                            self.alt_target_own_net))
 
-    def test_delete_subnets_tags(self):
+    def test_delete_subnet_tags(self):
         self.assertTrue(
-            policy.enforce(self.context, 'delete_subnets_tags', self.target))
+            policy.enforce(self.context, 'delete_subnet:tags', self.target))
         self.assertTrue(
-            policy.enforce(self.context, 'delete_subnets_tags',
+            policy.enforce(self.context, 'delete_subnet:tags',
                            self.target_net_alt_target))
         self.assertTrue(
-            policy.enforce(self.context, 'delete_subnets_tags',
+            policy.enforce(self.context, 'delete_subnet:tags',
                            self.alt_target))
         self.assertTrue(
-            policy.enforce(self.context, 'delete_subnets_tags',
+            policy.enforce(self.context, 'delete_subnet:tags',
                            self.alt_target_own_net))
 
 
-class ProjectMemberTests(AdminTests):
+class ProjectManagerTests(AdminTests):
 
     def setUp(self):
-        super(ProjectMemberTests, self).setUp()
-        self.context = self.project_member_ctx
+        super().setUp()
+        self.context = self.project_manager_ctx
 
     def test_create_subnet(self):
         self.assertTrue(
@@ -545,12 +612,29 @@ class ProjectMemberTests(AdminTests):
             self.context, 'create_subnet:service_types',
             self.alt_target_own_net)
 
+    def test_create_subnet_tags(self):
+        self.assertTrue(
+            policy.enforce(self.context, 'create_subnet:tags', self.target))
+        self.assertTrue(
+            policy.enforce(self.context, 'create_subnet:tags',
+                           self.target_net_alt_target))
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'create_subnet:tags', self.alt_target)
+        self.assertTrue(
+            policy.enforce(self.context, 'create_subnet:tags',
+                           self.alt_target_own_net))
+
     def test_get_subnet(self):
         self.assertTrue(
             policy.enforce(self.context, 'get_subnet', self.target))
         self.assertTrue(
             policy.enforce(self.context, 'get_subnet',
                            self.target_net_alt_target))
+        self.assertTrue(
+            policy.enforce(self.context, 'get_subnet',
+                           self.target_net_ext_alt_target))
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
             policy.enforce,
@@ -577,18 +661,21 @@ class ProjectMemberTests(AdminTests):
             policy.enforce,
             self.context, 'get_subnet:segment_id', self.alt_target_own_net)
 
-    def test_get_subnets_tags(self):
+    def test_get_subnet_tags(self):
         self.assertTrue(
-            policy.enforce(self.context, 'get_subnets_tags', self.target))
+            policy.enforce(self.context, 'get_subnet:tags', self.target))
         self.assertTrue(
-            policy.enforce(self.context, 'get_subnets_tags',
+            policy.enforce(self.context, 'get_subnet:tags',
                            self.target_net_alt_target))
+        self.assertTrue(
+            policy.enforce(self.context, 'get_subnet:tags',
+                           self.target_net_ext_alt_target))
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
             policy.enforce,
-            self.context, 'get_subnets_tags', self.alt_target)
+            self.context, 'get_subnet:tags', self.alt_target)
         self.assertTrue(
-            policy.enforce(self.context, 'get_subnets_tags',
+            policy.enforce(self.context, 'get_subnet:tags',
                            self.alt_target_own_net))
 
     def test_update_subnet(self):
@@ -644,18 +731,18 @@ class ProjectMemberTests(AdminTests):
             self.context, 'update_subnet:service_types',
             self.alt_target_own_net)
 
-    def test_update_subnets_tags(self):
+    def test_update_subnet_tags(self):
         self.assertTrue(
-            policy.enforce(self.context, 'update_subnets_tags', self.target))
+            policy.enforce(self.context, 'update_subnet:tags', self.target))
         self.assertTrue(
-            policy.enforce(self.context, 'update_subnets_tags',
+            policy.enforce(self.context, 'update_subnet:tags',
                            self.target_net_alt_target))
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
             policy.enforce,
-            self.context, 'update_subnets_tags', self.alt_target)
+            self.context, 'update_subnet:tags', self.alt_target)
         self.assertTrue(
-            policy.enforce(self.context, 'update_subnets_tags',
+            policy.enforce(self.context, 'update_subnet:tags',
                            self.alt_target_own_net))
 
     def test_delete_subnet(self):
@@ -672,25 +759,32 @@ class ProjectMemberTests(AdminTests):
             policy.enforce(self.context, 'delete_subnet',
                            self.alt_target_own_net))
 
-    def test_delete_subnets_tags(self):
+    def test_delete_subnet_tags(self):
         self.assertTrue(
-            policy.enforce(self.context, 'delete_subnets_tags', self.target))
+            policy.enforce(self.context, 'delete_subnet:tags', self.target))
         self.assertTrue(
-            policy.enforce(self.context, 'delete_subnets_tags',
+            policy.enforce(self.context, 'delete_subnet:tags',
                            self.target_net_alt_target))
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
             policy.enforce,
-            self.context, 'delete_subnets_tags', self.alt_target)
+            self.context, 'delete_subnet:tags', self.alt_target)
         self.assertTrue(
-            policy.enforce(self.context, 'delete_subnets_tags',
+                policy.enforce(self.context, 'delete_subnet:tags',
                            self.alt_target_own_net))
 
 
+class ProjectMemberTests(ProjectManagerTests):
+
+    def setUp(self):
+        super().setUp()
+        self.context = self.project_member_ctx
+
+
 class ProjectReaderTests(ProjectMemberTests):
 
     def setUp(self):
-        super(ProjectReaderTests, self).setUp()
+        super().setUp()
         self.context = self.project_reader_ctx
 
     def test_create_subnet(self):
@@ -711,6 +805,24 @@ class ProjectReaderTests(ProjectMemberTests):
             policy.enforce,
             self.context, 'create_subnet', self.alt_target_own_net)
 
+    def test_create_subnet_tags(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'create_subnet:tags', self.target)
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'create_subnet:tags', self.target_net_alt_target)
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'create_subnet:tags', self.alt_target)
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'create_subnet:tags', self.alt_target_own_net)
+
     def test_update_subnet(self):
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
@@ -729,23 +841,23 @@ class ProjectReaderTests(ProjectMemberTests):
             policy.enforce,
             self.context, 'update_subnet', self.alt_target_own_net)
 
-    def test_update_subnets_tags(self):
+    def test_update_subnet_tags(self):
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
             policy.enforce,
-            self.context, 'update_subnets_tags', self.target)
+            self.context, 'update_subnet:tags', self.target)
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
             policy.enforce,
-            self.context, 'update_subnets_tags', self.target_net_alt_target)
+            self.context, 'update_subnet:tags', self.target_net_alt_target)
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
             policy.enforce,
-            self.context, 'update_subnets_tags', self.alt_target)
+            self.context, 'update_subnet:tags', self.alt_target)
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
             policy.enforce,
-            self.context, 'update_subnets_tags', self.alt_target_own_net)
+            self.context, 'update_subnet:tags', self.alt_target_own_net)
 
     def test_delete_subnet(self):
         self.assertRaises(
@@ -765,20 +877,87 @@ class ProjectReaderTests(ProjectMemberTests):
             policy.enforce,
             self.context, 'delete_subnet', self.alt_target_own_net)
 
-    def test_delete_subnets_tags(self):
+    def test_delete_subnet_tags(self):
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
             policy.enforce,
-            self.context, 'delete_subnets_tags', self.target)
+            self.context, 'delete_subnet:tags', self.target)
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
             policy.enforce,
-            self.context, 'delete_subnets_tags', self.target_net_alt_target)
+            self.context, 'delete_subnet:tags', self.target_net_alt_target)
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
             policy.enforce,
-            self.context, 'delete_subnets_tags', self.alt_target)
+            self.context, 'delete_subnet:tags', self.alt_target)
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
             policy.enforce,
-            self.context, 'delete_subnets_tags', self.alt_target_own_net)
+            self.context, 'delete_subnet:tags', self.alt_target_own_net)
+
+
+class ServiceRoleTests(SubnetAPITestCase):
+
+    def setUp(self):
+        super().setUp()
+        self.context = self.service_ctx
+
+    def test_create_subnet(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'create_subnet', self.target)
+
+    def test_create_subnet_segment_id(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'create_subnet:segment_id', self.target)
+
+    def test_create_subnet_service_types(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'create_subnet:service_types', self.target)
+
+    def test_create_subnet_tags(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'create_subnet:tags', self.target)
+
+    def test_get_subnet(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'get_subnet', self.target)
+
+    def test_get_subnet_segment_id(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'get_subnet:segment_id', self.target)
+
+    def test_update_subnet(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'update_subnet', self.target)
+
+    def test_update_subnet_segment_id(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'update_subnet:segment_id', self.target)
+
+    def test_update_subnet_service_types(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'update_subnet:service_types', self.target)
+
+    def test_delete_subnet(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'delete_subnet', self.target)
