@@ -15,20 +15,22 @@
 import math
 
 import netaddr
+from neutron_lib.api.definitions import dns as dns_apidef
+from neutron_lib.api.definitions import l3 as l3_apdef
 from neutron_lib import constants
+from neutron_lib import context
+from neutron_lib.db import constants as db_const
+from neutron_lib.plugins import directory
 from oslo_config import cfg
 
 from neutron.common import utils
-from neutron import context
 from neutron.db import db_base_plugin_v2
 from neutron.extensions import dns
-from neutron import manager
-from neutron.plugins.ml2 import config
-from neutron.tests.unit.db import test_db_base_plugin_v2
+from neutron.tests.common import test_db_base_plugin_v2
 from neutron.tests.unit.plugins.ml2 import test_plugin
 
 
-class DnsExtensionManager(object):
+class DnsExtensionManager:
 
     def get_resources(self):
         return []
@@ -40,14 +42,14 @@ class DnsExtensionManager(object):
         return []
 
     def get_extended_resources(self, version):
-        return dns.get_extended_resources(version)
+        return dns.Dns().get_extended_resources(version)
 
 
 class DnsExtensionTestPlugin(db_base_plugin_v2.NeutronDbPluginV2):
     """Test plugin to mixin the DNS Integration extensions.
     """
 
-    supported_extension_aliases = ["dns-integration", "router"]
+    supported_extension_aliases = [dns_apidef.ALIAS, l3_apdef.ALIAS]
 
 
 class DnsExtensionTestCase(test_plugin.Ml2PluginV2TestCase):
@@ -57,10 +59,10 @@ class DnsExtensionTestCase(test_plugin.Ml2PluginV2TestCase):
     _extension_drivers = ['dns']
 
     def setUp(self):
-        config.cfg.CONF.set_override('extension_drivers',
-                                     self._extension_drivers,
-                                     group='ml2')
-        super(DnsExtensionTestCase, self).setUp()
+        cfg.CONF.set_override('extension_drivers',
+                              self._extension_drivers,
+                              group='ml2')
+        super().setUp()
 
     def _create_network(self, fmt, name, admin_state_up,
                         arg_list=None, set_context=False, tenant_id=None,
@@ -68,12 +70,11 @@ class DnsExtensionTestCase(test_plugin.Ml2PluginV2TestCase):
         new_arg_list = ('dns_domain',)
         if arg_list is not None:
             new_arg_list = arg_list + new_arg_list
-        return super(DnsExtensionTestCase,
-                     self)._create_network(fmt, name, admin_state_up,
-                                           arg_list=new_arg_list,
-                                           set_context=set_context,
-                                           tenant_id=tenant_id,
-                                           **kwargs)
+        return super()._create_network(fmt, name, admin_state_up,
+                                       arg_list=new_arg_list,
+                                       set_context=set_context,
+                                       tenant_id=tenant_id,
+                                       **kwargs)
 
     def _create_port(self, fmt, net_id, expected_res_status=None,
                      arg_list=None, set_context=False, tenant_id=None,
@@ -84,16 +85,16 @@ class DnsExtensionTestCase(test_plugin.Ml2PluginV2TestCase):
 
         for arg in (('admin_state_up', 'device_id',
                     'mac_address', 'name', 'fixed_ips',
-                    'tenant_id', 'device_owner', 'security_groups',
-                    'dns_name') + (arg_list or ())):
+                     'tenant_id', 'device_owner', 'security_groups',
+                     'dns_name') + (arg_list or ())):
             # Arg must be present
             if arg in kwargs:
                 data['port'][arg] = kwargs[arg]
         # create a dhcp port device id if one hasn't been supplied
         if ('device_owner' in kwargs and
-            kwargs['device_owner'] == constants.DEVICE_OWNER_DHCP and
-            'host' in kwargs and
-            'device_id' not in kwargs):
+                kwargs['device_owner'] == constants.DEVICE_OWNER_DHCP and
+                'host' in kwargs and
+                'device_id' not in kwargs):
             device_id = utils.get_dhcp_agent_device_id(net_id, kwargs['host'])
             data['port']['device_id'] = device_id
         port_req = self.new_create_request('ports', data, fmt)
@@ -107,13 +108,11 @@ class DnsExtensionTestCase(test_plugin.Ml2PluginV2TestCase):
             self.assertEqual(expected_res_status, port_res.status_int)
         return port_res
 
-    def _test_list_resources(self, resource, items, neutron_context=None,
-                             query_params=None):
+    def _test_list_resources(self, resource, items, query_params=None):
         res = self._list('%ss' % resource,
-                         neutron_context=neutron_context,
                          query_params=query_params)
         resource = resource.replace('-', '_')
-        self.assertItemsEqual([i['id'] for i in res['%ss' % resource]],
+        self.assertCountEqual([i['id'] for i in res['%ss' % resource]],
                               [i[resource]['id'] for i in items])
         return res
 
@@ -125,7 +124,7 @@ class DnsExtensionTestCase(test_plugin.Ml2PluginV2TestCase):
             self.assertIn('mac_address', port['port'])
             ips = port['port']['fixed_ips']
             self.assertEqual(1, len(ips))
-            subnet_db = manager.NeutronManager.get_plugin().get_subnet(
+            subnet_db = directory.get_plugin().get_subnet(
                     context.get_admin_context(), ips[0]['subnet_id'])
             self.assertIn(netaddr.IPAddress(ips[0]['ip_address']),
                           netaddr.IPSet(netaddr.IPNetwork(subnet_db['cidr'])))
@@ -134,8 +133,6 @@ class DnsExtensionTestCase(test_plugin.Ml2PluginV2TestCase):
                                        ips_list=[ips[0]['ip_address']])
 
     def test_list_ports(self):
-        # for this test we need to enable overlapping ips
-        cfg.CONF.set_default('allow_overlapping_ips', True)
         with self.port() as v1, self.port() as v2, self.port() as v3:
             ports = (v1, v2, v3)
             res = self._test_list_resources('port', ports)
@@ -234,7 +231,7 @@ class DnsExtensionTestCase(test_plugin.Ml2PluginV2TestCase):
             request_dns_name = dns_name
             request_fqdn = request_dns_name
             if not request_dns_name.endswith('.'):
-                request_fqdn = '%s.%s' % (dns_name, dns_domain)
+                request_fqdn = '{}.{}'.format(dns_name, dns_domain)
         return request_dns_name, request_fqdn
 
     def _get_hostname_and_fqdn(self, request_dns_name, request_fqdn,
@@ -248,7 +245,7 @@ class DnsExtensionTestCase(test_plugin.Ml2PluginV2TestCase):
                 '.', '-').replace(':', '-')
             fqdn = hostname
             if dns_domain:
-                fqdn = '%s.%s' % (hostname, dns_domain)
+                fqdn = '{}.{}'.format(hostname, dns_domain)
         return hostname, fqdn
 
     def _verify_ip_in_subnet(self, ip, subnets_list):
@@ -309,47 +306,46 @@ class DnsExtensionTestCase(test_plugin.Ml2PluginV2TestCase):
         self.assertEqual(201, res.status_code)
 
     def test_create_port_multiple_v4_v6_subnets_pqdn_and_dns_domain_no_period(
-        self):
+            self):
         cfg.CONF.set_override('dns_domain', 'example.com')
         res = self._test_create_port_with_multiple_ipv4_and_ipv6_subnets(
             dns_name='vm1')
         self.assertEqual(201, res.status_code)
 
     def test_create_port_multiple_v4_v6_subnets_pqdn_and_dns_domain_period(
-        self):
+            self):
         cfg.CONF.set_override('dns_domain', 'example.com.')
         res = self._test_create_port_with_multiple_ipv4_and_ipv6_subnets(
             dns_name='vm1')
         self.assertEqual(201, res.status_code)
 
     def test_create_port_multiple_v4_v6_subnets_pqdn_and_no_dns_domain(
-        self):
+            self):
         cfg.CONF.set_override('dns_domain', '')
         res = self._test_create_port_with_multiple_ipv4_and_ipv6_subnets()
         self.assertEqual(201, res.status_code)
 
     def test_create_port_multiple_v4_v6_subnets_fqdn_and_dns_domain_no_period(
-        self):
+            self):
         cfg.CONF.set_override('dns_domain', 'example.com')
         res = self._test_create_port_with_multiple_ipv4_and_ipv6_subnets(
             dns_name='vm1.example.com.')
         self.assertEqual(201, res.status_code)
 
     def test_create_port_multiple_v4_v6_subnets_fqdn_and_dns_domain_period(
-        self):
+            self):
         cfg.CONF.set_override('dns_domain', 'example.com.')
         res = self._test_create_port_with_multiple_ipv4_and_ipv6_subnets(
             dns_name='vm1.example.com.')
         self.assertEqual(201, res.status_code)
 
     def test_create_port_multiple_v4_v6_subnets_fqdn_default_domain_period(
-        self):
+            self):
         cfg.CONF.set_override('dns_domain', 'openstacklocal.')
         res = self._test_create_port_with_multiple_ipv4_and_ipv6_subnets()
         self.assertEqual(201, res.status_code)
 
-    def test_create_port_multiple_v4_v6_subnets_bad_fqdn_and_dns_domain(
-        self):
+    def test_create_port_multiple_v4_v6_subnets_bad_fqdn_and_dns_domain(self):
         cfg.CONF.set_override('dns_domain', 'example.com')
         res = self._test_create_port_with_multiple_ipv4_and_ipv6_subnets(
             dns_name='vm1.bad-domain.com.')
@@ -359,14 +355,13 @@ class DnsExtensionTestCase(test_plugin.Ml2PluginV2TestCase):
                           'neutron.conf')
         self.assertIn(expected_error, res.text)
 
-    def test_create_port_multiple_v4_v6_subnets_bad_pqdn_and_dns_domain(
-        self):
+    def test_create_port_multiple_v4_v6_subnets_bad_pqdn_and_dns_domain(self):
         cfg.CONF.set_override('dns_domain', 'example.com')
         num_labels = int(
-            math.floor(dns.FQDN_MAX_LEN / dns.DNS_LABEL_MAX_LEN))
+            math.floor(db_const.FQDN_FIELD_SIZE / constants.DNS_LABEL_MAX_LEN))
         filler_len = int(
-            math.floor(dns.FQDN_MAX_LEN % dns.DNS_LABEL_MAX_LEN))
-        dns_name = (('a' * (dns.DNS_LABEL_MAX_LEN - 1) + '.') *
+            math.floor(db_const.FQDN_FIELD_SIZE % constants.DNS_LABEL_MAX_LEN))
+        dns_name = (('a' * (constants.DNS_LABEL_MAX_LEN - 1) + '.') *
                     num_labels + 'a' * filler_len)
         res = self._test_create_port_with_multiple_ipv4_and_ipv6_subnets(
             dns_name=dns_name)
@@ -384,17 +379,21 @@ class DnsExtensionTestCase(test_plugin.Ml2PluginV2TestCase):
         network = self.deserialize(self.fmt, res)
         sub_dicts = [
             {'gateway': '10.0.0.1', 'cidr': '10.0.0.0/24',
-             'ip_version': 4, 'ra_addr_mode': None},
+             'ip_version': constants.IP_VERSION_4, 'ra_addr_mode': None},
             {'gateway': '10.0.1.1', 'cidr': '10.0.1.0/24',
-             'ip_version': 4, 'ra_addr_mode': None},
+             'ip_version': constants.IP_VERSION_4, 'ra_addr_mode': None},
             {'gateway': 'fe80::1', 'cidr': 'fe80::/64',
-             'ip_version': 6, 'ra_addr_mode': constants.IPV6_SLAAC},
+             'ip_version': constants.IP_VERSION_6,
+             'ra_addr_mode': constants.IPV6_SLAAC},
             {'gateway': 'fe81::1', 'cidr': 'fe81::/64',
-             'ip_version': 6, 'ra_addr_mode': constants.IPV6_SLAAC},
+             'ip_version': constants.IP_VERSION_6,
+             'ra_addr_mode': constants.IPV6_SLAAC},
             {'gateway': 'fe82::1', 'cidr': 'fe82::/64',
-             'ip_version': 6, 'ra_addr_mode': constants.DHCPV6_STATEFUL},
+             'ip_version': constants.IP_VERSION_6,
+             'ra_addr_mode': constants.DHCPV6_STATEFUL},
             {'gateway': 'fe83::1', 'cidr': 'fe83::/64',
-             'ip_version': 6, 'ra_addr_mode': constants.DHCPV6_STATEFUL}]
+             'ip_version': constants.IP_VERSION_6,
+             'ra_addr_mode': constants.DHCPV6_STATEFUL}]
         subnets = {}
         for sub_dict in sub_dicts:
             subnet = self._make_subnet(
@@ -437,21 +436,21 @@ class DnsExtensionTestCase(test_plugin.Ml2PluginV2TestCase):
 
     def test_api_extension_validation_with_bad_dns_names(self):
         num_labels = int(
-            math.floor(dns.FQDN_MAX_LEN / dns.DNS_LABEL_MAX_LEN))
+            math.floor(db_const.FQDN_FIELD_SIZE / constants.DNS_LABEL_MAX_LEN))
         filler_len = int(
-            math.floor(dns.FQDN_MAX_LEN % dns.DNS_LABEL_MAX_LEN))
+            math.floor(db_const.FQDN_FIELD_SIZE % constants.DNS_LABEL_MAX_LEN))
         dns_names = [555, '\f\n\r', '.', '-vm01', '_vm01', 'vm01-',
-                    '-vm01.test1', 'vm01.-test1', 'vm01._test1',
-                    'vm01.test1-', 'vm01.te$t1', 'vm0#1.test1.',
-                    'vm01.123.', '-' + 'a' * dns.DNS_LABEL_MAX_LEN,
-                    'a' * (dns.DNS_LABEL_MAX_LEN + 1),
-                    ('a' * (dns.DNS_LABEL_MAX_LEN - 1) + '.') *
-                    num_labels + 'a' * (filler_len + 1)]
+                     '-vm01.test1', 'vm01.-test1', 'vm01._test1',
+                     'vm01.test1-', 'vm01.te$t1', 'vm0#1.test1.',
+                     'vm01.123.', '-' + 'a' * constants.DNS_LABEL_MAX_LEN,
+                     'a' * (constants.DNS_LABEL_MAX_LEN + 1),
+                     ('a' * (constants.DNS_LABEL_MAX_LEN - 1) + '.') *
+                     num_labels + 'a' * (filler_len + 1)]
         res = self._create_network(fmt=self.fmt, name='net',
                                    admin_state_up=True)
         network = self.deserialize(self.fmt, res)
         sub_dict = {'gateway': '10.0.0.1', 'cidr': '10.0.0.0/24',
-                    'ip_version': 4, 'ra_addr_mode': None}
+                    'ip_version': constants.IP_VERSION_4, 'ra_addr_mode': None}
         self._make_subnet(self.fmt, network, gateway=sub_dict['gateway'],
                           cidr=sub_dict['cidr'],
                           ip_version=sub_dict['ip_version'],
@@ -464,31 +463,32 @@ class DnsExtensionTestCase(test_plugin.Ml2PluginV2TestCase):
             error_message = res.json['NeutronError']['message']
             is_expected_message = (
                 'cannot be converted to lowercase string' in error_message or
-                'not a valid PQDN or FQDN. Reason:' in error_message)
+                'not a valid PQDN or FQDN. Reason:' in error_message or
+                'must be string type' in error_message)
             self.assertTrue(is_expected_message)
 
     def test_api_extension_validation_with_good_dns_names(self):
         cfg.CONF.set_override('dns_domain', 'example.com')
         higher_labels_len = len('example.com.')
         num_labels = int(
-            math.floor((dns.FQDN_MAX_LEN - higher_labels_len) /
-                       dns.DNS_LABEL_MAX_LEN))
+            math.floor((db_const.FQDN_FIELD_SIZE - higher_labels_len) /
+                       constants.DNS_LABEL_MAX_LEN))
         filler_len = int(
-            math.floor((dns.FQDN_MAX_LEN - higher_labels_len) %
-                       dns.DNS_LABEL_MAX_LEN))
+            math.floor((db_const.FQDN_FIELD_SIZE - higher_labels_len) %
+                       constants.DNS_LABEL_MAX_LEN))
         dns_names = ['', 'www.1000.com', 'vM01', 'vm01.example.com.',
                      '8vm01', 'vm-01.example.com.', 'vm01.test',
                      'vm01.test.example.com.', 'vm01.test-100',
                      'vm01.test-100.example.com.',
-                     'a' * dns.DNS_LABEL_MAX_LEN,
-                     ('a' * dns.DNS_LABEL_MAX_LEN) + '.example.com.',
-                     ('a' * (dns.DNS_LABEL_MAX_LEN - 1) + '.') *
+                     'a' * constants.DNS_LABEL_MAX_LEN,
+                     ('a' * constants.DNS_LABEL_MAX_LEN) + '.example.com.',
+                     ('a' * (constants.DNS_LABEL_MAX_LEN - 1) + '.') *
                      num_labels + 'a' * (filler_len - 1)]
         res = self._create_network(fmt=self.fmt, name='net',
                                    admin_state_up=True)
         network = self.deserialize(self.fmt, res)
         sub_dict = {'gateway': '10.0.0.1', 'cidr': '10.0.0.0/24',
-                    'ip_version': 4, 'ra_addr_mode': None}
+                    'ip_version': constants.IP_VERSION_4, 'ra_addr_mode': None}
         self._make_subnet(self.fmt, network, gateway=sub_dict['gateway'],
                           cidr=sub_dict['cidr'],
                           ip_version=sub_dict['ip_version'],
@@ -501,12 +501,12 @@ class DnsExtensionTestCase(test_plugin.Ml2PluginV2TestCase):
 
 
 class DnsExtensionTestNetworkDnsDomain(
-    test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
+        test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
     def setUp(self):
         plugin = ('neutron.tests.unit.extensions.test_dns.' +
                   'DnsExtensionTestPlugin')
         ext_mgr = DnsExtensionManager()
-        super(DnsExtensionTestNetworkDnsDomain, self).setUp(
+        super().setUp(
             plugin=plugin, ext_mgr=ext_mgr)
 
     def test_update_network_dns_domain(self):
