@@ -18,7 +18,6 @@ from unittest import mock
 
 from neutron_lib.api.definitions import port_security as psec
 from neutron_lib.api import validators
-from neutron_lib import context
 from neutron_lib.db import api as db_api
 from neutron_lib.db import utils as db_utils
 from neutron_lib.exceptions import port_security as psec_exc
@@ -31,7 +30,7 @@ from neutron.db import portsecurity_db
 from neutron.db import securitygroups_db
 from neutron.extensions import securitygroup as ext_sg
 from neutron import quota
-from neutron.tests.unit.db import test_db_base_plugin_v2
+from neutron.tests.common import test_db_base_plugin_v2
 from neutron.tests.unit.extensions import test_securitygroup
 
 DB_PLUGIN_KLASS = ('neutron.tests.unit.extensions.test_portsecurity.'
@@ -46,7 +45,7 @@ class PortSecurityTestCase(test_securitygroup.SecurityGroupsTestCase,
         self.addCleanup(self._restore)
         ext_mgr = (
             test_securitygroup.SecurityGroupTestExtensionManager())
-        super(PortSecurityTestCase, self).setUp(plugin=plugin, ext_mgr=ext_mgr)
+        super().setUp(plugin=plugin, ext_mgr=ext_mgr)
 
         # Check if a plugin supports security groups
         plugin_obj = directory.get_plugin()
@@ -71,7 +70,7 @@ class PortSecurityTestPlugin(db_base_plugin_v2.NeutronDbPluginV2,
         with db_api.CONTEXT_WRITER.using(context):
             tenant_id = network['network'].get('tenant_id')
             self._ensure_default_security_group(context, tenant_id)
-            neutron_db = super(PortSecurityTestPlugin, self).create_network(
+            neutron_db = super().create_network(
                 context, network)
             neutron_db.update(network['network'])
             self._process_network_port_security_create(
@@ -80,7 +79,7 @@ class PortSecurityTestPlugin(db_base_plugin_v2.NeutronDbPluginV2,
 
     def update_network(self, context, id, network):
         with db_api.CONTEXT_WRITER.using(context):
-            neutron_db = super(PortSecurityTestPlugin, self).update_network(
+            neutron_db = super().update_network(
                 context, id, network)
             if psec.PORTSECURITY in network['network']:
                 self._process_network_port_security_update(
@@ -89,13 +88,13 @@ class PortSecurityTestPlugin(db_base_plugin_v2.NeutronDbPluginV2,
 
     def get_network(self, context, id, fields=None):
         with db_api.CONTEXT_READER.using(context):
-            net = super(PortSecurityTestPlugin, self).get_network(
+            net = super().get_network(
                 context, id)
         return db_utils.resource_fields(net, fields)
 
     def create_port(self, context, port):
         p = port['port']
-        neutron_db = super(PortSecurityTestPlugin, self).create_port(
+        neutron_db = super().create_port(
             context, port)
         p.update(neutron_db)
 
@@ -126,7 +125,7 @@ class PortSecurityTestPlugin(db_base_plugin_v2.NeutronDbPluginV2,
             port)
         has_security_groups = self._check_update_has_security_groups(port)
         with db_api.CONTEXT_WRITER.using(context):
-            ret_port = super(PortSecurityTestPlugin, self).update_port(
+            ret_port = super().update_port(
                 context, id, port)
             # copy values over - but not fixed_ips
             port['port'].pop('fixed_ips', None)
@@ -151,7 +150,7 @@ class PortSecurityTestPlugin(db_base_plugin_v2.NeutronDbPluginV2,
 
                 # get security groups on port
                 filters = {'port_id': [id]}
-                security_groups = (super(PortSecurityTestPlugin, self).
+                security_groups = (super().
                                    _get_port_security_group_bindings(
                                        context, filters))
                 if security_groups and not delete_security_groups:
@@ -176,7 +175,7 @@ class PortSecurityTestPlugin(db_base_plugin_v2.NeutronDbPluginV2,
 class PortSecurityDBTestCase(PortSecurityTestCase):
     def setUp(self, plugin=None, service_plugins=None):
         plugin = plugin or DB_PLUGIN_KLASS
-        super(PortSecurityDBTestCase, self).setUp(plugin)
+        super().setUp(plugin)
 
 
 class TestPortSecurity(PortSecurityDBTestCase):
@@ -298,7 +297,7 @@ class TestPortSecurity(PortSecurityDBTestCase):
         security_group_id = security_group['security_group']['id']
         res = self._create_port('json', net['network']['id'],
                                 arg_list=('security_groups',
-                                    'port_security_enabled'),
+                                          'port_security_enabled'),
                                 port_security_enabled=True,
                                 security_groups=[security_group_id])
         port = self.deserialize('json', res)
@@ -311,19 +310,18 @@ class TestPortSecurity(PortSecurityDBTestCase):
             self.skipTest("Plugin does not support security groups")
         res = self._create_network('json', 'net1', True,
                                    arg_list=('port_security_enabled',),
-                                   set_context=True,
                                    tenant_id='admin_tenant',
                                    port_security_enabled=False)
         net = self.deserialize('json', res)
-        self._create_subnet('json', net['network']['id'], '10.0.0.0/24')
+        self._create_subnet('json', net['network']['id'], '10.0.0.0/24',
+                            tenant_id='admin_tenant')
         security_group = self.deserialize(
             'json', self._create_security_group(self.fmt, 'asdf', 'asdf',
                                                 tenant_id='other_tenant'))
         security_group_id = security_group['security_group']['id']
         res = self._create_port('json', net['network']['id'],
                                 arg_list=('security_groups',
-                                    'port_security_enabled'),
-                                set_context=True,
+                                          'port_security_enabled'),
                                 is_admin=True,
                                 tenant_id='admin_tenant',
                                 port_security_enabled=True,
@@ -331,27 +329,25 @@ class TestPortSecurity(PortSecurityDBTestCase):
         port = self.deserialize('json', res)
         self.assertTrue(port['port'][psec.PORTSECURITY])
         self.assertEqual(port['port']['security_groups'], [security_group_id])
-        self._delete('ports', port['port']['id'])
+        self._delete('ports', port['port']['id'], tenant_id='admin_tenant')
 
     def test_create_port_with_no_admin_use_other_tenant_security_group(self):
         if self._skip_security_group:
             self.skipTest("Plugin does not support security groups")
         res = self._create_network('json', 'net1', True,
                                    arg_list=('port_security_enabled',),
-                                   set_context=True,
                                    tenant_id='demo_tenant',
                                    port_security_enabled=False)
         net = self.deserialize('json', res)
         self._create_subnet('json', net['network']['id'], '10.0.0.0/24',
-                            set_context=True, tenant_id='demo_tenant')
+                            tenant_id='demo_tenant')
         security_group = self.deserialize(
             'json', self._create_security_group(self.fmt, 'asdf', 'asdf',
                                                 tenant_id='other_tenant'))
         security_group_id = security_group['security_group']['id']
         res = self._create_port('json', net['network']['id'],
                                 arg_list=('security_groups',
-                                    'port_security_enabled'),
-                                set_context=True,
+                                          'port_security_enabled'),
                                 tenant_id='demo_tenant',
                                 port_security_enabled=True,
                                 security_groups=[security_group_id])
@@ -396,19 +392,22 @@ class TestPortSecurity(PortSecurityDBTestCase):
         with self.network() as net:
             with self.subnet(network=net):
                 res = self._create_port('json', net['network']['id'],
-                                        set_context=True, is_admin=True,
+                                        is_admin=True,
                                         tenant_id='admin_tenant',)
                 port = self.deserialize('json', res)
                 self.assertTrue(port['port'][psec.PORTSECURITY])
 
-                security_group = self.deserialize('json',
+                security_group = self.deserialize(
+                    'json',
                     self._create_security_group(self.fmt, 'asdf', 'asdf',
-                                                tenant_id='other_tenant'))
+                    tenant_id='other_tenant'))
                 security_group_id = security_group['security_group']['id']
                 update_port = {'port':
                                {'security_groups': [security_group_id]}}
                 req = self.new_update_request('ports', update_port,
-                                              port['port']['id'])
+                                              port['port']['id'],
+                                              tenant_id='admin_tenant',
+                                              as_admin=True)
                 port = self.deserialize('json', req.get_response(self.api))
                 security_groups = port['port']['security_groups']
                 self.assertIn(security_group_id, security_groups)
@@ -420,21 +419,20 @@ class TestPortSecurity(PortSecurityDBTestCase):
         with self.network(tenant_id='demo_tenant') as net:
             with self.subnet(network=net, tenant_id='demo_tenant'):
                 res = self._create_port('json', net['network']['id'],
-                                        set_context=True,
                                         tenant_id='demo_tenant',)
                 port = self.deserialize('json', res)
                 self.assertTrue(port['port'][psec.PORTSECURITY])
 
-                security_group = self.deserialize('json',
+                security_group = self.deserialize(
+                    'json',
                     self._create_security_group(self.fmt, 'asdf', 'asdf',
-                                                tenant_id='other_tenant'))
+                    tenant_id='other_tenant'))
                 security_group_id = security_group['security_group']['id']
                 update_port = {'port':
                                {'security_groups': [security_group_id]}}
                 req = self.new_update_request('ports', update_port,
-                                              port['port']['id'])
-                req.environ['neutron.context'] = context.Context(
-                    '', 'other_tenant')
+                                              port['port']['id'],
+                                              tenant_id='other_tenant')
                 res = req.get_response(self.api)
                 self.assertEqual(404, res.status_int)
 
@@ -490,29 +488,26 @@ class TestPortSecurity(PortSecurityDBTestCase):
                 self._delete('ports', port['port']['id'])
 
     def test_create_port_security_off_shared_network(self):
-        with self.network(shared=True) as net:
+        with self.network(as_admin=True, shared=True) as net:
             with self.subnet(network=net):
                 res = self._create_port('json', net['network']['id'],
                                         arg_list=('port_security_enabled',),
                                         port_security_enabled=False,
-                                        tenant_id='not_network_owner',
-                                        set_context=True)
+                                        tenant_id='not_network_owner')
                 self.deserialize('json', res)
                 self.assertEqual(403, res.status_int)
 
     def test_update_port_security_off_shared_network(self):
-        with self.network(shared=True) as net:
+        with self.network(as_admin=True, shared=True) as net:
             with self.subnet(network=net):
                 res = self._create_port('json', net['network']['id'],
-                                        tenant_id='not_network_owner',
-                                        set_context=True)
+                                        tenant_id='not_network_owner')
                 port = self.deserialize('json', res)
                 # remove security group on port
                 update_port = {'port': {ext_sg.SECURITYGROUPS: None,
                                         psec.PORTSECURITY: False}}
                 req = self.new_update_request('ports', update_port,
-                                              port['port']['id'])
-                req.environ['neutron.context'] = context.Context(
-                    '', 'not_network_owner')
+                                              port['port']['id'],
+                                              tenant_id='not_network_owner')
                 res = req.get_response(self.api)
                 self.assertEqual(exc.HTTPForbidden.code, res.status_int)
