@@ -13,19 +13,20 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
+from unittest import mock
+
 import netaddr
 from neutron_lib import constants
+from neutron_lib import context
 from neutron_lib import exceptions as n_exc
+from neutron_lib.plugins import directory
+from oslo_utils import uuidutils
 
-from neutron.common import constants as n_const
-from neutron import context
-from neutron.ipam.drivers.neutrondb_ipam import db_models
 from neutron.ipam.drivers.neutrondb_ipam import driver
 from neutron.ipam import exceptions as ipam_exc
 from neutron.ipam import requests as ipam_req
-from neutron import manager
-from neutron.tests.unit.db import test_db_base_plugin_v2 as test_db_plugin
+from neutron.objects import ipam as ipam_obj
+from neutron.tests.common import test_db_base_plugin_v2 as test_db_plugin
 from neutron.tests.unit import testlib_api
 
 
@@ -33,7 +34,7 @@ def convert_firstip_to_ipaddress(range_item):
     return netaddr.IPAddress(range_item['first_ip'])
 
 
-class TestNeutronDbIpamMixin(object):
+class TestNeutronDbIpamMixin:
 
     def _create_network(self, plugin, ctx, shared=False):
         network = {'network': {'name': 'net',
@@ -43,7 +44,8 @@ class TestNeutronDbIpamMixin(object):
         created_network = plugin.create_network(ctx, network)
         return (created_network, created_network['id'])
 
-    def _create_subnet(self, plugin, ctx, network_id, cidr, ip_version=4,
+    def _create_subnet(self, plugin, ctx, network_id, cidr,
+                       ip_version=constants.IP_VERSION_4,
                        v6_address_mode=constants.ATTR_NOT_SPECIFIED,
                        allocation_pools=constants.ATTR_NOT_SPECIFIED):
         subnet = {'subnet': {'name': 'sub',
@@ -66,14 +68,14 @@ class TestNeutronDbIpamPool(testlib_api.SqlTestCase,
     """Test case for the Neutron's DB IPAM driver subnet pool interface."""
 
     def setUp(self):
-        super(TestNeutronDbIpamPool, self).setUp()
+        super().setUp()
         self._tenant_id = 'test-tenant'
 
         # Configure plugin for tests
         self.setup_coreplugin(test_db_plugin.DB_PLUGIN_KLASS)
 
         # Prepare environment for tests
-        self.plugin = manager.NeutronManager.get_plugin()
+        self.plugin = directory.get_plugin()
         self.ctx = context.get_admin_context()
         self.network, self.net_id = self._create_network(self.plugin,
                                                          self.ctx)
@@ -206,9 +208,10 @@ class TestNeutronDbIpamPool(testlib_api.SqlTestCase,
 
     def test_get_details_for_invalid_subnet_id_fails(self):
         cidr = '10.0.0.0/24'
+        non_existent_id = uuidutils.generate_uuid()
         subnet_req = ipam_req.SpecificSubnetRequest(
             self._tenant_id,
-            'non-existent-id',
+            non_existent_id,
             cidr)
         self.ipam_pool.allocate_subnet(subnet_req)
         # Neutron subnet does not exist, so get_subnet should fail
@@ -229,14 +232,15 @@ class TestNeutronDbIpamSubnet(testlib_api.SqlTestCase,
     """
 
     def _create_and_allocate_ipam_subnet(
-        self, cidr, allocation_pools=constants.ATTR_NOT_SPECIFIED,
-        ip_version=4, v6_auto_address=False, tenant_id=None):
+            self, cidr, allocation_pools=constants.ATTR_NOT_SPECIFIED,
+            ip_version=constants.IP_VERSION_4, v6_auto_address=False,
+            tenant_id=None):
         v6_address_mode = constants.ATTR_NOT_SPECIFIED
         if v6_auto_address:
             # set ip version to 6 regardless of what's been passed to the
             # method
-            ip_version = 6
-            v6_address_mode = n_const.IPV6_SLAAC
+            ip_version = constants.IP_VERSION_6
+            v6_address_mode = constants.IPV6_SLAAC
         subnet = self._create_subnet(
             self.plugin, self.ctx, self.net_id, cidr,
             ip_version=ip_version,
@@ -257,14 +261,14 @@ class TestNeutronDbIpamSubnet(testlib_api.SqlTestCase,
         return ipam_subnet, subnet
 
     def setUp(self):
-        super(TestNeutronDbIpamSubnet, self).setUp()
+        super().setUp()
         self._tenant_id = 'test-tenant'
 
         # Configure plugin for tests
         self.setup_coreplugin(test_db_plugin.DB_PLUGIN_KLASS)
 
         # Prepare environment for tests
-        self.plugin = manager.NeutronManager.get_plugin()
+        self.plugin = directory.get_plugin()
         self.ctx = context.get_admin_context()
         self.network, self.net_id = self._create_network(self.plugin,
                                                          self.ctx)
@@ -275,14 +279,14 @@ class TestNeutronDbIpamSubnet(testlib_api.SqlTestCase,
     def test__verify_ip_succeeds(self):
         cidr = '10.0.0.0/24'
         ipam_subnet = self._create_and_allocate_ipam_subnet(cidr)[0]
-        ipam_subnet._verify_ip(self.ctx.session, '10.0.0.2')
+        ipam_subnet._verify_ip(self.ctx, '10.0.0.2')
 
     def test__verify_ip_not_in_subnet_fails(self):
         cidr = '10.0.0.0/24'
         ipam_subnet = self._create_and_allocate_ipam_subnet(cidr)[0]
         self.assertRaises(ipam_exc.InvalidIpForSubnet,
                           ipam_subnet._verify_ip,
-                          self.ctx.session,
+                          self.ctx,
                           '192.168.0.2')
 
     def test__verify_ip_bcast_and_network_fail(self):
@@ -290,11 +294,11 @@ class TestNeutronDbIpamSubnet(testlib_api.SqlTestCase,
         ipam_subnet = self._create_and_allocate_ipam_subnet(cidr)[0]
         self.assertRaises(ipam_exc.InvalidIpForSubnet,
                           ipam_subnet._verify_ip,
-                          self.ctx.session,
+                          self.ctx,
                           '10.0.0.255')
         self.assertRaises(ipam_exc.InvalidIpForSubnet,
                           ipam_subnet._verify_ip,
-                          self.ctx.session,
+                          self.ctx,
                           '10.0.0.0')
 
     def _allocate_address(self, cidr, ip_version, address_request):
@@ -340,7 +344,7 @@ class TestNeutronDbIpamSubnet(testlib_api.SqlTestCase,
 
     def test_allocate_specific_address_in_use_fails(self):
         ipam_subnet = self._create_and_allocate_ipam_subnet(
-            'fde3:abcd:4321:1::/64', ip_version=6)[0]
+            'fde3:abcd:4321:1::/64', ip_version=constants.IP_VERSION_6)[0]
         addr_req = ipam_req.SpecificAddressRequest('fde3:abcd:4321:1::33')
         ipam_subnet.allocate(addr_req)
         self.assertRaises(ipam_exc.IpAddressAlreadyAllocated,
@@ -350,13 +354,60 @@ class TestNeutronDbIpamSubnet(testlib_api.SqlTestCase,
     def test_allocate_any_address_exhausted_pools_fails(self):
         # Same as above, the ranges will be recalculated always
         ipam_subnet = self._create_and_allocate_ipam_subnet(
-            '192.168.0.0/30', ip_version=4)[0]
+            '192.168.0.0/30', ip_version=constants.IP_VERSION_4)[0]
         ipam_subnet.allocate(ipam_req.AnyAddressRequest)
         # The second address generation request on a /30 for v4 net must fail
         self.assertRaises(ipam_exc.IpAddressGenerationFailure,
                           ipam_subnet.allocate,
                           ipam_req.AnyAddressRequest)
 
+    def test_bulk_allocate_v4_address(self):
+        target_ip_count = 10
+        ipam_subnet = self._create_and_allocate_ipam_subnet(
+            '192.168.0.0/28', ip_version=constants.IP_VERSION_4)[0]
+        ip_addresses = ipam_subnet.bulk_allocate(
+                ipam_req.BulkAddressRequest(target_ip_count))
+        self.assertEqual(target_ip_count, len(ip_addresses))
+        self.assertRaises(ipam_exc.IpAddressGenerationFailure,
+                          ipam_subnet.bulk_allocate,
+                          ipam_req.BulkAddressRequest(target_ip_count))
+
+    def test_bulk_allocate_v6_address(self):
+        target_ip_count = 10
+        ipam_subnet = self._create_and_allocate_ipam_subnet(
+                'fd00::/124', ip_version=constants.IP_VERSION_6)[0]
+        ip_addresses = ipam_subnet.bulk_allocate(
+                ipam_req.BulkAddressRequest(target_ip_count))
+        self.assertEqual(target_ip_count, len(ip_addresses))
+        self.assertRaises(ipam_exc.IpAddressGenerationFailure,
+                          ipam_subnet.bulk_allocate,
+                          ipam_req.BulkAddressRequest(target_ip_count))
+
+    def test_bulk_allocate_multiple_address_pools(self):
+        target_ip_count = 10
+        # 11 addresses available
+        allocation_pools = [{'start': '192.168.0.5', 'end': '192.168.0.9'},
+                            {'start': '192.168.0.15', 'end': '192.168.0.20'}]
+        ipam_subnet = self._create_and_allocate_ipam_subnet(
+            '192.168.0.0/24', allocation_pools=allocation_pools,
+            ip_version=constants.IP_VERSION_4)[0]
+        ip_addresses = ipam_subnet.bulk_allocate(
+            ipam_req.BulkAddressRequest(target_ip_count))
+        self.assertEqual(target_ip_count, len(ip_addresses))
+        self.assertRaises(ipam_exc.IpAddressGenerationFailure,
+                          ipam_subnet.bulk_allocate,
+                          ipam_req.BulkAddressRequest(2))
+
+    def test_prefernext_allocate_multiple_address_pools(self):
+        ipam_subnet = self._create_and_allocate_ipam_subnet(
+            '192.168.0.0/30', ip_version=constants.IP_VERSION_4)[0]
+
+        ipam_subnet.allocate(ipam_req.PreferNextAddressRequest())
+        # The second address generation request on a /30 for v4 net must fail
+        self.assertRaises(ipam_exc.IpAddressGenerationFailure,
+                          ipam_subnet.allocate,
+                          ipam_req.PreferNextAddressRequest)
+
     def _test_deallocate_address(self, cidr, ip_version):
         ipam_subnet = self._create_and_allocate_ipam_subnet(
             cidr, ip_version=ip_version)[0]
@@ -372,12 +423,6 @@ class TestNeutronDbIpamSubnet(testlib_api.SqlTestCase,
         # future proofing in case v6-specific logic will be added.
         self._test_deallocate_address('fde3:abcd:4321:1::/64', 6)
 
-    def test_allocate_unallocated_address_fails(self):
-        ipam_subnet = self._create_and_allocate_ipam_subnet(
-            '10.0.0.0/24', ip_version=4)[0]
-        self.assertRaises(ipam_exc.IpAddressAllocationNotFound,
-                          ipam_subnet.deallocate, '10.0.0.2')
-
     def test_allocate_all_pool_addresses_triggers_range_recalculation(self):
         # This test instead might be made to pass, but for the wrong reasons!
         pass
@@ -386,8 +431,10 @@ class TestNeutronDbIpamSubnet(testlib_api.SqlTestCase,
         # This test should pass because ipam subnet is no longer
         # have foreign key relationship with neutron subnet.
         # Creating ipam subnet before neutron subnet is a valid case.
+        tenant_id = uuidutils.generate_uuid()
+        subnet_id = uuidutils.generate_uuid()
         subnet_req = ipam_req.SpecificSubnetRequest(
-            'tenant_id', 'meh', '192.168.0.0/24')
+            tenant_id, subnet_id, '192.168.0.0/24')
         self.ipam_pool.allocate_subnet(subnet_req)
 
     def test_update_allocation_pools_with_no_pool_change(self):
@@ -405,17 +452,19 @@ class TestNeutronDbIpamSubnet(testlib_api.SqlTestCase,
         self.assertFalse(ipam_subnet.create_allocation_pools.called)
 
     def _test__no_pool_changes(self, new_pools):
-        id = 'some-id'
+        id = uuidutils.generate_uuid()
         ipam_subnet = driver.NeutronDbSubnet(id, self.ctx)
-        pools = [db_models.IpamAllocationPool(ipam_subnet_id=id,
-                                              first_ip='192.168.10.20',
-                                              last_ip='192.168.10.41'),
-                 db_models.IpamAllocationPool(ipam_subnet_id=id,
-                                              first_ip='192.168.10.50',
-                                              last_ip='192.168.10.60')]
+        pools = [ipam_obj.IpamAllocationPool(self.ctx,
+                                             ipam_subnet_id=id,
+                                             first_ip='192.168.10.20',
+                                             last_ip='192.168.10.41'),
+                 ipam_obj.IpamAllocationPool(self.ctx,
+                                             ipam_subnet_id=id,
+                                             first_ip='192.168.10.50',
+                                             last_ip='192.168.10.60')]
 
         ipam_subnet.subnet_manager.list_pools = mock.Mock(return_value=pools)
-        return ipam_subnet._no_pool_changes(self.ctx.session, new_pools)
+        return ipam_subnet._no_pool_changes(self.ctx, new_pools)
 
     def test__no_pool_changes_negative(self):
         pool_list = [[netaddr.IPRange('192.168.10.2', '192.168.10.254')],
