@@ -10,8 +10,6 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import itertools
-
 from neutron.objects.port.extensions import extra_dhcp_opt
 from neutron.tests.unit.objects import test_base as obj_test_base
 from neutron.tests.unit import testlib_api
@@ -28,8 +26,6 @@ class ExtraDhcpOptDbObjectTestCase(obj_test_base.BaseDbObjectTestCase,
     _test_class = extra_dhcp_opt.ExtraDhcpOpt
 
     def setUp(self):
-        super(ExtraDhcpOptDbObjectTestCase, self).setUp()
-        self._create_test_network()
-        self._create_test_port(self._network)
-        for obj in itertools.chain(self.db_objs, self.obj_fields, self.objs):
-            obj['port_id'] = self._port['id']
+        super().setUp()
+        self.update_obj_fields(
+            {'port_id': lambda: self._create_test_port_id()})
