@@ -41,7 +41,7 @@ class SecurityGroupPortBindingDbObjectTestCase(
 class BasePortBindingDbObjectTestCase(obj_test_base._BaseObjectTestCase,
                                       testlib_api.SqlTestCase):
     def setUp(self):
-        super(BasePortBindingDbObjectTestCase, self).setUp()
+        super().setUp()
         self.update_obj_fields(
             {'port_id': lambda: self._create_test_port_id()})
 
@@ -56,14 +56,37 @@ class PortBindingDbObjectTestCase(obj_test_base.BaseDbObjectTestCase,
 
     def test_get_duplicated_port_bindings(self):
         port_id = self._create_test_port_id()
-        self.update_obj_fields({'port_id': port_id},
-                               objs=[self.objs[0], self.objs[1]])
+        self.update_obj_fields(
+            {'port_id': port_id, 'status': constants.ACTIVE},
+            obj_fields=[self.obj_fields[0]])
+        self.update_obj_fields(
+            {'port_id': port_id, 'status': constants.INACTIVE},
+            obj_fields=[self.obj_fields[1]])
         for i in range(3):
             _obj = self._make_object(self.obj_fields[i])
             _obj.create()
         dup_pb = ports.PortBinding.get_duplicated_port_bindings(self.context)
         self.assertEqual(1, len(dup_pb))
         self.assertEqual(port_id, dup_pb[0].port_id)
+        # The PB register returned is the INACTIVE one.
+        self.assertEqual(self.obj_fields[1]['host'], dup_pb[0].host)
+
+    def test_get_port_binding_by_vnic_type(self):
+        self.update_obj_fields({'vnic_type': portbindings.VNIC_NORMAL},
+                               obj_fields=[self.obj_fields[0]])
+        self.update_obj_fields({'vnic_type': portbindings.VNIC_DIRECT},
+                               obj_fields=[self.obj_fields[1],
+                                           self.obj_fields[2]])
+        for i in range(3):
+            _obj = self._make_object(self.obj_fields[i])
+            _obj.create()
+
+        for vnic_type, pb_num in [(portbindings.VNIC_NORMAL, 1),
+                                  (portbindings.VNIC_DIRECT, 2),
+                                  (portbindings.VNIC_MACVTAP, 0)]:
+            pb = ports.PortBinding.get_port_binding_by_vnic_type(self.context,
+                                                                 vnic_type)
+            self.assertEqual(pb_num, len(pb))
 
 
 class DistributedPortBindingIfaceObjTestCase(
@@ -93,7 +116,7 @@ class PortBindingVifDetailsTestCase(testscenarios.WithScenarios,
     ]
 
     def setUp(self):
-        super(PortBindingVifDetailsTestCase, self).setUp()
+        super().setUp()
         self._create_test_network()
         getter = lambda: self._create_port(network_id=self._network['id']).id
         self.update_obj_fields({'port_id': getter})
@@ -178,7 +201,7 @@ class IPAllocationDbObjectTestCase(obj_test_base.BaseDbObjectTestCase,
     _test_class = ports.IPAllocation
 
     def setUp(self):
-        super(IPAllocationDbObjectTestCase, self).setUp()
+        super().setUp()
         network_id = self._create_test_network_id()
         port_id = self._create_test_port_id(network_id=network_id)
         self.update_obj_fields(
@@ -197,7 +220,7 @@ class PortDNSDbObjectTestCase(obj_test_base.BaseDbObjectTestCase,
     _test_class = ports.PortDNS
 
     def setUp(self):
-        super(PortDNSDbObjectTestCase, self).setUp()
+        super().setUp()
         self.update_obj_fields(
             {'port_id': lambda: self._create_test_port_id()})
 
@@ -208,7 +231,7 @@ class PortBindingLevelIfaceObjTestCase(
     _test_class = ports.PortBindingLevel
 
     def setUp(self):
-        super(PortBindingLevelIfaceObjTestCase, self).setUp()
+        super().setUp()
         self.pager_map[self._test_class.obj_name()] = (
             obj_base.Pager(sorts=[('port_id', True), ('level', True)]))
 
@@ -219,7 +242,7 @@ class PortBindingLevelDbObjectTestCase(
     _test_class = ports.PortBindingLevel
 
     def setUp(self):
-        super(PortBindingLevelDbObjectTestCase, self).setUp()
+        super().setUp()
         self.update_obj_fields(
             {'port_id': lambda: self._create_test_port_id(),
              'segment_id': lambda: self._create_test_segment_id()})
@@ -230,7 +253,7 @@ class PortIfaceObjTestCase(obj_test_base.BaseObjectIfaceTestCase):
     _test_class = ports.Port
 
     def setUp(self):
-        super(PortIfaceObjTestCase, self).setUp()
+        super().setUp()
         self.pager_map[ports.PortBindingLevel.obj_name()] = (
             obj_base.Pager(sorts=[('port_id', True), ('level', True)]))
 
@@ -241,7 +264,7 @@ class PortDbObjectTestCase(obj_test_base.BaseDbObjectTestCase,
     _test_class = ports.Port
 
     def setUp(self):
-        super(PortDbObjectTestCase, self).setUp()
+        super().setUp()
         network_id = self._create_test_network_id()
         segment_id = self._create_test_segment_id(network_id)
         subnet_id = self._create_test_subnet_id(network_id)
@@ -506,6 +529,18 @@ class PortDbObjectTestCase(obj_test_base.BaseDbObjectTestCase,
         self.assertNotIn('device_profile',
                          port_v1_6['versioned_object.data'])
 
+    def test_v1_8_to_v1_7_drops_hints(self):
+        port_new = self._create_test_port()
+        port_v1_7 = port_new.obj_to_primitive(target_version='1.7')
+        self.assertNotIn('hints',
+                         port_v1_7['versioned_object.data'])
+
+    def test_v1_10_to_v1_9_drops_trusted(self):
+        port_new = self._create_test_port(trusted=True)
+        port_v1_9 = port_new.obj_to_primitive(target_version='1.9')
+        self.assertNotIn('trusted',
+                         port_v1_9['versioned_object.data'])
+
     def test_get_ports_ids_by_security_groups_except_router(self):
         sg_id = self._create_test_security_group_id()
         filter_owner = constants.ROUTER_INTERFACE_OWNERS_SNAT
