@@ -19,17 +19,18 @@
 from neutron_lib.api import validators
 from neutron_lib import constants
 from neutron_lib.db import model_base
+from neutron_lib.plugins.ml2 import api
 import oslo_db.sqlalchemy.session
 import sqlalchemy as sa
 from sqlalchemy import orm
 
 from neutron.api import extensions
 from neutron.db import models_v2
-from neutron.plugins.ml2 import driver_api
+from neutron.objects import subnet as subnet_obj
 from neutron.tests.unit.plugins.ml2 import extensions as test_extensions
 
 
-class TestExtensionDriverBase(driver_api.ExtensionDriver):
+class TestExtensionDriverBase(api.ExtensionDriver):
     _supported_extension_aliases = 'fake_extension'
 
     def initialize(self):
@@ -42,7 +43,7 @@ class TestExtensionDriverBase(driver_api.ExtensionDriver):
 
 class TestExtensionDriver(TestExtensionDriverBase):
     def initialize(self):
-        super(TestExtensionDriver, self).initialize()
+        super().initialize()
         # keep track of values
         self.val_by_id = {}
 
@@ -59,13 +60,21 @@ class TestExtensionDriver(TestExtensionDriverBase):
         assert(isinstance(result, dict))
         assert(result['id'] is not None)
 
-    def _check_extend(self, session, result, db_entry,
-                      expected_db_entry_class):
+    def _check_extend(self, session, result, entry,
+                      expected_db_entry_class, expected_obj_entry_class=None):
+        # TODO(slaweq): After converting all code to use Subnet OVO,
+        # expected_db_entry_class can be removed as only OVO object
+        # should be expected here
         assert(isinstance(session, oslo_db.sqlalchemy.session.Session))
         assert(isinstance(result, dict))
         assert(result['id'] is not None)
-        assert(isinstance(db_entry, expected_db_entry_class))
-        assert(db_entry.id == result['id'])
+        if expected_obj_entry_class:
+            assert(isinstance(entry,
+                              (expected_db_entry_class,
+                               expected_obj_entry_class)))
+        else:
+            assert(isinstance(entry, expected_db_entry_class))
+        assert(entry.id == result['id'])
 
     def _store_change(self, result, data, field):
         if field in data and data[field] != constants.ATTR_NOT_SPECIFIED:
@@ -102,7 +111,10 @@ class TestExtensionDriver(TestExtensionDriverBase):
         result['subnet_extension'] = self.val_by_id[result['id']]
 
     def extend_subnet_dict(self, session, subnet_db, result):
-        self._check_extend(session, result, subnet_db, models_v2.Subnet)
+        self._check_extend(
+            session, result, subnet_db,
+            expected_db_entry_class=models_v2.Subnet,
+            expected_obj_entry_class=subnet_obj.Subnet)
         result['subnet_extension'] = self.val_by_id.get(result['id'])
 
     def process_create_port(self, plugin_context, data, result):
@@ -129,7 +141,8 @@ class TestNetworkExtension(model_base.BASEV2):
     value = sa.Column(sa.String(64))
     network = orm.relationship(
         models_v2.Network,
-        backref=orm.backref('extension', cascade='delete', uselist=False))
+        backref=orm.backref('extension', cascade='delete', uselist=False,
+                            lazy='joined'))
 
 
 class TestSubnetExtension(model_base.BASEV2):
@@ -139,7 +152,8 @@ class TestSubnetExtension(model_base.BASEV2):
     value = sa.Column(sa.String(64))
     subnet = orm.relationship(
         models_v2.Subnet,
-        backref=orm.backref('extension', cascade='delete', uselist=False))
+        backref=orm.backref('extension', cascade='delete', uselist=False,
+                            lazy='joined'))
 
 
 class TestPortExtension(model_base.BASEV2):
@@ -149,7 +163,8 @@ class TestPortExtension(model_base.BASEV2):
     value = sa.Column(sa.String(64))
     port = orm.relationship(
         models_v2.Port,
-        backref=orm.backref('extension', cascade='delete', uselist=False))
+        backref=orm.backref('extension', cascade='delete', uselist=False,
+                            lazy='joined'))
 
 
 class TestDBExtensionDriver(TestExtensionDriverBase):
