@@ -15,6 +15,7 @@ import collections
 import copy
 from unittest import mock
 
+from oslo_utils import uuidutils
 from ovsdbapp.backend import ovs_idl
 
 from neutron.common.ovn import constants as ovn_const
@@ -161,20 +162,26 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
                               'lr-name-d'}},
             {'name': utils.ovn_name('lr-id-e'),
              'external_ids': {ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY:
-                              'lr-name-e'}}],
+                              'lr-name-e'}},
+            {'name': utils.ovn_name('lr-id-f'),
+             'external_ids': {ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY:
+                              'lr-name-f'}}],
         'lrouter_ports': [
             {'name': utils.ovn_lrouter_port_name('orp-id-a1'),
-             'external_ids': {ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY:
-                              'lr-id-a'},
+             'external_ids': {ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY: 'lr-id-a',
+                              ovn_const.OVN_ROUTER_IS_EXT_GW: str(True)},
              'networks': ['10.0.1.0/24'],
              'options': {ovn_const.OVN_GATEWAY_CHASSIS_KEY: 'host-1'}},
             {'name': utils.ovn_lrouter_port_name('orp-id-a2'),
-             'external_ids': {}, 'networks': ['10.0.2.0/24'],
+             'external_ids': {ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY: 'lr-id-a',
+                              ovn_const.OVN_ROUTER_IS_EXT_GW: str(True)},
+             'networks': ['10.0.2.0/24'],
              'options': {ovn_const.OVN_GATEWAY_CHASSIS_KEY: 'host-1'}},
             {'name': utils.ovn_lrouter_port_name('orp-id-a3'),
-             'external_ids': {}, 'networks': ['10.0.3.0/24'],
-             'options': {ovn_const.OVN_GATEWAY_CHASSIS_KEY:
-                         ovn_const.OVN_GATEWAY_INVALID_CHASSIS}},
+             'external_ids': {ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY: 'lr-id-a',
+                              ovn_const.OVN_ROUTER_IS_EXT_GW: str(True)},
+             'networks': ['10.0.3.0/24'],
+             'options': {ovn_const.OVN_GATEWAY_CHASSIS_KEY: None}},
             {'name': 'xrp-id-b1',
              'external_ids': {}, 'networks': ['20.0.1.0/24']},
             {'name': utils.ovn_lrouter_port_name('orp-id-b2'),
@@ -182,7 +189,15 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
              'options': {ovn_const.OVN_GATEWAY_CHASSIS_KEY: 'host-2'}},
             {'name': utils.ovn_lrouter_port_name('orp-id-b3'),
              'external_ids': {}, 'networks': ['20.0.3.0/24'],
+             'options': {}},
+            {'name': utils.ovn_lrouter_port_name('gwc'),
+             'external_ids': {ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY: 'lr-id-f',
+                              ovn_const.OVN_ROUTER_IS_EXT_GW: str(True)},
+             'networks': ['10.0.4.0/24'],
              'options': {}}],
+        'gateway_chassis': [
+            {'chassis_name': 'fake-chassis',
+             'name': utils.ovn_lrouter_port_name('gwc') + '_fake-chassis'}],
         'static_routes': [{'ip_prefix': '20.0.0.0/16',
                            'nexthop': '10.0.3.253'},
                           {'ip_prefix': '10.0.0.0/16',
@@ -193,11 +208,15 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
                   'type': 'snat'},
                  {'external_ip': '20.0.2.4', 'logical_ip': '10.0.0.4',
                   'type': 'dnat_and_snat', 'external_mac': [],
-                  'logical_port': []},
+                  'logical_port': [],
+                  'external_ids': {ovn_const.OVN_FIP_EXT_ID_KEY: 'fip_id_a'},
+                  'gateway_port': uuidutils.generate_uuid()},
                  {'external_ip': '20.0.2.5', 'logical_ip': '10.0.0.5',
                   'type': 'dnat_and_snat',
                   'external_mac': ['00:01:02:03:04:05'],
-                  'logical_port': ['lsp-id-001']}],
+                  'logical_port': ['lsp-id-001'],
+                  'external_ids': {ovn_const.OVN_FIP_EXT_ID_KEY: 'fip_id_b'},
+                  'gateway_port': []}],
         'acls': [
             {'unit_test_id': 1,
              'action': 'allow-related', 'direction': 'from-lport',
@@ -317,7 +336,12 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
                 utils.ovn_lrouter_port_name('orp-id-a3')],
             utils.ovn_name('lr-id-b'): [
                 'xrp-id-b1',
-                utils.ovn_lrouter_port_name('orp-id-b2')]},
+                utils.ovn_lrouter_port_name('orp-id-b2')],
+            utils.ovn_name('lr-id-f'): [
+                utils.ovn_lrouter_port_name('gwc')]},
+        'lrptogwc': {
+            utils.ovn_lrouter_port_name('gwc'): [
+                utils.ovn_lrouter_port_name('gwc') + '_fake-chassis']},
         'lrtosroute': {
             utils.ovn_name('lr-id-a'): ['20.0.0.0/16'],
             utils.ovn_name('lr-id-b'): ['10.0.0.0/16']
@@ -334,7 +358,7 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
     }
 
     def setUp(self):
-        super(TestNBImplIdlOvn, self).setUp()
+        super().setUp()
 
         self.lswitch_table = fakes.FakeOvsdbTable.create_one_ovsdb_table()
         self.lsp_table = fakes.FakeOvsdbTable.create_one_ovsdb_table()
@@ -346,6 +370,7 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
         self.dhcp_table = fakes.FakeOvsdbTable.create_one_ovsdb_table()
         self.address_set_table = fakes.FakeOvsdbTable.create_one_ovsdb_table()
         self.lb_table = fakes.FakeOvsdbTable.create_one_ovsdb_table()
+        self.gwc_table = fakes.FakeOvsdbTable.create_one_ovsdb_table()
 
         self._tables = {}
         self._tables['Logical_Switch'] = self.lswitch_table
@@ -358,6 +383,7 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
         self._tables['Address_Set'] = self.address_set_table
         self._tables['Load_Balancer'] = self.lb_table
         self._tables['NAT'] = self.nat_table
+        self._tables['Gateway_Chassis'] = self.gwc_table
 
         with mock.patch.object(impl_idl_ovn.OvsdbNbOvnIdl, 'from_worker',
                                return_value=mock.Mock()):
@@ -379,16 +405,23 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
             TestNBImplIdlOvn.fake_associations['lstolsp'],
             self.lswitch_table, self.lsp_table,
             'name', 'name', 'ports')
-        # Load Routers and Router Ports
+        # Load Routers, Router Ports and Gateway Chassis
         fake_lrouters = TestNBImplIdlOvn.fake_set['lrouters']
         self._load_ovsdb_fake_rows(self.lrouter_table, fake_lrouters)
         fake_lrps = TestNBImplIdlOvn.fake_set['lrouter_ports']
         self._load_ovsdb_fake_rows(self.lrp_table, fake_lrps)
+        fake_gwc = TestNBImplIdlOvn.fake_set['gateway_chassis']
+        self._load_ovsdb_fake_rows(self.gwc_table, fake_gwc)
         # Associate routers and router ports
         self._construct_ovsdb_references(
             TestNBImplIdlOvn.fake_associations['lrtolrp'],
             self.lrouter_table, self.lrp_table,
             'name', 'name', 'ports')
+        # Associate router ports and gateway chassis
+        self._construct_ovsdb_references(
+            TestNBImplIdlOvn.fake_associations['lrptogwc'],
+            self.lrp_table, self.gwc_table,
+            'name', 'name', 'gateway_chassis')
         # Load static routes
         fake_sroutes = TestNBImplIdlOvn.fake_set['static_routes']
         self._load_ovsdb_fake_rows(self.sroute_table, fake_sroutes)
@@ -446,13 +479,39 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
                      'provnet_ports': []}]
         self.assertCountEqual(mapping, expected)
 
-    def test_get_all_logical_routers_with_rports(self):
+    def _test_get_all_logical_routers_with_rports(self, is_gw_port):
         # Test empty
         mapping = self.nb_ovn_idl.get_all_logical_switches_with_ports()
         self.assertCountEqual(mapping, {})
         # Test loaded values
         self._load_nb_db()
+
+        # Test with gateway_port_support enabled
+        utils.is_nat_gateway_port_supported = mock.Mock()
+        utils.is_nat_gateway_port_supported.return_value = is_gw_port
         mapping = self.nb_ovn_idl.get_all_logical_routers_with_rports()
+        lra_nat = self._find_ovsdb_fake_row(self.nat_table,
+                                            'external_ip', '20.0.2.4')
+        lrb_nat = self._find_ovsdb_fake_row(self.nat_table,
+                                            'external_ip', '20.0.2.5')
+
+        lra_fip = {'external_ip': '20.0.2.4',
+                   'logical_ip': '10.0.0.4',
+                   'type': 'dnat_and_snat',
+                   'external_ids': {ovn_const.OVN_FIP_EXT_ID_KEY: 'fip_id_a'},
+                   'uuid': lra_nat.uuid}
+        lrb_fip = {'external_ip': '20.0.2.5',
+                   'logical_ip': '10.0.0.5',
+                   'type': 'dnat_and_snat',
+                   'external_mac': '00:01:02:03:04:05',
+                   'logical_port': 'lsp-id-001',
+                   'external_ids': {ovn_const.OVN_FIP_EXT_ID_KEY: 'fip_id_b'},
+                   'uuid': lrb_nat.uuid}
+
+        if is_gw_port:
+            lra_fip['gateway_port'] = lra_nat.gateway_port
+            lrb_fip['gateway_port'] = lrb_nat.gateway_port
+
         expected = [{'name': 'lr-id-a',
                      'ports': {'orp-id-a1': ['10.0.1.0/24'],
                                'orp-id-a2': ['10.0.2.0/24'],
@@ -471,22 +530,24 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
                      'snats': [{'external_ip': '20.0.2.1',
                                 'logical_ip': '10.0.0.0/24',
                                 'type': 'snat'}],
-                     'dnat_and_snats': [{'external_ip': '20.0.2.4',
-                                         'logical_ip': '10.0.0.4',
-                                         'type': 'dnat_and_snat'},
-                                        {'external_ip': '20.0.2.5',
-                                         'logical_ip': '10.0.0.5',
-                                         'type': 'dnat_and_snat',
-                                         'external_mac': '00:01:02:03:04:05',
-                                         'logical_port': 'lsp-id-001'}]},
+                     'dnat_and_snats': [lra_fip, lrb_fip]},
                     {'name': 'lr-id-c', 'ports': {}, 'static_routes': [],
                      'snats': [], 'dnat_and_snats': []},
                     {'name': 'lr-id-d', 'ports': {}, 'static_routes': [],
                      'snats': [], 'dnat_and_snats': []},
                     {'name': 'lr-id-e', 'ports': {}, 'static_routes': [],
+                     'snats': [], 'dnat_and_snats': []},
+                    {'name': 'lr-id-f', 'static_routes': [],
+                     'ports': {'gwc': ['10.0.4.0/24']},
                      'snats': [], 'dnat_and_snats': []}]
         self.assertCountEqual(mapping, expected)
 
+    def test_get_all_logical_routers_with_rports(self):
+        self._test_get_all_logical_routers_with_rports(True)
+
+    def test_get_all_logical_routers_with_rports_without_nat_gw_port(self):
+        self._test_get_all_logical_routers_with_rports(False)
+
     def test_get_acls_for_lswitches(self):
         self._load_nb_db()
         # Test neutron switches
@@ -556,12 +617,16 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
 
     def test_get_all_chassis_gateway_bindings(self):
         self._load_nb_db()
+
+        # NOTE(fnordahl): The `Gateway_Chassis` table being present without
+        # proper associations fools the test, remove for now.
+        del(self._tables['Gateway_Chassis'])
+
         bindings = self.nb_ovn_idl.get_all_chassis_gateway_bindings()
         expected = {'host-1': [utils.ovn_lrouter_port_name('orp-id-a1'),
                                utils.ovn_lrouter_port_name('orp-id-a2')],
                     'host-2': [utils.ovn_lrouter_port_name('orp-id-b2')],
-                    ovn_const.OVN_GATEWAY_INVALID_CHASSIS: [
-                        utils.ovn_name('orp-id-a3')]}
+                    }
         self.assertCountEqual(bindings, expected)
 
         bindings = self.nb_ovn_idl.get_all_chassis_gateway_bindings([])
@@ -574,6 +639,11 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
 
     def test_get_gateway_chassis_binding(self):
         self._load_nb_db()
+
+        # NOTE(fnordahl): The `Gateway_Chassis` table being present without
+        # proper associations fools the test, remove for now.
+        del(self._tables['Gateway_Chassis'])
+
         chassis = self.nb_ovn_idl.get_gateway_chassis_binding(
             utils.ovn_lrouter_port_name('orp-id-a1'))
         self.assertEqual(chassis, ['host-1'])
@@ -582,7 +652,7 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
         self.assertEqual(chassis, ['host-2'])
         chassis = self.nb_ovn_idl.get_gateway_chassis_binding(
             utils.ovn_lrouter_port_name('orp-id-a3'))
-        self.assertEqual(chassis, ['neutron-ovn-invalid-chassis'])
+        self.assertEqual([], chassis)
         chassis = self.nb_ovn_idl.get_gateway_chassis_binding(
             utils.ovn_lrouter_port_name('orp-id-b3'))
         self.assertEqual([], chassis)
@@ -591,6 +661,11 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
 
     def test_get_unhosted_gateways(self):
         self._load_nb_db()
+
+        # NOTE(fnordahl): The `Gateway_Chassis` table being present without
+        # proper associations fools the test, remove for now.
+        del(self._tables['Gateway_Chassis'])
+
         # Port physnet-dict
         port_physnet_dict = {
             'orp-id-a1': 'physnet1',  # scheduled
@@ -626,6 +701,11 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
 
     def test_get_unhosted_gateways_deleted_physnet(self):
         self._load_nb_db()
+
+        # NOTE(fnordahl): The `Gateway_Chassis` table being present without
+        # proper associations fools the test, remove for now.
+        del(self._tables['Gateway_Chassis'])
+
         # The LRP is on host-2 now
         router_row = self._find_ovsdb_fake_row(self.lrp_table,
                                                'name', 'lrp-orp-id-a1')
@@ -783,11 +863,6 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
         self.assertEqual(len(dhcp_options['subnets']), 3)
         self.assertEqual(len(dhcp_options['ports_v4']), 2)
 
-    def test_get_address_sets(self):
-        self._load_nb_db()
-        address_sets = self.nb_ovn_idl.get_address_sets()
-        self.assertEqual(len(address_sets), 4)
-
     def test_get_router_floatingip_lbs(self):
         lrouter_name = 'rtr_name'
         # Empty
@@ -813,6 +888,42 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
         lb = self.nb_ovn_idl.get_floatingip_in_nat_or_lb(fip_id)
         self.assertEqual(lb['_uuid'], lb_row.uuid)
 
+    def test_get_lrouter_gw_ports_legacy_option(self):
+        self._load_nb_db()
+
+        gw1_row = self._find_ovsdb_fake_row(
+            self.lrp_table, 'name', utils.ovn_lrouter_port_name('orp-id-a1'))
+        gw2_row = self._find_ovsdb_fake_row(
+            self.lrp_table, 'name', utils.ovn_lrouter_port_name('orp-id-a2'))
+        gw3_row = self._find_ovsdb_fake_row(
+            self.lrp_table, 'name', utils.ovn_lrouter_port_name('orp-id-a3'))
+
+        gw_ports = self.nb_ovn_idl.get_lrouter_gw_ports(
+            utils.ovn_name('lr-id-a'))
+        self.assertEqual([gw1_row, gw2_row, gw3_row], gw_ports)
+
+    def test_get_lrouter_gw_ports_gwc(self):
+        self._load_nb_db()
+        gw1_row = self._find_ovsdb_fake_row(
+            self.lrp_table, 'name', utils.ovn_lrouter_port_name('gwc'))
+
+        gw_ports = self.nb_ovn_idl.get_lrouter_gw_ports(
+            utils.ovn_name('lr-id-f'))
+        self.assertEqual([gw1_row], gw_ports)
+
+    def test_get_lrouter_by_lrouter_port(self):
+        self.assertIsNone(
+            self.nb_ovn_idl.get_lrouter_by_lrouter_port('not_there'))
+        self._load_nb_db()
+        self.assertIsNone(
+            self.nb_ovn_idl.get_lrouter_by_lrouter_port('not_there'))
+        lr_row = self._find_ovsdb_fake_row(
+            self.lrouter_table,
+            'name',
+            utils.ovn_name('lr-id-a'))
+        lr = self.nb_ovn_idl.get_lrouter_by_lrouter_port('lrp-orp-id-a1')
+        self.assertEqual(lr.uuid, lr_row.uuid)
+
 
 class TestSBImplIdlOvnBase(TestDBImplIdlOvn):
 
@@ -820,7 +931,7 @@ class TestSBImplIdlOvnBase(TestDBImplIdlOvn):
         'chassis': [
             {
                 'hostname': 'fake-smartnic-dpu-chassis.fqdn',
-                'external_ids': {
+                'other_config': {
                     ovn_const.OVN_CMS_OPTIONS: (
                         'firstoption,'
                         'card-serial-number=fake-serial,'
@@ -832,7 +943,7 @@ class TestSBImplIdlOvnBase(TestDBImplIdlOvn):
     fake_associations = {}
 
     def setUp(self):
-        super(TestSBImplIdlOvnBase, self).setUp()
+        super().setUp()
 
         self.chassis_table = fakes.FakeOvsdbTable.create_one_ovsdb_table()
 
