@@ -13,17 +13,17 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
+from unittest import mock
 
-import eventlet
+from neutron_lib.services.trunk import constants
 import oslo_messaging
 from oslo_serialization import jsonutils
 from oslo_utils import uuidutils
 import testtools
 
 from neutron.api.rpc.handlers import resources_rpc
+from neutron.common import utils
 from neutron.objects import trunk as trunk_obj
-from neutron.services.trunk import constants
 from neutron.services.trunk.drivers.openvswitch.agent import exceptions
 from neutron.services.trunk.drivers.openvswitch.agent import ovsdb_handler
 from neutron.services.trunk.drivers.openvswitch.agent import trunk_manager
@@ -32,7 +32,7 @@ from neutron.tests import base
 
 class TestLockOnBridgeName(base.BaseTestCase):
     def setUp(self):
-        super(TestLockOnBridgeName, self).setUp()
+        super().setUp()
         self.lock_mock = mock.patch('oslo_concurrency.lockutils.lock').start()
         self.method_called = False
 
@@ -90,7 +90,7 @@ class TestIsTrunkServicePort(base.BaseTestCase):
 
 class TestBridgeHasInstancePort(base.BaseTestCase):
     def setUp(self):
-        super(TestBridgeHasInstancePort, self).setUp()
+        super().setUp()
         self.bridge = mock.Mock()
         self.present_interfaces = []
         self.bridge.get_iface_name_list.return_value = self.present_interfaces
@@ -116,7 +116,7 @@ class TestBridgeHasInstancePort(base.BaseTestCase):
 class TestOVSDBHandler(base.BaseTestCase):
     """Test that RPC or OVSDB failures do not cause crash."""
     def setUp(self):
-        super(TestOVSDBHandler, self).setUp()
+        super().setUp()
         self.ovsdb_handler = ovsdb_handler.OVSDBHandler(mock.sentinel.manager)
         mock.patch.object(self.ovsdb_handler, 'trunk_rpc').start()
         mock.patch.object(self.ovsdb_handler, 'trunk_manager').start()
@@ -130,19 +130,19 @@ class TestOVSDBHandler(base.BaseTestCase):
         self.fake_port = {
             'name': 'foo',
             'external_ids': {
-                'trunk_id': 'trunk_id',
+                'trunk_id': self.trunk_id,
                 'subport_ids': jsonutils.dumps(
                     [s.id for s in self.fake_subports]),
             }
         }
         self.subport_bindings = {
-            'trunk_id': [
+            self.trunk_id: [
                 {'id': subport.port_id,
                  'mac_address': 'mac'} for subport in self.fake_subports]}
 
     @mock.patch('neutron.agent.common.ovs_lib.OVSBridge')
-    @mock.patch('neutron.common.utils.wait_until_true',
-                side_effect=eventlet.TimeoutError)
+    @mock.patch.object(utils, 'wait_until_true',
+                       side_effect=utils.WaitTimeout)
     def test_handle_trunk_add_interface_wont_appear(self, wut, br):
         self.ovsdb_handler.handle_trunk_add('foo')
         self.assertTrue(self.trunk_manager.dispose_trunk.called)
@@ -150,23 +150,23 @@ class TestOVSDBHandler(base.BaseTestCase):
     @mock.patch('neutron.agent.common.ovs_lib.OVSBridge')
     def test_handle_trunk_add_rpc_failure(self, br):
         with mock.patch.object(self.ovsdb_handler, '_wire_trunk',
-                side_effect=oslo_messaging.MessagingException):
+                               side_effect=oslo_messaging.MessagingException):
             with mock.patch.object(ovsdb_handler, 'bridge_has_instance_port',
-                    return_value=True):
+                                   return_value=True):
                 self.ovsdb_handler.handle_trunk_add('foo')
 
     @mock.patch('neutron.agent.common.ovs_lib.OVSBridge')
     def test_handle_trunk_add_ovsdb_failure(self, br):
         with mock.patch.object(self.ovsdb_handler, '_wire_trunk',
-                side_effect=RuntimeError):
+                               side_effect=RuntimeError):
             with mock.patch.object(ovsdb_handler, 'bridge_has_instance_port',
-                    return_value=True):
+                                   return_value=True):
                 self.ovsdb_handler.handle_trunk_add('foo')
 
     @mock.patch('neutron.agent.common.ovs_lib.OVSBridge')
     def test_handle_trunk_add_parent_port_not_found(self, br):
         with mock.patch.object(self.ovsdb_handler, '_get_parent_port',
-                side_effect=exceptions.ParentPortNotFound):
+                               side_effect=exceptions.ParentPortNotFound):
             # do not wait the default timeout
             self.ovsdb_handler.timeout = 1
             self.ovsdb_handler.handle_trunk_add('foo')
@@ -179,7 +179,8 @@ class TestOVSDBHandler(base.BaseTestCase):
             self.ovsdb_handler.handle_trunk_add('foo')
         self.assertFalse(has_port_mock.called)
 
-    def test_handle_trunk_remove_trunk_manager_failure(self):
+    @mock.patch('neutron.agent.common.ovs_lib.OVSBridge')
+    def test_handle_trunk_remove_trunk_manager_failure(self, br):
         with mock.patch.object(self.ovsdb_handler, '_get_trunk_metadata',
                 side_effect=trunk_manager.TrunkManagerError(error='error')):
             self.ovsdb_handler.handle_trunk_remove('foo', self.fake_port)
@@ -201,19 +202,19 @@ class TestOVSDBHandler(base.BaseTestCase):
                 trunk_manager.TrunkManagerError(error='error'))
 
             status = self.ovsdb_handler.wire_subports_for_trunk(
-                None, 'trunk_id', self.fake_subports)
+                None, self.trunk_id, self.fake_subports)
             self.assertTrue(f.call_count)
-            self.assertEqual(constants.DEGRADED_STATUS, status)
+            self.assertEqual(constants.TRUNK_DEGRADED_STATUS, status)
 
     @mock.patch('neutron.agent.common.ovs_lib.OVSBridge')
     def test_wire_subports_for_trunk_ovsdb_failure(self, br):
         self.ovsdb_handler.trunk_rpc.update_subport_bindings.return_value = (
             self.subport_bindings)
         with mock.patch.object(self.ovsdb_handler, '_set_trunk_metadata',
-                side_effect=RuntimeError):
+                               side_effect=RuntimeError):
             status = self.ovsdb_handler.wire_subports_for_trunk(
-                None, 'trunk_id', self.fake_subports)
-        self.assertEqual(constants.DEGRADED_STATUS, status)
+                None, self.trunk_id, self.fake_subports)
+        self.assertEqual(constants.TRUNK_DEGRADED_STATUS, status)
 
     @mock.patch('neutron.agent.common.ovs_lib.OVSBridge')
     def test_unwire_subports_for_trunk_port_not_found(self, br):
@@ -222,19 +223,19 @@ class TestOVSDBHandler(base.BaseTestCase):
         with mock.patch.object(self.ovsdb_handler, '_update_trunk_metadata',
                 side_effect=exceptions.ParentPortNotFound(bridge='foo_br')):
             status = self.ovsdb_handler.unwire_subports_for_trunk(
-                'trunk_id', ['subport_id'])
-        self.assertEqual(constants.ACTIVE_STATUS, status)
+                self.trunk_id, ['subport_id'])
+        self.assertEqual(constants.TRUNK_ACTIVE_STATUS, status)
 
     @mock.patch('neutron.agent.common.ovs_lib.OVSBridge')
     def test_unwire_subports_for_trunk_trunk_manager_failure(self, br):
         with mock.patch.object(
-            self.ovsdb_handler, '_update_trunk_metadata') as f:
+                self.ovsdb_handler, '_update_trunk_metadata') as f:
             self.trunk_manager.remove_sub_port.side_effect = (
                 trunk_manager.TrunkManagerError(error='error'))
             status = self.ovsdb_handler.unwire_subports_for_trunk(
                 'foo_trunk_id', ['subport_id'])
             self.assertTrue(f.call_count)
-            self.assertEqual(constants.DEGRADED_STATUS, status)
+            self.assertEqual(constants.TRUNK_DEGRADED_STATUS, status)
 
     def test__wire_trunk_get_trunk_details_failure(self):
         self.trunk_manager.get_port_uuid_from_external_ids.side_effect = (
@@ -255,7 +256,20 @@ class TestOVSDBHandler(base.BaseTestCase):
         self.ovsdb_handler._wire_trunk(mock.Mock(), self.fake_port)
         trunk_rpc = self.ovsdb_handler.trunk_rpc
         trunk_rpc.update_trunk_status.assert_called_once_with(
-            mock.ANY, mock.ANY, constants.ERROR_STATUS)
+            mock.ANY, mock.ANY, constants.TRUNK_ERROR_STATUS)
+
+    def test__wire_trunk_rewire_trunk_failure(self):
+        with mock.patch.object(self.ovsdb_handler,
+                               'unwire_subports_for_trunk') as f,\
+                mock.patch.object(self.ovsdb_handler,
+                                  'get_connected_subports_for_trunk') as g:
+            g.return_value = ['stale_port']
+            f.return_value = constants.TRUNK_DEGRADED_STATUS
+            self.ovsdb_handler._wire_trunk(
+                mock.Mock(), self.fake_port, rewire=True)
+            trunk_rpc = self.ovsdb_handler.trunk_rpc
+            trunk_rpc.update_trunk_status.assert_called_once_with(
+                mock.ANY, mock.ANY, constants.TRUNK_DEGRADED_STATUS)
 
     def test__wire_trunk_report_trunk_called_on_wiring(self):
         with mock.patch.object(self.trunk_manager, 'create_trunk'),\
@@ -278,11 +292,11 @@ class TestOVSDBHandler(base.BaseTestCase):
                  'subport_ids': '[]'})
 
     def test__get_current_status_active(self):
-        self.assertEqual(constants.ACTIVE_STATUS,
+        self.assertEqual(constants.TRUNK_ACTIVE_STATUS,
                          self.ovsdb_handler._get_current_status([], []))
 
     def test__get_current_status_degraded(self):
-        self.assertEqual(constants.DEGRADED_STATUS,
+        self.assertEqual(constants.TRUNK_DEGRADED_STATUS,
                          self.ovsdb_handler._get_current_status(
                              [mock.ANY], []))
 
@@ -322,3 +336,15 @@ class TestOVSDBHandler(base.BaseTestCase):
         expected_subport_ids = '["foo_subport_1"]'
         self._test__update_trunk_metadata_wire_flag(
             mock_br, False, external_ids, subport_ids, expected_subport_ids)
+
+    @mock.patch('neutron.agent.common.ovs_lib.OVSBridge')
+    def test__update_trunk_metadata_no_tbr_no_raise(self, br):
+        mock_br = br.return_value
+        mock_br.configure_mock(**{'bridge_exists.return_value': False})
+        try:
+            self.ovsdb_handler._update_trunk_metadata(
+                mock_br, None, self.trunk_id, ['foo_subport_1'])
+        except RuntimeError:
+            self.fail(
+                "_update_trunk_metadata() should not raise RuntimeError "
+                "when the trunk bridge does not exist")
