@@ -18,7 +18,6 @@ from neutron_lib.api.definitions import portbindings
 from neutron_lib.callbacks import events
 from neutron_lib.callbacks import registry
 from neutron_lib.callbacks import resources
-from neutron_lib import constants as nlib_consts
 from neutron_lib import exceptions as n_exc
 from neutron_lib.services.trunk import constants as trunk_consts
 from oslo_config import cfg
@@ -26,13 +25,9 @@ from oslo_config import cfg
 from neutron.common.ovn.constants import OVN_ML2_MECH_DRIVER_NAME
 from neutron.objects.ports import Port
 from neutron.objects.ports import PortBinding
-from neutron.objects import trunk as trunk_objects
-from neutron.services.trunk import drivers
 from neutron.services.trunk.drivers.ovn import trunk_driver
-from neutron.services.trunk import plugin as trunk_plugin
 from neutron.tests import base
 from neutron.tests.unit import fake_resources
-from neutron.tests.unit.plugins.ml2 import test_plugin
 
 
 class FakePayload:
@@ -43,7 +38,7 @@ class FakePayload:
 
 class TestTrunkHandler(base.BaseTestCase):
     def setUp(self):
-        super(TestTrunkHandler, self).setUp()
+        super().setUp()
         self.context = mock.Mock()
         self.plugin_driver = mock.Mock()
         self.plugin_driver._plugin = mock.Mock()
@@ -435,74 +430,7 @@ class TestTrunkHandler(base.BaseTestCase):
         m__unset_sub_ports.assert_not_called()
 
 
-class TestTrunkHandlerWithPlugin(test_plugin.Ml2PluginV2TestCase):
-    def setUp(self):
-        super().setUp()
-        self.drivers_patch = mock.patch.object(drivers, 'register').start()
-        self.compat_patch = mock.patch.object(
-            trunk_plugin.TrunkPlugin, 'check_compatibility').start()
-        self.trunk_plugin = trunk_plugin.TrunkPlugin()
-        self.trunk_plugin.add_segmentation_type('vlan', lambda x: True)
-        self.plugin_driver = mock.Mock()
-        self.trunk_handler = trunk_driver.OVNTrunkHandler(self.plugin_driver)
-
-    def _create_test_trunk(self, port, subports=None):
-        subports = subports if subports else []
-        trunk = {'port_id': port['port']['id'],
-                 'project_id': 'test_tenant',
-                 'sub_ports': subports}
-        response = (
-            self.trunk_plugin.create_trunk(self.context, {'trunk': trunk}))
-        return response
-
-    def _get_trunk_obj(self, trunk_id):
-        return trunk_objects.Trunk.get_object(self.context, id=trunk_id)
-
-    def test_parent_active_triggers_trunk_active(self):
-        with self.port() as new_parent:
-            new_parent['status'] = nlib_consts.PORT_STATUS_ACTIVE
-            old_parent = {'status': nlib_consts.PORT_STATUS_DOWN}
-            old_trunk = self._create_test_trunk(new_parent)
-            old_trunk = self._get_trunk_obj(old_trunk['id'])
-            old_trunk.update(status=trunk_consts.TRUNK_DOWN_STATUS)
-            trunk_details = {'trunk_id': old_trunk.id}
-            new_parent['trunk_details'] = trunk_details
-            old_parent['trunk_details'] = trunk_details
-            self.trunk_handler.port_updated(
-                resources.PORT,
-                events.AFTER_UPDATE,
-                None,
-                payload=events.DBEventPayload(
-                    self.context, states=(old_parent, new_parent)))
-            new_trunk = self._get_trunk_obj(old_trunk.id)
-            self.assertEqual(
-                trunk_consts.TRUNK_ACTIVE_STATUS, new_trunk.status)
-
-    def test_parent_build_does_not_trigger_trunk_active(self):
-        with self.port() as new_parent:
-            new_parent['status'] = nlib_consts.PORT_STATUS_BUILD
-            old_parent = {'status': nlib_consts.PORT_STATUS_DOWN}
-            old_trunk = self._create_test_trunk(new_parent)
-            old_trunk = self._get_trunk_obj(old_trunk['id'])
-            old_trunk.update(status=trunk_consts.TRUNK_DOWN_STATUS)
-            trunk_details = {'trunk_id': old_trunk.id}
-            new_parent['trunk_details'] = trunk_details
-            old_parent['trunk_details'] = trunk_details
-            self.trunk_handler.port_updated(
-                resources.PORT,
-                events.AFTER_UPDATE,
-                None,
-                payload=events.DBEventPayload(
-                    self.context, states=(old_parent, new_parent)))
-            new_trunk = self._get_trunk_obj(old_trunk.id)
-            self.assertNotEqual(
-                trunk_consts.TRUNK_ACTIVE_STATUS, new_trunk.status)
-
-
 class TestTrunkDriver(base.BaseTestCase):
-    def setUp(self):
-        super(TestTrunkDriver, self).setUp()
-
     def test_is_loaded(self):
         driver = trunk_driver.OVNTrunkDriver.create(mock.Mock())
         cfg.CONF.set_override('mechanism_drivers',
